@@ -3,7 +3,7 @@
  * @ingroup SQLiteCpp
  * @brief   Definition of the SQLITECPP_ASSERT() macro.
  *
- * Copyright (c) 2012-2020 Sebastien Rombauts (sebastien.rombauts@gmail.com)
+ * Copyright (c) 2012-2024 Sebastien Rombauts (sebastien.rombauts@gmail.com)
  *
  * Distributed under the MIT License (MIT) (See accompanying file LICENSE.txt
  * or copy at http://opensource.org/licenses/MIT)
@@ -12,7 +12,6 @@
 
 #include <cassert>
 
-
 /**
  * SQLITECPP_ASSERT SQLITECPP_ASSERT() is used in destructors, where exceptions shall not be thrown
  *
@@ -25,9 +24,10 @@
 // if an assert handler is provided by user code, use it instead of assert()
 namespace SQLite
 {
-    // declaration of the assert handler to define in user code
-    void assertion_failed(const char* apFile, const long apLine, const char* apFunc,
-                          const char* apExpr, const char* apMsg);
+
+// declaration of the assert handler to define in user code
+void assertion_failed(const char* apFile, const int apLine, const char* apFunc,
+                        const char* apExpr, const char* apMsg);
 
 #ifdef _MSC_VER
     #define __func__ __FUNCTION__
@@ -35,6 +35,7 @@ namespace SQLite
 // call the assert handler provided by user code
 #define SQLITECPP_ASSERT(expression, message) \
     if (!(expression))  SQLite::assertion_failed(__FILE__, __LINE__, __func__, #expression, message)
+
 } // namespace SQLite
 
 #else
