@@ -1,10 +1,11 @@
+#include <glm/ext/scalar_ulp.hpp>
+
 #define GLM_ENABLE_EXPERIMENTAL
 #include <glm/gtc/type_precision.hpp>
 #include <glm/gtx/fast_trigonometry.hpp>
 #include <glm/gtx/integer.hpp>
 #include <glm/gtx/common.hpp>
 #include <glm/gtc/constants.hpp>
-#include <glm/gtc/ulp.hpp>
 #include <glm/gtc/vec1.hpp>
 #include <glm/trigonometric.hpp>
 #include <cmath>
@@ -14,18 +15,18 @@
 
 namespace fastCos
 {
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastCos(i);
 
 		const std::clock_t timestamp2 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::cos(i);
 
 		const std::clock_t timestamp3 = std::clock();
@@ -34,6 +35,8 @@ namespace fastCos
 		std::printf("fastCos Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("cos Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastCos
@@ -48,18 +51,18 @@ namespace fastSin
 	}
 	*/
 
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastSin(i);
 
 		const std::clock_t timestamp2 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::sin(i);
 
 		const std::clock_t timestamp3 = std::clock();
@@ -68,24 +71,26 @@ namespace fastSin
 		std::printf("fastSin Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("sin Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastSin
 
 namespace fastTan
 {
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastTan(i);
 
 		const std::clock_t timestamp2 = std::clock();
-		for (float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for (float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::tan(i);
 
 		const std::clock_t timestamp3 = std::clock();
@@ -94,24 +99,26 @@ namespace fastTan
 		std::printf("fastTan Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("tan Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastTan
 
 namespace fastAcos
 {
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastAcos(i);
 
 		const std::clock_t timestamp2 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::acos(i);
 
 		const std::clock_t timestamp3 = std::clock();
@@ -121,22 +128,24 @@ namespace fastAcos
 		std::printf("fastAcos Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("acos Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastAcos
 
 namespace fastAsin
 {
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastAsin(i);
 		const std::clock_t timestamp2 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::asin(i);
 		const std::clock_t timestamp3 = std::clock();
 		const std::clock_t time_fast = timestamp2 - timestamp1;
@@ -144,22 +153,24 @@ namespace fastAsin
 		std::printf("fastAsin Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("asin Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastAsin
 
 namespace fastAtan
 {
-	int perf(bool NextFloat)
+	static int perf(bool NextFloat)
 	{
 		const float begin = -glm::pi<float>();
 		const float end = glm::pi<float>();
 		float result = 0.f;
 		const std::clock_t timestamp1 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::fastAtan(i);
 		const std::clock_t timestamp2 = std::clock();
-		for(float i = begin; i < end; i = NextFloat ? glm::next_float(i) : i += 0.1f)
+		for(float i = begin; i < end; i = NextFloat ? glm::nextFloat(i) : i + 0.1f)
 			result = glm::atan(i);
 		const std::clock_t timestamp3 = std::clock();
 		const std::clock_t time_fast = timestamp2 - timestamp1;
@@ -167,6 +178,8 @@ namespace fastAtan
 		std::printf("fastAtan Time %d clocks\n", static_cast<int>(time_fast));
 		std::printf("atan Time %d clocks\n", static_cast<int>(time_default));
 
+		(void) result; // Silence set but not used warning
+
 		return time_fast <= time_default ? 0 : 1;
 	}
 }//namespace fastAtan
@@ -175,11 +188,20 @@ namespace taylorCos
 {
 	using glm::qualifier;
 	using glm::length_t;
-	
+
+#	if (GLM_COMPILER & GLM_COMPILER_CLANG)
+#		pragma clang diagnostic push
+#		pragma clang diagnostic ignored "-Wglobal-constructors"
+#	endif
+
 	glm::vec4 const AngleShift(0.0f, glm::half_pi<float>(), glm::pi<float>(), glm::three_over_two_pi<float>());
 
+#	if (GLM_COMPILER & GLM_COMPILER_CLANG)
+#		pragma clang diagnostic pop
+#	endif
+
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesNewCos(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesNewCos(glm::vec<L, T, Q> const& x)
 	{
 		glm::vec<L, T, Q> const Powed2(x * x);
 		glm::vec<L, T, Q> const Powed4(Powed2 * Powed2);
@@ -192,9 +214,9 @@ namespace taylorCos
 			- Powed6 * static_cast<T>(0.00138888888888888888888888888889)
 			+ Powed8 * static_cast<T>(2.4801587301587301587301587301587e-5);
 	}
-
+/*
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesNewCos6(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesNewCos6(glm::vec<L, T, Q> const& x)
 	{
 		glm::vec<L, T, Q> const Powed2(x * x);
 		glm::vec<L, T, Q> const Powed4(Powed2 * Powed2);
@@ -205,9 +227,10 @@ namespace taylorCos
 			+ Powed4 * static_cast<T>(0.04166666666666666666666666666667)
 			- Powed6 * static_cast<T>(0.00138888888888888888888888888889);
 	}
-
+*/
+/*
 	template<glm::length_t L, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, float, Q> fastAbs(glm::vec<L, float, Q> x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, float, Q> fastAbs(glm::vec<L, float, Q> x)
 	{
 		int* Pointer = reinterpret_cast<int*>(&x[0]);
 		Pointer[0] &= 0x7fffffff;
@@ -218,32 +241,31 @@ namespace taylorCos
 	}
 
 	template<glm::length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastCosNew(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastCosNew(glm::vec<L, T, Q> const& x)
 	{
 		glm::vec<L, T, Q> const Angle0_PI(fastAbs(fmod(x + glm::pi<T>(), glm::two_pi<T>()) - glm::pi<T>()));
 		return taylorSeriesNewCos6(x);
-/*
-		vec<L, bool, Q> const FirstQuarterPi(lessThanEqual(Angle0_PI, vec<L, T, Q>(glm::half_pi<T>())));
+//		vec<L, bool, Q> const FirstQuarterPi(lessThanEqual(Angle0_PI, vec<L, T, Q>(glm::half_pi<T>())));
 
-		vec<L, T, Q> const RevertAngle(mix(vec<L, T, Q>(glm::pi<T>()), vec<L, T, Q>(0), FirstQuarterPi));
-		vec<L, T, Q> const ReturnSign(mix(vec<L, T, Q>(-1), vec<L, T, Q>(1), FirstQuarterPi));
-		vec<L, T, Q> const SectionAngle(RevertAngle - Angle0_PI);
+//		vec<L, T, Q> const RevertAngle(mix(vec<L, T, Q>(glm::pi<T>()), vec<L, T, Q>(0), FirstQuarterPi));
+//		vec<L, T, Q> const ReturnSign(mix(vec<L, T, Q>(-1), vec<L, T, Q>(1), FirstQuarterPi));
+//		vec<L, T, Q> const SectionAngle(RevertAngle - Angle0_PI);
 
-		return ReturnSign * taylorSeriesNewCos(SectionAngle);
-*/
+//		return ReturnSign * taylorSeriesNewCos(SectionAngle);
 	}
-
-	int perf_fastCosNew(float Begin, float End, std::size_t Samples)
+*/
+/*
+	static int perf_fastCosNew(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<glm::vec4> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = fastCosNew(AngleShift + glm::vec4(Begin + Steps * i));
+			Results[i] = fastCosNew(AngleShift + glm::vec4(Begin + Steps * static_cast<float>(i)));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -254,15 +276,15 @@ namespace taylorCos
 			Error += Results[i].x >= -1.0f && Results[i].x <= 1.0f ? 0 : 1;
 		return Error;
 	}
-
+*/
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> deterministic_fmod(glm::vec<L, T, Q> const& x, T y)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> deterministic_fmod(glm::vec<L, T, Q> const& x, T y)
 	{
 		return x - y * trunc(x / y);
 	}
 
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastCosDeterminisctic(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastCosDeterminisctic(glm::vec<L, T, Q> const& x)
 	{
 		glm::vec<L, T, Q> const Angle0_PI(abs(deterministic_fmod(x + glm::pi<T>(), glm::two_pi<T>()) - glm::pi<T>()));
 		glm::vec<L, bool, Q> const FirstQuarterPi(lessThanEqual(Angle0_PI, glm::vec<L, T, Q>(glm::half_pi<T>())));
@@ -274,17 +296,17 @@ namespace taylorCos
 		return ReturnSign * taylorSeriesNewCos(SectionAngle);
 	}
 
-	int perf_fastCosDeterminisctic(float Begin, float End, std::size_t Samples)
+	static int perf_fastCosDeterminisctic(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<glm::vec4> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = taylorCos::fastCosDeterminisctic(AngleShift + glm::vec4(Begin + Steps * i));
+			Results[i] = taylorCos::fastCosDeterminisctic(AngleShift + glm::vec4(Begin + Steps * static_cast<float>(i)));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -297,7 +319,7 @@ namespace taylorCos
 	}
 
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesRefCos(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> taylorSeriesRefCos(glm::vec<L, T, Q> const& x)
 	{
 		return static_cast<T>(1)
 			- (x * x) / glm::factorial(static_cast<T>(2))
@@ -307,7 +329,7 @@ namespace taylorCos
 	}
 
 	template<length_t L, typename T, qualifier Q>
-	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastRefCos(glm::vec<L, T, Q> const& x)
+	static GLM_FUNC_QUALIFIER glm::vec<L, T, Q> fastRefCos(glm::vec<L, T, Q> const& x)
 	{
 		glm::vec<L, T, Q> const Angle0_PI(glm::abs(fmod(x + glm::pi<T>(), glm::two_pi<T>()) - glm::pi<T>()));
 //		return taylorSeriesRefCos(Angle0_PI);
@@ -321,17 +343,17 @@ namespace taylorCos
 		return ReturnSign * taylorSeriesRefCos(SectionAngle);
 	}
 
-	int perf_fastCosRef(float Begin, float End, std::size_t Samples)
+	static int perf_fastCosRef(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<glm::vec4> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = taylorCos::fastRefCos(AngleShift + glm::vec4(Begin + Steps * i));
+			Results[i] = taylorCos::fastRefCos(AngleShift + glm::vec4(Begin + Steps * static_cast<float>(i)));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -343,17 +365,17 @@ namespace taylorCos
 		return Error;
 	}
 
-	int perf_fastCosOld(float Begin, float End, std::size_t Samples)
+	static int perf_fastCosOld(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<glm::vec4> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = glm::fastCos(AngleShift + glm::vec4(Begin + Steps * i));
+			Results[i] = glm::fastCos(AngleShift + glm::vec4(Begin + Steps * static_cast<float>(i)));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -365,17 +387,17 @@ namespace taylorCos
 		return Error;
 	}
 
-	int perf_cos(float Begin, float End, std::size_t Samples)
+	static int perf_cos(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<glm::vec4> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = glm::cos(AngleShift + glm::vec4(Begin + Steps * i));
+			Results[i] = glm::cos(AngleShift + glm::vec4(Begin + Steps * static_cast<float>(i)));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -387,7 +409,7 @@ namespace taylorCos
 		return Error;
 	}
 
-	int perf(std::size_t const Samples)
+	static int perf(std::size_t const Samples)
 	{
 		int Error = 0;
 
@@ -403,7 +425,7 @@ namespace taylorCos
 		return Error;
 	}
 
-	int test()
+	static int test()
 	{
 		int Error = 0;
 
@@ -431,9 +453,18 @@ namespace taylorCos
 
 namespace taylor2
 {
+#	if (GLM_COMPILER & GLM_COMPILER_CLANG)
+#		pragma clang diagnostic push
+#		pragma clang diagnostic ignored "-Wglobal-constructors"
+#	endif
+
 	glm::vec4 const AngleShift(0.0f, glm::pi<float>() * 0.5f, glm::pi<float>() * 1.0f, glm::pi<float>() * 1.5f);
 
-	float taylorCosA(float x)
+#	if (GLM_COMPILER & GLM_COMPILER_CLANG)
+#		pragma clang diagnostic pop
+#	endif
+
+	static float taylorCosA(float x)
 	{
 		return 1.f
 			- (x * x) * (1.f / 2.f)
@@ -442,7 +473,7 @@ namespace taylor2
 			+ (x * x * x * x * x * x * x * x) * (1.f / 40320.f);
 	}
 
-	float taylorCosB(float x)
+	static float taylorCosB(float x)
 	{
 		return 1.f
 			- (x * x) * (1.f / 2.f)
@@ -451,7 +482,7 @@ namespace taylor2
 			+ (x * x * x * x * x * x * x * x) * (1.f / 40320.f);
 	}
 
-	float taylorCosC(float x)
+	static float taylorCosC(float x)
 	{
 		return 1.f
 			- (x * x) * (1.f / 2.f)
@@ -460,17 +491,17 @@ namespace taylor2
 			+ (((x * x) * (x * x)) * ((x * x) * (x * x))) * (1.f / 40320.f);
 	}
 
-	int perf_taylorCosA(float Begin, float End, std::size_t Samples)
+	static int perf_taylorCosA(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<float> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = taylorCosA(AngleShift.x + Begin + Steps * i);
+			Results[i] = taylorCosA(AngleShift.x + Begin + Steps * static_cast<float>(i));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -482,17 +513,17 @@ namespace taylor2
 		return Error;
 	}
 
-	int perf_taylorCosB(float Begin, float End, std::size_t Samples)
+	static int perf_taylorCosB(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<float> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = taylorCosB(AngleShift.x + Begin + Steps * i);
+			Results[i] = taylorCosB(AngleShift.x + Begin + Steps * static_cast<float>(i));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -504,17 +535,17 @@ namespace taylor2
 		return Error;
 	}
 
-	int perf_taylorCosC(float Begin, float End, std::size_t Samples)
+	static int perf_taylorCosC(float Begin, float End, std::size_t Samples)
 	{
 		std::vector<float> Results;
 		Results.resize(Samples);
 
-		float Steps = (End - Begin) / Samples;
+		float const Steps = (End - Begin) / static_cast<float>(Samples);
 
 		std::clock_t const TimeStampBegin = std::clock();
 
 		for(std::size_t i = 0; i < Samples; ++i)
-			Results[i] = taylorCosC(AngleShift.x + Begin + Steps * i);
+			Results[i] = taylorCosC(AngleShift.x + Begin + Steps * static_cast<float>(i));
 
 		std::clock_t const TimeStampEnd = std::clock();
 
@@ -526,7 +557,7 @@ namespace taylor2
 		return Error;
 	}
 
-	int perf(std::size_t Samples)
+	static int perf(std::size_t Samples)
 	{
 		int Error = 0;
 
@@ -550,14 +581,12 @@ int main()
 	Error += ::taylorCos::test();
 	Error += ::taylorCos::perf(1000);
 
-#	ifdef NDEBUG
-		::fastCos::perf(false);
-		::fastSin::perf(false);
-		::fastTan::perf(false);
-		::fastAcos::perf(false);
-		::fastAsin::perf(false);
-		::fastAtan::perf(false);
-#	endif//NDEBUG
+	::fastCos::perf(false);
+	::fastSin::perf(false);
+	::fastTan::perf(false);
+	::fastAcos::perf(false);
+	::fastAsin::perf(false);
+	::fastAtan::perf(false);
 
 	return Error;
 }
