@@ -1,4 +1,3 @@
-
 include("script/campaign/libcampaign.js");
 include("script/campaign/templates.js");
 
@@ -6,7 +5,7 @@ const mis_newParadigmRes = [
 	"R-Wpn-MG1Mk1", "R-Vehicle-Body01", "R-Sys-Spade1Mk1", "R-Vehicle-Prop-Wheels",
 	"R-Sys-Engineering01", "R-Wpn-MG-Damage03", "R-Wpn-MG-ROF01", "R-Wpn-Cannon-Damage02",
 	"R-Wpn-Flamer-Damage03", "R-Wpn-Flamer-Range01", "R-Wpn-Flamer-ROF01",
-	"R-Defense-WallUpgrade03", "R-Struc-Materials03", "R-Vehicle-Engine02",
+	"R-Defense-WallUpgrade01", "R-Struc-Materials01", "R-Vehicle-Engine02",
 	"R-Struc-RprFac-Upgrade02", "R-Wpn-Rocket-Damage01", "R-Wpn-Rocket-ROF03",
 	"R-Vehicle-Metals01", "R-Wpn-Mortar-Damage02", "R-Wpn-Rocket-Accuracy01",
 	"R-Wpn-RocketSlow-Damage01", "R-Wpn-Mortar-ROF01",
@@ -15,8 +14,48 @@ const mis_scavengerRes = [
 	"R-Wpn-Flamer-Damage02", "R-Wpn-Flamer-Range01", "R-Wpn-Flamer-ROF01",
 	"R-Wpn-MG-Damage03", "R-Wpn-MG-ROF01", "R-Wpn-Rocket-Damage01",
 	"R-Wpn-Cannon-Damage02", "R-Wpn-Mortar-Damage02", "R-Wpn-Mortar-ROF01",
-	"R-Wpn-Rocket-ROF03", "R-Defense-WallUpgrade03", "R-Struc-Materials03",
+	"R-Wpn-Rocket-ROF03", "R-Defense-WallUpgrade01", "R-Struc-Materials01",
 ];
+const mis_newParadigmResClassic = [
+	"R-Defense-WallUpgrade01", "R-Struc-Factory-Upgrade01", "R-Struc-Materials01",
+	"R-Wpn-MG-ROF01", "R-Vehicle-Engine01", "R-Wpn-Cannon-Damage02", "R-Wpn-Flamer-Damage03",
+	"R-Wpn-MG-Damage03", "R-Wpn-Rocket-Damage01", "R-Wpn-Flamer-ROF01", "R-Vehicle-Metals01",
+	"R-Struc-RprFac-Upgrade03",
+];
+const mis_scavengerResClassic = [
+	"R-Wpn-MG-Damage03", "R-Wpn-Rocket-Damage01"
+];
+
+function activateLZDefenders()
+{
+	if (camClassicMode())
+	{
+		return;
+	}
+	camManageGroup(camMakeGroup("PatrolForce"), CAM_ORDER_PATROL, {
+		pos: [
+			camMakePos("PatrolPos1"),
+			camMakePos("PatrolPos2"),
+			camMakePos("PatrolPos3"),
+			camMakePos("PatrolPos4")
+		],
+		interval: camSecondsToMilliseconds(30),
+		regroup: false,
+	});
+}
+
+function activateScavBaseDefenders()
+{
+	if (camClassicMode())
+	{
+		return;
+	}
+	camManageGroup(camMakeGroup("DefendForce"), CAM_ORDER_DEFEND, {
+		pos: camMakePos("defensePos"),
+		radius: 16,
+		regroup: false
+	});
+}
 
 function sendRocketForce()
 {
@@ -122,26 +161,23 @@ function getDroidsForNPLZ(args)
 {
 	const scouts = [ cTempl.nppod, cTempl.nphmg ];
 	const heavies = [ cTempl.npslc, cTempl.npsmct ];
-	const USE_ARTILLERY = (camRand(100) < 50);
 	const list = [];
+	const LIMIT = ((difficulty >= INSANE) ? 10 : 8);
 	let numScouts = camRand(5) + 1;
 	let heavy = heavies[camRand(heavies.length)];
+	let loopRuns = 0;
 
-	if (USE_ARTILLERY)
+	if (camRand(100) < 50)
 	{
-		list[list.length] = cTempl.npsens; //sensor will count towards scout total
+		list.push(cTempl.npsens); //sensor will count towards scout total
 		numScouts -= 1;
 		heavy = cTempl.npmor;
 	}
 
-	for (let i = 0; i < numScouts; ++i)
+	while (list.length < LIMIT)
 	{
-		list[list.length] = scouts[camRand(scouts.length)];
-	}
-
-	for (let a = numScouts; a < 8; ++a)
-	{
-		list[list.length] = heavy;
+		list.push((loopRuns < numScouts) ? scouts[camRand(scouts.length)] : heavy);
+		++loopRuns;
 	}
 
 	return list;
@@ -154,12 +190,15 @@ camAreaEvent("NPLZ1Trigger", function()
 	camPlayVideos({video: "MB1C4_MSG", type: MISS_MSG});
 	camDetectEnemyBase("NPLZ1Group");
 
-	camSetBaseReinforcements("NPLZ1Group", camChangeOnDiff(camMinutesToMilliseconds(5)), "getDroidsForNPLZ",
+	camSetBaseReinforcements("NPLZ1Group", camChangeOnDiff(camMinutesToMilliseconds((difficulty >= INSANE) ? 4.5 : 5)), "getDroidsForNPLZ",
 		CAM_REINFORCE_TRANSPORT, {
 			entry: { x: 126, y: 76 },
-			exit: { x: 126, y: 36 }
+			exit: { x: 126, y: 36 },
+			posLZ: camMakePos("EastNPLZ")
 		}
 	);
+
+	camCallOnce("activateLZDefenders");
 });
 
 camAreaEvent("NPLZ2Trigger", function()
@@ -171,117 +210,145 @@ camAreaEvent("NPLZ2Trigger", function()
 	camSetBaseReinforcements("NPLZ2Group", camChangeOnDiff(camMinutesToMilliseconds(5)), "getDroidsForNPLZ",
 		CAM_REINFORCE_TRANSPORT, {
 			entry: { x: 126, y: 76 },
-			exit: { x: 126, y: 36 }
+			exit: { x: 126, y: 36 },
+			posLZ: camMakePos("WestNPLZ")
 		}
 	);
 });
 
+function insaneReinforcementSpawn()
+{
+	const units = [cTempl.npsmct, cTempl.nppod];
+	const limits = {minimum: 6, maxRandom: 4};
+	const location = ["reinforceNorth", "reinforceSouthEast"];
+	camSendGenericSpawn(CAM_REINFORCE_GROUND, CAM_NEW_PARADIGM, CAM_REINFORCE_CONDITION_ARTIFACTS, location, units, limits.minimum, limits.maxRandom);
+}
+
 function eventStartLevel()
 {
-	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "CAM_1CA");
+	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, cam_levels.alpha7);
 	const startPos = getObject("startPosition");
 	const lz = getObject("landingZone");
 	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
 
-	// make sure player doesn't build on enemy LZs of the next level
-	for (let i = 1; i <= 5; ++i)
-	{
-		const ph = getObject("PhantomLZ" + i);
-		// HACK: set LZs of bad players, namely 2...6,
-		// note: player 1 is NP, player 7 is scavs
-		setNoGoArea(ph.x, ph.y, ph.x2, ph.y2, i + 1);
-	}
+	camSetMissionTimer(camChangeOnDiff(camHoursToSeconds(2)));
 
-	if (difficulty === HARD)
+	setAlliance(CAM_NEW_PARADIGM, CAM_SCAV_7, true);
+
+	if (camClassicMode())
 	{
-		setMissionTime(camMinutesToSeconds(100));
-	}
-	else if (difficulty === INSANE)
-	{
-		setMissionTime(camMinutesToSeconds(90));
+		camClassicResearch(mis_newParadigmResClassic, CAM_NEW_PARADIGM);
+		camClassicResearch(mis_scavengerResClassic, CAM_SCAV_7);
+
+		camSetArtifacts({
+			"ScavSouthFactory": { tech: "R-Wpn-Rocket05-MiniPod" },
+			"NPResearchFacility": { tech: "R-Struc-Research-Module" },
+			"NPCentralFactory": { tech: "R-Vehicle-Prop-Tracks" },
+			"NPNorthFactory": { tech: "R-Vehicle-Engine01" },
+		});
 	}
 	else
 	{
-		setMissionTime(camChangeOnDiff(camHoursToSeconds(2)));
+		camCompleteRequiredResearch(mis_newParadigmRes, CAM_NEW_PARADIGM);
+		camCompleteRequiredResearch(mis_scavengerRes, CAM_SCAV_7);
+
+		camUpgradeOnMapTemplates(cTempl.bloke, cTempl.blokeheavy, CAM_SCAV_7);
+		camUpgradeOnMapTemplates(cTempl.trike, cTempl.trikeheavy, CAM_SCAV_7);
+		camUpgradeOnMapTemplates(cTempl.buggy, cTempl.buggyheavy, CAM_SCAV_7);
+		camUpgradeOnMapTemplates(cTempl.bjeep, cTempl.bjeepheavy, CAM_SCAV_7);
+
+		if (camAllowInsaneSpawns())
+		{
+			addDroid(CAM_NEW_PARADIGM, 90, 107, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 89, 107, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 88, 107, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 90, 110, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 89, 110, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 88, 110, "MRP Bug Wheels", tBody.tank.bug, tProp.tank.wheels, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 114, 25, "MRP Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 114, 26, "MRP Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 114, 27, "MRP Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.miniRocketPod);
+			addDroid(CAM_NEW_PARADIGM, 116, 25, "Lancer Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.lancer);
+			addDroid(CAM_NEW_PARADIGM, 116, 26, "Lancer Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.lancer);
+			addDroid(CAM_NEW_PARADIGM, 116, 27, "Lancer Scorpion Halftracks", tBody.tank.scorpion, tProp.tank.halfTracks, "", "", tWeap.tank.lancer);
+		}
+
+		camSetArtifacts({
+			"ScavSouthFactory": { tech: ["R-Wpn-Rocket05-MiniPod", "R-Wpn-Cannon2Mk1"] },
+			"NPResearchFacility": { tech: "R-Struc-Research-Module" },
+			"NPCentralFactory": { tech: "R-Vehicle-Prop-Tracks" },
+			"NPNorthFactory": { tech: "R-Vehicle-Engine01" },
+		});
 	}
 
-	setAlliance(CAM_NEW_PARADIGM, CAM_SCAV_7, true);
-	camCompleteRequiredResearch(mis_newParadigmRes, CAM_NEW_PARADIGM);
-	camCompleteRequiredResearch(mis_scavengerRes, CAM_SCAV_7);
-
-	camUpgradeOnMapTemplates(cTempl.bloke, cTempl.blokeheavy, CAM_SCAV_7);
-	camUpgradeOnMapTemplates(cTempl.trike, cTempl.trikeheavy, CAM_SCAV_7);
-	camUpgradeOnMapTemplates(cTempl.buggy, cTempl.buggyheavy, CAM_SCAV_7);
-	camUpgradeOnMapTemplates(cTempl.bjeep, cTempl.bjeepheavy, CAM_SCAV_7);
-
 	camSetEnemyBases({
 		"ScavSouthDerrickGroup": {
 			cleanup: "ScavSouthDerrick",
 			detectMsg: "C1C_BASE1",
-			detectSnd: "pcv374.ogg",
-			eliminateSnd: "pcv391.ogg"
+			detectSnd: cam_sounds.baseDetection.scavengerOutpostDetected,
+			eliminateSnd: cam_sounds.baseElimination.scavengerOutpostEradicated
 		},
 		"ScavSouthEastHighgroundGroup": {
 			cleanup: "ScavSouthEastHighground",
 			detectMsg: "C1C_BASE6",
-			detectSnd: "pcv374.ogg",
-			eliminateSnd: "pcv391.ogg"
+			detectSnd: cam_sounds.baseDetection.scavengerBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.scavengerBaseEradicated
 		},
 		"ScavNorthBaseGroup": {
 			cleanup: "ScavNorthBase",
 			detectMsg: "C1C_BASE3",
-			detectSnd: "pcv374.ogg",
-			eliminateSnd: "pcv391.ogg"
+			detectSnd: cam_sounds.baseDetection.scavengerBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.scavengerBaseEradicated
 		},
 		"ScavSouthPodPitsGroup": {
 			cleanup: "ScavSouthPodPits",
 			detectMsg: "C1C_BASE4",
-			detectSnd: "pcv374.ogg",
-			eliminateSnd: "pcv391.ogg"
+			detectSnd: cam_sounds.baseDetection.scavengerOutpostDetected,
+			eliminateSnd: cam_sounds.baseElimination.scavengerOutpostEradicated
 		},
 		"ScavCentralBaseGroup": {
 			cleanup: "MixedCentralBase", // two bases with same cleanup region
 			detectMsg: "C1C_BASE5",
-			detectSnd: "pcv374.ogg",
-			eliminateSnd: "pcv391.ogg",
+			detectSnd: cam_sounds.baseDetection.scavengerBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.scavengerBaseEradicated,
 			player: CAM_SCAV_7 // hence discriminate by player filter
 		},
 		"NPEastBaseGroup": {
 			cleanup: "NPEastBase",
 			detectMsg: "C1C_BASE7",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"NPNorthEastGeneratorGroup": {
 			cleanup: "NPNorthEastGenerator",
 			detectMsg: "C1C_BASE8",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"NPNorthEastBaseGroup": {
 			cleanup: "NPNorthEastBase",
 			detectMsg: "C1C_BASE9",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"NPCentralBaseGroup": {
 			cleanup: "MixedCentralBase", // two bases with same cleanup region
 			detectMsg: "C1C_BASE10",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 			player: CAM_NEW_PARADIGM // hence discriminate by player filter
 		},
 		"NPLZ1Group": {
 			cleanup: "NPLZ1", // kill the four towers to disable LZ
 			detectMsg: "C1C_LZ1",
-			eliminateSnd: "pcv394.ogg",
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 			player: CAM_NEW_PARADIGM // required for LZ-type bases
 		},
 		"NPLZ2Group": {
 			cleanup: "NPLZ2", // kill the four towers to disable LZ
 			detectMsg: "C1C_LZ2",
-			eliminateSnd: "pcv394.ogg",
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 			player: CAM_NEW_PARADIGM // required for LZ-type bases
 		},
 	});
@@ -289,52 +356,45 @@ function eventStartLevel()
 	hackAddMessage("C1C_OBJ1", PROX_MSG, CAM_HUMAN_PLAYER, false); // initial beacon
 	camPlayVideos([{video: "MB1C_MSG", type: CAMP_MSG}, {video: "MB1C2_MSG", type: CAMP_MSG}]);
 
-	camSetArtifacts({
-		"ScavSouthFactory": { tech: ["R-Wpn-Rocket05-MiniPod", "R-Wpn-Cannon2Mk1"] },
-		"NPResearchFacility": { tech: "R-Struc-Research-Module" },
-		"NPCentralFactory": { tech: "R-Vehicle-Prop-Tracks" },
-		"NPNorthFactory": { tech: "R-Vehicle-Engine01" },
-	});
-
 	camSetFactories({
 		"ScavSouthFactory": {
 			assembly: "ScavSouthFactoryAssembly",
 			order: CAM_ORDER_ATTACK,
 			groupSize: 4,
-			throttle: camChangeOnDiff(camSecondsToMilliseconds(25)),
-			templates: [ cTempl.buscan, cTempl.rbjeep8, cTempl.trikeheavy, cTempl.buggyheavy ]
+			throttle: camChangeOnDiff(camSecondsToMilliseconds((difficulty >= INSANE) ? 20 : 25)),
+			templates: (!camClassicMode()) ? [ cTempl.buscan, cTempl.rbjeep8, cTempl.trikeheavy, cTempl.buggyheavy ] : [ cTempl.buscan, cTempl.rbjeep, cTempl.trike, cTempl.buggy ]
 		},
 		"ScavCentralFactory": {
 			assembly: "ScavCentralFactoryAssembly",
 			order: CAM_ORDER_ATTACK,
 			groupSize: 4,
-			throttle: camChangeOnDiff(camSecondsToMilliseconds(25)),
-			templates: [ cTempl.firecan, cTempl.rbuggy, cTempl.bjeepheavy, cTempl.blokeheavy ]
+			throttle: camChangeOnDiff(camSecondsToMilliseconds((difficulty >= INSANE) ? 20 : 25)),
+			templates: (!camClassicMode()) ? [ cTempl.firecan, cTempl.rbuggy, cTempl.bjeepheavy, cTempl.blokeheavy ] : [ cTempl.firecan, cTempl.rbuggy, cTempl.bjeep, cTempl.bloke ]
 		},
 		"ScavNorthFactory": {
 			assembly: "ScavNorthFactoryAssembly",
 			order: CAM_ORDER_ATTACK,
 			groupSize: 4,
-			throttle: camChangeOnDiff(camSecondsToMilliseconds(20)),
-			templates: [ cTempl.firecan, cTempl.rbuggy, cTempl.buscan, cTempl.trikeheavy ]
+			throttle: camChangeOnDiff(camSecondsToMilliseconds((difficulty >= INSANE) ? 15 : 20)),
+			templates: (!camClassicMode()) ? [ cTempl.firecan, cTempl.rbuggy, cTempl.buscan, cTempl.trikeheavy ] : [ cTempl.firecan, cTempl.rbuggy, cTempl.buscan, cTempl.trike ]
 		},
 		"NPCentralFactory": {
 			assembly: "NPCentralFactoryAssembly",
 			order: CAM_ORDER_ATTACK,
 			groupSize: 4,
-			throttle: camChangeOnDiff(camSecondsToMilliseconds(60)),
+			throttle: camChangeOnDiff(camSecondsToMilliseconds((difficulty >= INSANE) ? 40 : 60)),
 			data: {
 				regroup: false,
 				repair: 40,
 				count: -1,
 			},
-			templates: [ cTempl.npmor, cTempl.npsens, cTempl.npslc ]
+			templates: (difficulty >= INSANE) ? [ cTempl.npmor, cTempl.npsens, cTempl.npsmc ] : [ cTempl.npmor, cTempl.npsens, cTempl.npslc ]
 		},
 		"NPNorthFactory": {
 			assembly: "NPNorthFactoryAssembly",
 			order: CAM_ORDER_ATTACK,
 			groupSize: 4,
-			throttle: camChangeOnDiff(camSecondsToMilliseconds(50)),
+			throttle: camChangeOnDiff(camSecondsToMilliseconds((difficulty >= INSANE) ? 40 : 50)),
 			data: {
 				regroup: false,
 				repair: 66,
@@ -356,4 +416,9 @@ function eventStartLevel()
 	queue("sendTankScoutForce", camSecondsToMilliseconds(30));
 	queue("sendTankForce", camSecondsToMilliseconds(100)); // in wzcam it moves back and then forward
 	queue("enableNPFactory", camMinutesToMilliseconds(5));
+	queue("activateScavBaseDefenders", camSecondsToMilliseconds(3));
+	if (camAllowInsaneSpawns())
+	{
+		setTimer("insaneReinforcementSpawn", camMinutesToMilliseconds(4));
+	}
 }
