@@ -8,7 +8,10 @@ var startedFromMenu;
 
 camAreaEvent("vtolRemoveZone", function(droid)
 {
-	camSafeRemoveObject(droid, false);
+	if ((droid.player !== CAM_HUMAN_PLAYER) && camVtolCanDisappear(droid))
+	{
+		camSafeRemoveObject(droid, false);
+	}
 	resetLabel("vtolRemoveZone", CAM_THE_COLLECTIVE);
 });
 
@@ -54,7 +57,7 @@ function getDroidsForCOLZ()
 	}
 	else
 	{
-		templates = [cTempl.cohct, cTempl.commrl, cTempl.comorb];
+		templates = (!camClassicMode()) ? [cTempl.cohct, cTempl.commrl, cTempl.comorb] : [cTempl.cohct, cTempl.commc, cTempl.comorb];
 		usingHeavy = true;
 	}
 
@@ -111,10 +114,13 @@ function sendPlayerTransporter()
 	}
 
 	const droids = [];
-	const bodyList = ["Body12SUP", "Body11ABT"];
-	const propulsionList = ["tracked01", "tracked01", "hover01"];
-	const weaponList = ["Cannon375mmMk1", "Cannon375mmMk1", "Cannon375mmMk1", "Rocket-LtA-T", "Rocket-LtA-T", "Mortar2Mk1", "Rocket-MRL"];
-	const specialList = ["SensorTurret1Mk1", "CommandBrain01"];
+	const bodyList = [tBody.tank.mantis, tBody.tank.python];
+	const propulsionList = [tProp.tank.tracks, tProp.tank.tracks, tProp.tank.hover];
+	const weaponList = [
+		tWeap.tank.heavyCannon, tWeap.tank.heavyCannon, tWeap.tank.heavyCannon,
+		tWeap.tank.lancer, tWeap.tank.lancer, tWeap.tank.bombard, tWeap.tank.miniRocketArray
+	];
+	const specialList = [tSensor.sensor, tCommand.commander];
 	const BODY = bodyList[camRand(bodyList.length)];
 	const PROP = propulsionList[camRand(propulsionList.length)];
 
@@ -133,24 +139,58 @@ function sendPlayerTransporter()
 }
 
 //Continuously spawns heavy units on the north part of the map every 7 minutes
-function mapEdgeDroids()
+function mapEdgeDroids(useInsaneLocations)
 {
-	const TANK_NUM = 8 + camRand(6);
-	const list = [cTempl.npcybm, cTempl.npcybr, cTempl.commrp, cTempl.cohct];
-
-	const droids = [];
-	for (let i = 0; i < TANK_NUM; ++i)
+	let location = ["groundUnitPos", "groundUnitPos"];
+	if (camDef(useInsaneLocations) && useInsaneLocations)
 	{
-		droids.push(list[camRand(list.length)]);
+		location = ["groundUnitPos", "insaneSpawnPos1", "insaneSpawnPos2", "insaneSpawnPos3", "insaneSpawnPos4"];
 	}
+	const units = (!camClassicMode()) ? [cTempl.npcybm, cTempl.npcybr, cTempl.commrp, cTempl.cohct] : [cTempl.npcybm, cTempl.npcybr, cTempl.commc, cTempl.cohct];
+	const limits = {minimum: 8, maxRandom: 5};
+	camSendGenericSpawn(CAM_REINFORCE_GROUND, CAM_THE_COLLECTIVE, CAM_REINFORCE_CONDITION_NONE, location, units, limits.minimum, limits.maxRandom);
+}
 
-	camSendReinforcement(CAM_THE_COLLECTIVE, camMakePos("groundUnitPos"), droids, CAM_REINFORCE_GROUND);
+function insaneReinforcementSpawn()
+{
+	mapEdgeDroids(true);
+}
+
+function wave2()
+{
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	const CONDITION = ((camAllowInsaneSpawns()) ? CAM_REINFORCE_CONDITION_ARTIFACTS : "COCommandCenter");
+	const list = [cTempl.colatv, cTempl.colatv];
+	const ext = {limit: [3, 3], alternate: true, altIdx: 0};
+	camSetVtolData(CAM_THE_COLLECTIVE, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(4)), CONDITION, ext);
+}
+
+function wave3()
+{
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	const CONDITION = ((camAllowInsaneSpawns()) ? CAM_REINFORCE_CONDITION_ARTIFACTS : "COCommandCenter");
+	const list = [cTempl.colcbv, cTempl.colcbv];
+	const ext = {limit: [2, 2], alternate: true, altIdx: 0};
+	camSetVtolData(CAM_THE_COLLECTIVE, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(4)), CONDITION, ext);
 }
 
 function vtolAttack()
 {
-	const list = [cTempl.colcbv];
-	camSetVtolData(CAM_THE_COLLECTIVE, "vtolAppearPos", "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(3)), "COCommandCenter");
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	const CONDITION = ((camAllowInsaneSpawns()) ? CAM_REINFORCE_CONDITION_ARTIFACTS : "COCommandCenter");
+	if (camClassicMode())
+	{
+		const list = [cTempl.colcbv, cTempl.colcbv];
+		camSetVtolData(CAM_THE_COLLECTIVE, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(3)), CONDITION);
+	}
+	else
+	{
+		const list = [cTempl.colpbv, cTempl.colpbv];
+		const ext = {limit: [2, 2], alternate: true, altIdx: 0};
+		camSetVtolData(CAM_THE_COLLECTIVE, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(4)), CONDITION, ext);
+		queue("wave2", camChangeOnDiff(camSecondsToMilliseconds(30)));
+		queue("wave3", camChangeOnDiff(camSecondsToMilliseconds(60)));
+	}
 }
 
 function groupPatrol()
@@ -193,7 +233,7 @@ function cam2Setup()
 {
 	const collectiveRes = [
 		"R-Wpn-MG1Mk1", "R-Sys-Engineering02",
-		"R-Defense-WallUpgrade06", "R-Struc-Materials06",
+		"R-Defense-WallUpgrade04", "R-Struc-Materials04",
 		"R-Vehicle-Engine03", "R-Vehicle-Metals03", "R-Cyborg-Metals03",
 		"R-Wpn-Cannon-Accuracy02", "R-Wpn-Cannon-Damage04",
 		"R-Wpn-Cannon-ROF01", "R-Wpn-Flamer-Damage03", "R-Wpn-Flamer-ROF01",
@@ -203,31 +243,66 @@ function cam2Setup()
 		"R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03",
 		"R-Wpn-RocketSlow-Damage04", "R-Sys-Sensor-Upgrade01"
 	];
+	const collectiveResClassic = [
+		"R-Defense-WallUpgrade03", "R-Struc-Materials03", "R-Struc-Factory-Upgrade03",
+		"R-Vehicle-Engine03", "R-Vehicle-Metals03", "R-Cyborg-Metals03",
+		"R-Vehicle-Armor-Heat01", "R-Cyborg-Armor-Heat01", "R-Cyborg-Armor-Heat01",
+		"R-Wpn-Cannon-Damage03", "R-Wpn-Cannon-ROF01", "R-Wpn-Flamer-Damage03",
+		"R-Wpn-Flamer-ROF01", "R-Wpn-MG-Damage04", "R-Wpn-MG-ROF02",
+		"R-Wpn-Mortar-Damage03", "R-Wpn-Mortar-ROF01", "R-Wpn-Rocket-Accuracy02",
+		"R-Wpn-Rocket-Damage03", "R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03",
+		"R-Wpn-RocketSlow-Damage03", "R-Sys-Sensor-Upgrade01"
+	];
 
 	for (let x = 0, l = mis_structsAlpha.length; x < l; ++x)
 	{
 		enableStructure(mis_structsAlpha[x], CAM_HUMAN_PLAYER);
 	}
 
-	camCompleteRequiredResearch(mis_playerResBeta, CAM_HUMAN_PLAYER);
-	camCompleteRequiredResearch(mis_alphaResearchNew, CAM_THE_COLLECTIVE);
-	camCompleteRequiredResearch(collectiveRes, CAM_THE_COLLECTIVE);
-	camCompleteRequiredResearch(mis_alphaResearchNew, CAM_HUMAN_PLAYER);
-
-	if (difficulty >= HARD)
+	if (camClassicMode())
 	{
-		camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
+		camCompleteRequiredResearch(mis_alphaResearchNewClassic, CAM_HUMAN_PLAYER);
+		camCompleteRequiredResearch(mis_playerResBetaClassic, CAM_HUMAN_PLAYER);
+
+		if (tweakOptions.camClassic_balance32)
+		{
+			camClassicResearch(mis_betaStartingResearchClassic, CAM_HUMAN_PLAYER);
+			completeResearch("CAM2RESEARCH-UNDO", CAM_HUMAN_PLAYER);
+			//The Collective have no research in 3.2
+		}
+		else
+		{
+			completeResearch("CAM2RESEARCH-UNDO-Rockets", CAM_HUMAN_PLAYER);
+			camCompleteRequiredResearch(mis_betaStartingResearchClassic, CAM_HUMAN_PLAYER);
+			camCompleteRequiredResearch(mis_alphaResearchNewClassic, CAM_THE_COLLECTIVE);
+			camCompleteRequiredResearch(collectiveResClassic, CAM_THE_COLLECTIVE);
+		}
+
+		enableResearch("R-Wpn-Cannon-Accuracy02", CAM_HUMAN_PLAYER);
+	}
+	else
+	{
+		camCompleteRequiredResearch(mis_playerResBeta, CAM_HUMAN_PLAYER);
+		camCompleteRequiredResearch(mis_alphaResearchNew, CAM_THE_COLLECTIVE);
+		camCompleteRequiredResearch(collectiveRes, CAM_THE_COLLECTIVE);
+		camCompleteRequiredResearch(mis_alphaResearchNew, CAM_HUMAN_PLAYER);
+
+		if (difficulty >= HARD)
+		{
+			camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
+		}
+
+		enableResearch("R-Wpn-Cannon-Damage04", CAM_HUMAN_PLAYER);
+		enableResearch("R-Wpn-Rocket-Damage04", CAM_HUMAN_PLAYER);
 	}
 
-	enableResearch("R-Wpn-Cannon-Damage04", CAM_HUMAN_PLAYER);
-	enableResearch("R-Wpn-Rocket-Damage04", CAM_HUMAN_PLAYER);
 	preDamageStuff();
 }
 
 //Get some higher rank droids.
 function setUnitRank(transport)
 {
-	const droidExp = [128, 64, 32, 16];
+	const ranks = ["elite", "veteran", "professional", "regular"];
 	let droids;
 	let mapRun = false;
 
@@ -247,8 +322,8 @@ function setUnitRank(transport)
 		const droid = droids[i];
 		if (droid.droidType !== DROID_CONSTRUCT && droid.droidType !== DROID_REPAIR)
 		{
-			const MOD = (droid.droidType === DROID_COMMAND || droid.droidType === DROID_SENSOR) ? 2 : 1;
-			setDroidExperience(droid, MOD * droidExp[mapRun ? 0 : (transporterIndex - 1)]);
+			const USE_COMMAND_RANK = (droid.droidType === DROID_COMMAND || droid.droidType === DROID_SENSOR);
+			setDroidExperience(droid, camGetRankThreshold(ranks[mapRun ? 0 : (transporterIndex - 1)], USE_COMMAND_RANK));
 		}
 	}
 }
@@ -285,7 +360,7 @@ function reallyDownTransporter()
 		removeTimer("sendPlayerTransporter");
 	}
 	setReinforcementTime(LZ_COMPROMISED_TIME);
-	playSound("pcv443.ogg");
+	playSound(cam_sounds.transport.transportUnderAttack);
 }
 
 function downTransporter()
@@ -314,16 +389,14 @@ function eventStartLevel()
 	const PLAYER_POWER = 5000;
 	const startPos = getObject("startPosition");
 	const lz = getObject("landingZone"); //player lz
-	const enemyLz = getObject("COLandingZone");
 	const tEnt = getObject("transporterEntry");
 	const tExt = getObject("transporterExit");
 
-	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "SUB_2_1S");
+	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, cam_levels.beta2.pre);
 	setReinforcementTime(LZ_COMPROMISED_TIME);
 
 	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
-	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, 5);
 	startTransporterEntry(tEnt.x, tEnt.y, CAM_HUMAN_PLAYER);
 	setTransporterExit(tExt.x, tExt.y, CAM_HUMAN_PLAYER);
 
@@ -333,7 +406,7 @@ function eventStartLevel()
 		"COArtiCBTower": { tech: "R-Sys-Sensor-Upgrade01" },
 	});
 
-	setMissionTime(camChangeOnDiff(camHoursToSeconds(1)));
+	camSetMissionTimer(camChangeOnDiff(camHoursToSeconds(1)));
 	setPower(PLAYER_POWER, CAM_HUMAN_PLAYER);
 	cam2Setup();
 
@@ -342,14 +415,14 @@ function eventStartLevel()
 		"CONorthBase": {
 			cleanup: "CONorth",
 			detectMsg: "C2A_BASE1",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"CONorthWestBase": {
 			cleanup: "CONorthWest",
 			detectMsg: "C2A_BASE2",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 	});
 
@@ -376,6 +449,10 @@ function eventStartLevel()
 	setTimer("truckDefense", camChangeOnDiff(camMinutesToMilliseconds(3)));
 	setTimer("sendCOTransporter", camChangeOnDiff(camMinutesToMilliseconds(4)));
 	setTimer("mapEdgeDroids", camChangeOnDiff(camMinutesToMilliseconds(7)));
+	if (camAllowInsaneSpawns())
+	{
+		setTimer("insaneReinforcementSpawn", camMinutesToMilliseconds(2.5));
+	}
 
 	truckDefense();
 }
