@@ -1,4 +1,3 @@
-
 include("script/campaign/libcampaign.js");
 include("script/campaign/templates.js");
 
@@ -7,7 +6,7 @@ var civilianPosIndex; //Current location of civilian groups.
 var shepardGroup; //Enemy group that protects civilians.
 var lastSoundTime; //Only play the "civilian rescued" sound every so often.
 const mis_collectiveRes = [
-	"R-Defense-WallUpgrade06", "R-Struc-Materials06", "R-Sys-Engineering02",
+	"R-Defense-WallUpgrade05", "R-Struc-Materials05", "R-Sys-Engineering02",
 	"R-Vehicle-Engine04", "R-Vehicle-Metals05", "R-Cyborg-Metals05",
 	"R-Wpn-Cannon-Accuracy02", "R-Wpn-Cannon-Damage05", "R-Wpn-Cannon-ROF01",
 	"R-Wpn-Flamer-Damage06", "R-Wpn-Flamer-ROF03", "R-Wpn-MG-Damage06",
@@ -16,9 +15,21 @@ const mis_collectiveRes = [
 	"R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03", "R-Wpn-RocketSlow-Damage05",
 	"R-Sys-Sensor-Upgrade01", "R-Wpn-RocketSlow-ROF01", "R-Wpn-Howitzer-ROF01",
 	"R-Wpn-Howitzer-Damage07", "R-Cyborg-Armor-Heat01", "R-Vehicle-Armor-Heat01",
-	"R-Wpn-Bomb-Damage01", "R-Wpn-AAGun-Damage03", "R-Wpn-AAGun-ROF02",
+	"R-Wpn-Bomb-Damage01", "R-Wpn-AAGun-Damage02", "R-Wpn-AAGun-ROF02",
 	"R-Wpn-AAGun-Accuracy01", "R-Struc-VTOLPad-Upgrade01"
 ];
+const mis_collectiveResClassic = [
+	"R-Defense-WallUpgrade03", "R-Struc-Materials04", "R-Struc-Factory-Upgrade04",
+	"R-Struc-VTOLPad-Upgrade01", "R-Vehicle-Engine04", "R-Vehicle-Metals03",
+	"R-Cyborg-Metals04", "R-Vehicle-Armor-Heat02", "R-Cyborg-Armor-Heat02",
+	"R-Sys-Engineering02", "R-Wpn-Cannon-Accuracy02", "R-Wpn-Cannon-Damage04",
+	"R-Wpn-Cannon-ROF03", "R-Wpn-Flamer-Damage06", "R-Wpn-Flamer-ROF03",
+	"R-Wpn-Howitzer-Accuracy02", "R-Wpn-Howitzer-Damage02", "R-Sys-Sensor-Upgrade01",
+	"R-Wpn-MG-Damage07", "R-Wpn-MG-ROF03", "R-Wpn-Mortar-Acc02", "R-Wpn-Mortar-Damage06",
+	"R-Wpn-Mortar-ROF03", "R-Wpn-Rocket-Accuracy02", "R-Wpn-Rocket-Damage06",
+	"R-Wpn-Rocket-ROF03", "R-Wpn-RocketSlow-Accuracy03", "R-Wpn-RocketSlow-Damage05",
+	"R-Wpn-RocketSlow-ROF03"
+];
 
 //Play video about civilians being captured by the Collective. Triggered
 //by destroying the air base or crossing the base3Trigger area.
@@ -26,7 +37,7 @@ function videoTrigger()
 {
 	camSetExtraObjectiveMessage(_("Rescue the civilians from The Collective before too many are captured"));
 
-	setMissionTime(getMissionTime() + camChangeOnDiff(camMinutesToSeconds(30)));
+	camSetMissionTimer(getMissionTime() + camChangeOnDiff(camMinutesToSeconds(30)));
 	setTimer("civilianOrders", camSecondsToMilliseconds(2));
 	setTimer("captureCivilians", camChangeOnDiff(camSecondsToMilliseconds(10)));
 
@@ -67,6 +78,23 @@ function camEnemyBaseEliminated_COAirBase()
 	camCallOnce("videoTrigger");
 }
 
+function insaneReinforcementSpawn()
+{
+	const units = [cTempl.comatt, cTempl.comit, cTempl.cohct, cTempl.commrl, cTempl.comhpv, cTempl.npcybc];
+	const limits = {minimum: 8, maxRandom: 8};
+	const location = camMakePos(getObject("southWestSpawnPos"));
+	camSendGenericSpawn(CAM_REINFORCE_GROUND, CAM_THE_COLLECTIVE, CAM_REINFORCE_CONDITION_UNITS, location, units, limits.minimum, limits.maxRandom);
+}
+
+function insaneTransporterAttack()
+{
+	const DISTANCE_FROM_POS = 40;
+	const units = [cTempl.cohct, cTempl.commrl, cTempl.comhpv, cTempl.comtathh];
+	const limits = {minimum: 10, maxRandom: 0};
+	const location = camGenerateRandomMapCoordinate(getObject("startPosition"), CAM_GENERIC_LAND_STAT, DISTANCE_FROM_POS);
+	camSendGenericSpawn(CAM_REINFORCE_TRANSPORT, CAM_THE_COLLECTIVE, CAM_REINFORCE_CONDITION_ARTIFACTS, location, units, limits.minimum, limits.maxRandom);
+}
+
 function enableFactories()
 {
 	camEnableFactory("COHeavyFac-Leopard");
@@ -148,7 +176,7 @@ function captureCivilians()
 		for (let i = 0; i < NUM; ++i)
 		{
 			addDroid(CAM_SCAV_7, currPos.x, currPos.y, "Civilian",
-					"B1BaBaPerson01", "BaBaLegs", "", "", "BabaMG");
+					tBody.scav.human, tProp.scav.legs, "", "", tWeap.scav.machinegun);
 		}
 
 		//Only count civilians that are not in the the transporter base.
@@ -205,22 +233,21 @@ function civilianOrders()
 	//Play the "Civilian rescued" sound and throttle it.
 	if (rescued && ((lastSoundTime + camSecondsToMilliseconds(30)) < gameTime))
 	{
-		const RESCUE_SND = "pcv612.ogg";	//"Civilian Rescued".
 		lastSoundTime = gameTime;
-		playSound(RESCUE_SND);
+		playSound(cam_sounds.rescue.civilianRescued);
 	}
 }
 
 //Capture civilans.
 function eventTransporterLanded(transport)
 {
+	const SCAN_RADIUS = 4;
 	const position = getObject("COTransportPos");
-	const civs = enumRange(position.x, position.y, 15, CAM_SCAV_7, false);
+	const civs = enumRange(position.x, position.y, SCAN_RADIUS, CAM_SCAV_7, false);
 
-	if (civs.length)
+	if ((civs.length > 0) && (camDist(transport, position) <= SCAN_RADIUS))
 	{
-		const ESCAPE_SND = "pcv632.ogg"; //"Enemy escaping".
-		playSound(ESCAPE_SND);
+		playSound(cam_sounds.enemyEscaping);
 		capturedCivCount += civs.length - 1;
 		for (let i = 0; i < civs.length; ++i)
 		{
@@ -274,7 +301,7 @@ function extraVictoryCondition()
 
 function eventStartLevel()
 {
-	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "SUB_2_5S", {
+	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, cam_levels.beta6.pre, {
 		callback: "extraVictoryCondition"
 	});
 
@@ -283,48 +310,63 @@ function eventStartLevel()
 	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
 
-	const enemyLz = getObject("COLandingZone");
-	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, 5);
+	if (camClassicMode())
+	{
+		camClassicResearch(mis_collectiveResClassic, CAM_THE_COLLECTIVE);
 
-	camSetArtifacts({
-		"rippleRocket": { tech: "R-Wpn-Rocket06-IDF" },
-		"quadbof": { tech: "R-Wpn-AAGun02" },
-		"howitzer": { tech: "R-Wpn-HowitzerMk1" },
-		"COHeavyFac-Leopard": { tech: "R-Vehicle-Body06" }, //Panther
-		"COHeavyFac-Upgrade": { tech: "R-Struc-Factory-Upgrade04" },
-		"COVtolFacLeft-Prop": { tech: "R-Vehicle-Prop-VTOL" },
-		"COInfernoEmplacement-Arti": { tech: "R-Wpn-Flamer-ROF02" },
-	});
+		camSetArtifacts({
+			"rippleRocket": { tech: "R-Wpn-Rocket06-IDF" },
+			"COResearchLab": { tech: "R-Struc-Factory-Upgrade04" },
+			"quadbof": { tech: "R-Wpn-AAGun02" },
+			"howitzer": { tech: "R-Wpn-HowitzerMk1" },
+			"COHeavyFac-Leopard": { tech: "R-Vehicle-Body02" }, //Leopard
+			"COVtolFacLeft-Prop": { tech: "R-Vehicle-Prop-VTOL" },
+		});
+	}
+	else
+	{
+		camCompleteRequiredResearch(mis_collectiveRes, CAM_THE_COLLECTIVE);
 
-	setMissionTime(camChangeOnDiff(camHoursToSeconds(2)));
+		if (difficulty >= MEDIUM)
+		{
+			camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
+		}
+		camUpgradeOnMapTemplates(cTempl.npcybf, cTempl.cocybth, CAM_THE_COLLECTIVE);
+
+		camSetArtifacts({
+			"rippleRocket": { tech: "R-Wpn-Rocket06-IDF" },
+			"quadbof": { tech: "R-Wpn-AAGun02" },
+			"howitzer": { tech: "R-Wpn-HowitzerMk1" },
+			"COHeavyFac-Leopard": { tech: "R-Vehicle-Body06" }, //Panther
+			"COHeavyFac-Upgrade": { tech: "R-Struc-Factory-Upgrade04" },
+			"COVtolFacLeft-Prop": { tech: "R-Vehicle-Prop-VTOL" },
+			"COInfernoEmplacement-Arti": { tech: "R-Wpn-Flamer-ROF02" },
+		});
+	}
+
+	camSetMissionTimer(camChangeOnDiff(camHoursToSeconds(2)));
 
 	setAlliance(CAM_THE_COLLECTIVE, CAM_SCAV_7, true);
 	setAlliance(CAM_HUMAN_PLAYER, CAM_SCAV_7, true);
-	camCompleteRequiredResearch(mis_collectiveRes, CAM_THE_COLLECTIVE);
-
-	if (difficulty >= MEDIUM)
-	{
-		camUpgradeOnMapTemplates(cTempl.commc, cTempl.commrp, CAM_THE_COLLECTIVE);
-	}
 
 	camSetEnemyBases({
 		"COAirBase": {
 			cleanup: "airBaseCleanup",
 			detectMsg: "C2C_BASE1",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"COCyborgBase": {
 			cleanup: "cyborgBaseCleanup",
 			detectMsg: "C2C_BASE2",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 		"COtransportBase": {
 			cleanup: "transportBaseCleanup",
 			detectMsg: "C2C_BASE3",
-			detectSnd: "pcv379.ogg",
-			eliminateSnd: "pcv394.ogg",
+			detectSnd: cam_sounds.baseDetection.enemyBaseDetected,
+			eliminateSnd: cam_sounds.baseElimination.enemyBaseEradicated,
 		},
 	});
 
@@ -363,7 +405,7 @@ function eventStartLevel()
 				repair: 40,
 				count: -1,
 			},
-			templates: [cTempl.npcybf, cTempl.npcybc, cTempl.npcybr]
+			templates: (!camClassicMode()) ? [cTempl.cocybth, cTempl.npcybc, cTempl.npcybr] : [cTempl.npcybf, cTempl.npcybc, cTempl.npcybr]
 		},
 		"COCyborgFactoryR": {
 			assembly: "COCyborgFactoryRAssembly",
@@ -375,7 +417,7 @@ function eventStartLevel()
 				repair: 40,
 				count: -1,
 			},
-			templates: [cTempl.npcybf, cTempl.npcybc, cTempl.npcybr]
+			templates: (!camClassicMode()) ? [cTempl.cocybth, cTempl.npcybc, cTempl.npcybr] : [cTempl.npcybf, cTempl.npcybc, cTempl.npcybr]
 		},
 		"COVtolFacLeft-Prop": {
 			order: CAM_ORDER_ATTACK,
@@ -385,7 +427,7 @@ function eventStartLevel()
 				regroup: false,
 				count: -1,
 			},
-			templates: [cTempl.commorv, cTempl.colagv]
+			templates: (!camClassicMode()) ? [cTempl.commorv, cTempl.comhcv, cTempl.commorv, cTempl.colpbv] : [cTempl.commorv, cTempl.colagv]
 		},
 		"COVtolFacRight": {
 			order: CAM_ORDER_ATTACK,
@@ -395,7 +437,7 @@ function eventStartLevel()
 				regroup: false,
 				count: -1,
 			},
-			templates: [cTempl.colagv, cTempl.commorv]
+			templates: (!camClassicMode()) ? [cTempl.colagv, cTempl.commorv, cTempl.colatv, cTempl.commorv] : [cTempl.colagv, cTempl.commorv]
 		},
 	});
 
@@ -411,6 +453,11 @@ function eventStartLevel()
 
 	queue("activateGroups", camChangeOnDiff(camMinutesToMilliseconds(8)));
 	setTimer("truckDefense", camChangeOnDiff(camMinutesToMilliseconds(3)));
+	if (camAllowInsaneSpawns())
+	{
+		setTimer("insaneTransporterAttack", camMinutesToMilliseconds(4));
+		setTimer("insaneReinforcementSpawn", camMinutesToMilliseconds(5));
+	}
 
 	truckDefense();
 }
