@@ -5,7 +5,7 @@ const mis_nexusRes = [
 	"R-Sys-Engineering03", "R-Defense-WallUpgrade09", "R-Struc-Materials09",
 	"R-Struc-VTOLPad-Upgrade06", "R-Wpn-Bomb-Damage03", "R-Sys-NEXUSrepair",
 	"R-Vehicle-Prop-Hover02", "R-Vehicle-Prop-VTOL02", "R-Cyborg-Legs02",
-	"R-Wpn-Mortar-Acc03", "R-Wpn-MG-Damage09", "R-Wpn-Mortar-ROF04",
+	"R-Wpn-Mortar-Acc03", "R-Wpn-MG-Damage10", "R-Wpn-Mortar-ROF04",
 	"R-Vehicle-Engine08", "R-Vehicle-Metals08", "R-Vehicle-Armor-Heat05",
 	"R-Cyborg-Metals08", "R-Cyborg-Armor-Heat05", "R-Wpn-RocketSlow-ROF06",
 	"R-Wpn-AAGun-Damage06", "R-Wpn-AAGun-ROF06", "R-Wpn-Howitzer-Damage09",
@@ -13,76 +13,94 @@ const mis_nexusRes = [
 	"R-Wpn-Missile-Damage01", "R-Wpn-Missile-ROF01", "R-Wpn-Missile-Accuracy01",
 	"R-Wpn-Rail-Damage01", "R-Wpn-Rail-ROF01", "R-Wpn-Rail-Accuracy01",
 	"R-Wpn-Energy-Damage03", "R-Wpn-Energy-ROF03", "R-Wpn-Energy-Accuracy01",
-	"R-Wpn-AAGun-Accuracy03", "R-Wpn-Howitzer-Accuracy03",
+	"R-Wpn-AAGun-Accuracy03", "R-Wpn-Howitzer-Accuracy03", "R-Sys-NEXUSsensor",
+];
+const mis_nexusResClassic = [
+	"R-Defense-WallUpgrade09", "R-Struc-Materials09", "R-Struc-Factory-Upgrade06",
+	"R-Struc-VTOLPad-Upgrade06", "R-Vehicle-Engine09", "R-Vehicle-Metals07",
+	"R-Cyborg-Metals07", "R-Vehicle-Armor-Heat05", "R-Cyborg-Armor-Heat05",
+	"R-Sys-Engineering03", "R-Vehicle-Prop-Hover02", "R-Vehicle-Prop-VTOL02",
+	"R-Wpn-Bomb-Damage03", "R-Wpn-Energy-Accuracy01", "R-Wpn-Energy-Damage02",
+	"R-Wpn-Energy-ROF02", "R-Wpn-Missile-Accuracy01", "R-Wpn-Missile-Damage02",
+	"R-Wpn-Rail-Damage02", "R-Wpn-Rail-ROF02", "R-Sys-Sensor-Upgrade01",
+	"R-Sys-NEXUSrepair", "R-Wpn-Flamer-Damage06", "R-Sys-NEXUSsensor",
 ];
-var edgeMapCounter; //how many Nexus reinforcement runs have happened.
 var hackFailChance; //chance the Nexus Intruder Program will fail
 var winFlag;
 
 //Remove Nexus VTOL droids.
 camAreaEvent("vtolRemoveZone", function(droid)
 {
-	if (droid.player !== CAM_HUMAN_PLAYER)
+	if (droid.player !== CAM_HUMAN_PLAYER && camVtolCanDisappear(droid))
 	{
-		if (isVTOL(droid))
-		{
-			camSafeRemoveObject(droid, false);
-		}
+		camSafeRemoveObject(droid, false);
 	}
-
 	resetLabel("vtolRemoveZone", CAM_NEXUS);
 });
 
 function sendEdgeMapDroids()
 {
-	let unitCount = 16 + camRand(5); // 16 - 20.
-	if (difficulty === INSANE)
+	const ADD_EXTRA = (!camClassicMode() && (difficulty >= HARD));
+	const units = {
+		units: [cTempl.nxcyrail, cTempl.nxcyscou, cTempl.nxcylas, cTempl.nxlflash,
+				cTempl.nxmrailh, cTempl.nxmlinkh, cTempl.nxmscouh, cTempl.nxmsamh],
+		appended: (ADD_EXTRA) ? cTempl.nxmsens : undefined
+	};
+	if (ADD_EXTRA)
 	{
-		unitCount = 14 + camRand(3); // 14 - 16.
+		units.units.push(cTempl.nxmangel);
 	}
-	const edge = ["SWPhantomFactory", "NWPhantomFactory"];
-	let list = [
-		cTempl.nxcyrail, cTempl.nxcyscou, cTempl.nxcylas,
-		cTempl.nxlflash, cTempl.nxmrailh, cTempl.nxmlinkh,
-		cTempl.nxmscouh, cTempl.nxmsamh,
-	];
-	if (difficulty >= HARD)
-	{
-		list = list.concat(cTempl.nxmangel);
-	}
-	let droids = [];
+	const limits = {minimum: (difficulty >= INSANE) ? 14 : 16, maxRandom: (difficulty >= INSANE) ? 2 : 4};
+	const location = ["SWPhantomFactory", "NWPhantomFactory"];
+	camSendGenericSpawn(CAM_REINFORCE_GROUND, CAM_NEXUS, CAM_REINFORCE_CONDITION_NONE, location, units, limits.minimum, limits.maxRandom);
+}
 
-	if (!camDef(edgeMapCounter))
-	{
-		edgeMapCounter = 0;
-	}
+function wave2()
+{
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	const list = [cTempl.nxlscouv, cTempl.nxlscouv];
+	const ext = {limit: [3, 3], alternate: true, altIdx: 0};
+	camSetVtolData(CAM_NEXUS, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(3.5)), undefined, ext);
+}
 
-	for (let i = 0; i < unitCount; ++i)
-	{
-		droids.push(list[camRand(list.length)]);
-	}
-
-	droids = droids.concat(cTempl.nxmsens);
-
-	camSendReinforcement(CAM_NEXUS, camMakePos(edge[camRand(edge.length)]), droids,
-		CAM_REINFORCE_GROUND, {
-			data: {regroup: false, count: -1}
-		}
-	);
-
-	edgeMapCounter += 1;
+function wave3()
+{
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	const list = [cTempl.nxlneedv, cTempl.nxlneedv];
+	const ext = {limit: [3, 3], alternate: true, altIdx: 0};
+	camSetVtolData(CAM_NEXUS, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(3.5)), undefined, ext);
 }
 
 //Setup Nexus VTOL hit and runners. NOTE: These do not go away in this mission.
 function vtolAttack()
 {
-	const list = [cTempl.nxlscouv, cTempl.nxmtherv, cTempl.nxlscouv, cTempl.nxmheapv];
-	const ext = {
-		limit: [2, 4, 2, 4],
-		alternate: true,
-		altIdx: 0
-	};
-	camSetVtolData(CAM_NEXUS, (difficulty === INSANE) ? undefined : "vtolAppearPos", "vtolRemovePos", list, camChangeOnDiff(camMinutesToMilliseconds(2)), undefined, ext);
+	const APPEAR_POS = ((camAllowInsaneSpawns()) ? undefined : "vtolAppearPos");
+	if (camClassicMode())
+	{
+		const list = [cTempl.nxmheapv, cTempl.nxmheapv, cTempl.nxmtherv];
+		const ext = {limit: [4, 4, 5], alternate: true, altIdx: 0};
+		camSetVtolData(CAM_NEXUS, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(2)), undefined, ext);
+	}
+	else
+	{
+		const list = [cTempl.nxmtherv, cTempl.nxmheapv];
+		const ext = {limit: [3, 3], alternate: true, altIdx: 0};
+		camSetVtolData(CAM_NEXUS, APPEAR_POS, "vtolRemoveZone", list, camChangeOnDiff(camMinutesToMilliseconds(3.5)), undefined, ext);
+		queue("wave2", camChangeOnDiff(camSecondsToMilliseconds(30)));
+		queue("wave3", camChangeOnDiff(camSecondsToMilliseconds(60)));
+	}
+}
+
+function insaneTransporterAttack()
+{
+	if (getResearch(cam_resistance_circuits.third).done)
+	{
+		return;
+	}
+	const units = {units: [cTempl.nxmangel, cTempl.nxmangel], appended: cTempl.nxmsens};
+	const limits = {minimum: 9, maxRandom: 0};
+	const location = camMakePos("nexusEastTransportPos");
+	camSendGenericSpawn(CAM_REINFORCE_TRANSPORT, CAM_NEXUS, CAM_REINFORCE_CONDITION_NONE, location, units, limits.minimum, limits.maxRandom);
 }
 
 // Order any absorbed trucks to start building defenses near themselves.
@@ -151,8 +169,8 @@ function nexusManufacture()
 
 function manualGrouping()
 {
-	const vtols = enumDroid(CAM_NEXUS).filter((obj) => (obj.group === null && isVTOL(obj)));
-	const nonVtols = enumDroid(CAM_NEXUS).filter((obj) => (obj.group === null && !isVTOL(obj)));
+	const vtols = enumDroid(CAM_NEXUS).filter((obj) => (obj.group === null && !camIsTransporter(obj) && isVTOL(obj)));
+	const nonVtols = enumDroid(CAM_NEXUS).filter((obj) => (obj.group === null && !camIsTransporter(obj) && !isVTOL(obj)));
 	if (vtols.length)
 	{
 		camManageGroup(camMakeGroup(vtols), CAM_ORDER_ATTACK, { regroup: false, count: -1 });
@@ -180,24 +198,32 @@ function eventObjectTransfer(obj, from)
 function powerTransfer()
 {
 	setPower(playerPower(CAM_HUMAN_PLAYER) + 5000);
-	playSound("power-transferred.ogg");
+	playSound(cam_sounds.powerTransferred);
 }
 
 function eventResearched(research, structure, player)
 {
-	if (research.name === "R-Sys-Resistance-Upgrade01")
+	if (research.name === cam_resistance_circuits.first)
 	{
-		hackFailChance = 55;
+		hackFailChance = 60;
+		if (camClassicMode())
+		{
+			camSetNexusState(false);
+		}
 	}
-	else if (research.name === "R-Sys-Resistance-Upgrade02")
+	else if (research.name === cam_resistance_circuits.second)
 	{
-		hackFailChance = 70;
+		hackFailChance = 75;
 	}
-	else if (research.name === "R-Sys-Resistance-Upgrade03")
+	else if (research.name === cam_resistance_circuits.third)
 	{
-		hackFailChance = 85;
+		hackFailChance = 90;
+		if (camClassicMode())
+		{
+			winFlag = true;
+		}
 	}
-	else if (research.name === "R-Sys-Resistance-Upgrade04")
+	else if (research.name === cam_resistance_circuits.fourth)
 	{
 		winFlag = true;
 		hackFailChance = 100;
@@ -222,15 +248,14 @@ function hackPlayer()
 
 function synapticsSound()
 {
-	playSound(CAM_SYNAPTICS_ACTIVATED_SND);
+	playSound(cam_sounds.nexus.synapticLinksActivated);
 	camHackIntoPlayer(CAM_HUMAN_PLAYER, CAM_NEXUS);
 }
 
 //winFlag is set in eventResearched.
 function resistanceResearched()
 {
-	const MAP_EDGE_COUNT = (difficulty >= MEDIUM) ? 15 : 8;
-	if (winFlag && edgeMapCounter >= MAP_EDGE_COUNT)
+	if (winFlag)
 	{
 		return true;
 	}
@@ -242,8 +267,9 @@ function eventStartLevel()
 
 	const startPos = getObject("startPosition");
 	const lz = getObject("landingZone");
+	const nextLev = ((camDef(tweakOptions.gammaBonusLevel) && tweakOptions.gammaBonusLevel) ? cam_levels.gammaBonus.pre : cam_levels.gamma6);
 
-	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "CAM3C", {
+	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, nextLev, {
 		callback: "resistanceResearched"
 	});
 
@@ -252,16 +278,20 @@ function eventStartLevel()
 
 	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
-	setMissionTime(camChangeOnDiff(camHoursToSeconds(1)));
+	camSetMissionTimer(camChangeOnDiff(camHoursToSeconds(1)));
 
-	const enemyLz = getObject("NXlandingZone");
-	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, CAM_NEXUS);
+	if (camClassicMode())
+	{
+		camClassicResearch(mis_nexusResClassic, CAM_NEXUS);
+	}
+	else
+	{
+		camCompleteRequiredResearch(mis_nexusRes, CAM_NEXUS);
+	}
 
-	camCompleteRequiredResearch(mis_nexusRes, CAM_NEXUS);
-
-	enableResearch("R-Sys-Resistance-Upgrade01", CAM_HUMAN_PLAYER);
+	enableResearch(cam_resistance_circuits.first, CAM_HUMAN_PLAYER);
 	winFlag = false;
-	hackFailChance = (difficulty <= EASY) ? 40 : 30;
+	hackFailChance = (difficulty <= EASY) ? 45 : 33;
 
 	queue("vtolAttack", camChangeOnDiff(camMinutesToMilliseconds(2)));
 
@@ -272,5 +302,9 @@ function eventStartLevel()
 	setTimer("truckDefense", camSecondsToMilliseconds(2));
 	setTimer("hackPlayer", camChangeOnDiff(camSecondsToMilliseconds(8)));
 	setTimer("nexusManufacture", camSecondsToMilliseconds(10));
-	setTimer("sendEdgeMapDroids", camChangeOnDiff(camMinutesToMilliseconds(3)));
+	setTimer("sendEdgeMapDroids", camChangeOnDiff(camMinutesToMilliseconds(4)));
+	if (camAllowInsaneSpawns())
+	{
+		setTimer("insaneTransporterAttack", camMinutesToMilliseconds(5));
+	}
 }
