@@ -33,8 +33,8 @@
 //;;
 function camSetEnemyBases(bases)
 {
-	var reload = !camDef(bases);
-	if (!reload)
+	const __RELOAD = !camDef(bases);
+	if (!__RELOAD)
 	{
 		__camEnemyBases = bases;
 		__camNumEnemyBases = 0;
@@ -42,8 +42,8 @@ function camSetEnemyBases(bases)
 	// convert label strings to groups and store
 	for (const baseLabel in __camEnemyBases)
 	{
-		var bi = __camEnemyBases[baseLabel];
-		var obj = getObject(baseLabel);
+		const bi = __camEnemyBases[baseLabel];
+		const obj = getObject(baseLabel);
 		if (camDef(obj) && obj) // group already defined
 		{
 			if (!camDef(bi.group))
@@ -52,11 +52,11 @@ function camSetEnemyBases(bases)
 			}
 			else
 			{
-				var structures = enumGroup(bi.group);
+				const structures = enumGroup(bi.group);
 				addLabel({ type: GROUP, id: bi.group }, baseLabel);
 				for (let idx = 0, len = structures.length; idx < len; ++idx)
 				{
-					var s = structures[idx];
+					const s = structures[idx];
 					if (s.type !== STRUCTURE || __camIsValidLeftover(s))
 					{
 						continue;
@@ -70,11 +70,11 @@ function camSetEnemyBases(bases)
 			}
 			if (!camDef(bi.cleanup)) // auto-detect cleanup area
 			{
-				var objs = enumGroup(bi.group);
+				const objs = enumGroup(bi.group);
 				if (objs.length > 0)
 				{
-					const OFFSET = 2; // increases size of the auto-detected base area a bit
-					var a = {
+					const __OFFSET = 2; // increases size of the auto-detected base area a bit
+					const a = {
 						type: AREA,
 						x: mapWidth, y: mapHeight,
 						x2: 0, y2: 0
@@ -82,13 +82,28 @@ function camSetEnemyBases(bases)
 					// smallest rectangle to contain all objects
 					for (let idx = 0, len = objs.length; idx < len; ++idx)
 					{
-						var o = objs[idx];
-						if (o.x < a.x) a.x = o.x;
-						if (o.y < a.y) a.y = o.y;
-						if (o.x > a.x2) a.x2 = o.x;
-						if (o.y > a.y2) a.y2 = o.y;
+						const o = objs[idx];
+						if (o.x < a.x)
+						{
+							a.x = o.x;
+						}
+						if (o.y < a.y)
+						{
+							a.y = o.y;
+						}
+						if (o.x > a.x2)
+						{
+							a.x2 = o.x;
+						}
+						if (o.y > a.y2)
+						{
+							a.y2 = o.y;
+						}
 					}
-					a.x -= OFFSET; a.y -= OFFSET; a.x2 += OFFSET; a.y2 += OFFSET;
+					a.x -= __OFFSET;
+					a.y -= __OFFSET;
+					a.x2 += __OFFSET;
+					a.y2 += __OFFSET;
 					camTrace("Auto-detected cleanup area for", baseLabel, ":", a.x, a.y, a.x2, a.y2);
 					bi.cleanup = "__cam_enemy_base_cleanup__" + baseLabel;
 					addLabel(a, bi.cleanup);
@@ -104,10 +119,10 @@ function camSetEnemyBases(bases)
 			}
 			bi.group = camNewGroup();
 			addLabel({ type: GROUP, id: bi.group }, baseLabel);
-			var structs = enumArea(bi.cleanup, ENEMIES, false);
+			const structs = enumArea(bi.cleanup, ENEMIES, false);
 			for (let idx = 0, len = structs.length; idx < len; ++idx)
 			{
-				var s = structs[idx];
+				const s = structs[idx];
 				if (s.type !== STRUCTURE || __camIsValidLeftover(s))
 				{
 					continue;
@@ -123,7 +138,7 @@ function camSetEnemyBases(bases)
 		{
 			//camDebug("Base", baseLabel, "defined as empty group");
 		}
-		if (!reload)
+		if (!__RELOAD)
 		{
 			bi.detected = false;
 			bi.eliminated = false;
@@ -143,7 +158,7 @@ function camSetEnemyBases(bases)
 //;;
 function camDetectEnemyBase(baseLabel)
 {
-	var bi = __camEnemyBases[baseLabel];
+	const bi = __camEnemyBases[baseLabel];
 	if (bi.detected || bi.eliminated)
 	{
 		return;
@@ -152,13 +167,13 @@ function camDetectEnemyBase(baseLabel)
 	bi.detected = true;
 	if (camDef(bi.detectSnd))
 	{
-		var pos = camMakePos(bi.cleanup);
+		let pos = camMakePos(bi.cleanup);
 		if (!camDef(pos)) // auto-detect sound position by group object pos
 		{
-			var objs = enumGroup(bi.group);
+			const objs = enumGroup(bi.group);
 			if (objs.length > 0)
 			{
-				var firstObject = objs[0];
+				const firstObject = objs[0];
 				pos = camMakePos(firstObject);
 			}
 		}
@@ -171,7 +186,7 @@ function camDetectEnemyBase(baseLabel)
 	{
 		hackAddMessage(bi.detectMsg, PROX_MSG, CAM_HUMAN_PLAYER, false);
 	}
-	var callback = __camGlobalContext()["camEnemyBaseDetected_" + baseLabel];
+	const callback = __camGlobalContext()["camEnemyBaseDetected_" + baseLabel];
 	if (camDef(callback))
 	{
 		callback();
@@ -194,7 +209,7 @@ function camAllEnemyBasesEliminated()
 
 function __camCheckBaseSeen(seen)
 {
-	var group = seen; // group?
+	let group = seen; // group?
 	if (camDef(seen.group)) // object?
 	{
 		group = seen.group;
@@ -206,7 +221,7 @@ function __camCheckBaseSeen(seen)
 	// FIXME: O(n) lookup here
 	for (const baseLabel in __camEnemyBases)
 	{
-		var bi = __camEnemyBases[baseLabel];
+		const bi = __camEnemyBases[baseLabel];
 		if (bi.group !== group)
 		{
 			continue;
@@ -236,7 +251,7 @@ function __camIsValidLeftover(obj)
 
 function __camShouldDestroyLeftover(objInfo, basePlayer)
 {
-	var object = getObject(objInfo.type, objInfo.player, objInfo.id);
+	const object = getObject(objInfo.type, objInfo.player, objInfo.id);
 	if (object === null)
 	{
 		return false;
@@ -253,8 +268,8 @@ function __camCheckBaseEliminated(group)
 	// FIXME: O(n) lookup here
 	for (const baseLabel in __camEnemyBases)
 	{
-		var bi = __camEnemyBases[baseLabel];
-		var leftovers = [];
+		const bi = __camEnemyBases[baseLabel];
+		const leftovers = [];
 		if (bi.eliminated || (bi.group !== group))
 		{
 			continue;
@@ -265,11 +280,11 @@ function __camCheckBaseEliminated(group)
 		}
 		if (camDef(bi.cleanup))
 		{
-			var objects = enumArea(bi.cleanup, ENEMIES, false);
+			const objects = enumArea(bi.cleanup, ENEMIES, false);
 			for (let i = 0, len = objects.length; i < len; ++i)
 			{
-				var object = objects[i];
-				var objInfo = {
+				const object = objects[i];
+				const objInfo = {
 					type: object.type,
 					player: object.player,
 					id: object.id
@@ -282,13 +297,13 @@ function __camCheckBaseEliminated(group)
 			for (let i = 0, len = leftovers.length; i < len; ++i)
 			{
 				// remove with special effect
-				var leftover = leftovers[i];
+				const leftover = leftovers[i];
 				camSafeRemoveObject(leftover, true);
 			}
 			if (camDef(bi.eliminateSnd))
 			{
 				// play sound
-				var pos = camMakePos(bi.cleanup);
+				const pos = camMakePos(bi.cleanup);
 				playSound(bi.eliminateSnd, pos.x, pos.y, 0);
 			}
 		}
@@ -306,12 +321,11 @@ function __camCheckBaseEliminated(group)
 		// bump counter before the callback, so that it was
 		// actual during the callback
 		++__camNumEnemyBases;
-		var callback = __camGlobalContext()["camEnemyBaseEliminated_" + baseLabel];
+		const callback = __camGlobalContext()["camEnemyBaseEliminated_" + baseLabel];
 		if (camDef(callback))
 		{
 			callback();
 		}
-
 		__camSetupConsoleForVictoryConditions();
 	}
 }
@@ -320,11 +334,15 @@ function __camBasesTick()
 {
 	for (const baseLabel in __camEnemyBases)
 	{
-		var bi = __camEnemyBases[baseLabel];
+		const bi = __camEnemyBases[baseLabel];
 		if (bi.eliminated || !camDef(bi.reinforce_kind))
 		{
 			continue;
 		}
+		if (bi.reinforce_kind === CAM_REINFORCE_NONE)
+		{
+			continue;
+		}
 		if (gameTime - bi.reinforce_last < bi.reinforce_interval)
 		{
 			continue;
@@ -340,8 +358,9 @@ function __camBasesTick()
 			return;
 		}
 		bi.reinforce_last = gameTime;
-		var list = profile(bi.reinforce_callback);
-		var pos = camMakePos(bi.cleanup);
+		const list = profile(bi.reinforce_callback);
+		const __DEFINED_LZ = (camDef(bi.reinforce_data) && camDef(bi.reinforce_data.posLZ));
+		const pos = (__DEFINED_LZ) ? camMakePos(bi.reinforce_data.posLZ) : camMakePos(bi.cleanup);
 		camSendReinforcement(bi.player, pos, list, bi.reinforce_kind, bi.reinforce_data);
 	}
 }
