@@ -62,7 +62,7 @@ function cam_eventChat(from, to, message)
 		return;
 	}
 	camTrace(from, to, message);
-	if (message === "let me win" && __camNextLevel !== "SUB_1_1")
+	if (message === "let me win" && __camNextLevel !== cam_levels.alpha3.offWorld)
 	{
 		__camLetMeWin();
 	}
@@ -146,7 +146,8 @@ function cam_eventStartLevel()
 	setTimer("__camShowVictoryConditions", camMinutesToMilliseconds(5));
 	setTimer("__camTacticsTick", camSecondsToMilliseconds(0.1));
 	queue("__camShowBetaHintEarly", camSecondsToMilliseconds(4));
-	queue("__camGrantSpecialResearch", camSecondsToMilliseconds(6));
+	queue("__camGrantSpecialResearch", camSecondsToMilliseconds(0.5));
+	queue("__camEnableGuideTopics", camSecondsToMilliseconds(0.1)); // delayed to handle when mission scripts add research
 }
 
 function cam_eventDroidBuilt(droid, structure)
@@ -163,10 +164,22 @@ function cam_eventDroidBuilt(droid, structure)
 	{
 		return;
 	}
-	if (camGetNexusState() && droid.player === CAM_NEXUS && __camNextLevel === "CAM3C" && camRand(100) < 7)
+	if (camGetNexusState() && droid.player === CAM_NEXUS && __camNextLevel === cam_levels.gamma6 && camRand(100) < 7)
 	{
 		// Occasionally hint that NEXUS is producing units on Gamma 5.
-		playSound(CAM_PRODUCTION_COMPLETE_SND);
+		playSound(cam_sounds.nexus.productionCompleted);
+	}
+	if (droid.player === CAM_HUMAN_PLAYER)
+	{
+		// handling guide topics for built units
+		if (droid.isVTOL)
+		{
+			camCallOnce("__camDoAddVTOLUseTopics");
+		}
+		else if (droid.droidType === DROID_COMMAND)
+		{
+			camCallOnce("__camDoAddCommanderUseTopics");
+		}
 	}
 	if (!camDef(__camFactoryInfo))
 	{
@@ -178,6 +191,10 @@ function cam_eventDroidBuilt(droid, structure)
 
 function cam_eventDestroyed(obj)
 {
+	if (obj.type === FEATURE && obj.stattype === ARTIFACT)
+	{
+		return;
+	}
 	__camCheckPlaceArtifact(obj);
 	if (obj.type === DROID)
 	{
@@ -215,11 +232,9 @@ function cam_eventGroupSeen(viewer, group)
 function cam_eventTransporterExit(transport)
 {
 	camTrace("Transporter for player", transport.player + " has exited");
-
 	if (transport.player === CAM_HUMAN_PLAYER)
 	{
 		__camNumTransporterExits += 1;
-
 		//Audio cue to let the player know they can bring in reinforcements. This
 		//assumes the player can bring in reinforcements immediately after the first
 		//transporter leaves the map. Mission scripts can handle special situations.
@@ -228,8 +243,7 @@ function cam_eventTransporterExit(transport)
 			__camVictoryData.reinforcements > -1) ||
 			__camWinLossCallback === CAM_VICTORY_STANDARD))
 		{
-			const __REINFORCEMENTS_AVAILABLE_SOUND = "pcv440.ogg";
-			playSound(__REINFORCEMENTS_AVAILABLE_SOUND);
+			playSound(cam_sounds.reinforcementsAreAvailable);
 			//Show the transporter reinforcement timer when it leaves for the first time.
 			if (__camWinLossCallback === CAM_VICTORY_OFFWORLD)
 			{
@@ -242,7 +256,6 @@ function cam_eventTransporterExit(transport)
 			setReinforcementTime(__camVictoryData.reinforcements);
 		}
 	}
-
 	if (transport.player !== CAM_HUMAN_PLAYER ||
 		(__camWinLossCallback === CAM_VICTORY_STANDARD &&
 		transport.player === CAM_HUMAN_PLAYER))
@@ -269,6 +282,8 @@ function cam_eventTransporterLanded(transport)
 		{
 			setReinforcementTime(-1);
 		}
+		// Handle enabling guide topics relevant to units potentially "gifted" by libcampaign
+		__camEnableGuideTopicsForTransport(transport);
 	}
 }
 
@@ -293,9 +308,9 @@ function cam_eventMissionTimeout()
 
 function cam_eventAttacked(victim, attacker)
 {
-	if (camDef(victim) && victim && victim.type === DROID)
+	if (camDef(victim) && victim)
 	{
-		if (victim.player !== CAM_HUMAN_PLAYER && !allianceExistsBetween(CAM_HUMAN_PLAYER, victim.player))
+		if (victim.type === DROID && victim.player !== CAM_HUMAN_PLAYER && !allianceExistsBetween(CAM_HUMAN_PLAYER, victim.player))
 		{
 			//Try dynamically creating a group of nearby droids not part
 			//of a group. Only supports those who can hit ground units.
@@ -321,21 +336,27 @@ function cam_eventAttacked(victim, attacker)
 					repair: 70
 				});
 			}
-
 			if (camDef(__camGroupInfo[victim.group]))
 			{
 				__camGroupInfo[victim.group].lastHit = gameTime;
-
-				//Increased Nexus intelligence if struck on cam3-4
-				if (__camNextLevel === CAM_GAMMA_OUT)
+				if (__camGroupInfo[victim.group].order === CAM_ORDER_PATROL)
 				{
-					if (__camGroupInfo[victim.group].order === CAM_ORDER_PATROL)
+					if (camDef(__camGroupInfo[victim.group].data) &&
+						camDef(__camGroupInfo[victim.group].data.reactToAttack) &&
+						__camGroupInfo[victim.group].data.reactToAttack)
 					{
 						__camGroupInfo[victim.group].order = CAM_ORDER_ATTACK;
 					}
 				}
 			}
 		}
+		if (victim.player === CAM_HUMAN_PLAYER && camDef(attacker) && attacker && attacker.player !== CAM_HUMAN_PLAYER)
+		{
+			if (attacker.type === DROID && attacker.isVTOL)
+			{
+				camCallOnce("__camDoAddVTOLDefenseTopics");
+			}
+		}
 	}
 }
 
@@ -345,10 +366,11 @@ function cam_eventGameLoaded()
 	receiveAllEvents(true);
 	__camSaveLoading = true;
 	const scavKevlarMissions = [
-		"CAM_1CA", "SUB_1_4AS", "SUB_1_4A", "SUB_1_5S", "SUB_1_5",
-		"CAM_1A-C", "SUB_1_7S", "SUB_1_7", "SUB_1_DS", "CAM_1END", "SUB_2_5S"
+		cam_levels.alpha7, cam_levels.alpha8.pre, cam_levels.alpha8.offWorld,
+		cam_levels.alpha9.pre, cam_levels.alpha9.offWorld, cam_levels.alpha10,
+		cam_levels.alpha11.pre, cam_levels.alpha11.offWorld, cam_levels.alpha12.pre,
+		cam_levels.alphaEnd, cam_levels.beta6.pre
 	];
-
 	//Need to set the scavenger kevlar vests when loading a save from later Alpha
 	//missions or else it reverts to the original texture.
 	for (let i = 0, l = scavKevlarMissions.length; i < l; ++i)
@@ -369,20 +391,18 @@ function cam_eventGameLoaded()
 			break;
 		}
 	}
-
-	if (__camWinLossCallback === CAM_VICTORY_TIMEOUT
-		&& enumDroid(CAM_HUMAN_PLAYER, DROID_SUPERTRANSPORTER).length === 0)
+	if (__camWinLossCallback === CAM_VICTORY_TIMEOUT &&
+		enumDroid(CAM_HUMAN_PLAYER, DROID_SUPERTRANSPORTER).length === 0)
 	{
 		// If the transport is gone on Beta End, put a timer up to show when it'll be back
 		setReinforcementTime(__camVictoryData.reinforcements);
 	}
-
 	//Subscribe to eventGroupSeen again.
 	camSetEnemyBases();
-
+	// Ensure appropriate guide topics are displayed
+	__camEnableGuideTopics();
 	//Reset any vars
 	__camCheatMode = false;
-
 	__camSaveLoading = false;
 }
 
@@ -396,22 +416,21 @@ function cam_eventObjectTransfer(obj, from)
 		{
 			if (obj.stattype === DEFENSE)
 			{
-				snd = CAM_DEFENSE_ABSORBED_SND;
+				snd = cam_sounds.nexus.defensesAbsorbed;
 			}
 			else if (obj.stattype === RESEARCH_LAB)
 			{
-				snd = CAM_RES_ABSORBED_SND;
+				snd = cam_sounds.nexus.researchAbsorbed;
 			}
 			else
 			{
-				snd = CAM_STRUCTURE_ABSORBED_SND;
+				snd = cam_sounds.nexus.structureAbsorbed;
 			}
 		}
 		else if (obj.type === DROID)
 		{
-			snd = CAM_UNIT_ABSORBED_SND;
+			snd = cam_sounds.nexus.unitAbsorbed;
 		}
-
 		if (camDef(snd))
 		{
 			playSound(snd);
@@ -424,3 +443,27 @@ function cam_eventVideoDone()
 {
 	__camEnqueueVideos(); //Play any remaining videos automatically.
 }
+
+function cam_eventDroidRankGained(droid, rankNum)
+{
+	if (droid.player === CAM_HUMAN_PLAYER)
+	{
+		addGuideTopic("wz2100::units::experience", SHOWTOPIC_FIRSTADD);
+	}
+}
+
+function cam_eventResearched(research, structure, player)
+{
+	if (player !== CAM_HUMAN_PLAYER)
+	{
+		return;
+	}
+	const __RESEARCHED_BY_STRUCT = (camDef(structure) && structure);
+	if (!__RESEARCHED_BY_STRUCT)
+	{
+		return; // for now, return - don't think we need to process if researched by API call here?
+	}
+	// only pass the research in if it was completed by a structure (not if given by an API call, in which structure would be null)
+	//__camProcessResearchGatedGuideTopics((__RESEARCHED_BY_STRUCT) ? research : null);
+	__camProcessResearchGatedGuideTopics(research);
+}
