@@ -3,6 +3,17 @@
 // Misc useful stuff.
 ////////////////////////////////////////////////////////////////////////////////
 
+//;; ## camClassicMode()
+//;;
+//;; Returns `true` if classic balance mod is enabled.
+//;;
+//;; @returns {boolean}
+//;;
+function camClassicMode()
+{
+	return __camClassicModActive;
+}
+
 //;; ## camDef(something)
 //;;
 //;; Returns `false` if something is JavaScript-undefined, `true` otherwise.
@@ -183,7 +194,8 @@ function camDist(x1, y1, x2, y2)
 //;;
 function camPlayerMatchesFilter(playerId, playerFilter)
 {
-	switch (playerFilter) {
+	switch (playerFilter)
+	{
 		case ALL_PLAYERS:
 			return true;
 		case ALLIES:
@@ -206,7 +218,6 @@ function camRemoveDuplicates(items)
 {
 	let prims = {"boolean":{}, "number":{}, "string":{}};
 	const objs = [];
-
 	return items.filter((item) => {
 		const type = typeof item;
 		if (type in prims)
@@ -247,6 +258,36 @@ function camCountStructuresInArea(label, playerFilter)
 	return ret;
 }
 
+//;; ## camCleanTileOfObstructions(x, y | pos)
+//;;
+//;; Obliterates player structures and features near the tile around certain coordinates.
+//;; Can be used for spawn locations or transport reinforcement spots. May not
+//;; delete very large objects like factories or skyscrapers.
+//;;
+//;; @param {number|Object} x
+//;; @param {number} [y]
+//;; @returns {void}
+//;;
+function camCleanTileOfObstructions(x, y)
+{
+	if (!camDef(x))
+	{
+		camDebug("invalid parameters?");
+		return;
+	}
+	const __TILE_SWEEP_RADIUS = 1;
+	const pos = (camDef(y)) ? {x: x, y: y} : x;
+	const objects = enumRange(pos.x, pos.y, __TILE_SWEEP_RADIUS, CAM_HUMAN_PLAYER, false);
+	for (let i = 0, len = objects.length; i < len; ++i)
+	{
+		const obj = objects[i];
+		if (obj.type !== DROID)
+		{
+			camSafeRemoveObject(obj, true);
+		}
+	}
+}
+
 //;; ## camChangeOnDiff(numericValue)
 //;;
 //;; Change a numeric value based on campaign difficulty.
@@ -257,7 +298,6 @@ function camCountStructuresInArea(label, playerFilter)
 function camChangeOnDiff(numericValue)
 {
 	let modifier = 0;
-
 	switch (difficulty)
 	{
 		case SUPEREASY:
@@ -279,10 +319,26 @@ function camChangeOnDiff(numericValue)
 			modifier = 1;
 			break;
 	}
-
 	return Math.floor(numericValue * modifier);
 }
 
+//;; ## camAllowInsaneSpawns()
+//;;
+//;; Allow additional Insane difficulty (or higher) spawns and behavior.
+//;;
+//;; @returns {boolean}
+//;;
+function camAllowInsaneSpawns()
+{
+	if (!camDef(tweakOptions.insanePlus) || !camDef(tweakOptions.insanePlusLowDiff))
+	{
+		return false;
+	}
+	const __INSANE_SPAWNS = ((difficulty >= INSANE) && tweakOptions.insanePlus);
+	const __LOWER_DIFF_SPAWNS = ((difficulty < INSANE) && tweakOptions.insanePlusLowDiff);
+	return (__INSANE_SPAWNS || __LOWER_DIFF_SPAWNS);
+}
+
 //;; ## camIsSystemDroid(gameObject)
 //;;
 //;; Determine if the passed in object is a non-weapon based droid.
@@ -296,13 +352,11 @@ function camIsSystemDroid(gameObject)
 	{
 		return false;
 	}
-
 	if (gameObject.type !== DROID)
 	{
 		camTrace("Non-droid: " + gameObject.type + " pl: " + gameObject.name);
 		return false;
 	}
-
 	return (gameObject.droidType === DROID_SENSOR || gameObject.droidType === DROID_CONSTRUCT || gameObject.droidType === DROID_REPAIR);
 }
 
@@ -339,7 +393,8 @@ function camMakeGroup(what, playerFilter)
 	}
 	if (camDef(obj))
 	{
-		switch (obj.type) {
+		switch (obj.type)
+		{
 			case POSITION:
 				obj = getObject(obj.x, obj.y);
 				// fall-through
@@ -372,7 +427,7 @@ function camMakeGroup(what, playerFilter)
 				camDebug("Trying to add", o);
 				continue;
 			}
-			if (o.type === DROID && o.droidType !== DROID_CONSTRUCT && camPlayerMatchesFilter(o.player, playerFilter))
+			if (o.type === DROID && o.droidType !== DROID_CONSTRUCT && !camIsTransporter(o) && camPlayerMatchesFilter(o.player, playerFilter))
 			{
 				groupAdd(group, o);
 			}
@@ -402,114 +457,232 @@ function camBreakAlliances()
 	}
 }
 
-//;; ## camGenerateRandomMapEdgeCoordinate(reachPosition)
+//;; ## camIsWaterPropulsion(propulsion)
 //;;
-//;; Returns a random coordinate anywhere on the edge of the map that reachs a position.
+//;; Check if a propulsion can traverse water tiles. Until the Stats object can tell us this
+//;; information, it simply uses a very basic name check against the default hover propulsions.
+//;;
+//;; @param {String} propulsion
+//;; @returns {Boolean}
+//;;
+function camIsWaterPropulsion(propulsion)
+{
+	return (propulsion.indexOf("hover") !== -1);
+}
+
+//;; ## camNearInaccessibleAreas(start, destination [, propulsion [, distance]])
+//;;
+//;; Determine if a start position can reach a destination position within the limits of
+//;; the chosen propulsion, and if there are nearby cliffs/water tiles nearby within a certain distance.
+//;; if `destination` is undefined it will cause this function to just scan around the start position.
+//;;
+//;; @param {Object} start
+//;; @param {Object} destination
+//;; @param {String} propulsion
+//;; @param {Number} distance
+//;; @returns {Boolean}
+//;;
+function camNearInaccessibleAreas(start, destination, propulsion, distance)
+{
+	if (!camDef(start) || !start)
+	{
+		camDebug("Undefined start position when scanning for inaccessible areas.");
+		return true;
+	}
+	if (!camDef(propulsion))
+	{
+		propulsion = CAM_GENERIC_LAND_STAT;
+	}
+	if (!camDef(distance))
+	{
+		distance = 1;
+	}
+	if (!camDef(destination) || !destination)
+	{
+		destination = start; //Quick way to just scan for pits/cliffs at the start position.
+	}
+	for (let x = -distance; x <= distance; ++x)
+	{
+		for (let y = -distance; y <= distance; ++y)
+		{
+			const tmpStart = {x: start.x + x, y: start.y + y};
+			if ((tmpStart.x < 0) || (tmpStart.y < 0) || (tmpStart.x >= mapWidth) || (tmpStart.y >= mapHeight))
+			{
+				continue; // We don't care about being outside the map here.
+			}
+			const TTYPE = terrainType(tmpStart.x, tmpStart.y);
+			if (!propulsionCanReach(propulsion, destination.x, destination.y, tmpStart.x, tmpStart.y) ||
+				(TTYPE === TER_CLIFFFACE) ||
+				((TTYPE === TER_WATER) && !camIsWaterPropulsion(propulsion)))
+			{
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+//;; ## camGenerateRandomMapEdgeCoordinate(reachPosition [, propulsion [, distFromReach [, scanObjectRadius]]])
+//;;
+//;; Returns a random coordinate anywhere on the edge of the map that reaches a position.
+//;; `reachPosition` may be undefined if you just want a random edge coordinate, without object scans.
+//;; Which can be useful for spawning transporter entry/exit points or VTOL spawn positions.
+//;; `scanObjectRadius` may be defined to scan possible spawn points for nearby objects,
+//;; and should be above one tile if there are large skyscrapers at the edges of some maps.
 //;;
 //;; @param {Object} reachPosition
+//;; @param {String} propulsion
+//;; @param {Number} distFromReach
+//;; @param {Number} scanObjectRadius
 //;; @returns {Object}
 //;;
-function camGenerateRandomMapEdgeCoordinate(reachPosition)
+function camGenerateRandomMapEdgeCoordinate(reachPosition, propulsion, distFromReach, scanObjectRadius)
 {
-	const limits = getScrollLimits();
-	let loc;
-
-	do
+	if (!camDef(propulsion))
 	{
+		propulsion = CAM_GENERIC_LAND_STAT;
+	}
+	if (!camDef(distFromReach))
+	{
+		distFromReach = 0;
+	}
+	const limits = getScrollLimits();
+	const __MAX_ATTEMPTS = 10000;
+	const __DEFINED_POS = (camDef(reachPosition) && reachPosition);
+	const __DEFINED_SCAN = (camDef(scanObjectRadius) && scanObjectRadius);
+	const __OFFSET = 3; // Gives transporters enough space to turn around near map edges.
+	const __SCAN_DIST = 1;
+	let attempts = 0;
+	let breakOut = false;
+	let loc;
+	while (!breakOut)
+	{
+		++attempts;
 		const location = {x: 0, y: 0};
 		let xWasRandom = false;
-
 		if (camRand(100) < 50)
 		{
 			location.x = camRand(limits.x2 + 1);
-			if (location.x < (limits.x + 2))
+			if (location.x < (limits.x + __OFFSET))
 			{
-				location.x = limits.x + 2;
+				location.x = limits.x + __OFFSET;
 			}
-			else if (location.x > (limits.x2 - 2))
+			else if (location.x > (limits.x2 - __OFFSET))
 			{
-				location.x = limits.x2 - 2;
+				location.x = limits.x2 - __OFFSET;
 			}
 			xWasRandom = true;
 		}
 		else
 		{
-			location.x = (camRand(100) < 50) ? (limits.x2 - 2) : (limits.x + 2);
+			location.x = (camRand(100) < 50) ? (limits.x2 - __OFFSET) : (limits.x + __OFFSET);
 		}
-
 		if (!xWasRandom && (camRand(100) < 50))
 		{
 			location.y = camRand(limits.y2 + 1);
-			if (location.y < (limits.y + 2))
+			if (location.y < (limits.y + __OFFSET))
 			{
-				location.y = limits.y + 2;
+				location.y = limits.y + __OFFSET;
 			}
-			else if (location.y > (limits.y2 - 2))
+			else if (location.y > (limits.y2 - __OFFSET))
 			{
-				location.y = limits.y2 - 2;
+				location.y = limits.y2 - __OFFSET;
 			}
 		}
 		else
 		{
-			location.y = (camRand(100) < 50) ? (limits.y2 - 2) : (limits.y + 2);
+			location.y = (camRand(100) < 50) ? (limits.y2 - __OFFSET) : (limits.y + __OFFSET);
 		}
-
 		loc = location;
-	} while (camDef(reachPosition) && reachPosition && !propulsionCanReach("wheeled01", reachPosition.x, reachPosition.y, loc.x, loc.y));
-
+		if ((attempts > __MAX_ATTEMPTS) ||
+			(!__DEFINED_POS ||
+			((camDist(reachPosition.x, reachPosition.y, loc.x, loc.y) >= distFromReach) &&
+			(!__DEFINED_SCAN || !enumRange(loc.x, loc.y, scanObjectRadius, ALL_PLAYERS, false).length) &&
+			!camNearInaccessibleAreas(loc, reachPosition, propulsion, __SCAN_DIST))))
+		{
+			breakOut = true;
+		}
+	}
 	return loc;
 }
 
-//;; ## camGenerateRandomMapCoordinate(reachPosition)
+//;; ## camGenerateRandomMapCoordinate(reachPosition [, propulsion [, distFromReach [, scanObjectRadius, [, avoidNearbyCliffs]]]])
 //;;
-//;; Returns a random coordinate anywhere on the map
+//;; Returns a random coordinate anywhere on the map.
 //;;
 //;; @param {Object} reachPosition
+//;; @param {String} propulsion
+//;; @param {Number} distFromReach
+//;; @param {Number} scanObjectRadius
+//;; @param {Boolean} avoidNearbyCliffs
 //;; @returns {Object}
 //;;
-function camGenerateRandomMapCoordinate(reachPosition, distFromReach, scanObjectRadius)
+function camGenerateRandomMapCoordinate(reachPosition, propulsion, distFromReach, scanObjectRadius, avoidNearbyCliffs)
 {
+	if (!camDef(reachPosition) || !reachPosition)
+	{
+		camDebug("Undefined reachPosition when attempting to generate random coordinate.");
+		return {x: (mapWidth / 2), y: (mapHeight / 2)}; // Better than nothing.
+	}
 	if (!camDef(distFromReach))
 	{
 		distFromReach = 10;
 	}
 	if (!camDef(scanObjectRadius))
 	{
-		scanObjectRadius = 2;
+		scanObjectRadius = 1;
 	}
-
-	const limits = getScrollLimits();
-	let pos;
-
-	do
+	if (!camDef(propulsion))
 	{
+		propulsion = CAM_GENERIC_LAND_STAT;
+	}
+	if (!camDef(avoidNearbyCliffs))
+	{
+		avoidNearbyCliffs = true;
+	}
+	const limits = getScrollLimits();
+	const __MAX_ATTEMPTS = 10000;
+	const __OFFSET = 3; // Gives transporters enough space to turn around near map edges.
+	const __SCAN_DIST = 2;
+	let attempts = 0;
+	let breakOut = false;
+	let pos;
+	while (!breakOut)
+	{
+		++attempts;
 		let randomPos = {x: camRand(limits.x2), y: camRand(limits.y2)};
-
-		if (randomPos.x < (limits.x + 2))
+		let nearPitOrCliff = false;
+		if (randomPos.x < (limits.x + __OFFSET))
 		{
-			randomPos.x = limits.x + 2;
+			randomPos.x = limits.x + __OFFSET;
 		}
-		else if (randomPos.x > (limits.x2 - 2))
+		else if (randomPos.x > (limits.x2 - __OFFSET))
 		{
-			randomPos.x = limits.x2 - 2;
+			randomPos.x = limits.x2 - __OFFSET;
 		}
-
-		if (randomPos.y < (limits.y + 2))
+		if (randomPos.y < (limits.y + __OFFSET))
 		{
-			randomPos.y = limits.y;
+			randomPos.y = limits.y + __OFFSET;
 		}
-		else if (randomPos.y > (limits.y2 - 2))
+		else if (randomPos.y > (limits.y2 - __OFFSET))
 		{
-			randomPos.y = limits.y2 - 2;
+			randomPos.y = limits.y2 - __OFFSET;
 		}
-
 		pos = randomPos;
-	} while (camDef(reachPosition) &&
-		reachPosition &&
-		!propulsionCanReach("wheeled01", reachPosition.x, reachPosition.y, pos.x, pos.y) &&
-		(camDist(pos, reachPosition) < distFromReach) &&
-		(enumRange(pos.x, pos.y, scanObjectRadius, ALL_PLAYERS, false).length > 0));
-
+		// Scan for nearby pits/hills so transporters don't put units inside inaccessible areas.
+		if (avoidNearbyCliffs)
+		{
+			nearPitOrCliff = camNearInaccessibleAreas(pos, reachPosition, propulsion, __SCAN_DIST);
+		}
+		if ((attempts > __MAX_ATTEMPTS) ||
+			((camDist(pos, reachPosition) >= distFromReach) &&
+			propulsionCanReach(propulsion, reachPosition.x, reachPosition.y, pos.x, pos.y) &&
+			(!avoidNearbyCliffs || !nearPitOrCliff) &&
+			!enumRange(pos.x, pos.y, scanObjectRadius, ALL_PLAYERS, false).length))
+		{
+			breakOut = true;
+		}
+	}
 	return pos;
 }
 
@@ -546,11 +719,57 @@ function camDiscoverCampaign()
 	return __CAM_UNKNOWN_CAMPAIGN_NUMBER;
 }
 
-function camSetExpLevel(number)
+//;; ## camGetRankThreshold(rank [, command [, player]])
+//;;
+//;; Returns the rank threshold for a given rank.
+//;;
+//;; @param {String|Number} rank
+//;; @param {Boolean} command
+//;; @param {Number} player
+//;; @returns {number}
+//;;
+function camGetRankThreshold(rankName, command, player)
 {
-	__camExpLevel = number;
+	if (!camDef(command))
+	{
+		command = false;
+	}
+	if (!camDef(player))
+	{
+		player = CAM_HUMAN_PLAYER;
+	}
+	const __BRAIN_TYPE = (command) ? "Command Turret" : "Z NULL BRAIN";
+	let rank = 0;
+	if (typeof rankName === "string")
+	{
+		rank = __camRankStringToNumber(rankName);
+	}
+	return Upgrades[player]["Brain"][__BRAIN_TYPE]["RankThresholds"][rank];
 }
 
+//;; ## camSetExpLevel(rank)
+//;;
+//;; Sets what rank will be used for the AI when it creates units. Can be a rank threshold
+//;; index or the name of the rank.
+//;;
+//;; @param {Number|String} rank
+//;; @returns {void}
+//;;
+function camSetExpLevel(rank)
+{
+	if (!camDef(rank))
+	{
+		rank = 0;
+	}
+	__camExpLevel = (typeof rank === "string") ? __camRankStringToNumber(rank) : rank;
+}
+
+//;; ## camSetOnMapEnemyUnitExp()
+//;;
+//;; Sets all non-player units to the chosen rank set through camSetExpLevel().
+//;;
+//;; @returns {void}
+//;;
 function camSetOnMapEnemyUnitExp()
 {
 	enumDroid(CAM_NEW_PARADIGM)
@@ -638,21 +857,53 @@ function __camAiPowerReset()
 	}
 }
 
-function __camGetExpRangeLevel()
+function __camRankStringToNumber(rankName)
 {
-	const ranks = {
-		rookie: 0,
-		green: 4,
-		trained: 8,
-		regular: 16,
-		professional: 32,
-		veteran: 64,
-		elite: 128,
-		special: 256,
-		hero: 512,
-	}; //see brain.json
-	let exp = [];
+	if (!camDef(rankName))
+	{
+		camDebug("Undefined parameter");
+		return 0;
+	}
+	if (typeof rankName !== "string")
+	{
+		camDebug("Please specify rank as a string");
+		return 0;
+	}
+	let rank = 0;
+	switch (rankName.toLowerCase())
+	{
+		case "rookie": rank = 0; break;
+		case "green": rank = 1; break;
+		case "trained": rank = 2; break;
+		case "regular": rank = 3; break;
+		case "professional": rank = 4; break;
+		case "veteran": rank = 5; break;
+		case "elite": rank = 6; break;
+		case "special": rank = 7; break;
+		case "hero": rank = 8; break;
+		default: camDebug("Unknown rank encountered");
+	}
+	return rank;
+}
 
+function __camGetExpRangeLevel(useCommanderRanks)
+{
+	if (!camDef(useCommanderRanks))
+	{
+		useCommanderRanks = false;
+	}
+	const ranks = {
+		rookie: camGetRankThreshold("rookie", useCommanderRanks),
+		green: camGetRankThreshold("green", useCommanderRanks),
+		trained: camGetRankThreshold("trained", useCommanderRanks),
+		regular: camGetRankThreshold("regular", useCommanderRanks),
+		professional: camGetRankThreshold("professional", useCommanderRanks),
+		veteran: camGetRankThreshold("veteran", useCommanderRanks),
+		elite: camGetRankThreshold("elite", useCommanderRanks),
+		special: camGetRankThreshold("special", useCommanderRanks),
+		hero: camGetRankThreshold("hero", useCommanderRanks)
+	};
+	let exp = [];
 	switch (__camExpLevel)
 	{
 		case 0: // fall-through
@@ -687,7 +938,6 @@ function __camGetExpRangeLevel()
 			__camExpLevel = 0;
 			exp = [ranks.rookie, ranks.rookie];
 	}
-
 	return exp;
 }
 
@@ -701,14 +951,24 @@ function camSetDroidExperience(droid)
 	{
 		return;
 	}
+	const __CMD_RANK = (droid.droidType === DROID_COMMAND || droid.droidType === DROID_SENSOR);
+	const expRange = __camGetExpRangeLevel(__CMD_RANK);
+	const __EXP = expRange[camRand(expRange.length)];
+	setDroidExperience(droid, __EXP);
+}
 
-	const expRange = __camGetExpRangeLevel();
-	let exp = expRange[camRand(expRange.length)];
-
-	if (droid.droidType === DROID_COMMAND || droid.droidType === DROID_SENSOR)
+// Only to prevent prebuilt units from team Gamma on Gamma 6 from having the NavGunSensor.
+function __camRemoveNavGunSensorResearch()
+{
+	if (camDiscoverCampaign() !== __CAM_GAMMA_CAMPAIGN_NUMBER)
+	{
+		return;
+	}
+	for (let i = 0; i < __CAM_MAX_PLAYERS; ++i)
 	{
-		exp = exp * 2;
+		if (i !== CAM_NEXUS && getResearch("R-Sys-NEXUSsensor", i).done)
+		{
+			completeResearch("R-Sys-NEXUSsensorUndo", i, true);
+		}
 	}
-
-	setDroidExperience(droid, exp);
 }
