@@ -60,20 +60,16 @@ function __camEnqueueVideos()
 	{
 		return; //Nothing to play
 	}
-
-	const SOUND_IDENTIFER = ".ogg";
-	var what = __camVideoSequences[0];
-
+	const what = __camVideoSequences[0];
 	// Check if this is a sound to play before some sequence.
-	if (typeof what === "string" && what.indexOf(SOUND_IDENTIFER) !== -1)
+	if (typeof what === "string" && what.indexOf(cam_sounds.soundIdentifier) !== -1)
 	{
 		playSound(what);
 		queue("__camEnqueueVideos", camSecondsToMilliseconds(3.2)); //more than enough for most sounds.
 	}
 	else if (typeof what === "object")
 	{
-		var play = true;
-
+		let play = true;
 		if (!camDef(what.video) || !camIsString(what.video))
 		{
 			camDebug("Problem with video property.");
@@ -88,7 +84,6 @@ function __camEnqueueVideos()
 		{
 			what.immediate = true;
 		}
-
 		if (play)
 		{
 			hackAddMessage(what.video, what.type, CAM_HUMAN_PLAYER, what.immediate);
@@ -98,6 +93,5 @@ function __camEnqueueVideos()
 	{
 		camDebug("Funky camPlayVideos() parameter!");
 	}
-
 	__camVideoSequences.shift();
 }
