@@ -3,7 +3,7 @@
 // Base Under Attack
 function eventAttacked(victimObj, attackerObj)
 {
-	if (gameTime > lastHitTime + 5000 && victimObj.player === selectedPlayer)
+	if ((victimObj.player === selectedPlayer) && (attackerObj.player !== selectedPlayer) && (gameTime > (lastHitTime + 5000)))
 	{
 		lastHitTime = gameTime;
 		if (victimObj.type === STRUCTURE)
