@@ -1,24 +1,24 @@
 function setupPowerModifier(player)
 {
-	if (powerType == 0)
+	if (powerType === 0)
 	{
 		setPowerModifier(85, player);
 	}
-	else if (powerType == 2)
+	else if (powerType === 2)
 	{
 		setPowerModifier(125, player);
 	}
 
 	// insane difficulty is meant to be insane...
-	if (playerData[player].difficulty == INSANE)
+	if (playerData[player].difficulty === INSANE)
 	{
 		setPowerModifier(200 + 15 * powerType, player);
 	}
-	else if (playerData[player].difficulty == HARD)
+	else if (playerData[player].difficulty === HARD)
 	{
 		setPowerModifier(150 + 10 * powerType, player);
 	}
-	else if (playerData[player].difficulty == EASY)
+	else if (playerData[player].difficulty === EASY)
 	{
 		setPowerModifier(70 + 5 * powerType, player);
 	}
