@@ -1,19 +1,19 @@
 function setupTechLevel(player)
 {
-	
+
 	//global function, doc/js-functions
 	var techLevel = getMultiTechLevel();
-	if (techLevel == 2)
+	if (techLevel === 2)
 	{
 		completeResearchOnTime(timeT2, player);
 	}
-	else if (techLevel == 3)
+	else if (techLevel === 3)
 	{
 		completeResearchOnTime(timeT3, player);
 	}
-	else if (techLevel == 4)
+	else if (techLevel === 4)
 	{
 		completeAllResearch(player);
 	}
-	
+
 }
