@@ -62,7 +62,7 @@ var debugName = me;
 
 
 //Массив конкретных технологий (tech.js)
-var tech = [];
+var tech = {};
 
 include("multiplay/skirmish/"+vernum+"/names.js");
 
@@ -195,10 +195,10 @@ else rage = HARD;
 var buildersMain = newGroup();
 var buildersHunters = newGroup();
 
-var policy = [];
+var policy = {};
 
 //Фитчи, не совместимые с 3.1.5
-var nf = [];
+var nf = {};
 nf['policy'] = false;
 
 var enemyDist = 0;
@@ -238,7 +238,7 @@ var credit = 0;
 var lassat_charged = false;
 
 
-var eventsRun=[];
+var eventsRun={};
 eventsRun['targetCyborgs'] = 0;
 eventsRun['targetArmy'] = 0;
 eventsRun['targetRegular'] = 0;
@@ -422,7 +422,7 @@ function init(){
 //			debugMsg("TEST: "+bc_ally.length, 'research');
 			//			debugName = colors[data.colour];
 		}
-		else if (playerLoose(player)) {msg+=" отсутствует";}
+		else if (playerLose(player)) {msg+=" отсутствует";}
 		else if (playerSpectator(player)) {msg+=" наблюдатель";}
 		else if (allianceExistsBetween(me,player)) {
 			msg+=" мой союзник ";
