@@ -26,17 +26,19 @@ function getInfoNear(x,y,command,range,time,obj,cheat,inc){
 
 	debugMsg(x+'x'+y+' '+command, 'gi');
 
-	if (typeof _globalInfoNear[x+'_'+y+'_'+command] !== "undefined"
-	&& gameTime < (_globalInfoNear[x+'_'+y+'_'+command].setTime + _globalInfoNear[x+'_'+y+'_'+command].updateIn)) {
+	if (typeof _globalInfoNear[x+'_'+y+'_'+command] !== "undefined" &&
+		gameTime < (_globalInfoNear[x+'_'+y+'_'+command].setTime + _globalInfoNear[x+'_'+y+'_'+command].updateIn))
+	{
 		if (inc) {
 			_globalInfoNear[x+'_'+y+'_'+command].value++;
 		}
 		return _globalInfoNear[x+'_'+y+'_'+command];
 	} else {
+		var view;
 		if (typeof time === "undefined") time = 30000;
 		if (typeof range === "undefined") range = 7;
-		if (typeof cheat === "undefined") var view = me;
-		else if (cheat) var view = -1;
+		if (typeof cheat === "undefined") view = me;
+		else if (cheat) view = -1;
 //		_globalInfoNear[x+'_'+y+'_'+command] = [];
 		_globalInfoNear[x+'_'+y+'_'+command] = { setTime: gameTime, updateIn: time };
 
@@ -120,7 +122,7 @@ function checkAlly(){
 			+", ally=spectator, dist="+dist, "ally");
 			return;
 		}
-		if (playerLoose(player)) {
+		if (playerLose(player)) {
 			debugMsg("#"+player+" name=\"Empty Slot\""
 			+", ally=empty, dist="+dist, "ally");
 			return;
@@ -342,8 +344,8 @@ function checkProcess(){
 	if (!running)return;
 	if (Math.floor(gameTime / 1000) < 300) return;
 
-	if (playerLoose(me)) {
-		gameStop("loose");
+	if (playerLose(me)) {
+		gameStop("lose");
 	}
 
 	for (const plally in bc_ally) {
@@ -358,8 +360,9 @@ function checkProcess(){
 			}
 
 			if (enumDroid(bc_ally[plally], DROID_CONSTRUCT).length < 3) {
+				var truck;
 				if (groupSize(buildersMain) >= 2) {
-					var truck = enumGroup(buildersMain);
+					truck = enumGroup(buildersMain);
 					if (truck.length > 0) {
 						debugMsg("Send builder["+truck[0].id+"] from "+me+" to "+bc_ally[plally], 'ally');
 						donateObject(truck[0], bc_ally[plally]);
@@ -368,7 +371,7 @@ function checkProcess(){
 				}
 
 				if (groupSize(buildersHunters) > 1) {
-					var truck = enumGroup(buildersHunters);
+					truck = enumGroup(buildersHunters);
 					if (truck.length > 0) {
 						donateObject(truck[0], bc_ally[plally]);
 						debugMsg("Send hunter["+truck[0].id+"] from "+me+" to "+bc_ally[plally], 'ally');
@@ -383,11 +386,11 @@ function checkProcess(){
 function gameStop(condition){
 
 
-	if (condition === 'loose') {
-		debugMsg("I guess, i'm loose.. Give up", 'end');
+	if (condition === 'lose') {
+		debugMsg("I guess, I'm loser.. Give up", 'end');
 		if (running) {
 			playerData.forEach((data, player) => {
-				if (!asPlayer)chat(player, ' from '+debugName+': '+chatting('loose'));
+				if (!asPlayer)chat(player, ' from '+debugName+': '+chatting('lose'));
 			});
 		}
 
@@ -396,7 +399,7 @@ function gameStop(condition){
 		var _ally=false;
 		playerData.forEach((data, player) => {
 			if (player === me) return;
-			if (playerLoose(player)) return;
+			if (playerLose(player)) return;
 			if (playerSpectator(player)) return;
 			if (!allianceExistsBetween(me,player)) return;
 			if (allianceExistsBetween(me,player)) {
@@ -422,22 +425,22 @@ function gameStop(condition){
 		running = false;
 }
 
-function playerLoose(player){
-	var loose = false;
-	if (enumStruct(player,"A0LightFactory").length === 0
-		&& enumDroid(player, DROID_CONSTRUCT).length === 0
-		&& enumStruct(player,"A0CyborgFactory").length === 0
-		&& enumDroid(player, 10).length === 0) loose = true;
-	return loose;
+function playerLose(player){
+	var lose = false;
+	if (enumStruct(player,"A0LightFactory").length === 0 &&
+		enumDroid(player, DROID_CONSTRUCT).length === 0 &&
+		enumStruct(player,"A0CyborgFactory").length === 0 &&
+		enumDroid(player, 10).length === 0) lose = true;
+	return lose;
 }
 
 function playerSpectator(player){
-	var loose = false;
-	if ((enumStruct(player, "A0Sat-linkCentre").length === 1 || enumStruct(player, "A0CommandCentre").length === 1)
-		&& enumStruct(player,"A0LightFactory").length === 0
-		&& enumStruct(player,"A0CyborgFactory").length === 0
-		&& enumDroid(player, 10).length === 0) loose = true;
-	return loose;
+	var lose = false;
+	if ((enumStruct(player, "A0Sat-linkCentre").length === 1 || enumStruct(player, "A0CommandCentre").length === 1) &&
+		enumStruct(player,"A0LightFactory").length === 0 &&
+		enumStruct(player,"A0CyborgFactory").length === 0 &&
+		enumDroid(player, 10).length === 0) lose = true;
+	return lose;
 }
 
 //функция отфильтровывает объекты, которые находяться близко
@@ -446,7 +449,7 @@ function filterNearAlly(obj){
 	for (let p = 0; p < maxPlayers; ++p) {
 		if (p === me) continue; //Выкидываем себя
 		if (!allianceExistsBetween(me,p)) continue; //Выкидываем вражеские
-//		if (playerLoose(p)) continue; //Пропускаем проигравших
+//		if (playerLose(p)) continue; //Пропускаем проигравших
 		if (playerSpectator(p)) continue;
 
 		//Если союзнику доступно 40 вышек, не уступаем ему свободную.
@@ -496,7 +499,7 @@ function getEnemyNearAlly(){
 	for (let p = 0; p < maxPlayers; ++p) {
 		if (p === me) continue;
 		if (!allianceExistsBetween(me,p)) continue;
-//		if (playerLoose(p)) continue; //Пропускаем проигравших
+//		if (playerLose(p)) continue; //Пропускаем проигравших
 		if (playerSpectator(p)) continue;
 //		enemy = enemy.concat(targ.filter((e) => {if (distBetweenTwoPoints_p(e.x,e.y,startPositions[p].x,startPositions[p].y) < (base_range/2)) {
 		enemy = enemy.concat(targ.filter((e) => {if (distBetweenTwoPoints_p(e.x,e.y,startPositions[p].x,startPositions[p].y) < base_range) {
@@ -677,7 +680,7 @@ function getNumEnemies(){
 	for (let e = 0; e < maxPlayers; ++e) {
 		if (allianceExistsBetween(me,e)) continue;
 		if (playerSpectator(e)) continue;
-		if (playerLoose(e)) continue;
+		if (playerLose(e)) continue;
 		if (e === me) continue;
 		enemies++;
 	}
@@ -751,7 +754,7 @@ function getEnemyStartPos(){
 	for (let e = 0; e < maxPlayers; ++e) {
 		if (allianceExistsBetween(me,e)) continue;
 		if (playerSpectator(e)) continue;
-		if (playerLoose(e)) continue;
+		if (playerLose(e)) continue;
 		targ = targ.concat(startPositions[e]);
 	}
 	return targ;
@@ -1008,7 +1011,7 @@ var isFixVTOL = function(obj) {
 	} catch(e) {
 //		debugMsg("isFixVTOL(): "+e.message, 'error');
 	}
-}
+};
 
 //from: http://stackoverflow.com/questions/6274339/how-can-i-shuffle-an-array
 function shuffle(a) {
@@ -1100,7 +1103,7 @@ function longCycle(){
 		playerData.forEach((data, player) => {
 			if (!access) return;
 			if (player === me) return;
-			if (playerLoose(player)) return;
+			if (playerLose(player)) return;
 			if (playerSpectator(player)) return;
 			if (allianceExistsBetween(me,player)) return;
 			if (propulsionCanReach('wheeled01', base.x, base.y, startPositions[player].x, startPositions[player].y)) { access = false; debugMsg("LongCycle: "+player+', land - exit', 'debug'); return; }
@@ -1151,7 +1154,7 @@ function longCycle(){
 	if (playerPower(me) > 10000) {
 		playerData.forEach((data, player) => {
 			if (player === me) return;
-			if (playerLoose(player)) return;
+			if (playerLose(player)) return;
 			if (playerSpectator(player)) return;
 			if (!allianceExistsBetween(me,player)) return;
 			if (allianceExistsBetween(me,player)) {
@@ -1266,7 +1269,7 @@ function getFleetPoint(droid){
 	droidsNear = sortByDistance(droidsNear, base);
 //	debugMsg('dl:'+droidsNear.length, 'temp');
 //	debugMsg('dr:'+droidCanReach(droid, base.x, base.y), 'temp');
-	if (droidsNear.length === 0 || droidsNear[0].id === droid.id) return base
+	if (droidsNear.length === 0 || droidsNear[0].id === droid.id) return base;
 	return droidsNear[0];
 
 }
