@@ -3,7 +3,7 @@
 //they destroy whatever thay are attacking or need repair.
 function droidReady(droidID)
 {
-	var droid = getObject(DROID, me, droidID);
+	let droid = getObject(DROID, me, droidID);
 	if (droid === null)
 	{
 		return false;
@@ -33,7 +33,7 @@ function beaconAreaHasEnemies()
 
 	const MAX_SCAN_RANGE = 15;
 
-	return enumRange(beacon.x, beacon.y, MAX_SCAN_RANGE, ENEMIES, false).length !== 0;
+	return enumRange(beacon.x, beacon.y, MAX_SCAN_RANGE, ENEMIES, false).length > 0;
 }
 
 //Modified from Nullbot.
@@ -54,7 +54,7 @@ function returnClosestEnemyFactory(enemyNumber)
 			enemyNumber = getMostHarmfulPlayer();
 		}
 
-		var facs = enumStruct(enemyNumber, FACTORY)
+		let facs = enumStruct(enemyNumber, FACTORY)
 		.concat(enumStruct(enemyNumber, CYBORG_FACTORY))
 		.concat(enumStruct(enemyNumber, VTOL_FACTORY));
 
@@ -73,7 +73,7 @@ function returnClosestEnemyFactory(enemyNumber)
 //Should the vtol attack when ammo is high enough?
 function vtolReady(droidID)
 {
-	var droid = getObject(DROID, me, droidID);
+	let droid = getObject(DROID, me, droidID);
 	if (droid === null)
 	{
 		return true;  //Pretend it is busy
@@ -104,7 +104,7 @@ function vtolReady(droidID)
 //Repair a droid with the option of forcing it to.
 function repairDroid(droidID, force)
 {
-	var droid = getObject(DROID, me, droidID);
+	let droid = getObject(DROID, me, droidID);
 	if (droid === null)
 	{
 		return true; //pretend it is busy
@@ -115,7 +115,7 @@ function repairDroid(droidID, force)
 		return false; //Plasma Cannon shall never repair
 	}
 
-	var highOil = highOilMap();
+	let highOil = highOilMap();
 
 	if (droid.health < 15 || (highOil && gameTime < 600000))
 	{
@@ -124,14 +124,14 @@ function repairDroid(droidID, force)
 
 	const SAFE_EXTREME_OIL_IGNORE_NUM = 80;
 
-	var forceRepairPercent = 50;
+	let forceRepairPercent = 50;
 	const EXPERIENCE_DIVISOR = 26;
 	const HEALTH_TO_REPAIR = forceRepairPercent + Math.floor(droid.experience / EXPERIENCE_DIVISOR);
 
 	//Ignore repairing combat units if one super high oil map if we have X amount of units.
 	if (countDroid(DROID_ANY, me) >= SAFE_EXTREME_OIL_IGNORE_NUM && highOilMap)
 	{
-		if ((droid.droidType === DROID_WEAPON || droid.droidType == DROID_CYBORG) && droid.health <= forceRepairPercent)
+		if ((droid.droidType === DROID_WEAPON || droid.droidType === DROID_CYBORG) && droid.health <= forceRepairPercent)
 		{
 			return false;
 		}
@@ -152,7 +152,7 @@ function repairDroid(droidID, force)
 		return true;
 	}
 
-	if (countStruct(structures.repair) && (force || (Math.floor(droid.health) <= HEALTH_TO_REPAIR)))
+	if (countStruct(structures.repair, me) && (force || (Math.floor(droid.health) <= HEALTH_TO_REPAIR)))
 	{
 		orderDroid(droid, DORDER_RTR);
 		return true;
@@ -164,13 +164,13 @@ function repairDroid(droidID, force)
 //Continuously check a random ground group for repair
 function checkAllForRepair()
 {
-	if (!countStruct(structures.repair))
+	if (!countStruct(structures.repair, me))
 	{
 		return;
 	}
 
-	var droids = enumGroup(attackGroup);
-	for (var i = 0, l = droids.length; i < l; ++i)
+	let droids = enumGroup(attackGroup);
+	for (let i = 0, l = droids.length; i < l; ++i)
 	{
 		repairDroid(droids[i].id);
 	}
@@ -187,10 +187,10 @@ function findNearestEnemyDroid(enemy)
 			enemy = getMostHarmfulPlayer();
 		}
 
-		var badDroids = enumDroid(enemy);
+		let badDroids = enumDroid(enemy);
 		if (badDroids.length > 0)
 		{
-			var temp = badDroids.filter(function(dr) { return !isVTOL(dr); });
+			let temp = badDroids.filter((dr) => (!isVTOL(dr)));
 			if (temp.length === 0)
 			{
 				temp = badDroids;
@@ -216,7 +216,7 @@ function findNearestEnemyStructure(enemy)
 			enemy = getMostHarmfulPlayer();
 		}
 
-		var s = enumStruct(enemy).filter(function(obj) { return (obj.stattype !== WALL); });
+		let s = enumStruct(enemy).filter((obj) => (obj.stattype !== WALL));
 		if (s.length === 0)
 		{
 			s = enumStruct(enemy);
@@ -244,7 +244,7 @@ function findNearestEnemyDerrick(enemy)
 			enemy = getMostHarmfulPlayer();
 		}
 
-		var derr = enumStruct(enemy, structures.derricks).sort(distanceToBase);
+		let derr = enumStruct(enemy, structures.derricks).sort(distanceToBase);
 
 		if (derr.length > 0)
 		{
@@ -264,9 +264,7 @@ function artilleryTactics()
 	{
 		return;
 	}
-	var sensors = enumGroup(sensorGroup).filter(function(dr) {
-		return droidReady(dr.id);
-	});
+	let sensors = enumGroup(sensorGroup).filter((dr) => (droidReady(dr.id)));
 	const ARTILLERY_UNITS = enumGroup(artilleryGroup);
 	const ARTI_LEN = ARTILLERY_UNITS.length;
 	const SENS_LEN = sensors.length;
@@ -274,18 +272,18 @@ function artilleryTactics()
 	if (SENS_LEN + ARTI_LEN > 0)
 	{
 		sensors = sortAndReverseDistance(sensors);
-		var obj = rangeStep();
+		let obj = rangeStep();
 
 		if (isDefined(obj))
 		{
-			var tempObj = getObject(obj.typeInfo, obj.playerInfo, obj.idInfo);
+			let tempObj = getObject(obj.typeInfo, obj.playerInfo, obj.idInfo);
 
 			if (SENS_LEN)
 			{
 				orderDroidObj(sensors[0], DORDER_OBSERVE, tempObj);
 			}
 
-			for (var i = 0; i < ARTI_LEN; ++i)
+			for (let i = 0; i < ARTI_LEN; ++i)
 			{
 				//Send artillery to help at beacon, if possible
 				if ((beacon.endTime > gameTime) &&
@@ -295,10 +293,10 @@ function artilleryTactics()
 					if (!beacon.wasVtol || (beacon.wasVtol && ARTILLERY_UNITS[i].weapons[0].canHitAir))
 					{
 						//Attack something in this area, if possible.
-						var xRand = (random(100) < 50) ? random(15) : -random(15);
-						var yRand = (random(100) < 50) ? random(15) : -random(15);
-						var xPos = beacon.x + xRand;
-						var yPos = beacon.y + yRand;
+						let xRand = (random(100) < 50) ? random(15) : -random(15);
+						let yRand = (random(100) < 50) ? random(15) : -random(15);
+						let xPos = beacon.x + xRand;
+						let yPos = beacon.y + yRand;
 
 						if (xPos < 2)
 						{
@@ -335,21 +333,21 @@ function groundTactics()
 	{
 		donateSomePower();
 
-		var target = rangeStep();
+		let target = rangeStep();
 		if (isDefined(target))
 		{
-			const UNITS = enumGroup(attackGroup).filter(function(dr) {
-				return droidReady(dr.id);
-			});
+			const UNITS = enumGroup(attackGroup).filter((dr) => (
+				droidReady(dr.id)
+			));
 
 			if (UNITS.length < MIN_ATTACK_DROIDS)
 			{
 				return;
 			}
 
-			for (var i = 0, l = UNITS.length; i < l; ++i)
+			for (let i = 0, l = UNITS.length; i < l; ++i)
 			{
-				var id = UNITS[i].id;
+				let id = UNITS[i].id;
 
 				//Send most of army to beacon explicitly
 				if ((beacon.endTime > gameTime) &&
@@ -359,10 +357,10 @@ function groundTactics()
 					if (!beacon.wasVtol || (beacon.wasVtol && UNITS[i].weapons[0].canHitAir))
 					{
 						//Attack something in this area, if possible.
-						var xRand = (random(100) < 50) ? random(15) : -random(15);
-						var yRand = (random(100) < 50) ? random(15) : -random(15);
-						var xPos = beacon.x + xRand;
-						var yPos = beacon.y + yRand;
+						let xRand = (random(100) < 50) ? random(15) : -random(15);
+						let yRand = (random(100) < 50) ? random(15) : -random(15);
+						let xPos = beacon.x + xRand;
+						let yPos = beacon.y + yRand;
 
 						if (xPos < 2)
 						{
@@ -405,17 +403,17 @@ function recycleForHover()
 	}
 
 	const MIN_FACTORY = 1;
-	var systems = enumDroid(me, DROID_CONSTRUCT).concat(enumDroid(me, DROID_SENSOR)).filter(function(dr) {
-		return (dr.body !== "CyborgLightBody" && dr.propulsion !== "hover01");
-	});
-	var unfinished = unfinishedStructures();
+	let systems = enumDroid(me, DROID_CONSTRUCT).concat(enumDroid(me, DROID_SENSOR)).filter((dr) => (
+		dr.body !== "CyborgLightBody" && dr.propulsion !== "hover01"
+	));
+	let unfinished = unfinishedStructures();
 	const NON_HOVER_SYSTEMS = systems.length;
 
-	if ((countStruct(structures.factory) > MIN_FACTORY) && componentAvailable("hover01"))
+	if ((countStruct(structures.factory, me) > MIN_FACTORY) && componentAvailable("hover01"))
 	{
 		if (!unfinished.length && NON_HOVER_SYSTEMS)
 		{
-			for (var i = 0; i < NON_HOVER_SYSTEMS; ++i)
+			for (let i = 0; i < NON_HOVER_SYSTEMS; ++i)
 			{
 				orderDroid(systems[i], DORDER_RECYCLE);
 			}
@@ -428,9 +426,9 @@ function recycleForHover()
 
 		if (forceHover)
 		{
-			var tanks = enumGroup(attackGroup).filter(function(dr) { return (dr.propulsion !== "hover01"); });
+			let tanks = enumGroup(attackGroup).filter((dr) => (dr.propulsion !== "hover01"));
 			const NON_HOVER_TANKS = tanks.length;
-			for (var j = 0; j < NON_HOVER_TANKS; ++j)
+			for (let j = 0; j < NON_HOVER_TANKS; ++j)
 			{
 				orderDroid(tanks[j], DORDER_RECYCLE);
 			}
@@ -449,14 +447,18 @@ function targetPlayer(playerNumber)
 	const PREVIOUS_TARGET = getMostHarmfulPlayer();
 	if (isDefined(scavengerPlayer) && ((playerNumber === scavengerPlayer) || (PREVIOUS_TARGET === scavengerPlayer)))
 	{
-		return; //No targeting scavs.
+		return false; //No targeting scavs.
 	}
 
-	if (playerNumber !== PREVIOUS_TARGET)
+	if (playerNumber === PREVIOUS_TARGET)
 	{
-		const INC = 400;
-		grudgeCount[playerNumber] = grudgeCount[PREVIOUS_TARGET] + INC;
+		return false;
 	}
+
+	const INC = 400;
+	grudgeCount[playerNumber] = grudgeCount[PREVIOUS_TARGET] + INC;
+
+	return true;
 }
 
 //VTOL units do there own form of tactics.
@@ -468,28 +470,26 @@ function vtolTactics()
 	}
 
 	const MIN_VTOLS = 5;
-	var vtols = enumGroup(vtolGroup).filter(function(dr) {
-		return droidReady(dr.id);
-	});
+	let vtols = enumGroup(vtolGroup).filter((dr) => (droidReady(dr.id)));
 	const LEN = vtols.length;
 	const D_CIRCLE = 40; //DORDER_CIRCLE = 40
 	const SCOUT_TO_CIRCLE_DIST = 2; //when to switch from scout to circle order for beacon
 
 	if (LEN >= MIN_VTOLS)
 	{
-		var target = rangeStep();
+		let target = rangeStep();
 
 		if (isDefined(target))
 		{
-			for (var i = 0; i < LEN; ++i)
+			for (let i = 0; i < LEN; ++i)
 			{
-				var id = vtols[i].id;
+				let id = vtols[i].id;
 
 				if ((beacon.endTime > gameTime) &&
 					(i < Math.floor(LEN * subPersonalities[personality].beaconVtolPercentage)) &&
 					(!beacon.wasVtol || (beacon.wasVtol && vtols[i].weapons[0].canHitAir)))
 				{
-					var pos = {x: vtols[i].x, y: vtols[i].y};
+					let pos = {x: vtols[i].x, y: vtols[i].y};
 					//Patrol this area for a bit.
 					if (vtols[i].order === D_CIRCLE)
 					{
@@ -515,19 +515,19 @@ function vtolTactics()
 //the object returned by objectInformation().
 function attackThisObject(droidID, target)
 {
-	var d = getObject(DROID, me, droidID);
+	let d = getObject(DROID, me, droidID);
 	if (d === null || !isDefined(d.weapons[0]))
 	{
 		return;
 	}
 
-	var t = getObject(target.typeInfo, target.playerInfo, target.idInfo);
+	let t = getObject(target.typeInfo, target.playerInfo, target.idInfo);
 	if (t === null)
 	{
 		return;
 	}
 
-	var isScav = isDefined(scavengerPlayer) && t.player === scavengerPlayer;
+	let isScav = isDefined(scavengerPlayer) && t.player === scavengerPlayer;
 	if (droidReady(droidID) && droidCanReach(d, t.x, t.y))
 	{
 		if (isScav && getMostHarmfulPlayer() !== scavengerPlayer)
@@ -553,10 +553,10 @@ function attackThisObject(droidID, target)
 //Check if enemy units are in or around Cobra base.
 function enemyUnitsInBase()
 {
-	var area = cobraBaseArea();
-	var enemyUnits = enumArea(area.x1, area.y1, area.x2, area.y2, ENEMIES, false).filter(function(obj) {
-		return (obj.type === DROID && (obj.droidType === DROID_WEAPON || obj.droidType === DROID_CYBORG));
-	}).sort(distanceToBase);
+	let area = cobraBaseArea();
+	let enemyUnits = enumArea(area.x1, area.y1, area.x2, area.y2, ENEMIES, false).filter((obj) => (
+		obj.type === DROID && (obj.droidType === DROID_WEAPON || obj.droidType === DROID_CYBORG)
+	)).sort(distanceToBase);
 
 	//The attack code automatically chooses the closest object of the
 	//most harmful player anyway so this should suffice for defense.
@@ -574,7 +574,7 @@ function enemyUnitsInBase()
 		//Send a beacon that enemies are in my base area! Allied Cobra AI can interpret and help friends through this drop.
 		if (!beacon.disabled && (beacon.endTime < gameTime))
 		{
-			var mes = isVTOL(enemyUnits[0]) ? BEACON_VTOL_ALARM : undefined;
+			let mes = isVTOL(enemyUnits[0]) ? BEACON_VTOL_ALARM : undefined;
 			addBeacon(enemyUnits[0].x, enemyUnits[0].y, ALLIES, mes);
 			//Set beacon data for me also since we won't receive our own beacon.
 			eventBeacon(enemyUnits[0].x, enemyUnits[0].y, me, me, mes);
@@ -593,18 +593,18 @@ function donateSomePower()
 	{
 		return;
 	}
-	if (!countStruct(structures.gen) || !countStruct(structures.derrick))
+	if (!countStruct(structures.gen, me) || !countStruct(structures.derrick, me))
 	{
 		return;
 	}
 
-	const ALLY_PLAYERS = playerAlliance(true).filter(function(player) { return playerData[player].isAI; });
+	const ALLY_PLAYERS = playerAlliance(true).filter((player) => (playerData[player].isAI));
 	const LEN = ALLY_PLAYERS.length;
 	const ALIVE_ENEMIES = findLivingEnemies().length;
 
 	if (LEN > 0 && ALIVE_ENEMIES > 0)
 	{
-		var ally = ALLY_PLAYERS[random(LEN)];
+		let ally = ALLY_PLAYERS[random(LEN)];
 		if (getRealPower() > 300 && (getRealPower() > Math.floor(1.5 * getRealPower(ally))))
 		{
 			donatePower(playerPower(me) / 2, ally);
@@ -617,7 +617,7 @@ function haveEnoughUnitsForFirstAttack()
 {
 	if (!startAttacking)
 	{
-		var amountOfAttackers = groupSize(attackGroup) + groupSize(artilleryGroup) + groupSize(vtolGroup);
+		let amountOfAttackers = groupSize(attackGroup) + groupSize(artilleryGroup) + groupSize(vtolGroup);
 		// These amounts of units will build up in base if unprovoked
 		startAttacking = amountOfAttackers >= (highOilMap() ? 120 : 20);
 	}
@@ -632,14 +632,14 @@ function baseShuffleDefensePattern()
 		return; //Prevent the dreadful jitter movement defense pattern.
 	}
 
-	var attackers = enumGroup(attackGroup).concat(enumGroup(artilleryGroup)).concat(enumGroup(vtolGroup));
+	let attackers = enumGroup(attackGroup).concat(enumGroup(artilleryGroup)).concat(enumGroup(vtolGroup));
 	if (attackers.length === 0)
 	{
 		return;
 	}
 
-	var area = cobraBaseArea();
-	var quad = [
+	let area = cobraBaseArea();
+	let quad = [
 		{x1: area.x1, x2: area.x2, y1: area.y1, y2: area.y1 + 10,},
 		{x1: area.x1, x2: area.x2 + 10, y1: area.y1, y2: area.y2,},
 		{x1: area.x2 - 10, x2: area.x2, y1: area.y1, y2: area.y2,},
@@ -647,10 +647,10 @@ function baseShuffleDefensePattern()
 	];
 
 	const MAX_ATTEMPTS = 100;
-	var sector;
-	var x;
-	var y;
-	var attempts = 0;
+	let sector;
+	let x;
+	let y;
+	let attempts = 0;
 
 	do
 	{
@@ -684,7 +684,7 @@ function baseShuffleDefensePattern()
 	// Given that the base area has an additional 20 tiles of territory around the furthest base structure in a rectangel/square
 	// we can safely tell units to go into this territory zone to keep trucks from being obstructed, maybe.
 	const MAX_NEARBY_STRUCTURES = 2;
-	for (var i = 0, len = attackers.length; i < len; ++i)
+	for (let i = 0, len = attackers.length; i < len; ++i)
 	{
 		if (attackers[i].order !== DORDER_HOLD)
 		{
@@ -709,21 +709,21 @@ function lowOilDefensePattern()
 		return; //visit a derrick for a bit... maybe
 	}
 
-	var derricks = enumStruct(me, structures.derrick);
+	let derricks = enumStruct(me, structures.derrick);
 	if (derricks.length === 0)
 	{
 		return;
 	}
 
-	var attackers = enumGroup(attackGroup).concat(enumGroup(artilleryGroup)).concat(enumGroup(vtolGroup));
+	let attackers = enumGroup(attackGroup).concat(enumGroup(artilleryGroup)).concat(enumGroup(vtolGroup));
 	if (attackers.length === 0)
 	{
 		return;
 	}
 
-	for (var i = 0, len = attackers.length; i < len; ++i)
+	for (let i = 0, len = attackers.length; i < len; ++i)
 	{
-		var derr = derricks[random(derricks.length)];
+		let derr = derricks[random(derricks.length)];
 		orderDroidLoc(attackers[i], DORDER_SCOUT, derr.x, derr.y);
 	}
 
@@ -756,15 +756,15 @@ function shouldCobraAttack()
 function retreatTactics()
 {
 	const SCAN_RADIUS = subPersonalities[personality].retreatScanRange;
-	var droids = enumGroup(retreatGroup);
+	let droids = enumGroup(retreatGroup);
 
 	//Flee!
-	for (var i = 0, len = droids.length; i < len; ++i)
+	for (let i = 0, len = droids.length; i < len; ++i)
 	{
-		var droid = droids[i];
-		var friends = enumRange(droid.x, droid.y, SCAN_RADIUS, ALLIES, false).filter(function(obj) {
-			return obj.type === DROID;
-		});
+		let droid = droids[i];
+		let friends = enumRange(droid.x, droid.y, SCAN_RADIUS, ALLIES, false).filter((obj) => (
+			obj.type === DROID
+		));
 
 		if (enumRange(droid.x, droid.y, SCAN_RADIUS, ENEMIES, !highOilMap()).length > friends.length)
 		{
