@@ -35,7 +35,7 @@ function isEnemy(player) {
 function enumLivingPlayers() {
 	function uncached() {
 		var ret = [];
-		for (var i = 0; i < maxPlayers; ++i) {
+		for (let i = 0; i < maxPlayers; ++i) {
 			if (countStructList(targets, i) > 0) {
 				ret.push(i);
 				continue;
@@ -44,7 +44,7 @@ function enumLivingPlayers() {
 				ret.push(i);
 		}
 		return ret;
-	};
+	}
 	return cached(uncached, 30000);
 }
 
@@ -58,7 +58,7 @@ function goEasy() {
 
 function areThereGW() {
 	function uncached() {	//should only need to be evaluated once at the start
-		if(gateways.length > 0)
+		if (gateways.length > 0)
 			return true;
 		return false;
 	}
