@@ -8,24 +8,24 @@
 ////////////////////////////////////////////////////////////////////////////////////////////
 
 _global.isHoverPropulsion = function(str) {
-	return propulsionStats.some(function(val) { return val.usage === PROPULSIONUSAGE.HOVER && val.stat === str; });
+	return propulsionStats.some((val) => (val.usage === PROPULSIONUSAGE.HOVER && val.stat === str));
 }
 
 _global.isVtolPropulsion = function(str) {
-	return propulsionStats.some(function(val) { return val.usage === PROPULSIONUSAGE.VTOL && val.stat === str; });
+	return propulsionStats.some((val) => (val.usage === PROPULSIONUSAGE.VTOL && val.stat === str));
 }
 
 _global.iHaveHover = function() {
-	return propulsionStats.some(function(val) { return val.usage === PROPULSIONUSAGE.HOVER && componentAvailable(val.stat);} );
+	return propulsionStats.some((val) => (val.usage === PROPULSIONUSAGE.HOVER && componentAvailable(val.stat)));
 }
 
 _global.iHaveVtol = function() {
-    return propulsionStats.some(function(val) { return val.usage === PROPULSIONUSAGE.VTOL && componentAvailable(val.stat);} );
+	return propulsionStats.some((val) => (val.usage === PROPULSIONUSAGE.VTOL && componentAvailable(val.stat)));
 }
 
 _global.iHaveArty = function() {
-	for (var stat in weaponStats)
-		for (var i = 0; i < weaponStats[stat].defenses.length; ++i)
+	for (const stat in weaponStats)
+		for (let i = 0; i < weaponStats[stat].defenses.length; ++i)
 			if (weaponStats[stat].defenses[i].defrole === DEFROLE.ARTY)
 				if (countStruct(weaponStats[stat].defenses[i].stat) > 0)
 					return true;
@@ -40,13 +40,13 @@ _global.safeIsVtol = function(droid) {
 _global.enumStructList = function(list, player) {
 	if (!defined(player))
 		player = me;
-	return list.reduce(function(summ, new_value) { return summ.concat(enumStruct(player, new_value)); }, []);
+	return list.reduce((summ, new_value) => (summ.concat(enumStruct(player, new_value))), []);
 }
 
 _global.countStructList = function(list, player) {
 	if (!defined(player))
 		player = me;
-	return list.reduce(function(summ, new_value) { return summ + countStruct(new_value, player); }, 0);
+	return list.reduce((summ, new_value) => (summ + countStruct(new_value, player)), 0);
 }
 
 _global.enumFinishedStructList = function(list, player) {
@@ -62,15 +62,15 @@ _global.enumIdleStructList = function(list, player) {
 }
 
 _global.structListLimit = function(list) {
-	return list.reduce(function (summ, val) {return summ + getStructureLimit(val)}, 0)
+	return list.reduce((summ, val) => (summ + getStructureLimit(val)), 0);
 }
 
 _global.guessWeaponRole = function (name) {
-	for (var stat in weaponStats) {
+	for (const stat in weaponStats) {
 		if (
 			weaponStats[stat].weapons.someProperty("stat", name) ||
 			weaponStats[stat].vtols.someProperty("stat", name) ||
-			weaponStats[stat].templates.some(function (i) {return i.weapons.indexOf(name) > -1;})
+			weaponStats[stat].templates.some((i) => (i.weapons.indexOf(name) > -1))
 		)
 		return weaponStats[stat].roles;
 	}
@@ -79,11 +79,11 @@ _global.guessWeaponRole = function (name) {
 
 function guessWeaponMicro(name) {
 	function uncached() {
-		for (var stat in weaponStats)
+		for (const stat in weaponStats)
 		{
 			if (weaponStats[stat].weapons.someProperty("stat", name))
 				return weaponStats[stat].micro;
-			if (weaponStats[stat].templates.some(function(i) { return i.weapons.indexOf(name) > -1; }))
+			if (weaponStats[stat].templates.some((i) => (i.weapons.indexOf(name) > -1)))
 				return weaponStats[stat].micro;
 		}
 	}
@@ -91,7 +91,7 @@ function guessWeaponMicro(name) {
 }
 
 _global.guessDroidMicro = function(droid) {
-	for (var i = 0; i < droid.weapons.length; ++i) {
+	for (let i = 0; i < droid.weapons.length; ++i) {
 		var ret = guessWeaponMicro(droid.weapons[i].name);
 		if (ret !== MICRO.RANGED)
 			return ret;
@@ -100,28 +100,28 @@ _global.guessDroidMicro = function(droid) {
 }
 
 _global.guessBodyArmor = function(name) {
-    var body = bodyStats.filterProperty("stat", name).last()
-    if (defined(body))
-        return body.armor;
-    else
-        niceDebug("Ruleset warning: Couldn't guess body class of", name);
+	var body = bodyStats.filterProperty("stat", name).last()
+	if (defined(body))
+		return body.armor;
+	else
+		niceDebug("Ruleset warning: Couldn't guess body class of", name);
 }
 
 function weaponPathIsAvailable(path, objectType, defrole) {
-	switch(objectType) {
+	switch (objectType) {
 		case 0:
-			return path.weapons.some(function(val) { return componentAvailable(val.stat); })
+			return path.weapons.some((val) => (componentAvailable(val.stat)))
 		case 1:
-			return path.templates.some(function(val) {
-				for (var i = 0; i < val.weapons.length; ++i)
+			return path.templates.some((val) => {
+				for (let i = 0; i < val.weapons.length; ++i)
 					if (!componentAvailable(val.weapons[i]))
 						return false;
 				return componentAvailable(val.body) && componentAvailable(val.prop);
 			});
 		case 2:
-			return path.defenses.some(function(val) { return val.defrole === defrole && isAvailable(val.stat); });
+			return path.defenses.some((val) => (val.defrole === defrole && isAvailable(val.stat)));
 		case 3:
-			return path.vtols.some(function(val) { return componentAvailable(val.stat); });
+			return path.vtols.some((val) => (componentAvailable(val.stat)));
 		default: // research
 			return true;
 	}
@@ -135,11 +135,11 @@ _global.getProductionPaths = function() {
 
 _global.chooseAvailableWeaponPathByRoleRatings = function(paths, rating, objectType, defrole) {
 	var minDist = Infinity, minPath;
-	paths.forEach(function(path) {
+	paths.forEach((path) => {
 		if (!weaponPathIsAvailable(path, objectType, defrole))
 			return;
 		var dist = 0;
-		for (var i = 0; i < ROLE.LENGTH; ++i) {
+		for (let i = 0; i < ROLE.LENGTH; ++i) {
 			var newDist = Math.abs(rating[i] - path.roles[i])
 			if (newDist > dist)
 				dist = newDist;
@@ -157,12 +157,12 @@ _global.chooseAvailableWeaponPathByRoleRatings = function(paths, rating, objectT
 //
 
 function statsToResList(path) {
-	return path.map(function (val) { return val.res; });
+	return path.map((val) => (val.res));
 }
 
 // todo make one StatsToResList and do filtering for path outside
 _global.bodyStatsToResList = function(armor) {
-    return statsToResList(filterBodyStatsByUsage(armor)).reverse();
+	return statsToResList(filterBodyStatsByUsage(armor)).reverse();
 }
 
 _global.propulsionStatsToResList = function(usage) {
@@ -173,7 +173,7 @@ _global.weaponStatsToResList = function(path, objType) {
 	if (!defined(path))
 		return [];
 	var ret = [];
-	switch(objType) {
+	switch (objType) {
 		case 0:
 			ret = statsToResList(path.weapons); break;
 		case 1:
@@ -200,21 +200,21 @@ _global.weaponStatsToResList = function(path, objType) {
 
 // TODO: move this to math. If we have same structure of data <list of objects> we can use it all around.
 _global.filterDataByFlag = function(data, attr_name, flag) {
-     return data.filter(function(obj) { return obj[attr_name] & flag; });
+		return data.filter((obj) => (obj[attr_name] & flag));
 }
 
 _global.filterBodyStatsByUsage = function(usage, armor) {
 	var data;
-    if (defined(armor))
-        data = filterDataByFlag(bodyStats, 'armor', armor)
-    else
-        data = bodyStats;
-    return filterDataByFlag(data, 'usage', usage).reverse();
+	if (defined(armor))
+		data = filterDataByFlag(bodyStats, 'armor', armor)
+	else
+		data = bodyStats;
+	return filterDataByFlag(data, 'usage', usage).reverse();
 }
 
 _global.getPropulsionStatsComponents = function(usage) {
-    var data = filterDataByFlag(propulsionStats, 'usage', usage)
-	return data.map(function(val) { return val.stat; }).reverse()
+	var data = filterDataByFlag(propulsionStats, 'usage', usage)
+	return data.map((val) => (val.stat)).reverse()
 }
 
 //
@@ -225,7 +225,7 @@ _global.weaponStatsToDefenses = function(stats, defrole) {
 	if (!defined(stats))
 		return [];
 	var ret = [];
-	for (var i = 0; i < stats.defenses.length; ++i)
+	for (let i = 0; i < stats.defenses.length; ++i)
 		if (!defined(defrole) || stats.defenses[i].defrole === defrole)
 			ret.push(stats.defenses[i].stat);
 	// reverse not needed here
