@@ -26,24 +26,24 @@ function safeGetObject(label) {
 
 function groupsBySize() {
 	var ret = [];
-	for (var i = 0; i < MAX_GROUPS; ++i)
+	for (let i = 0; i < MAX_GROUPS; ++i)
 		if (isEnemy(i))
 			ret.push(i);
-	ret.sort(function(one, two) { return groupSize(two) - groupSize(one); });
+	ret.sort((one, two) => (groupSize(two) - groupSize(one)));
 	return ret;
 }
 
 function findLargestGroupIn(list) {
 	var sizes = []
-	for (var i = 0; i < MAX_GROUPS; ++i)
+	for (let i = 0; i < MAX_GROUPS; ++i)
 		sizes[i] = 0;
-	list.forEach(function(object) {
+	list.forEach((object) => {
 		if (!defined(object.group) || object.group < 0 && object.group >= MAX_GROUPS)
 			return;
 		++sizes[object.group];
 	});
 	var maxCount = 0, maxIdx = 0;
-	for (var i = 0; i < sizes.length; ++i)
+	for (let i = 0; i < sizes.length; ++i)
 		if (sizes[i] > maxCount) {
 			maxCount = sizes[i];
 			maxIdx = i;
@@ -52,14 +52,14 @@ function findLargestGroupIn(list) {
 }
 
 function findNearestGroup(x, y) {
-	var ret = naiveFindClusters(enumDroid(me).filter(function(droid) {
-		return !isVTOL(droid) && (droid.droidType === DROID_WEAPON || droid.droidType === DROID_CYBORG);
-	}), baseScale / 3);
+	var ret = naiveFindClusters(enumDroid(me).filter((droid) => (
+		!isVTOL(droid) && (droid.droidType === DROID_WEAPON || droid.droidType === DROID_CYBORG)
+	)), baseScale / 3);
 	if (ret.maxCount === 0)
 		return undefined;
 	var minDist = Infinity, minIdx;
 	var gr = [];
-	for (var i = 0; i < ret.clusters.length; ++i) {
+	for (let i = 0; i < ret.clusters.length; ++i) {
 		gr[i] = findLargestGroupIn(ret.clusters[i]);
 		if (groupSize(gr[i]) > attackGroupSize()) {
 			var dist = distance(ret.xav[i], ret.yav[i], x, y);
@@ -79,9 +79,9 @@ function findNearestGroup(x, y) {
 
 function targetSuitableForHarass(object) {
 	function uncached() {
-		var ret = enumRange(object.x, object.y, baseScale / 2, ENEMIES, false).filter(function(obj) {
-			return !(obj.type == STRUCTURE && obj.stattype != DEFENSE);
-		}).length;
+		var ret = enumRange(object.x, object.y, baseScale / 2, ENEMIES, false).filter((obj) => (
+			obj.type !== STRUCTURE || obj.stattype === DEFENSE
+		)).length;
 		return ret <= groupSize(miscGroup);
 	}
 	return cached(uncached, 60000, object.id);
@@ -127,17 +127,13 @@ function findTarget(gr) {
 	// find harass targets for the misc group
 	if (gr === miscGroup) {
 		var list = enumStructList(miscTargets, enumLivingPlayers().filter(isEnemy).random());
-		powerUps.forEach(function(stat) { // pick up oil drums and artifacts
+		powerUps.forEach((stat) => { // pick up oil drums and artifacts
 			list = list.concat(enumFeature(ALL_PLAYERS, stat));
 		});
-		list = list.filter(targetSuitableForHarass).filter(function(feature) {
-			if (iHaveHover())
-				if (canReachFromBase(getPropulsionStatsComponents(PROPULSIONUSAGE.HOVER)[0], feature))
-					return true;
-			return canReachFromBase(getPropulsionStatsComponents(PROPULSIONUSAGE.GROUND)[0], feature);
-		}).sort(function(one, two) {
-			return distanceToBase(one) - distanceToBase(two);
-		});
+		list = list.filter(targetSuitableForHarass).filter((feature) => (
+			(iHaveHover() && canReachFromBase(getPropulsionStatsComponents(PROPULSIONUSAGE.HOVER)[0], feature)) ||
+			canReachFromBase(getPropulsionStatsComponents(PROPULSIONUSAGE.GROUND)[0], feature)
+		)).sort((one, two) => (distanceToBase(one) - distanceToBase(two)));
 		obj = list[random(Math.min(3, list.length))];
 		if (obj) {
 			addLabel(obj, groupTargetLabel(gr));
@@ -176,12 +172,12 @@ function regroup(gr) {
 	var ret = naiveFindClusters(enumGroup(gr).filter(checkRepaired), (baseScale / 3));
 	if (ret.maxCount === 0)
 		return [];
-	for (var i = 0; i < ret.clusters.length; ++i)
+	for (let i = 0; i < ret.clusters.length; ++i)
 		if (i !== ret.maxIdx)
-			for (var j = 0; j < ret.clusters[i].length; ++j)
+			for (let j = 0; j < ret.clusters[i].length; ++j)
 				orderDroidLoc(ret.clusters[i][j], DORDER_MOVE, ret.xav[ret.maxIdx], ret.yav[ret.maxIdx]);
 	if (ret.maxCount < size) {
-		for (var j = 0; j < ret.clusters[ret.maxIdx].length; ++j) {
+		for (let j = 0; j < ret.clusters[ret.maxIdx].length; ++j) {
 			if (groupInDanger(gr))
 				orderDroid(ret.clusters[ret.maxIdx][j], DORDER_RTB);
 			else
@@ -225,7 +221,7 @@ function attackTarget(droid) {
 					orderDroidObj(droid, DORDER_OBSERVE, target);
 				else if (droid.canHitGround === true && !isVTOL(target))
 					orderDroidObj(droid, DORDER_ATTACK, target);
-				else if(droid.canHitAir === true && isVTOL(target))
+				else if (droid.canHitAir === true && isVTOL(target))
 					orderDroidObj(droid, DORDER_ATTACK, target);
 				else
 					orderDroidLoc(droid, DORDER_SCOUT, target.x, target.y);
@@ -269,11 +265,11 @@ function pickVtolTarget(droid) {
 }
 
 function vtolReady(droid) {
-	if (droid.order == DORDER_ATTACK)
+	if (droid.order === DORDER_ATTACK)
 		return false;
 	if (vtolArmed(droid, 99))
 		return true;
-	if (droid.order != DORDER_REARM) {
+	if (droid.order !== DORDER_REARM) {
 		orderDroid(droid, DORDER_REARM);
 		buildVtols() // actually pads
 	}
@@ -288,11 +284,11 @@ _global.vtolCanHit = function(droid, obj) {
 }
 
 _global.vtolArmed = function(obj, percent) {
-	if (obj.type != DROID)
+	if (obj.type !== DROID)
 		return;
 	if (!isVTOL(obj))
 		return false;
-	for (var i = 0; i < obj.weapons.length; ++i)
+	for (let i = 0; i < obj.weapons.length; ++i)
 		if (obj.weapons[i].armed >= percent)
 			return true;
 	return false;
@@ -324,7 +320,7 @@ _global.setTarget = function(object, group) {
 }
 
 _global.unsetTarget = function(player) {
-	for (var i = 0; i <= MAX_GROUPS; ++i) {
+	for (let i = 0; i <= MAX_GROUPS; ++i) {
 		var obj = safeGetObject(groupTargetLabel(i));
 		if (defined(obj) && obj.type === POSITION && findBeaconPlayer(obj.x, obj.y) === player)
 			removeLabel(groupTargetLabel(i));
@@ -342,18 +338,12 @@ _global.groupDroid = function(droid) {
 			groupAdd(miscGroup, droid);
 			return;
 		}
-		var grp = groupsBySize().filter(function(i) {
-			if (isAlly(i))
-				return false;
-			if (!defined(findTarget(i)))
-				return false;
-			if (!droidCanReach(droid, findTarget(i).x, findTarget(i).y))
-				return false;
-			return true;
-		});
-		var ret = grp.filter(function(i) {
-			return groupSize(i) < attackGroupSize() * 2 && defined(findTarget(i));
-		});
+		var grp = groupsBySize().filter((i) => (
+			!isAlly(i) && defined(findTarget(i)) && droidCanReach(droid, findTarget(i).x, findTarget(i).y)
+		));
+		var ret = grp.filter((i) => (
+			groupSize(i) < attackGroupSize() * 2 && defined(findTarget(i))
+		));
 		if (ret.length === 0)
 			ret = grp;
 		if (ret.length === 0)
@@ -369,15 +359,15 @@ _global.rebalanceGroups = function() {
 		return;
 	if (groupSize(miscGroup) > personality.maxMiscTanks) {
 		var list = enumGroup(miscGroup).shuffle();
-		for (var i = personality.maxMiscTanks; i < personality.maxMiscTanks + 5 && i < list.length; ++i)
+		for (let i = personality.maxMiscTanks; i < personality.maxMiscTanks + 5 && i < list.length; ++i)
 			groupDroid(list[i]);
 	}
 	var ret = groupsBySize();
 	if (ret.length > 0)
 		if (ret[0] > 0 && ret[0] < attackGroupSize())
-			for (var i = 1; i < ret.length; ++i) {
+			for (let i = 1; i < ret.length; ++i) {
 				var list = enumGroup(ret[i]);
-				for (var j = 0; j < list.length; ++j) {
+				for (let j = 0; j < list.length; ++j) {
 					var target = findTarget(ret[0]);
 					if (defined(target))
 						if (droidCanReach(list[j], target.x, target.y)) {
@@ -412,7 +402,7 @@ _global.dangerLevel = function(loc) {
 _global.checkAttack = function() {
 	if (enumLivingPlayers().filter(isEnemy).length === 0)
 		return;
-	for (var i = 0; i < MAX_GROUPS; ++i)
+	for (let i = 0; i < MAX_GROUPS; ++i)
 		if (!throttled(3000, i)) {
 			regroup(i).forEach(attackTarget);
 			break;
@@ -424,7 +414,7 @@ _global.checkAttack = function() {
 		return;
 	var droids = enumGroup(vtolGroup).filter(vtolReady);
 	if (droids.length > attackGroupSize() / 3.)
-		droids.forEach(function(droid) {
+		droids.forEach((droid) => {
 			var target = pickVtolTarget(droid);
 			if (defined(target))
 				orderDroidObj(droid, DORDER_ATTACK, target);
@@ -434,9 +424,9 @@ _global.checkAttack = function() {
 _global.pushVtols = function(object) {
 	var vtols = enumRange(object.x, object.y, 20, me, false);
 	var enemies = enumRange(object.x, object.y, 8, ENEMIES, true);
-	for (var i = 0; i < vtols.length; ++i)
+	for (let i = 0; i < vtols.length; ++i)
 		if (vtolArmed(vtols[i], 1))
-			for (var j = 0; j < enemies.length; ++j)
+			for (let j = 0; j < enemies.length; ++j)
 				if (vtolCanHit(vtols[i], enemies[j])) {
 					orderDroidObj(vtols[i], DORDER_ATTACK, enemies[j]);
 					break;
@@ -446,9 +436,9 @@ _global.pushVtols = function(object) {
 _global.inPanic = function() {
 	function uncached() {
 		var badGuys = enumRange(baseLocation.x, baseLocation.y, baseScale, ENEMIES).length;
-		var goodGuys = enumRange(baseLocation.x, baseLocation.y, baseScale, ALLIES).filter(function(object) {
-			return object.type === DROID && (object.droidType === DROID_WEAPON || object.droidType === DROID_CYBORG);
-		}).length;
+		var goodGuys = enumRange(baseLocation.x, baseLocation.y, baseScale, ALLIES).filter((object) => (
+			object.type === DROID && (object.droidType === DROID_WEAPON || object.droidType === DROID_CYBORG)
+		)).length;
 		return 3 * badGuys > 2 * goodGuys;
 	}
 	return cached(uncached, 10000);
