@@ -71,8 +71,10 @@ struct screeninfo
 
 void wzMain(int &argc, char **argv);
 bool wzMainScreenSetup(optional<video_backend> backend, int antialiasing = 0, WINDOW_MODE fullscreen = WINDOW_MODE::windowed, int vsync = 1, int lodDistanceBiasPercentage = 0, uint32_t depthMapResolution = 0, bool highDPI = true);
+optional<video_backend> wzGetInitializedGfxBackend();
 video_backend wzGetDefaultGfxBackendForCurrentSystem();
-bool wzPromptToChangeGfxBackendOnFailure(std::string additionalErrorDetails = "");
+bool wzPromptToChangeGfxBackendOnFailure(const std::string& additionalErrorDetails = "");
+void wzDisplayFatalGfxBackendFailure(const std::string& additionalErrorDetails = "");
 void wzResetGfxSettingsOnFailure();
 void wzGetGameToRendererScaleFactor(float *horizScaleFactor, float *vertScaleFactor);
 void wzGetGameToRendererScaleFactorInt(unsigned int *horizScalePercentage, unsigned int *vertScalePercentage);
@@ -89,8 +91,11 @@ bool wzSetToggleFullscreenMode(WINDOW_MODE fullscreenMode);
 WINDOW_MODE wzGetToggleFullscreenMode();
 bool wzChangeWindowMode(WINDOW_MODE mode, bool silent = false);
 WINDOW_MODE wzGetCurrentWindowMode();
+bool wzIsMaximized();
 bool wzIsFullscreen();
+bool wzWindowHasFocus();
 void wzSetWindowIsResizable(bool resizable);
+void wzPostChangedSwapInterval();
 bool wzIsWindowResizable();
 bool wzChangeDisplayScale(unsigned int displayScale);
 bool wzChangeCursorScale(unsigned int cursorScale);
@@ -150,6 +155,7 @@ bool wzBackendAttemptOpenURL(const char *url);
 
 // System information related
 uint64_t wzGetCurrentSystemRAM(); // gets the system RAM in MiB
+uint32_t wzGetLogicalCPUCount();
 
 // Thread related
 WZ_THREAD *wzThreadCreate(int (*threadFunc)(void *), void *data, const char* name = nullptr);
