@@ -117,7 +117,7 @@ WzConfig::WzConfig(const WzString &name, WzConfig::warning warning)
 	ASSERT(!mRoot.is_null(), "JSON document from %s is null", name.toUtf8().c_str());
 	ASSERT(mRoot.is_object(), "JSON document from %s is not an object. Read: \n%s", name.toUtf8().c_str(), data);
 	free(data);
-	WZ_PHYSFS_enumerateFiles("diffs", [&](const char *i) -> bool {
+	WZ_PHYSFS_enumerateFolders("diffs", [&](const char *i) -> bool {
 		std::string str(std::string("diffs/") + i + std::string("/") + name.toUtf8().c_str());
 		if (!PHYSFS_exists(str.c_str()))
 		{
@@ -163,6 +163,8 @@ std::string WzConfig::compactStringRepresentation(const bool ensure_ascii) const
 
 std::vector<WzString> WzConfig::childGroups() const
 {
+	ASSERT_OR_RETURN({}, pCurrentObj != nullptr, "current object is nullptr");
+	ASSERT_OR_RETURN({}, pCurrentObj->is_object(), "current pointer is not an object");
 	std::vector<WzString> keys;
 	for (auto it = pCurrentObj->begin(); it != pCurrentObj->end(); ++it)
 	{
@@ -176,6 +178,8 @@ std::vector<WzString> WzConfig::childGroups() const
 
 std::vector<WzString> WzConfig::childKeys() const
 {
+	ASSERT_OR_RETURN({}, pCurrentObj != nullptr, "current object is nullptr");
+	ASSERT_OR_RETURN({}, pCurrentObj->is_object(), "current pointer is not an object");
 	std::vector<WzString> keys;
 	for (auto it = pCurrentObj->begin(); it != pCurrentObj->end(); ++it)
 	{
