@@ -27,6 +27,7 @@
 #include <functional>
 #include <typeinfo>
 #include <typeindex>
+#include <array>
 
 #include "lib/framework/frame.h"
 #include "lib/framework/wzstring.h"
@@ -94,11 +95,18 @@ namespace gfx_api
 		virtual ~abstract_texture() {};
 	};
 
+	struct texture2dDimensions
+	{
+		size_t width = 0;
+		size_t height = 0;
+	};
+
 	struct texture : abstract_texture
 	{
 		virtual bool upload(const size_t& mip_level, const iV_BaseImage& image) = 0;
 		virtual bool upload_sub(const size_t& mip_level, const size_t& offset_x, const size_t& offset_y, const iV_Image& image) = 0;
 		virtual unsigned id() = 0;
+		virtual texture2dDimensions get_dimensions() const = 0;
 		bool isArray() const { return false; }
 
 		texture( const texture& other ) = delete; // non construction-copyable
@@ -319,17 +327,19 @@ namespace gfx_api
 		{}
 	};
 
-	struct shadow_constants
+	struct lighting_constants
 	{
 		uint32_t shadowMode = 1;
 		uint32_t shadowFilterSize = 5;
 		uint32_t shadowCascadesCount = WZ_MAX_SHADOW_CASCADES;
+		bool isPointLightPerPixelEnabled = false;
 
-		bool operator==(const shadow_constants& rhs) const
+		bool operator==(const lighting_constants& rhs) const
 		{
 			return shadowMode == rhs.shadowMode
 			&& shadowFilterSize == rhs.shadowFilterSize
-			&& shadowCascadesCount == rhs.shadowCascadesCount;
+			&& shadowCascadesCount == rhs.shadowCascadesCount
+			&& isPointLightPerPixelEnabled == rhs.isPointLightPerPixelEnabled;
 		}
 	};
 
@@ -412,22 +422,26 @@ namespace gfx_api
 		virtual bool getScreenshot(std::function<void (std::unique_ptr<iV_Image>)> callback) = 0;
 		virtual void handleWindowSizeChange(unsigned int oldWidth, unsigned int oldHeight, unsigned int newWidth, unsigned int newHeight) = 0;
 		virtual std::pair<uint32_t, uint32_t> getDrawableDimensions() = 0;
+		virtual bool isYAxisInverted() const = 0;
 		virtual bool shouldDraw() = 0;
 		virtual void shutdown() = 0;
 		virtual const size_t& current_FrameNum() const = 0;
-		virtual bool setSwapInterval(swap_interval_mode mode) = 0;
+		typedef std::function<void()> SetSwapIntervalCompletionHandler;
+		virtual bool setSwapInterval(swap_interval_mode mode, const SetSwapIntervalCompletionHandler& completionHandler) = 0;
 		virtual swap_interval_mode getSwapInterval() const = 0;
 		virtual bool textureFormatIsSupported(pixel_format_target target, pixel_format format, pixel_format_usage::flags usage) = 0;
 		virtual bool supportsMipLodBias() const = 0;
 		virtual bool supports2DTextureArrays() const = 0;
 		virtual bool supportsIntVertexAttributes() const = 0;
 		virtual size_t maxFramesInFlight() const = 0;
-		virtual shadow_constants getShadowConstants() = 0;
-		virtual bool setShadowConstants(shadow_constants values) = 0;
+		virtual lighting_constants getShadowConstants() = 0;
+		virtual bool setShadowConstants(lighting_constants values) = 0;
 		// instanced rendering APIs
 		virtual bool supportsInstancedRendering() = 0;
 		virtual void draw_instanced(const std::size_t& offset, const std::size_t &count, const primitive_type &primitive, std::size_t instance_count) = 0;
 		virtual void draw_elements_instanced(const std::size_t& offset, const std::size_t& count, const primitive_type& primitive, const index_type& index, std::size_t instance_count) = 0;
+		// debug apis for recompiling pipelines
+		virtual bool debugRecompileAllPipelines() = 0;
 	public:
 		// High-level API for getting a texture object from file / uncompressed bitmap
 		gfx_api::texture* loadTextureFromFile(const char *filename, gfx_api::texture_type textureType, int maxWidth = -1, int maxHeight = -1, bool quiet = false);
@@ -728,6 +742,10 @@ namespace gfx_api
 	using Draw3DShapeAlphaNoDepthWRT = Draw3DShape<REND_ALPHA, SHADER_COMPONENT, DEPTH_CMP_LEQ_WRT_OFF>;
 	using Draw3DShapeNoLightAlphaNoDepthWRT = Draw3DShape<REND_ALPHA, SHADER_NOLIGHT, DEPTH_CMP_LEQ_WRT_OFF>;
 
+	constexpr size_t max_lights = 128;
+	constexpr size_t max_indexed_lights = 512;
+	constexpr size_t bucket_dimension = 8;
+
 	// Only change once per frame
 	struct Draw3DShapeInstancedGlobalUniforms
 	{
@@ -747,6 +765,14 @@ namespace gfx_api
 		float fogBegin;
 		float timeState; // graphicsCycle
 		int fogEnabled;
+		int viewportWidth;
+		int viewportheight;
+		float unused2;
+		std::array<glm::vec4, max_lights> PointLightsPosition;
+		std::array<glm::vec4, max_lights> PointLightsColorAndEnergy;
+		std::array<glm::ivec4, bucket_dimension * bucket_dimension> bucketOffsetAndSize;
+		std::array<glm::ivec4, max_indexed_lights> indexed_lights;
+		int bucketDimensionUsed;
 	};
 
 	// Only change per mesh
@@ -756,6 +782,7 @@ namespace gfx_api
 		int normalMap;
 		int specularMap;
 		int hasTangents;
+		int shieldEffect;
 	};
 
 	// interleaved vertex data
@@ -818,6 +845,8 @@ namespace gfx_api
 	using Draw3DShapeNoLightAlphaNoDepthWRT_Instanced = Draw3DShapeInstanced<REND_ALPHA, SHADER_NOLIGHT_INSTANCED, DEPTH_CMP_LEQ_WRT_OFF>;
 	using Draw3DShapeAdditiveNoDepthWRT_Instanced = Draw3DShapeInstanced<REND_ADDITIVE, SHADER_COMPONENT_INSTANCED, DEPTH_CMP_LEQ_WRT_OFF>;
 	using Draw3DShapeNoLightAdditiveNoDepthWRT_Instanced = Draw3DShapeInstanced<REND_ADDITIVE, SHADER_NOLIGHT_INSTANCED, DEPTH_CMP_LEQ_WRT_OFF>;
+	using Draw3DShapePremulNoDepthWRT_Instanced = Draw3DShapeInstanced<REND_PREMULTIPLIED, SHADER_COMPONENT_INSTANCED, DEPTH_CMP_LEQ_WRT_OFF>;
+	using Draw3DShapeNoLightPremulNoDepthWRT_Instanced = Draw3DShapeInstanced<REND_PREMULTIPLIED, SHADER_NOLIGHT_INSTANCED, DEPTH_CMP_LEQ_WRT_OFF>;
 
 	struct Draw3DShapeInstancedDepthOnlyGlobalUniforms
 	{
@@ -978,6 +1007,8 @@ namespace gfx_api
 	};
 
 	static_assert(WZ_MAX_SHADOW_CASCADES <= 4, "Packing the ShadowMapCascadeSplits into a vec4 won't work...");
+
+
 	struct TerrainCombinedUniforms
 	{
 		glm::mat4 ModelViewProjectionMatrix;
@@ -998,6 +1029,14 @@ namespace gfx_api
 		float fog_begin;
 		float fog_end;
 		int quality;
+		int viewportWidth;
+		int viewportheight;
+		float  unused2;
+		std::array<glm::vec4, max_lights> PointLightsPosition;
+		std::array<glm::vec4, max_lights> PointLightsColorAndEnergy;
+		std::array<glm::ivec4, bucket_dimension * bucket_dimension> bucketOffsetAndSize;
+		std::array<glm::ivec4, max_indexed_lights> indexed_lights;
+		int bucketDimensionUsed;
 	};
 
 	template<REND_MODE render_mode, SHADER_MODE shader>
@@ -1064,9 +1103,9 @@ namespace gfx_api
 	struct constant_buffer_type<SHADER_WATER_HIGH>
 	{
 		glm::mat4 ModelViewProjectionMatrix;
+		glm::mat4 ViewMatrix;
 		glm::mat4 ModelUVLightmapMatrix;
-		glm::mat4 ModelUV1Matrix;
-		glm::mat4 ModelUV2Matrix;
+		glm::mat4 ShadowMapMVPMatrix[WZ_MAX_SHADOW_CASCADES];
 		glm::vec4 cameraPos; // in modelSpace
 		glm::vec4 sunPos; // in modelSpace
 		glm::vec4 emissiveLight; // light colors/intensity
@@ -1074,6 +1113,8 @@ namespace gfx_api
 		glm::vec4 diffuseLight;
 		glm::vec4 specularLight;
 		glm::vec4 fog_colour;
+		glm::vec4 ShadowMapCascadeSplits; // Can't use float[4] (because of std140 layout alignment rules, which don't match C/C++ and waste a lot of space)
+		int ShadowMapSize;
 		int fog_enabled;
 		float fog_begin;
 		float fog_end;
@@ -1088,7 +1129,8 @@ namespace gfx_api
 		texture_description<0, sampler_type::anisotropic_repeat, pixel_format_target::texture_2d_array>, // textures
 		texture_description<1, sampler_type::anisotropic_repeat, pixel_format_target::texture_2d_array>, // normal maps
 		texture_description<2, sampler_type::anisotropic_repeat, pixel_format_target::texture_2d_array>, // specular maps
-		texture_description<3, sampler_type::bilinear> // lightmap
+		texture_description<3, sampler_type::bilinear>, // lightmap
+		texture_description<4, sampler_type::bilinear_border, pixel_format_target::depth_map, border_color::opaque_white>  // depth / shadow map
 	>, SHADER_WATER_HIGH>;
 
 	template<>
@@ -1129,7 +1171,7 @@ namespace gfx_api
 		glm::vec2 offset;
 		glm::vec2 size;
 		glm::vec4 color;
-		int texture;
+		int texture; // IGNORED
 	};
 
 	template<>
@@ -1180,7 +1222,7 @@ namespace gfx_api
 		glm::vec2 offset;
 		glm::vec2 size;
 		glm::vec4 color;
-		int texture;
+		int texture; // IGNORED
 	};
 
 	using DrawImageTextPSO = typename gfx_api::pipeline_state_helper<rasterizer_state<REND_TEXT, DEPTH_CMP_ALWAYS_WRT_OFF, 255, polygon_offset::disabled, stencil_mode::stencil_disabled, cull_mode::none>, primitive_type::triangle_strip, index_type::u16,
@@ -1217,7 +1259,7 @@ namespace gfx_api
 		glm::vec2 offset;
 		glm::vec2 size;
 		glm::vec4 color;
-		int texture;
+		int texture; // IGNORED
 	};
 
 	using DrawImagePSO = typename gfx_api::pipeline_state_helper<rasterizer_state<REND_ALPHA, DEPTH_CMP_ALWAYS_WRT_OFF, 255, polygon_offset::disabled, stencil_mode::stencil_disabled, cull_mode::back>, primitive_type::triangle_strip, index_type::u16,
