@@ -145,38 +145,6 @@ static inline void iV_Box(int x0, int y0, int x1, int y1, PIELIGHT first)
 void pie_BoxFill(int x0, int y0, int x1, int y1, PIELIGHT colour);
 void pie_BoxFillf(float x0, float y0, float x1, float y1, PIELIGHT colour);
 void pie_BoxFill_alpha(int x0, int y0, int x1, int y1, PIELIGHT colour);
-struct PIERECT_DrawRequest
-{
-	PIERECT_DrawRequest(int x0, int y0, int x1, int y1, PIELIGHT color)
-	: x0(x0)
-	, y0(y0)
-	, x1(x1)
-	, y1(y1)
-	, color(color)
-	{ }
-
-	int x0;
-	int y0;
-	int x1;
-	int y1;
-	PIELIGHT color;
-};
-struct PIERECT_DrawRequest_f
-{
-	PIERECT_DrawRequest_f(float x0, float y0, float x1, float y1, PIELIGHT color)
-	: x0(x0)
-	, y0(y0)
-	, x1(x1)
-	, y1(y1)
-	, color(color)
-	{ }
-
-	float x0;
-	float y0;
-	float x1;
-	float y1;
-	PIELIGHT color;
-};
 void pie_DrawMultiRect(std::vector<PIERECT_DrawRequest> rects);
 class BatchedMultiRectRenderer
 {
@@ -274,10 +242,11 @@ private:
 
 void iV_DrawImageAnisotropic(gfx_api::texture& TextureID, Vector2i Position, Vector2f offset, Vector2f size, float angle, PIELIGHT colour);
 void iV_DrawImageText(gfx_api::texture& TextureID, Vector2f Position, Vector2f offset, Vector2f size, float angle, PIELIGHT colour);
-void iV_DrawImageTextClipped(gfx_api::texture& TextureID, Vector2i textureSize, Vector2f Position, Vector2f offset, Vector2f size, float angle, PIELIGHT colour, WzRect clippingRect);
+void iV_DrawImageTextClipped(gfx_api::texture& TextureID, Vector2i textureSize, Vector2f Position, Vector2f offset, Vector2f size, float angle, PIELIGHT colour, const WzClippingRectF& clippingRect);
 void iV_DrawImage(IMAGEFILE *ImageFile, UWORD ID, int x, int y, const glm::mat4 &modelViewProjection = defaultProjectionMatrix(), BatchedImageDrawRequests* pBatchedRequests = nullptr, uint8_t alpha = 255);
 void iV_DrawImageTint(IMAGEFILE *ImageFile, UWORD ID, float x, float y, PIELIGHT color, optional<Vector2f> size = nullopt, const glm::mat4 &modelViewProjection = defaultProjectionMatrix(), BatchedImageDrawRequests* pBatchedRequests = nullptr);
 void iV_DrawImageFileAnisotropic(IMAGEFILE *ImageFile, UWORD ID, int x, int y, Vector2f size, const glm::mat4 &modelViewProjection = defaultProjectionMatrix(), uint8_t alpha = 255);
+void iV_DrawImageFileAnisotropicTint(IMAGEFILE *ImageFile, UWORD ID, int x, int y, Vector2f size, PIELIGHT color, const glm::mat4 &modelViewProjection = defaultProjectionMatrix());
 void iV_DrawImage2(const WzString &filename, float x, float y, float width = -0.0f, float height = -0.0f);
 void iV_DrawImage2(const AtlasImageDef *image, float x, float y, float width = -0.0f, float height = -0.0f);
 void iV_DrawImageTc(AtlasImage image, AtlasImage imageTc, int x, int y, PIELIGHT colour, const glm::mat4 &modelViewProjection = defaultProjectionMatrix());
@@ -296,6 +265,8 @@ static inline void iV_DrawImageTc(IMAGEFILE *imageFile, unsigned id, unsigned id
 void iV_TransBoxFill(float x0, float y0, float x1, float y1);
 void pie_UniTransBoxFill(float x0, float y0, float x1, float y1, PIELIGHT colour);
 
+bool assertValidImage(IMAGEFILE *imageFile, unsigned id);
+
 bool pie_InitRadar();
 bool pie_ShutdownRadar();
 void pie_DownLoadRadar(const iV_Image& bitmap);
