@@ -26,8 +26,12 @@
 
 int32_t pie_RotateProjectWithPerspective(const Vector3i *v3d, const glm::mat4 &perspectiveViewMatrix, Vector2i *v2d);
 const glm::mat4& pie_PerspectiveGet();
+const glm::mat4& pie_SkyboxPerspectiveGet();
+const glm::mat4& pie_UIPerspectiveGet();
+glm::mat4 pie_PerspectiveGetConstrained(float zNear, float zFar);
 void pie_SetGeometricOffset(int x, int y);
 void pie_Begin3DScene();
 void pie_BeginInterface();
+float pie_getPerspectiveZFar();
 
 #endif
