@@ -82,7 +82,7 @@ const Vector3f& getDefaultSunPosition();
 
 int pie_GetMaxAntialiasing();
 
-bool pie_LoadShaders(uint32_t shadowFilterSize);
+bool pie_LoadShaders(uint32_t shadowFilterSize, bool pointLightEnabled);
 void pie_FreeShaders();
 
 namespace pie_internal
