@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2020  Warzone 2100 Project
+	Copyright (C) 2005-2024  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -29,10 +29,16 @@
 #include "lib/framework/crc.h"
 #include "src/factionid.h"
 #include "nettypes.h"
+#include "wzfile.h"
+#include "netlog.h"
+#include "sync_debug.h"
+#include "port_mapping_manager.h"
+
 #include <physfs.h>
 #include <vector>
 #include <functional>
 #include <memory>
+
 // Lobby Connection errors
 
 enum LOBBY_ERROR_TYPES
@@ -46,7 +52,8 @@ enum LOBBY_ERROR_TYPES
 	ERROR_WRONGPASSWORD,
 	ERROR_HOSTDROPPED,
 	ERROR_WRONGDATA,
-	ERROR_UNKNOWNFILEISSUE
+	ERROR_UNKNOWNFILEISSUE,
+	ERROR_REDIRECT
 };
 
 enum CONNECTION_STATUS
@@ -131,13 +138,19 @@ enum MESSAGE_TYPES
 	GAME_DEBUG_REMOVE_FEATURE,      ///< Remove feature.
 	GAME_DEBUG_FINISH_RESEARCH,     ///< Research has been completed.
 	// End of debug messages.
+	GAME_SYNC_OPT_CHANGE,			///< Change synchronized options for a player (ex formation options)
 	GAME_MAX_TYPE,                  ///< Maximum+1 valid GAME_ type, *MUST* be last.
 
 	// The following messages are used for playing back replays.
-	REPLAY_ENDED					///< A special message for signifying the end of the replay
+	REPLAY_ENDED = 255				///< A special message for signifying the end of the replay
 	// End of replay messages.
 };
 
+enum SYNC_OPT_TYPES
+{
+	SYNC_OPT_FORMATION_SPEED_LIMITING = 1
+};
+
 #define SYNC_FLAG 0x10000000	//special flag used for logging. (Not sure what this is. Was added in trunk, NUM_GAME_PACKETS not in newnet.)
 
 #define WZ_SERVER_DISCONNECT 0
@@ -146,7 +159,7 @@ enum MESSAGE_TYPES
 
 // Constants
 // @NOTE / FIXME: We need a way to detect what should happen if the msg buffer exceeds this.
-#define MaxMsgSize		16384		// max size of a message in bytes.
+#define MaxMsgSize		32768		// max size of a message in bytes.
 #define	StringSize		64			// size of strings used.
 #define extra_string_size	157		// extra 199 char for future use
 #define map_string_size		40
@@ -156,14 +169,19 @@ enum MESSAGE_TYPES
 
 #define MAX_NET_TRANSFERRABLE_FILE_SIZE	0x8000000
 
+static_assert(MaxMsgSize <= UINT16_MAX, "NetMessage/NetMessageBuilder encodes message length as a uint16_t");
+
 struct SESSIONDESC  //Available game storage... JUST FOR REFERENCE!
 {
 	int32_t dwSize;
-	int32_t dwFlags;
+	uint8_t alliances;
+	uint8_t techLevel;
+	uint8_t powerLevel;
+	uint8_t basesLevel;
 	char host[40];	// host's ip address (can fit a full IPv4 and IPv6 address + terminating NUL)
 	int32_t dwMaxPlayers;
 	int32_t dwCurrentPlayers;
-	uint32_t dwUserFlags[4]; // {game.type, openSpectatorSlots, unused, unused)
+	uint32_t dwUserFlags[4]; // {game.type, openSpectatorSlots, blindMode, unused)
 };
 
 /**
@@ -203,6 +221,12 @@ struct GAMESTRUCT
 // ////////////////////////////////////////////////////////////////////////
 // Message information. ie. the packets sent between machines.
 
+// For NET_JOIN messages
+enum NET_JOIN_PLAYERTYPE : uint8_t {
+	NET_JOIN_PLAYER = 0,
+	NET_JOIN_SPECTATOR = 1,
+};
+
 #define NET_ALL_PLAYERS 255
 #define NET_HOST_ONLY 0
 // the following structure is going to be used to track if we sync or not
@@ -217,40 +241,6 @@ struct SYNC_COUNTER
 	uint16_t	rejected;
 };
 
-void physfs_file_safe_close(PHYSFS_file* f);
-
-struct WZFile
-{
-public:
-//	WZFile() : handle_(nullptr, physfs_file_safe_close), size(0), pos(0) { hash.setZero(); }
-	WZFile(PHYSFS_file *handle, const std::string &filename, Sha256 hash, uint32_t size = 0) : handle_(handle, physfs_file_safe_close), filename(filename), hash(hash), size(size), pos(0) {}
-
-	~WZFile();
-
-	// Prevent copies
-	WZFile(const WZFile&) = delete;
-	void operator=(const WZFile&) = delete;
-
-	// Allow move semantics
-	WZFile(WZFile&& other) = default;
-	WZFile& operator=(WZFile&& other) = default;
-
-public:
-	bool closeFile();
-	inline PHYSFS_file* handle() const
-	{
-		return handle_.get();
-	}
-
-private:
-	std::unique_ptr<PHYSFS_file, void(*)(PHYSFS_file*)> handle_;
-public:
-	std::string filename;
-	Sha256 hash;
-	uint32_t size = 0;
-	uint32_t pos = 0;  // Current position, the range [0; currPos[ has been sent or received already.
-};
-
 enum class AIDifficulty : int8_t
 {
 	SUPEREASY,
@@ -291,6 +281,7 @@ struct PLAYER
 	bool                autoGame;           ///< if we are running a autogame (AI controls us)
 	FactionID			faction;			///< which faction the player has
 	bool				isSpectator;		///< whether this slot is a spectator slot
+	bool				isAdmin;			///< whether this slot has admin privs
 
 	// used on host-ONLY (not transmitted to other clients):
 	std::shared_ptr<std::vector<WZFile>> wzFiles = std::make_shared<std::vector<WZFile>>();            ///< for each player, we keep track of map/mod download progress
@@ -318,6 +309,7 @@ struct PLAYER
 		IPtextAddress[0] = '\0';
 		faction = FACTION_NORMAL;
 		isSpectator = false;
+		isAdmin = false;
 	}
 };
 
@@ -332,16 +324,13 @@ struct NETPLAY
 	uint32_t	hostPlayer;		///< Index of host in player array
 	bool		bComms;			///< Actually do the comms?
 	bool		isHost;			///< True if we are hosting the game
-	bool		isUPNP;				// if we want the UPnP detection routines to run
-	bool		isUPNP_CONFIGURED;	// if UPnP was successful
-	bool		isUPNP_ERROR;		//If we had a error during detection/config process
+	bool		isPortMappingEnabled;				// if we want the automatic Port mapping setup routines to run
 	bool		isHostAlive;	/// if the host is still alive
 	char gamePassword[password_string_size];		//
 	bool GamePassworded;				// if we have a password or not.
 	bool ShowedMOTD;					// only want to show this once
 	bool HaveUpgrade;					// game updates available
-	char MOTDbuffer[255];				// buffer for MOTD
-	char *MOTD = nullptr;
+	std::string MOTD;
 
 	std::vector<std::unordered_map<std::string, std::string>> scriptSetPlayerDataStrings;
 	std::vector<std::shared_ptr<PlayerReference>> playerReferences;
@@ -360,6 +349,8 @@ extern char iptoconnect[PATH_MAX]; // holds IP/hostname from command line
 extern bool cliConnectToIpAsSpectator; // = false; (for cli option)
 extern bool netGameserverPortOverride; // = false; (for cli override)
 
+extern PortMappingAsyncRequestHandle ipv4MappingRequest;
+
 #define ASSERT_HOST_ONLY(failAction) \
 	if (!NetPlay.isHost) \
 	{ \
@@ -367,11 +358,12 @@ extern bool netGameserverPortOverride; // = false; (for cli override)
 		failAction; \
 	}
 
+enum class ConnectionProviderType : uint8_t;
 
 // ////////////////////////////////////////////////////////////////////////
 // functions available to you.
-int NETinit(bool bFirstCall);
-WZ_DECL_NONNULL(2) bool NETsend(NETQUEUE queue, NetMessage const *message);   ///< send to player, or broadcast if player == NET_ALL_PLAYERS.
+int NETinit(ConnectionProviderType pt);
+bool NETsend(NETQUEUE queue, NetMessage const& message);   ///< send to player, or broadcast if player == NET_ALL_PLAYERS.
 void NETsendProcessDelayedActions();
 WZ_DECL_NONNULL(1, 2) bool NETrecvNet(NETQUEUE *queue, uint8_t *type);        ///< recv a message from the net queues if possible.
 WZ_DECL_NONNULL(1, 2) bool NETrecvGame(NETQUEUE *queue, uint8_t *type);       ///< recv a message from the game queues which is sceduled to execute by time, if possible.
@@ -384,14 +376,19 @@ unsigned NETgetDownloadProgress(unsigned player);     ///< Returns 100 when done
 int NETclose();					// close current game
 int NETshutdown();					// leave the game in play.
 
+bool netplayShutDown();
+
 void NETaddRedirects();
 void NETremRedirects();
-void NETdiscoverUPnPDevices();
+/// Initializes the port mapping infrastructure and spawns a background thread,
+/// which will automatically add a port mapping rule and signal the main thread
+/// (NETrecvNet, in particular) when the operation is complete.
+void NETinitPortMapping();
 
 enum NetStatisticType {NetStatisticRawBytes, NetStatisticUncompressedBytes, NetStatisticPackets};
 size_t NETgetStatistic(NetStatisticType type, bool sent, bool isTotal = false);     // Return some statistic. Call regularly for good results.
 
-void NETplayerKicked(UDWORD index);			// Cleanup after player has been kicked
+void NETplayerKicked(UDWORD index, bool quiet = false);			// Cleanup after player has been kicked
 
 bool NETplayerHasConnection(uint32_t index);
 
@@ -453,17 +450,23 @@ SDWORD NETgetGameFlags(UDWORD flag);			// return one of the four flags(dword) ab
 uint32_t NETgetGameUserFlagsUnjoined(const GAMESTRUCT& game, unsigned int flag);	// return one of the four flags(dword) about the game.
 bool NETsetGameFlags(UDWORD flag, SDWORD value);	// set game flag(1-4) to value.
 bool NEThaltJoining();				// stop new players joining this game
-bool NETenumerateGames(const std::function<bool (const GAMESTRUCT& game)>& handleEnumerateGameFunc);
-bool NETfindGames(std::vector<GAMESTRUCT>& results, size_t startingIndex, size_t resultsLimit, bool onlyMatchingLocalVersion = false);
+
+class WzConnectionProvider;
+std::shared_ptr<WzConnectionProvider> NET_getLobbyConnectionProvider();
+bool NETenumerateGames(const std::shared_ptr<WzConnectionProvider>& connProvider, const std::function<bool (const GAMESTRUCT& game)>& handleEnumerateGameFunc, const std::function<void(std::string&& lobbyMOTD)>& lobbyMotdFunc = nullptr);
+bool NETfindGames(std::vector<GAMESTRUCT>& results, std::string& lobbyMOTD, size_t startingIndex, size_t resultsLimit, bool onlyMatchingLocalVersion = false);
 bool NETfindGame(uint32_t gameId, GAMESTRUCT& output);
-bool NETjoinGame(const char *host, uint32_t port, const char *playername, const EcKey& playerIdentity, bool asSpectator = false); // join game given with playername
+
+class IClientConnection;
+class IConnectionPollGroup;
+
+bool NETpromoteJoinAttemptToEstablishedConnectionToHost(uint32_t hostPlayer, uint8_t index, const char* playername, NETQUEUE joiningQUEUEInfo, IClientConnection** client_joining_socket, IConnectionPollGroup** client_joining_socket_set);
 bool NEThostGame(const char *SessionName, const char *PlayerName, bool spectatorHost, // host a game
-                 uint32_t gameType, uint32_t two, uint32_t three, uint32_t four, UDWORD plyrs);
+                 uint32_t gameType, uint32_t two, uint32_t three, uint32_t four, UDWORD plyrs,
+                 uint8_t alliancesType, uint8_t techLevel, uint8_t powerLevel, uint8_t basesLevel);
 bool NETchangePlayerName(UDWORD player, char *newName);// change a players name.
 void NETfixDuplicatePlayerNames();  // Change a player's name automatically, if there are duplicates.
 
-#include "netlog.h"
-
 void NETsetMasterserverName(const char *hostname);
 const char *NETgetMasterserverName();
 void NETsetMasterserverPort(unsigned int port);
@@ -474,11 +477,15 @@ void NETsetJoinPreferenceIPv6(bool bTryIPv6First);
 bool NETgetJoinPreferenceIPv6();
 void NETsetDefaultMPHostFreeChatPreference(bool enabled);
 bool NETgetDefaultMPHostFreeChatPreference();
+void NETsetEnableTCPNoDelay(bool enabled);
+bool NETgetEnableTCPNoDelay();
+uint32_t NETgetJoinConnectionNETPINGChallengeFromHostSize();
+uint32_t NETgetJoinConnectionNETPINGChallengeFromClientSize();
 
-bool NETsetupTCPIP(const char *machine);
 void NETsetGamePassword(const char *password);
 void NETBroadcastPlayerInfo(uint32_t index);
 void NETBroadcastTwoPlayerInfo(uint32_t index1, uint32_t index2);
+void NETSendAllPlayerInfoTo(unsigned to);
 bool NETisCorrectVersion(uint32_t game_version_major, uint32_t game_version_minor);
 uint32_t NETGetMajorVersion();
 uint32_t NETGetMinorVersion();
@@ -487,6 +494,7 @@ void NET_InitPlayers(bool initTeams = false, bool initSpectator = false);
 
 uint8_t NET_numHumanPlayers(void);
 void NETsetLobbyOptField(const char *Value, const NET_LOBBY_OPT_FIELD Field);
+void NETsetLobbyConfigFlagsFields(uint8_t alliancesType, uint8_t techLevel, uint8_t powerLevel, uint8_t basesLevel);
 std::vector<uint8_t> NET_getHumanPlayers(void);
 
 const std::vector<WZFile>& NET_getDownloadingWzFiles();
@@ -496,49 +504,32 @@ void NET_clearDownloadingWZFiles();
 bool NET_getLobbyDisabled();
 const std::string& NET_getLobbyDisabledInfoLinkURL();
 void NET_setLobbyDisabled(const std::string& infoLinkURL);
+uint32_t NET_getCurrentHostedLobbyGameId();
+
+// If a client, retrieve the current host's address
+optional<std::string> NET_getCurrentHostTextAddress();
 
 bool NETGameIsLocked();
 void NETGameLocked(bool flag);
 void NETresetGamePassword();
 bool NETregisterServer(int state);
-bool NETprocessQueuedServerUpdates();
 void NETsetPlayerConnectionStatus(CONNECTION_STATUS status, unsigned player);    ///< Cumulative, except that CONNECTIONSTATUS_NORMAL resets.
 bool NETcheckPlayerConnectionStatus(CONNECTION_STATUS status, unsigned player);  ///< True iff connection status icon hasn't expired for this player. CONNECTIONSTATUS_NORMAL means any status, NET_ALL_PLAYERS means all players.
 
 void NETsetAsyncJoinApprovalRequired(bool enabled);
+bool NETgetAsyncJoinApprovalRequired();
+
+enum class AsyncJoinApprovalAction
+{
+	Approve,
+	ApproveSpectators,
+	Reject
+};
 //	NOTE: *MUST* be called from the main thread!
-bool NETsetAsyncJoinApprovalResult(const std::string& uniqueJoinID, bool approve, LOBBY_ERROR_TYPES rejectedReason = ERROR_NOERROR);
+bool NETsetAsyncJoinApprovalResult(const std::string& uniqueJoinID, AsyncJoinApprovalAction action, optional<uint8_t> explicitPlayerIdx, LOBBY_ERROR_TYPES rejectedReason = ERROR_NOERROR, optional<std::string> customRejectionMessage = nullopt);
 
 const char *messageTypeToString(unsigned messageType);
 
-/// Sync debugging. Only prints anything, if different players would print different things.
-#define syncDebug(...) do { _syncDebug(__FUNCTION__, __VA_ARGS__); } while(0)
-#ifdef WZ_CC_MINGW
-void _syncDebug(const char *function, const char *str, ...) WZ_DECL_FORMAT(__MINGW_PRINTF_FORMAT, 2, 3);
-#else
-void _syncDebug(const char *function, const char *str, ...) WZ_DECL_FORMAT(printf, 2, 3);
-#endif
-
-/// Faster than syncDebug. Make sure that str is a format string that takes ints only.
-void _syncDebugIntList(const char *function, const char *str, int *ints, size_t numInts);
-#define syncDebugBacktrace() do { _syncDebugBacktrace(__FUNCTION__); } while(0)
-void _syncDebugBacktrace(const char *function);                  ///< Adds a backtrace to syncDebug, if the platform supports it. Can be a bit slow, don't call way too often, unless desperate.
-uint32_t syncDebugGetCrc();                                      ///< syncDebug() calls between uint32_t crc = syncDebugGetCrc(); and syncDebugSetCrc(crc); appear in synch debug logs, but without triggering a desynch if different.
-void syncDebugSetCrc(uint32_t crc);                              ///< syncDebug() calls between uint32_t crc = syncDebugGetCrc(); and syncDebugSetCrc(crc); appear in synch debug logs, but without triggering a desynch if different.
-
-typedef uint16_t GameCrcType;  // Truncate CRC of game state to 16 bits, to save a bit of bandwidth.
-void resetSyncDebug();                                              ///< Resets the syncDebug, so syncDebug from a previous game doesn't cause a spurious desynch dump.
-GameCrcType nextDebugSync();                                        ///< Returns a CRC corresponding to all syncDebug() calls since the last nextDebugSync() or resetSyncDebug() call.
-bool checkDebugSync(uint32_t checkGameTime, GameCrcType checkCrc);  ///< Dumps all syncDebug() calls from that gameTime, if the CRC doesn't match.
-
-
-// Set whether verbose debug mode - outputting the current player's sync log for every single game tick - is enabled until a specific gameTime value
-// WARNING: This may significantly impact performance *and* will fill up your drive with a lot of logs data!
-// It is only intended to be used for debugging issues such as: replays desyncing when gameplay does not, etc. (And don't let the game run too long / set untilGameTime too high!)
-void NET_setDebuggingModeVerboseOutputAllSyncLogs(uint32_t untilGameTime = 0);
-void debugVerboseLogSyncIfNeeded();
-
-
 /**
  * This structure provides read-only access to a player, and can be used to identify players uniquely.
  *
@@ -583,4 +574,14 @@ private:
 	uint32_t index;
 };
 
+void NETacceptIncomingConnections();
+/// <summary>
+/// Increase the connected timeout for all player connection objects when transitioning
+/// from the lobby room to the main game loop.
+///
+/// Currently, this will set timeout value to 60 seconds, so that automatic lag-kick mechanism
+/// would be able to close stalled connections.
+/// </summary>
+void NETadjustConnectedTimeoutForClients();
+
 #endif
