@@ -51,8 +51,8 @@ public:
 	void display(int xOffset, int yOffset) override;
 
 	void screenSizeDidChange(int oldWidth, int oldHeight, int newWidth, int newHeight) override;
-	bool hitTest(int x, int y) override;
-	bool processClickRecursive(W_CONTEXT *psContext, WIDGET_KEY key, bool wasPressed) override;
+	bool hitTest(int x, int y) const override;
+	std::shared_ptr<WIDGET> findMouseTargetRecursive(W_CONTEXT *psContext, WIDGET_KEY key, bool wasPressed) override;
 	void displayRecursive(WidgetGraphicsContext const &context) override;
 	using WIDGET::displayRecursive;
 
@@ -65,6 +65,10 @@ public:
 	bool            disableChildren;        ///< Disable all child widgets if true
 	bool			userMovable = false;	///< Whether the user can drag the form around (NOTE: should only be used with forms on overlay screens, currently)
 
+protected:
+	bool capturesMouseDrag(WIDGET_KEY) override;
+	void mouseDragged(WIDGET_KEY, W_CONTEXT *start, W_CONTEXT *current) override;
+
 private:
 	Vector2i calcMinimizedSize() const;
 	const WzRect& minimizedGeometry() const;
@@ -105,7 +109,7 @@ public:
 		return &(help.value());
 	}
 
-	unsigned getState() override;
+	unsigned getState() const override;
 	void setState(unsigned state) override;
 	void setFlash(bool enable) override;
 	void setTip(std::string string) override;
@@ -149,7 +153,7 @@ public:
 	PIELIGHT backgroundColor = pal_RGBA(0, 0, 0, 125);
 	std::function<void ()> onClickedFunc;
 	std::function<void ()> onCancelPressed;
-private:
+protected:
 	std::weak_ptr<WIDGET> cutoutWidget;
 };
 
