@@ -41,6 +41,8 @@
 # include <windows.h>
 #endif
 
+constexpr uint64_t WzMapZipDefaultEmbeddedFileMaxFileSize = 104857600; // 100 MiB
+
 class WrappedZipArchive
 {
 public:
@@ -49,7 +51,12 @@ public:
 	WrappedZipArchive(zip_t* pZip, PostCloseFunc postCloseFunc = nullptr)
 	: pZip(pZip)
 	, postCloseFunc(postCloseFunc)
-	{ }
+	{
+		if (pZip)
+		{
+			readOnly = zip_get_archive_flag(pZip, ZIP_AFL_RDONLY, 0) == 1;
+		}
+	}
 	~WrappedZipArchive()
 	{
 		close();
@@ -260,6 +267,142 @@ private:
 	bool m_fixedLastMod = false;
 };
 
+WzZipIOSourceReadProvider::WzZipIOSourceReadProvider()
+{
+	error_ = new zip_error_t();
+	zip_error_init(static_cast<zip_error_t*>(error_));
+}
+WzZipIOSourceReadProvider::~WzZipIOSourceReadProvider()
+{
+#if DEBUG
+	assert(retainCount == 0);
+#else
+	(void)retainCount;
+#endif
+	if (error_)
+	{
+		zip_error_fini(static_cast<zip_error_t*>(error_));
+	}
+	delete static_cast<zip_error_t*>(error_);
+	error_ = nullptr;
+}
+
+void* WzZipIOSourceReadProvider::error()
+{
+	return error_;
+}
+
+void WzZipIOSourceReadProvider::inform_source_keep()
+{
+	++retainCount;
+}
+
+void WzZipIOSourceReadProvider::inform_source_free()
+{
+	if (retainCount > 0)
+	{
+		--retainCount;
+	}
+}
+
+static inline zip_error_t* getZipIOCtxError(WzZipIOSourceReadProvider *ctx)
+{
+	return static_cast<zip_error_t*>(ctx->error());
+}
+
+zip_int64_t wzZipIOSourceProviderCallback(void *state, void *data, zip_uint64_t len, zip_source_cmd_t cmd)
+{
+	WzZipIOSourceReadProvider *ctx = (WzZipIOSourceReadProvider *)state;
+
+	switch (cmd)
+	{
+		case ZIP_SOURCE_OPEN:
+			ctx->seek(0);
+			return 0;
+
+		case ZIP_SOURCE_READ:
+			if (len > ZIP_INT64_MAX)
+			{
+				zip_error_set(getZipIOCtxError(ctx), ZIP_ER_INVAL, 0);
+				return -1;
+			}
+			return ctx->readBytes(data, len).value_or(-1);
+
+		case ZIP_SOURCE_CLOSE:
+			return 0;
+
+		case ZIP_SOURCE_STAT: {
+			zip_stat_t *st = ZIP_SOURCE_GET_ARGS(zip_stat_t, data, len, getZipIOCtxError(ctx));
+			if (st == NULL)
+			{
+				return -1;
+			}
+			zip_stat_init(st);
+			if (auto modTime = ctx->modTime())
+			{
+				st->mtime = modTime.value();
+				st->valid |= ZIP_STAT_MTIME;
+			}
+			if (auto fileSize = ctx->fileSize())
+			{
+				st->size = fileSize.value();
+				st->valid |= ZIP_STAT_SIZE;
+			}
+			return sizeof(struct zip_stat);
+		}
+
+		case ZIP_SOURCE_ERROR:
+			return zip_error_to_data(getZipIOCtxError(ctx), data, len);
+
+		case ZIP_SOURCE_FREE:
+			ctx->inform_source_free();
+			return 0;
+
+		case ZIP_SOURCE_TELL:
+		{
+			auto currentOffset = ctx->tell();
+			if (!currentOffset.has_value())
+			{
+				zip_error_set(getZipIOCtxError(ctx), ZIP_ER_TELL, ECANCELED);
+				return -1;
+			}
+			if (currentOffset.value() > ZIP_INT64_MAX) {
+				zip_error_set(getZipIOCtxError(ctx), ZIP_ER_TELL, EOVERFLOW);
+				return -1;
+			}
+			return (zip_int64_t)currentOffset.value();
+		}
+
+		case ZIP_SOURCE_SEEK:
+		{
+			auto currentOffset = ctx->tell();
+			if (!currentOffset.has_value())
+			{
+				zip_error_set(getZipIOCtxError(ctx), ZIP_ER_TELL, ECANCELED);
+				return -1;
+			}
+			zip_int64_t new_offset = zip_source_seek_compute_offset(static_cast<zip_uint64_t>(currentOffset.value()), static_cast<zip_uint64_t>(ctx->fileSize().value_or(0)), data, len, getZipIOCtxError(ctx));
+			if (new_offset < 0)
+			{
+				return -1;
+			}
+			if (!ctx->seek(static_cast<uint64_t>(new_offset)))
+			{
+				// seek failed
+				return -1;
+			}
+			return 0;
+		}
+
+		case ZIP_SOURCE_SUPPORTS:
+			return zip_source_make_command_bitmap(ZIP_SOURCE_OPEN, ZIP_SOURCE_READ, ZIP_SOURCE_CLOSE, ZIP_SOURCE_STAT, ZIP_SOURCE_ERROR, ZIP_SOURCE_FREE, ZIP_SOURCE_SEEK, ZIP_SOURCE_TELL /*, ZIP_SOURCE_SUPPORTS_REOPEN*/ /* Requires libzip >= 1.10 */, -1);
+
+		default:
+			zip_error_set(getZipIOCtxError(ctx), ZIP_ER_OPNOTSUPP, 0);
+			return -1;
+	}
+}
+
 static zip_int64_t wz_zip_name_locate_impl(zip_t *archive, const char *fname, zip_flags_t flags, bool useWindowsPathWorkaroundIfNeeded)
 {
 	auto zipLocateResult = zip_name_locate(archive, fname, flags);
@@ -328,10 +471,96 @@ std::shared_ptr<WzMapZipIO> WzMapZipIO::openZipArchiveFS(const char* fileSystemP
 	return result;
 }
 
-std::shared_ptr<WzMapZipIO> WzMapZipIO::createZipArchiveFS(const char* fileSystemPath, bool fixedLastMod /*= false*/)
+std::shared_ptr<WzMapZipIO> WzMapZipIO::openZipArchiveMemory(std::unique_ptr<std::vector<uint8_t>> zipFileContents, bool extraConsistencyChecks /*= false*/)
 {
-	if (fileSystemPath == nullptr) { return nullptr; }
-	if (*fileSystemPath == '\0') { return nullptr; }
+	if (zipFileContents == nullptr) { return nullptr; }
+	struct zip_error error;
+	zip_error_init(&error);
+
+	// Create new (empty) in-memory source buffer
+	zip_source_t *pMemSource = zip_source_buffer_create(zipFileContents->data(), zipFileContents->size(), 0, &error);
+	if (pMemSource == NULL)
+	{
+		// Failed to create source
+		zip_error_fini(&error);
+		return nullptr;
+	}
+	int flags = ZIP_RDONLY;
+	if (extraConsistencyChecks)
+	{
+		flags |= ZIP_CHECKCONS;
+	}
+	zip_t* pZip = zip_open_from_source(pMemSource, flags, &error);
+	if (pZip == NULL)
+	{
+		// Failed to open from source
+		zip_source_free(pMemSource);
+		zip_error_fini(&error);
+		return nullptr;
+	}
+	zip_error_fini(&error);
+	zip_source_keep(pMemSource); // explicitly keep the zip source buffer around after the in-memory zip file is "closed"
+
+	auto result = std::shared_ptr<WzMapZipIO>(new WzMapZipIO());
+	auto retainedZipFileContents = std::make_shared<std::unique_ptr<std::vector<uint8_t>>>(std::move(zipFileContents));
+	result->m_zipArchive = std::make_shared<WrappedZipArchive>(pZip, [pMemSource, retainedZipFileContents]() { // effectively, retain ownership of zipFileContents to ensure it sticks around
+		// This closure is run after the zip file is "closed"
+		zip_source_free(pMemSource);
+		// retainedZipFileContents will stick around until *after* this call to zip_source_free, which is required
+		retainedZipFileContents->reset();
+	});
+	return result;
+}
+
+std::shared_ptr<WzMapZipIO> WzMapZipIO::openZipArchiveReadIOProvider(std::shared_ptr<WzZipIOSourceReadProvider> zipSourceProvider, WzMap::LoggingProtocol* pCustomLogger /*= nullptr*/, bool extraConsistencyChecks /*= false*/)
+{
+	if (zipSourceProvider == nullptr) { return nullptr; }
+	struct zip_error error;
+	zip_error_init(&error);
+
+	zip_source_t *pProviderSource = zip_source_function_create(wzZipIOSourceProviderCallback, zipSourceProvider.get(), &error);
+	if (pProviderSource == NULL)
+	{
+		// Failed to create source
+		zip_error_fini(&error);
+		return nullptr;
+	}
+	int flags = ZIP_RDONLY;
+	if (extraConsistencyChecks)
+	{
+		flags |= ZIP_CHECKCONS;
+	}
+	zip_t* pZip = zip_open_from_source(pProviderSource, flags, &error);
+	if (pZip == NULL)
+	{
+		// Failed to open from source
+		if (pCustomLogger)
+		{
+			const char* pErrorStr = zip_error_strerror(&error);
+			pCustomLogger->printLog(WzMap::LoggingProtocol::LogLevel::Error, __FUNCTION__, __LINE__, (pErrorStr) ? pErrorStr : "<n/a>");
+			pErrorStr = nullptr;
+		}
+		zip_source_free(pProviderSource);
+		zip_error_fini(&error);
+		return nullptr;
+	}
+	zip_error_fini(&error);
+	zip_source_keep(pProviderSource); // explicitly keep the zip source buffer around after the in-memory zip file is "closed"
+	zipSourceProvider->inform_source_keep();
+
+	auto result = std::shared_ptr<WzMapZipIO>(new WzMapZipIO());
+	result->m_zipArchive = std::make_shared<WrappedZipArchive>(pZip, [pProviderSource, zipSourceProvider]() mutable { // keep zipSourceProvider around until actual close
+		// This closure is run after the zip file is "closed"
+		zip_source_free(pProviderSource);
+		// zipSourceProvider must stick around until *after* this call to zip_source_free, which is required
+		zipSourceProvider.reset();
+	});
+	return result;
+}
+
+std::shared_ptr<WzMapZipIO> WzMapZipIO::createZipArchiveMemory(CreatedMemoryZipOnCloseFunc onCloseFunc, bool fixedLastMod /*= false*/)
+{
+	if (!onCloseFunc) { return nullptr; }
 
 	struct zip_error error;
 	zip_error_init(&error);
@@ -359,14 +588,14 @@ std::shared_ptr<WzMapZipIO> WzMapZipIO::createZipArchiveFS(const char* fileSyste
 	zip_source_keep(pMemSource); // explicitly keep the buffer around after the in-memory zip file is "closed"
 
 	auto result = std::shared_ptr<WzMapZipIO>(new WzMapZipIO());
-	std::string writeNewZipOutputPath = fileSystemPath;
-	result->m_zipArchive = std::make_shared<WrappedZipArchive>(pZip, [pMemSource, writeNewZipOutputPath]() {
+	result->m_zipArchive = std::make_shared<WrappedZipArchive>(pZip, [pMemSource, onCloseFunc]() {
 		// This closure is run after the zip file is "closed"
 
 		if (zip_source_is_deleted(pMemSource))
 		{
 			// the zip is empty, so do nothing
 			zip_source_free(pMemSource);
+			onCloseFunc(nullptr);
 			return;
 		}
 
@@ -374,43 +603,95 @@ std::shared_ptr<WzMapZipIO> WzMapZipIO::createZipArchiveFS(const char* fileSyste
 		if (zip_source_stat(pMemSource, &zst) < 0)
 		{
 			zip_source_free(pMemSource);
+			onCloseFunc(nullptr);
 			return;
 		}
 
 		if (zip_source_open(pMemSource) < 0)
 		{
 			zip_source_free(pMemSource);
+			onCloseFunc(nullptr);
 			return;
 		}
 
-		std::vector<char> zipDataBuffer(zst.size, 0);
+		auto zipDataBuffer = std::make_unique<std::vector<uint8_t>>(zst.size, 0);
 
-		auto readResult = zip_source_read(pMemSource, zipDataBuffer.data(), zst.size);
+		auto readResult = zip_source_read(pMemSource, zipDataBuffer->data(), zst.size);
 		if (readResult < 0 || static_cast<zip_uint64_t>(readResult) < zst.size)
 		{
 			zip_source_close(pMemSource);
 			zip_source_free(pMemSource);
+			onCloseFunc(nullptr);
 			return;
 		}
 
 		zip_source_close(pMemSource);
 		zip_source_free(pMemSource);
 
-		//  Write out the zipDataBuffer to a file at the writeNewZipOutputPath
-		WzMap::StdIOProvider stdIOProvider;
-		if (!stdIOProvider.writeFullFile(writeNewZipOutputPath, zipDataBuffer.data(), static_cast<uint32_t>(zipDataBuffer.size())))
-		{
-			// Failed to write out zip buffer data
-			return;
-		}
+		onCloseFunc(std::move(zipDataBuffer));
 	});
 	result->m_fixedLastMod = fixedLastMod;
 	return result;
 }
 
+std::shared_ptr<WzMapZipIO> WzMapZipIO::createZipArchiveFS(const char* fileSystemPath, bool fixedLastMod /*= false*/)
+{
+	if (fileSystemPath == nullptr) { return nullptr; }
+	if (*fileSystemPath == '\0') { return nullptr; }
+
+	std::string writeNewZipOutputPath = fileSystemPath;
+	auto result = createZipArchiveMemory([writeNewZipOutputPath](std::unique_ptr<std::vector<uint8_t>> zipDataBuffer) {
+		if (!zipDataBuffer)
+		{
+			return;
+		}
+
+		//  Write out the zipDataBuffer to a file at the writeNewZipOutputPath
+		WzMap::StdIOProvider stdIOProvider;
+		if (!stdIOProvider.writeFullFile(writeNewZipOutputPath, (const char*)zipDataBuffer->data(), static_cast<uint32_t>(zipDataBuffer->size())))
+		{
+			// Failed to write out zip buffer data
+			return;
+		}
+	}, fixedLastMod);
+	return result;
+}
+
 WzMapZipIO::~WzMapZipIO()
 { }
 
+enum class ZipSanityCheckResult
+{
+	PASSED,
+	FAILURE_EXCEEDS_MAXFILESIZE,
+	FAILURE_UNSUPPORTED_COMP_METHOD
+};
+
+static ZipSanityCheckResult wzMapZipIOSanityCheckStat(const struct zip_stat& st, uint64_t fileSizeLimit = WzMapZipDefaultEmbeddedFileMaxFileSize)
+{
+	if (st.valid & ZIP_STAT_SIZE)
+	{
+		if (fileSizeLimit < st.size)
+		{
+			// size is too big!
+			return ZipSanityCheckResult::FAILURE_EXCEEDS_MAXFILESIZE;
+		}
+	}
+
+	bool compressedFile = (st.valid & ZIP_STAT_COMP_METHOD) && (st.comp_method != ZIP_CM_STORE);
+	if (compressedFile)
+	{
+		// Check for permitted compression methods
+		// (This is a subset of all methods that latest libzip itself may support, but we want to ensure consistent support across all WZ target platforms)
+		if (st.comp_method != ZIP_CM_DEFLATE)
+		{
+			return ZipSanityCheckResult::FAILURE_UNSUPPORTED_COMP_METHOD;
+		}
+	}
+
+	return ZipSanityCheckResult::PASSED;
+}
+
 std::unique_ptr<WzMap::BinaryIOStream> WzMapZipIO::openBinaryStream(const std::string& filename, WzMap::BinaryIOStream::OpenMode mode)
 {
 	std::unique_ptr<WzMap::BinaryIOStream> pStream;
@@ -425,6 +706,17 @@ std::unique_ptr<WzMap::BinaryIOStream> WzMapZipIO::openBinaryStream(const std::s
 				return nullptr;
 			}
 			zip_uint64_t zipFileIndex = static_cast<zip_uint64_t>(zipLocateResult);
+			// Get file stats
+			struct zip_stat st;
+			if (zip_stat_index(m_zipArchive->handle(), zipFileIndex, 0, &st) != 0)
+			{
+				// Failed to stat file?
+				return nullptr;
+			}
+			if (wzMapZipIOSanityCheckStat(st) != ZipSanityCheckResult::PASSED)
+			{
+				return nullptr;
+			}
 			pStream = WzMapBinaryZipIOStream::openForReading(zipFileIndex, m_zipArchive);
 			break;
 		}
@@ -435,55 +727,64 @@ std::unique_ptr<WzMap::BinaryIOStream> WzMapZipIO::openBinaryStream(const std::s
 	return pStream;
 }
 
-bool WzMapZipIO::loadFullFile(const std::string& filename, std::vector<char>& fileData)
+WzMap::IOProvider::LoadFullFileResult WzMapZipIO::loadFullFile(const std::string& filename, std::vector<char>& fileData, uint32_t maxFileSize /*= 0*/, bool appendNullCharacter /*= false*/)
 {
 	auto zipLocateResult = wz_zip_name_locate(m_zipArchive->handle(), filename.c_str(), ZIP_FL_ENC_GUESS);
 	if (zipLocateResult < 0)
 	{
 		// Failed to find a file with this name
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_OPEN;
 	}
 	zip_uint64_t zipFileIndex = static_cast<zip_uint64_t>(zipLocateResult);
 	// Get the expected length of the file
 	struct zip_stat st;
-	zip_stat_init(&st);
 	if (zip_stat_index(m_zipArchive->handle(), zipFileIndex, 0, &st) != 0)
 	{
 		// zip_stat failed for file??
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_OPEN;
 	}
 	if (!(st.valid & ZIP_STAT_SIZE))
 	{
 		// couldn't get the file size??
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_OPEN;
 	}
-	if (std::numeric_limits<uint32_t>::max() < st.size)
+	switch (wzMapZipIOSanityCheckStat(st, (maxFileSize) ? maxFileSize : WzMapZipDefaultEmbeddedFileMaxFileSize))
 	{
-		// size is too big!
-		return false;
+		case ZipSanityCheckResult::PASSED:
+			break;
+		case ZipSanityCheckResult::FAILURE_EXCEEDS_MAXFILESIZE:
+			return WzMap::IOProvider::LoadFullFileResult::FAILURE_EXCEEDS_MAXFILESIZE;
+		default:
+			// failed some other sanity check
+			return WzMap::IOProvider::LoadFullFileResult::FAILURE_OPEN;
 	}
 	auto readStream = WzMapBinaryZipIOStream::openForReading(zipFileIndex, m_zipArchive);
 	if (!readStream)
 	{
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_OPEN;
 	}
 	// read the entire file
 	fileData.clear();
-	fileData.resize(static_cast<size_t>(st.size));
+	size_t expectedFileSize = static_cast<size_t>(st.size);
+	fileData.resize(expectedFileSize + ((appendNullCharacter) ? 1 : 0));
 	auto result = readStream->readBytes(fileData.data(), fileData.size());
 	if (!result.has_value())
 	{
 		// read failed
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_READ;
 	}
-	if (result.value() != fileData.size())
+	if (result.value() != expectedFileSize)
 	{
 		// read was short
 		fileData.clear();
-		return false;
+		return WzMap::IOProvider::LoadFullFileResult::FAILURE_READ;
+	}
+	if (appendNullCharacter)
+	{
+		fileData[fileData.size() - 1] = 0;
 	}
 	readStream->close();
-	return true;
+	return WzMap::IOProvider::LoadFullFileResult::SUCCESS;
 }
 
 bool WzMapZipIO::writeFullFile(const std::string& filename, const char *ppFileData, uint32_t fileSize)
@@ -522,6 +823,17 @@ const char* WzMapZipIO::pathSeparator() const
 	return "/";
 }
 
+bool WzMapZipIO::fileExists(const std::string& filename)
+{
+	auto zipLocateResult = wz_zip_name_locate(m_zipArchive->handle(), filename.c_str(), ZIP_FL_ENC_GUESS);
+	if (zipLocateResult < 0)
+	{
+		// Failed to find a file with this name
+		return false;
+	}
+	return true;
+}
+
 static inline bool isUnsafeZipEntryName(const std::string& filename)
 {
 	if (filename.empty())
@@ -642,7 +954,7 @@ bool WzMapZipIO::enumerateFoldersInternal(const std::string& basePath, bool recu
 	{
 		return false;
 	}
-	
+
 	if (m_cachedDirectoriesList.empty())
 	{
 		// Valid ZIP files may or may not contain dedicated directory entries (which end in "/")
@@ -687,7 +999,6 @@ bool WzMapZipIO::enumerateFoldersInternal(const std::string& basePath, bool recu
 
 			// entries that end with "/" are dedicated directory entries
 
-			size_t directoryDepth = 0;
 			while (!nameStr.empty())
 			{
 				if (nameStr.back() != '/')
@@ -708,7 +1019,6 @@ bool WzMapZipIO::enumerateFoldersInternal(const std::string& basePath, bool recu
 				{
 					m_cachedDirectoriesList.push_back(nameStr);
 				}
-				directoryDepth++;
 
 				// remove any trailing "/"
 				size_t numTrailingSlash = 0;
