@@ -33,6 +33,7 @@
 
 #include "action.h"
 #include "combat.h"
+#include "formation.h"
 #include "geometry.h"
 #include "mission.h"
 #include "projectile.h"
@@ -167,21 +168,21 @@ bool actionInRange(const DROID *psDroid, const BASE_OBJECT *psObj, int weapon_sl
 	}
 
 	const unsigned compIndex = psDroid->asWeaps[weapon_slot].nStat;
-	ASSERT_OR_RETURN(false, compIndex < numWeaponStats, "Invalid range referenced for numWeaponStats, %d > %d", compIndex, numWeaponStats);
-	const WEAPON_STATS *psStats = asWeaponStats + compIndex;
+	ASSERT_OR_RETURN(false, compIndex < asWeaponStats.size(), "Invalid range referenced for numWeaponStats, %d > %zu", compIndex, asWeaponStats.size());
+	const WEAPON_STATS *psStats = &asWeaponStats[compIndex];
 
 	const int dx = (SDWORD)psDroid->pos.x - (SDWORD)psObj->pos.x;
 	const int dy = (SDWORD)psDroid->pos.y - (SDWORD)psObj->pos.y;
 
 	const int radSq = dx * dx + dy * dy;
-	const int longRange = proj_GetLongRange(psStats, psDroid->player);
-	const int shortRange = proj_GetShortRange(psStats, psDroid->player);
+	const int longRange = proj_GetLongRange(*psStats, psDroid->player);
+	const int shortRange = proj_GetShortRange(*psStats, psDroid->player);
 
 	int rangeSq = 0;
 	switch (psDroid->secondaryOrder & DSS_ARANGE_MASK)
 	{
 		case DSS_ARANGE_OPTIMUM:
-			if (!useLongWithOptimum && weaponShortHit(psStats, psDroid->player) > weaponLongHit(psStats, psDroid->player))
+			if (!useLongWithOptimum && weaponShortHit(*psStats, psDroid->player) > weaponLongHit(*psStats, psDroid->player))
 			{
 				rangeSq = shortRange * shortRange;
 			}
@@ -206,7 +207,7 @@ bool actionInRange(const DROID *psDroid, const BASE_OBJECT *psObj, int weapon_sl
 	if (radSq <= rangeSq)
 	{
 		/* check min range */
-		const int minrange = proj_GetMinRange(psStats, psDroid->player);
+		const int minrange = proj_GetMinRange(*psStats, psDroid->player);
 		if (radSq >= minrange * minrange || !proj_Direct(psStats))
 		{
 			return true;
@@ -227,7 +228,7 @@ static bool actionInsideMinRange(DROID *psDroid, BASE_OBJECT *psObj, WEAPON_STAT
 
 	if (!psStats)
 	{
-		psStats = getWeaponStats(psDroid, 0);
+		psStats = psDroid->getWeaponStats(0);
 	}
 
 	/* if I am a multi-turret droid */
@@ -239,7 +240,7 @@ static bool actionInsideMinRange(DROID *psDroid, BASE_OBJECT *psObj, WEAPON_STAT
 	const int dx = psDroid->pos.x - psObj->pos.x;
 	const int dy = psDroid->pos.y - psObj->pos.y;
 	const int radSq = dx * dx + dy * dy;
-	const int minRange = proj_GetMinRange(psStats, psDroid->player);
+	const int minRange = proj_GetMinRange(*psStats, psDroid->player);
 	const int rangeSq = minRange * minRange;
 
 	// check min range
@@ -304,7 +305,7 @@ void actionAlignTurret(BASE_OBJECT *psObj, int weapon_slot)
 /* returns true if on target */
 bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *psWeapon)
 {
-	WEAPON_STATS *psWeapStats = asWeaponStats + psWeapon->nStat;
+	WEAPON_STATS *psWeapStats = &asWeaponStats[psWeapon->nStat];
 	uint16_t tRotation, tPitch;
 	uint16_t targetRotation;
 	int32_t  rotationTolerance = 0;
@@ -335,12 +336,12 @@ bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *
 
 	//set the pitch limits based on the weapon stats of the attacker
 	pitchLowerLimit = pitchUpperLimit = 0;
-	Vector3i attackerMuzzlePos = psAttacker->pos;  // Using for calculating the pitch, but not the direction, in case using the exact direction causes bugs somewhere.
+	Vector3i attackerTurretPos = psAttacker->pos;  // Using for calculating the pitch, but not the direction, in case using the exact direction causes bugs somewhere.
 	if (psAttacker->type == OBJ_STRUCTURE)
 	{
 		STRUCTURE *psStructure = (STRUCTURE *)psAttacker;
 		int weapon_slot = psWeapon - psStructure->asWeaps;  // Should probably be passed weapon_slot instead of psWeapon.
-		calcStructureMuzzleLocation(psStructure, &attackerMuzzlePos, weapon_slot);
+		calcStructureMuzzleBaseLocation(psStructure, &attackerTurretPos, weapon_slot);
 		pitchLowerLimit = DEG(psWeapStats->minElevation);
 		pitchUpperLimit = DEG(psWeapStats->maxElevation);
 	}
@@ -348,9 +349,9 @@ bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *
 	{
 		DROID *psDroid = (DROID *)psAttacker;
 		int weapon_slot = psWeapon - psDroid->asWeaps;  // Should probably be passed weapon_slot instead of psWeapon.
-		calcDroidMuzzleLocation(psDroid, &attackerMuzzlePos, weapon_slot);
+		calcDroidMuzzleBaseLocation(psDroid, &attackerTurretPos, weapon_slot);
 
-		if (psDroid->droidType == DROID_WEAPON || isTransporter(psDroid)
+		if (psDroid->droidType == DROID_WEAPON || psDroid->isTransporter()
 		    || psDroid->droidType == DROID_COMMAND || psDroid->droidType == DROID_CYBORG
 		    || psDroid->droidType == DROID_CYBORG_SUPER)
 		{
@@ -377,7 +378,7 @@ bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *
 	int rotationError = angleDelta(targetRotation - (tRotation + psAttacker->rot.direction));
 
 	tRotation += clip(rotationError, -rotRate, rotRate);  // Addition wrapping intentional.
-	if (psAttacker->type == OBJ_DROID && isVtolDroid((DROID *)psAttacker))
+	if (psAttacker->type == OBJ_DROID && ((DROID*)psAttacker)->isVtol())
 	{
 		// limit the rotation for vtols
 		int32_t limit = VTOL_TURRET_LIMIT;
@@ -391,11 +392,11 @@ bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *
 	bool onTarget = abs(angleDelta(targetRotation - (tRotation + psAttacker->rot.direction))) <= rotationTolerance;
 
 	/* Set muzzle pitch if not repairing or outside minimum range */
-	const int minRange = proj_GetMinRange(psWeapStats, psAttacker->player);
+	const int minRange = proj_GetMinRange(*psWeapStats, psAttacker->player);
 	if (!bRepair && (unsigned)objPosDiffSq(psAttacker, psTarget) > minRange * minRange && proj_Direct(psWeapStats))
 	{
 		/* get target distance */
-		Vector3i delta = psTarget->pos - attackerMuzzlePos;
+		Vector3i delta = psTarget->pos - attackerTurretPos;
 		int32_t dxy = iHypot(delta.x, delta.y);
 
 		uint16_t targetPitch = iAtan2(delta.z, dxy);
@@ -423,7 +424,7 @@ bool actionVisibleTarget(DROID *psDroid, BASE_OBJECT *psTarget, int weapon_slot)
 	{
 		return false;
 	}
-	if ((psDroid->numWeaps == 0 || isVtolDroid(psDroid)) && visibleObject(psDroid, psTarget, false))
+	if ((psDroid->numWeaps == 0 || psDroid->isVtol()) && visibleObject(psDroid, psTarget, false))
 	{
 		return true;
 	}
@@ -487,7 +488,7 @@ static void actionUpdateVtolAttack(DROID *psDroid)
 	}
 
 	/* circle around target if hovering and not cyborg */
-	if (psDroid->sMove.Status == MOVEHOVER && !cyborgDroid(psDroid))
+	if (psDroid->sMove.Status == MOVEHOVER && !psDroid->isCyborg())
 	{
 		actionAddVtolAttackRun(psDroid);
 	}
@@ -585,7 +586,7 @@ static bool actionRemoveDroidsFromBuildPos(unsigned player, Vector2i pos, uint16
 		}
 
 		Vector2i delta = map_coord(droid->pos.xy()) - b.map;
-		if (delta.x < 0 || delta.x >= b.size.x || delta.y < 0 || delta.y >= b.size.y || isFlying(droid))
+		if (delta.x < 0 || delta.x >= b.size.x || delta.y < 0 || delta.y >= b.size.y || droid->isFlying() || droid->isFlightBasedTransporter())
 		{
 			continue;  // Droid not under new structure (just near it).
 		}
@@ -605,7 +606,7 @@ static bool actionRemoveDroidsFromBuildPos(unsigned player, Vector2i pos, uint16
 			{
 				Vector2i dest = world_coord(b.map + Vector2i(x, y)) + Vector2i(TILE_UNITS, TILE_UNITS) / 2;
 				unsigned dist = iHypot(droid->pos.xy() - dest);
-				if (dist < bestDist && !fpathBlockingTile(map_coord(dest.x), map_coord(dest.y), getPropulsionStats(droid)->propulsionType))
+				if (dist < bestDist && !fpathBlockingTile(map_coord(dest.x), map_coord(dest.y), droid->getPropulsionStats()->propulsionType))
 				{
 					bestDest = dest;
 					bestDist = dist;
@@ -634,9 +635,13 @@ void actionSanity(DROID *psDroid)
 	// clear the target if it has died
 	for (int i = 0; i < MAX_WEAPONS; i++)
 	{
-		bDirect = proj_Direct(asWeaponStats + psDroid->asWeaps[i].nStat);
+		bDirect = proj_Direct(psDroid->getWeaponStats(i));
 		if (psDroid->psActionTarget[i] && (avoidOverkill ? aiObjectIsProbablyDoomed(psDroid->psActionTarget[i], bDirect) : psDroid->psActionTarget[i]->died))
 		{
+			if (psDroid->action == DACTION_DROIDREPAIR)
+			{
+				droidRepairStopped(castDroid(psDroid->psActionTarget[0]), psDroid);
+			}
 			syncDebugObject(psDroid->psActionTarget[i], '-');
 			setDroidActionTarget(psDroid, nullptr, i);
 			if (i == 0)
@@ -647,7 +652,7 @@ void actionSanity(DROID *psDroid)
 				{
 					psDroid->action = DACTION_NONE;
 					// if VTOL - return to rearm pad if not patrolling
-					if (isVtolDroid(psDroid))
+					if (psDroid->isVtol())
 					{
 						if ((psDroid->order.type == DORDER_PATROL || psDroid->order.type == DORDER_CIRCLE) && (!vtolEmpty(psDroid) || (psDroid->secondaryOrder & DSS_ALEV_MASK) == DSS_ALEV_NEVER))
 						{
@@ -681,7 +686,7 @@ void actionUpdateDroid(DROID *psDroid)
 
 	CHECK_DROID(psDroid);
 
-	PROPULSION_STATS *psPropStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION];
+	PROPULSION_STATS *psPropStats = psDroid->getPropulsionStats();
 	ASSERT_OR_RETURN(, psPropStats != nullptr, "Invalid propulsion stats pointer");
 
 	bool secHoldActive = secondaryGetState(psDroid, DSO_HALTTYPE) == DSS_HALT_HOLD;
@@ -729,7 +734,7 @@ void actionUpdateDroid(DROID *psDroid)
 	case DACTION_WAITFORREPAIR:
 		// doing nothing
 		// see if there's anything to shoot.
-		if (psDroid->numWeaps > 0 && !isVtolDroid(psDroid)
+		if (psDroid->numWeaps > 0 && !psDroid->isVtol()
 		    && (order->type == DORDER_NONE || order->type == DORDER_HOLD || order->type == DORDER_RTR || order->type == DORDER_GUARD))
 		{
 			for (unsigned i = 0; i < psDroid->numWeaps; ++i)
@@ -738,26 +743,29 @@ void actionUpdateDroid(DROID *psDroid)
 				{
 					BASE_OBJECT *psTemp = nullptr;
 
-					WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
+					WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(i);
 					if (psDroid->asWeaps[i].nStat > 0
 					    && psWeapStats->rotate
 						&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
-						// don't bother doing this costly calculation again if aiUpdateDroid already checked this tick
-						// (aiUpdateDroid would have set an attack target if one was available)
-						&& psDroid->lastCheckNearestTarget[i] != gameTime
+						&& (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
+						// don't bother doing this costly calculation again if aiUpdateDroid already checked this tick and failed
+						&& psDroid->lastCheckNearestTargetFailed[i] != gameTime
 					    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0)
 					{
-						if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
-						{
-							psDroid->action = DACTION_ATTACK;
-							setDroidActionTarget(psDroid, psTemp, i);
-						}
+						psDroid->action = DACTION_ATTACK;
+						setDroidActionTarget(psDroid, psTemp, i);
 					}
 				}
 			}
 		}
 		break;
 	case DACTION_WAITDURINGREPAIR:
+		// don't want to be in a formation for this move
+		if (psDroid->sMove.psFormation != nullptr)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
 		// Check that repair facility still exists
 		if (!order->psObj)
 		{
@@ -838,19 +846,17 @@ void actionUpdateDroid(DROID *psDroid)
 					BASE_OBJECT *psTemp = nullptr;
 
 					//I moved psWeapStats flag update there
-					WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
-					if (!isVtolDroid(psDroid)
+					WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(i);
+					if (!psDroid->isVtol()
 					    && psDroid->asWeaps[i].nStat > 0
 					    && psWeapStats->rotate
 					    && psWeapStats->fireOnMove
 						&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
+						&& (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
 					    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0)
 					{
-						if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
-						{
-							psDroid->action = DACTION_MOVEFIRE;
-							setDroidActionTarget(psDroid, psTemp, i);
-						}
+						psDroid->action = DACTION_MOVEFIRE;
+						setDroidActionTarget(psDroid, psTemp, i);
 					}
 				}
 			}
@@ -876,7 +882,7 @@ void actionUpdateDroid(DROID *psDroid)
 		bHasTarget = false;
 		for (unsigned i = 0; i < psDroid->numWeaps; ++i)
 		{
-			bDirect = proj_Direct(asWeaponStats + psDroid->asWeaps[i].nStat);
+			bDirect = proj_Direct(psDroid->getWeaponStats(i));
 			blockingWall = nullptr;
 			// Does this weapon have a target?
 			if (psDroid->psActionTarget[i] != nullptr)
@@ -927,7 +933,7 @@ void actionUpdateDroid(DROID *psDroid)
 			for (unsigned i = 0; i < psDroid->numWeaps; ++i)
 			{
 				const unsigned compIndex = psDroid->asWeaps[i].nStat;
-				const WEAPON_STATS *psStats = asWeaponStats + compIndex;
+				const WEAPON_STATS *psStats = &asWeaponStats[compIndex];
 				wallBlocked = false;
 
 				// has weapon a target? is target valid?
@@ -982,7 +988,7 @@ void actionUpdateDroid(DROID *psDroid)
 		}
 		//check its a VTOL unit since adding Transporter's into multiPlayer
 		/* check vtol attack runs */
-		if (isVtolDroid(psDroid))
+		if (psDroid->isVtol())
 		{
 			actionUpdateVtolAttack(psDroid);
 		}
@@ -1019,6 +1025,16 @@ void actionUpdateDroid(DROID *psDroid)
 			psDroid->action = DACTION_ATTACK;
 		}
 
+		if (psDroid->action == DACTION_ATTACK)
+		{
+			// don't wan't formations for this one
+			if (psDroid->sMove.psFormation)
+			{
+				formationLeave(psDroid->sMove.psFormation, psDroid);
+				psDroid->sMove.psFormation = nullptr;
+			}
+		}
+
 		//check the target hasn't become one the same player ID - Electronic Warfare
 		if (electronicDroid(psDroid) && psDroid->player == psDroid->psActionTarget[0]->player)
 		{
@@ -1071,7 +1087,7 @@ void actionUpdateDroid(DROID *psDroid)
 			    && actionVisibleTarget(psDroid, psActionTarget, i)
 			    && actionInRange(psDroid, psActionTarget, i))
 			{
-				WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
+				WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(i);
 				WEAPON_EFFECT weapEffect = psWeapStats->weaponEffect;
 				blockingWall = visGetBlockingWall(psDroid, psActionTarget);
 
@@ -1126,6 +1142,14 @@ void actionUpdateDroid(DROID *psDroid)
 					else if (!psWeapStats->rotate ||
 					         actionTargetTurret(psDroid, psActionTarget, &psDroid->asWeaps[i]))
 					{
+						// Prevents artillery units attached to a leader sometimes not stopping.
+						if (!psDroid->isVtol() &&
+							!proj_Direct(psWeapStats) &&
+							psDroid->order.type == DORDER_FIRESUPPORT &&
+							!DROID_STOPPED(psDroid))
+						{
+							moveStopDroid(psDroid);
+						}
 						/* In range - fire !!! */
 						combFire(&psDroid->asWeaps[i], psDroid, psActionTarget, i);
 					}
@@ -1146,7 +1170,7 @@ void actionUpdateDroid(DROID *psDroid)
 		if (!bHasTarget || wallBlocked)
 		{
 			BASE_OBJECT *psTarget;
-			bool supportsSensorTower = !isVtolDroid(psDroid) && (psTarget = orderStateObj(psDroid, DORDER_FIRESUPPORT)) && psTarget->type == OBJ_STRUCTURE;
+			bool supportsSensorTower = !psDroid->isVtol() && (psTarget = orderStateObj(psDroid, DORDER_FIRESUPPORT)) && psTarget->type == OBJ_STRUCTURE;
 
 			if (secHoldActive && (order->type == DORDER_ATTACKTARGET || order->type == DORDER_FIRESUPPORT))
 			{
@@ -1226,7 +1250,7 @@ void actionUpdateDroid(DROID *psDroid)
 					    && validTarget(psDroid, psDroid->psActionTarget[0], i))
 					{
 						//I moved psWeapStats flag update there
-						psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
+						psWeapStats = psDroid->getWeaponStats(i);
 						if (actionVisibleTarget(psDroid, psDroid->psActionTarget[0], i))
 						{
 							if (actionInRange(psDroid, psDroid->psActionTarget[0], i))
@@ -1278,7 +1302,7 @@ void actionUpdateDroid(DROID *psDroid)
 				else if (psWeapStats)
 				{
 					// if the vtol is far enough away head for the target again
-					const int maxRange = proj_GetLongRange(psWeapStats, psDroid->player);
+					const int maxRange = proj_GetLongRange(*psWeapStats, psDroid->player);
 					if (rangeSq > maxRange * maxRange)
 					{
 						// don't do another attack run if already heading for the target
@@ -1293,8 +1317,15 @@ void actionUpdateDroid(DROID *psDroid)
 			break;
 		}
 	case DACTION_MOVETOATTACK:
+		// don't wan't formations for this one
+		if (psDroid->sMove.psFormation)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
+
 		// send vtols back to rearm
-		if (isVtolDroid(psDroid) && vtolEmpty(psDroid))
+		if (psDroid->isVtol() && vtolEmpty(psDroid))
 		{
 			moveToRearm(psDroid);
 			break;
@@ -1325,14 +1356,14 @@ void actionUpdateDroid(DROID *psDroid)
 					    && actionVisibleTarget(psDroid, psDroid->psActionTarget[0], i))
 					{
 						bool chaseBloke = false;
-						WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
+						WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(i);
 						const bool actionIsInRange = actionInRange(psDroid, psDroid->psActionTarget[0], i);
 						if (psWeapStats->rotate)
 						{
 							actionTargetTurret(psDroid, psDroid->psActionTarget[0], &psDroid->asWeaps[i]);
 						}
 
-						if (!isVtolDroid(psDroid) &&
+						if (!psDroid->isVtol() &&
 						    psDroid->psActionTarget[0]->type == OBJ_DROID &&
 						    ((DROID *)psDroid->psActionTarget[0])->droidType == DROID_PERSON &&
 						    psWeapStats->fireOnMove)
@@ -1403,7 +1434,7 @@ void actionUpdateDroid(DROID *psDroid)
 				/* Stopped moving but haven't reached the target - possibly move again */
 
 				//'hack' to make the droid to check the primary turrent instead of all
-				WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[0].nStat];
+				WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(0);
 
 				if (order->type == DORDER_ATTACKTARGET && secHoldActive)
 				{
@@ -1518,6 +1549,13 @@ void actionUpdateDroid(DROID *psDroid)
 			}
 		} // End of check for whether the droid can still succesfully build the ordered structure
 
+		// The droid cannot be in a formation
+		if (psDroid->sMove.psFormation != nullptr)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
+
 		// The droid can still build or help with a build, and is moving to a location to do so - are we there yet, are we there yet, are we there yet?
 		if (actionReachedBuildPos(psDroid, psDroid->actionPos.x, psDroid->actionPos.y, order->direction, order->psStats))
 		{
@@ -1693,6 +1731,12 @@ void actionUpdateDroid(DROID *psDroid)
 			psDroid->action = DACTION_NONE;
 			break;
 		}
+		// The droid cannot be in a formation
+		if (psDroid->sMove.psFormation != nullptr)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
 		if (DROID_STOPPED(psDroid) &&
 		    !actionReachedBuildPos(psDroid, psDroid->actionPos.x, psDroid->actionPos.y, order->direction, order->psStats))
 		{
@@ -1744,7 +1788,7 @@ void actionUpdateDroid(DROID *psDroid)
 				{
 					cantDoRepairLikeAction = true;
 				}
-				else if (order->type != DORDER_DEMOLISH && structureAtPos->body == structureBody(structureAtPos))
+				else if (order->type != DORDER_DEMOLISH && structureAtPos->body == structureAtPos->structureBody())
 				{
 					cantDoRepairLikeAction = true;
 				}
@@ -1766,6 +1810,14 @@ void actionUpdateDroid(DROID *psDroid)
 				}
 			}
 		}
+
+		// The droid cannot be in a formation
+		if (psDroid->sMove.psFormation != nullptr)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
+
 		// see if the droid is at the edge of what it is moving to
 		if (actionReachedBuildPos(psDroid, psDroid->actionPos.x, psDroid->actionPos.y, ((STRUCTURE *)psDroid->psActionTarget[0])->rot.direction, order->psStats))
 		{
@@ -1861,6 +1913,12 @@ void actionUpdateDroid(DROID *psDroid)
 		}
 		break;
 	case DACTION_MOVETOREPAIRPOINT:
+		// don't want to be in a formation for this move
+		if (psDroid->sMove.psFormation != nullptr)
+		{
+			formationLeave(psDroid->sMove.psFormation, psDroid);
+			psDroid->sMove.psFormation = nullptr;
+		}
 		if (psDroid->order.rtrType == RTR_TYPE_REPAIR_FACILITY)
 		{
 			/* moving from front to rear of repair facility or rearm pad */
@@ -1965,11 +2023,11 @@ void actionUpdateDroid(DROID *psDroid)
 
 		//don't move VTOL's
 		// also don't move closer to sensor towers
-		if (!isVtolDroid(psDroid) && order->psObj->type != OBJ_STRUCTURE)
+		if (!psDroid->isVtol() && order->psObj->type != OBJ_STRUCTURE)
 		{
 			Vector2i diff = (psDroid->pos - order->psObj->pos).xy();
 			//Consider .shortRange here
-			int rangeSq = asWeaponStats[psDroid->asWeaps[0].nStat].upgrade[psDroid->player].maxRange / 2; // move close to sensor
+			int rangeSq = psDroid->getWeaponStats(0)->upgrade[psDroid->player].maxRange / 2; // move close to sensor
 			rangeSq = rangeSq * rangeSq;
 			if (dot(diff, diff) < rangeSq)
 			{
@@ -2028,6 +2086,7 @@ void actionUpdateDroid(DROID *psDroid)
 				actionTargetTurret(psDroid, psDroid->psActionTarget[0], &psDroid->asWeaps[0]);
 				droidStartAction(psDroid);
 				psDroid->action = DACTION_DROIDREPAIR;
+				droidRepairStarted((DROID *)psDroid->psActionTarget[0], psDroid);
 			}
 			if (DROID_STOPPED(psDroid))
 			{
@@ -2046,7 +2105,7 @@ void actionUpdateDroid(DROID *psDroid)
 				psDroid->action = DACTION_NONE;
 				break;
 			}
-			
+
 			// If not doing self-repair (psActionTarget[0] is repair target)
 			if (psDroid->psActionTarget[0] != psDroid)
 			{
@@ -2054,7 +2113,7 @@ void actionUpdateDroid(DROID *psDroid)
 			}
 			// Just self-repairing.
 			// See if there's anything to shoot.
-			else if (psDroid->numWeaps > 0 && !isVtolDroid(psDroid)
+			else if (psDroid->numWeaps > 0 && !psDroid->isVtol()
 			         && (order->type == DORDER_NONE || order->type == DORDER_HOLD || order->type == DORDER_RTR))
 			{
 				for (unsigned i = 0; i < psDroid->numWeaps; ++i)
@@ -2063,12 +2122,13 @@ void actionUpdateDroid(DROID *psDroid)
 					{
 						BASE_OBJECT *psTemp = nullptr;
 
-						WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
+						WEAPON_STATS *const psWeapStats = psDroid->getWeaponStats(i);
 						if (psDroid->asWeaps[i].nStat > 0 && psWeapStats->rotate
 						    && secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS
 							&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
 						    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0 && psTemp)
 						{
+							droidRepairStopped(castDroid(psDroid->psActionTarget[0]), psDroid);
 							psDroid->action = DACTION_ATTACK;
 							setDroidActionTarget(psDroid, psTemp, 0);
 							break;
@@ -2086,6 +2146,7 @@ void actionUpdateDroid(DROID *psDroid)
 			ydiff = (SDWORD)psDroid->pos.y - (SDWORD)psDroid->psActionTarget[0]->pos.y;
 			if (xdiff * xdiff + ydiff * ydiff > REPAIR_RANGE * REPAIR_RANGE)
 			{
+				droidRepairStopped(castDroid(psDroid->psActionTarget[0]), psDroid);
 				if (order->type == DORDER_DROIDREPAIR)
 				{
 					// damaged droid has moved off - follow if we're not holding position!
@@ -2102,6 +2163,7 @@ void actionUpdateDroid(DROID *psDroid)
 			{
 				if (!droidUpdateDroidRepair(psDroid))
 				{
+					droidRepairStopped(castDroid(psDroid->psActionTarget[0]), psDroid);
 					psDroid->action = DACTION_NONE;
 					moveStopDroid(psDroid);
 					//if the order is RTR then resubmit order so that the unit will go to repair facility point
@@ -2231,7 +2293,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 {
 	ASSERT_OR_RETURN(, psAction->psObj == nullptr || !psAction->psObj->died, "Droid dead");
 
-	WEAPON_STATS *psWeapStats = getWeaponStats(psDroid, 0);
+	WEAPON_STATS *psWeapStats = psDroid->getWeaponStats(0);
 	Vector2i pos(0, 0);
 
 	CHECK_DROID(psDroid);
@@ -2243,6 +2305,9 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 	syncDebug("%d does %s", psDroid->id, getDroidActionName(psAction->action));
 	objTrace(psDroid->id, "base set action to %s (was %s)", getDroidActionName(psAction->action), getDroidActionName(psDroid->action));
 
+	DROID_ACTION priorDroidAction = psDroid->action;
+	BASE_OBJECT* priorActionTarget0 = psDroid->psActionTarget[0];
+
 	DROID_ORDER_DATA *order = &psDroid->order;
 	bool hasValidWeapon = false;
 	for (int i = 0; i < MAX_WEAPONS; i++)
@@ -2279,7 +2344,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		break;
 
 	case DACTION_ATTACK:
-		if (psDroid->asWeaps[0].nStat == 0 || isTransporter(psDroid) || psAction->psObj == psDroid)
+		if (psDroid->asWeaps[0].nStat == 0 || psDroid->isTransporter() || psAction->psObj == psDroid)
 		{
 			break;
 		}
@@ -2289,7 +2354,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 			// for ex. AA gun can't attack ground unit
 			break;
 		}
-		if (vtolEmpty(psDroid) && vtolRearming(psDroid))
+		if (vtolEmpty(psDroid) && psDroid->isVtolRearming())
 		{
 			// ignore attack order: we have no ammo and we are rearming right now, or going to rearm, or waiting for it
 			break;
@@ -2308,7 +2373,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 			//in multiPlayer cannot electronically attack a transporter
 			if (bMultiPlayer
 			    && psAction->psObj->type == OBJ_DROID
-			    && isTransporter((DROID *)psAction->psObj))
+			    && ((DROID*)psAction->psObj)->isTransporter())
 			{
 				psDroid->action = DACTION_NONE;
 				break;
@@ -2327,7 +2392,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		   || (order->type == DORDER_GUARD && hasCommander(psDroid))
 		   || order->type == DORDER_FIRESUPPORT)
 		   && secHoldActive)
-		   || (!isVtolDroid(psDroid) && (orderStateObj(psDroid, DORDER_FIRESUPPORT) != nullptr)))
+		   || (!psDroid->isVtol() && (orderStateObj(psDroid, DORDER_FIRESUPPORT) != nullptr)))
 		{
 			psDroid->action = DACTION_ATTACK;		// holding, try attack straightaway
 		}
@@ -2369,7 +2434,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		}
 		else if (actionIsInRange || (order->type != DORDER_HOLD && secondaryGetState(psDroid, DSO_HALTTYPE) == DSS_HALT_HOLD))
 		{
-			if (!isVtolDroid(psDroid))
+			if (!psDroid->isVtol())
 			{
 				moveStopDroid(psDroid);
 				psDroid->action = DACTION_ATTACK;
@@ -2441,7 +2506,15 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		psDroid->action = DACTION_MOVETOBUILD;
 		psDroid->actionPos.x = psAction->x;
 		psDroid->actionPos.y = psAction->y;
-		moveDroidToNoFormation(psDroid, psDroid->actionPos.x, psDroid->actionPos.y);
+		// Move a droid in a tile if it has never been moved before
+		if (psDroid->actionPos == psDroid->pos.xy())
+		{
+			actionRemoveDroidsFromBuildPos(psDroid->player, psDroid->actionPos, order->direction, order->psStats);
+		}
+		else
+		{
+			moveDroidToNoFormation(psDroid, psDroid->actionPos.x, psDroid->actionPos.y);
+		}
 		break;
 	case DACTION_DEMOLISH:
 		ASSERT_OR_RETURN(, order->type == DORDER_DEMOLISH, "cannot start demolish action without a demolish order");
@@ -2489,7 +2562,7 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		break;
 	case DACTION_FIRESUPPORT:
 		psDroid->action = DACTION_FIRESUPPORT;
-		if (!isVtolDroid(psDroid) && !secHoldActive && order->psObj->type != OBJ_STRUCTURE)
+		if (!psDroid->isVtol() && !secHoldActive && order->psObj->type != OBJ_STRUCTURE)
 		{
 			moveDroidTo(psDroid, order->psObj->pos.x, order->psObj->pos.y);		// movetotarget.
 		}
@@ -2549,6 +2622,10 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 				psDroid->action = DACTION_MOVETODROIDREPAIR;
 				moveDroidTo(psDroid, psAction->x, psAction->y);
 			}
+			if (psDroid->action == DACTION_DROIDREPAIR && priorDroidAction != DACTION_DROIDREPAIR)
+			{
+				droidRepairStarted(castDroid(psDroid->psActionTarget[0]), psDroid);
+			}
 			break;
 		}
 	case DACTION_RESTORE:
@@ -2569,6 +2646,10 @@ static void actionDroidBase(DROID *psDroid, DROID_ACTION_DATA *psAction)
 		ASSERT(!"unknown action", "actionUnitBase: unknown action");
 		break;
 	}
+	if (priorDroidAction == DACTION_DROIDREPAIR && psDroid->action != DACTION_DROIDREPAIR)
+	{
+		droidRepairStopped(castDroid(priorActionTarget0), psDroid);
+	}
 	syncDebugDroid(psDroid, '+');
 	CHECK_DROID(psDroid);
 }
@@ -2630,13 +2711,13 @@ void moveToRearm(DROID *psDroid)
 {
 	CHECK_DROID(psDroid);
 
-	if (!isVtolDroid(psDroid))
+	if (!psDroid->isVtol())
 	{
 		return;
 	}
 
 	//if droid is already returning - ignore
-	if (vtolRearming(psDroid))
+	if (psDroid->isVtolRearming())
 	{
 		return;
 	}
@@ -2790,7 +2871,7 @@ bool actionVTOLLandingPos(DROID const *psDroid, Vector2i *p)
 	int startY = map_coord(p->y);
 
 	// set blocking flags for all the other droids
-	for (const DROID *psCurr = apsDroidLists[psDroid->player]; psCurr; psCurr = psCurr->psNext)
+	for (const DROID *psCurr : apsDroidLists[psDroid->player])
 	{
 		Vector2i t(0, 0);
 		if (DROID_STOPPED(psCurr))
@@ -2822,7 +2903,7 @@ bool actionVTOLLandingPos(DROID const *psDroid, Vector2i *p)
 	}
 
 	// clear blocking flags for all the other droids
-	for (DROID *psCurr = apsDroidLists[psDroid->player]; psCurr; psCurr = psCurr->psNext)
+	for (const DROID *psCurr : apsDroidLists[psDroid->player])
 	{
 		Vector2i t(0, 0);
 		if (DROID_STOPPED(psCurr))
