@@ -51,8 +51,10 @@
 #include <vector>
 #include <algorithm>
 #include <memory>
+#include <iterator>
+#include <cstddef>
 
-#include "lib/netplay/netplay.h"
+#include "lib/netplay/sync_debug.h"
 
 /// A coordinate.
 struct PathCoord
@@ -171,12 +173,12 @@ struct PathfindContext
 	{
 		return !blockingMap->dangerMap.empty() && blockingMap->dangerMap[x + y * mapWidth];
 	}
-	bool matches(std::shared_ptr<PathBlockingMap> &blockingMap_, PathCoord tileS_, PathNonblockingArea dstIgnore_) const
+	bool matches(const std::shared_ptr<const PathBlockingMap> &blockingMap_, PathCoord tileS_, PathNonblockingArea dstIgnore_) const
 	{
 		// Must check myGameTime == blockingMap_->type.gameTime, otherwise blockingMap could be a deleted pointer which coincidentally compares equal to the valid pointer blockingMap_.
 		return myGameTime == blockingMap_->type.gameTime && blockingMap == blockingMap_ && tileS == tileS_ && dstIgnore == dstIgnore_;
 	}
-	void assign(std::shared_ptr<PathBlockingMap> &blockingMap_, PathCoord tileS_, PathNonblockingArea dstIgnore_)
+	void assign(const std::shared_ptr<const PathBlockingMap> &blockingMap_, PathCoord tileS_, PathNonblockingArea dstIgnore_)
 	{
 		blockingMap = blockingMap_;
 		tileS = tileS_;
@@ -206,13 +208,10 @@ struct PathfindContext
 
 	std::vector<PathNode> nodes;        ///< Edge of explored region of the map.
 	std::vector<PathExploredTile> map;  ///< Map, with paths leading back to tileS.
-	std::shared_ptr<PathBlockingMap> blockingMap; ///< Map of blocking tiles for the type of object which needs a path.
+	std::shared_ptr<const PathBlockingMap> blockingMap; ///< Map of blocking tiles for the type of object which needs a path.
 	PathNonblockingArea dstIgnore;      ///< Area of structure at destination which should be considered nonblocking.
 };
 
-/// Last recently used list of contexts.
-static std::list<PathfindContext> fpathContexts;
-
 /// Lists of blocking maps from current tick.
 static std::vector<std::shared_ptr<PathBlockingMap>> fpathBlockingMaps;
 /// Game time for all blocking maps in fpathBlockingMaps.
@@ -234,7 +233,6 @@ static const Vector2i aDirOffset[] =
 
 void fpathHardTableReset()
 {
-	fpathContexts.clear();
 	fpathBlockingMaps.clear();
 }
 
@@ -432,7 +430,7 @@ static PathCoord fpathAStarExplore(PathfindContext &context, PathCoord tileF)
 	return nearestCoord;
 }
 
-static void fpathInitContext(PathfindContext &context, std::shared_ptr<PathBlockingMap> &blockingMap, PathCoord tileS, PathCoord tileRealS, PathCoord tileF, PathNonblockingArea dstIgnore)
+static void fpathInitContext(PathfindContext &context, const std::shared_ptr<const PathBlockingMap> &blockingMap, PathCoord tileS, PathCoord tileRealS, PathCoord tileF, PathNonblockingArea dstIgnore)
 {
 	context.assign(blockingMap, tileS, dstIgnore);
 
@@ -441,20 +439,161 @@ static void fpathInitContext(PathfindContext &context, std::shared_ptr<PathBlock
 	ASSERT(!context.nodes.empty(), "fpathNewNode failed to add node.");
 }
 
-ASR_RETVAL fpathAStarRoute(MOVE_CONTROL *psMove, PATHJOB *psJob)
+class PathfindContextList
+{
+public:
+	struct Iterator
+	{
+		using iterator_category = std::random_access_iterator_tag;
+		using difference_type = std::ptrdiff_t;
+		using value_type = PathfindContext;
+		using pointer = value_type*;
+		using reference = value_type&;
+
+		Iterator(PathfindContextList& list, size_t idx) : m_list(list), m_idx(idx)
+		{}
+
+		Iterator(const Iterator& other)
+		: m_list(other.m_list)
+		, m_idx(other.m_idx)
+		{}
+
+		Iterator& operator=(const Iterator& other)
+		{
+			m_list = other.m_list;
+			m_idx = other.m_idx;
+			return *this;
+		}
+
+		reference operator*() const { return m_list.contexts[m_list.orderedIndexes[m_idx]]; }
+		pointer operator->() const { return &m_list.contexts[m_list.orderedIndexes[m_idx]]; }
+
+		Iterator& operator++() { ++m_idx; return *this; }
+		Iterator operator++(int) { Iterator tmp = *this; ++(*this); return tmp; }
+
+		Iterator& operator--() { --m_idx; return *this; }
+		Iterator operator--(int) { Iterator tmp = *this; --(*this); return tmp; }
+
+		Iterator operator+(difference_type n) const { return Iterator(m_list, m_idx + n); }
+		Iterator operator-(difference_type n) const { return Iterator(m_list, m_idx - n); }
+		difference_type operator-(const Iterator& other) const { return m_idx - other.m_idx; }
+		Iterator& operator+=(difference_type n) { m_idx += n; return *this; }
+		Iterator& operator-=(difference_type n) { m_idx -= n; return *this; }
+
+		reference operator[](difference_type n) const { return m_list.contexts[m_list.orderedIndexes[n]]; }
+
+		bool operator== (const Iterator& other) { return m_idx == other.m_idx; }
+		bool operator!= (const Iterator& other) { return m_idx != other.m_idx; }
+		bool operator< (const Iterator& other) const { return m_idx < other.m_idx; }
+		bool operator> (const Iterator& other) const { return m_idx > other.m_idx; }
+		bool operator<= (const Iterator& other) const { return m_idx <= other.m_idx; }
+		bool operator>= (const Iterator& other) const { return m_idx >= other.m_idx; }
+
+	private:
+		friend class PathfindContextList;
+
+		PathfindContextList& m_list;
+		size_t m_idx;
+	};
+
+public:
+
+	Iterator push_back(PathfindContext& ctx);
+	Iterator push_back(PathfindContext&& ctx);
+
+	void moveToFront(Iterator it); // invalidates any iterators
+
+	Iterator begin() { return Iterator(*this, 0); }
+	Iterator end() { return Iterator(*this, orderedIndexes.size()); }
+
+	void clear();
+
+	bool empty() const { return contexts.empty(); }
+	PathfindContext& front() { return contexts[orderedIndexes.front()]; }
+
+	size_t size() const { return contexts.size(); }
+
+private:
+	std::vector<PathfindContext> contexts;
+	std::vector<size_t> orderedIndexes;
+};
+
+PathfindContextList::Iterator PathfindContextList::push_back(PathfindContext& ctx)
+{
+	contexts.push_back(ctx);
+	orderedIndexes.push_back(contexts.size() - 1);
+	return Iterator(*this, orderedIndexes.size() - 1);
+}
+
+PathfindContextList::Iterator PathfindContextList::push_back(PathfindContext&& ctx)
+{
+	contexts.push_back(std::move(ctx));
+	orderedIndexes.push_back(contexts.size() - 1);
+	return Iterator(*this, orderedIndexes.size() - 1);
+}
+
+void PathfindContextList::moveToFront(Iterator it)
+{
+	auto oIt = orderedIndexes.begin() + it.m_idx;
+	std::rotate(orderedIndexes.begin(), oIt, oIt + 1);
+}
+
+void PathfindContextList::clear()
+{
+	contexts.clear();
+	orderedIndexes.clear();
+}
+
+class FPathExecuteContextImpl : public FPathExecuteContext
+{
+public:
+	virtual ~FPathExecuteContextImpl();
+
+	void resetForNewGameTimeIfNeeded(const PATHJOB& job);
+public:
+	/// Last recently used list of contexts.
+	PathfindContextList fpathContexts;
+	/// Used to avoid extra allocations in fpathAStarRoute
+	std::vector<Vector2i> pathBuffer;
+};
+
+FPathExecuteContext::~FPathExecuteContext()
+{ }
+
+FPathExecuteContextImpl::~FPathExecuteContextImpl()
+{ }
+
+void FPathExecuteContextImpl::resetForNewGameTimeIfNeeded(const PATHJOB& job)
+{
+	if (!fpathContexts.empty() && job.blockingMap->type.gameTime != fpathContexts.front().myGameTime)
+	{
+		fpathContexts.clear();
+	}
+}
+
+std::shared_ptr<FPathExecuteContext> makeFPathExecuteContext()
+{
+	return std::make_shared<FPathExecuteContextImpl>();
+}
+
+ASR_RETVAL fpathAStarRoute(const std::shared_ptr<FPathExecuteContext>& ctx, MOVE_CONTROL *psMove, PATHJOB *psJob)
 {
 	ASR_RETVAL      retval = ASR_OK;
 
 	bool            mustReverse = true;
 
+	auto ctxImpl = std::static_pointer_cast<FPathExecuteContextImpl>(ctx);
+	ctxImpl->resetForNewGameTimeIfNeeded(*psJob);
+
+	auto& fpathContexts = ctxImpl->fpathContexts;
 	const PathCoord tileOrig(map_coord(psJob->origX), map_coord(psJob->origY));
 	const PathCoord tileDest(map_coord(psJob->destX), map_coord(psJob->destY));
 	const PathNonblockingArea dstIgnore(psJob->dstStructure);
 
 	PathCoord endCoord;  // Either nearest coord (mustReverse = true) or orig (mustReverse = false).
 
-	std::list<PathfindContext>::iterator contextIterator = fpathContexts.begin();
-	for (contextIterator = fpathContexts.begin(); contextIterator != fpathContexts.end(); ++contextIterator)
+	auto contextIterator = fpathContexts.begin();
+	for (; contextIterator != fpathContexts.end(); ++contextIterator)
 	{
 		if (!contextIterator->matches(psJob->blockingMap, tileDest, dstIgnore))
 		{
@@ -490,12 +629,7 @@ ASR_RETVAL fpathAStarRoute(MOVE_CONTROL *psMove, PATHJOB *psJob)
 	if (contextIterator == fpathContexts.end())
 	{
 		// We did not find an appropriate context. Make one.
-
-		if (fpathContexts.size() < 30)
-		{
-			fpathContexts.push_back(PathfindContext());
-		}
-		--contextIterator;
+		contextIterator = fpathContexts.push_back(PathfindContext());
 
 		// Init a new context, overwriting the oldest one if we are caching too many.
 		// We will be searching from orig to dest, since we don't know where the nearest reachable tile to dest is.
@@ -513,7 +647,7 @@ ASR_RETVAL fpathAStarRoute(MOVE_CONTROL *psMove, PATHJOB *psJob)
 	}
 
 	// Get route, in reverse order.
-	static std::vector<Vector2i> path;  // Declared static to save allocations.
+	std::vector<Vector2i>& path = ctxImpl->pathBuffer;
 	path.clear();
 
 	Vector2i newP(0, 0);
@@ -590,7 +724,7 @@ ASR_RETVAL fpathAStarRoute(MOVE_CONTROL *psMove, PATHJOB *psJob)
 	// Move context to beginning of last recently used list.
 	if (contextIterator != fpathContexts.begin())  // Not sure whether or not the splice is a safe noop, if equal.
 	{
-		fpathContexts.splice(fpathContexts.begin(), fpathContexts, contextIterator);
+		fpathContexts.moveToFront(contextIterator);
 	}
 
 	psMove->destination = psMove->asPath[path.size() - 1];
@@ -621,8 +755,8 @@ void fpathSetBlockingMap(PATHJOB *psJob)
 	if (i == fpathBlockingMaps.end())
 	{
 		// Didn't find the map, so i does not point to a map.
-		PathBlockingMap *blockMap = new PathBlockingMap();
-		fpathBlockingMaps.emplace_back(blockMap);
+		auto blockMap = std::make_shared<PathBlockingMap>();
+		fpathBlockingMaps.push_back(blockMap);
 
 		// blockMap now points to an empty map with no data. Fill the map.
 		blockMap->type = type;
@@ -648,7 +782,7 @@ void fpathSetBlockingMap(PATHJOB *psJob)
 		}
 		syncDebug("blockingMap(%d,%d,%d,%d) = %08X %08X", gameTime, psJob->propulsion, psJob->owner, psJob->moveType, checksumMap, checksumDangerMap);
 
-		psJob->blockingMap = fpathBlockingMaps.back();
+		psJob->blockingMap = blockMap;
 	}
 	else
 	{
