@@ -34,6 +34,8 @@
 #include "display3d.h"
 #include "effects.h"
 #include "miscimd.h"
+#include "profiling.h"
+#include "droid.h"
 
 #include <algorithm>
 
@@ -108,7 +110,7 @@ static SDWORD bucketCalculateZ(RENDER_TYPE objectType, void *pObject, const glm:
 		{
 
 			//the weapon stats holds the reference to which graphic to use
-			pImd = ((PROJECTILE *)pObject)->psWStats->pInFlightGraphic->displayModel();
+			pImd = (((PROJECTILE *)pObject)->psWStats->pInFlightGraphic) ? ((PROJECTILE *)pObject)->psWStats->pInFlightGraphic->displayModel() : nullptr;
 
 			psSimpObj = (SIMPLE_OBJECT *) pObject;
 			position.x = psSimpObj->pos.x;
@@ -118,7 +120,7 @@ static SDWORD bucketCalculateZ(RENDER_TYPE objectType, void *pObject, const glm:
 
 			z = pie_RotateProjectWithPerspective(&position, perspectiveViewMatrix, &pixel);
 
-			if (z > 0)
+			if (z > 0 && pImd != nullptr)
 			{
 				//particle use the image radius
 				radius = pImd->radius;
@@ -195,7 +197,7 @@ static SDWORD bucketCalculateZ(RENDER_TYPE objectType, void *pObject, const glm:
 		position.z = -(psSimpObj->pos.y);
 		position.y = psSimpObj->pos.z;
 
-		psBStats = asBodyStats + psDroid->asBits[COMP_BODY];
+		psBStats = psDroid->getBodyStats();
 		droidSize = psBStats->pIMD->radius;
 		z = pie_RotateProjectWithPerspective(&position, perspectiveViewMatrix, &pixel) - (droidSize * 2);
 
@@ -384,6 +386,7 @@ void bucketAddTypeToList(RENDER_TYPE objectType, void *pObject, const glm::mat4
 /* render Objects in list */
 void bucketRenderCurrentList(const glm::mat4 &viewMatrix, const glm::mat4 &perspectiveViewMatrix)
 {
+	WZ_PROFILE_SCOPE(bucketRenderCurrentList);
 	std::sort(bucketArray.begin(), bucketArray.end());
 
 	for (auto thisTag = bucketArray.cbegin(); thisTag != bucketArray.cend(); ++thisTag)
