@@ -71,11 +71,10 @@ bool setPlayerColour(UDWORD player, UDWORD col)
 
 UBYTE getPlayerColour(UDWORD pl)
 {
-	if (pl == MAX_PLAYERS)
+	if (pl >= MAX_PLAYERS)
 	{
-		return 0; // baba
+		return 0; // baba, or spectator slot
 	}
-	ASSERT_OR_RETURN(0, pl < MAX_PLAYERS, "Invalid player number %u", pl);
 	return NetPlay.players[pl].colour;
 }
 
@@ -88,22 +87,22 @@ static glm::mat4 setMatrix(const Vector3i *Position, const Vector3i *Rotation, i
 		glm::scale(glm::vec3(scale / 100.f));
 }
 
-UDWORD getComponentDroidRadius(DROID *)
+UDWORD getComponentDroidRadius(const DROID *)
 {
 	return 100;
 }
 
 
-UDWORD getComponentDroidTemplateRadius(DROID_TEMPLATE *)
+UDWORD getComponentDroidTemplateRadius(const DROID_TEMPLATE *)
 {
 	return 100;
 }
 
 
-UDWORD getComponentRadius(BASE_STATS *psComponent) // DISPLAY ONLY
+UDWORD getComponentRadius(const BASE_STATS *psComponent) // DISPLAY ONLY
 {
-	iIMDShape *ComponentIMD = nullptr;
-	iIMDShape *MountIMD = nullptr;
+	const iIMDShape *ComponentIMD = nullptr;
+	const iIMDShape *MountIMD = nullptr;
 	SDWORD compID;
 
 	compID = StatIsComponent(psComponent);
@@ -118,8 +117,8 @@ UDWORD getComponentRadius(BASE_STATS *psComponent) // DISPLAY ONLY
 
 	/* VTOL bombs are only stats allowed to have NULL ComponentIMD */
 	if (StatIsComponent(psComponent) != COMP_WEAPON
-	    || (((WEAPON_STATS *)psComponent)->weaponSubClass != WSC_BOMB
-	        && ((WEAPON_STATS *)psComponent)->weaponSubClass != WSC_EMP))
+	    || (((const WEAPON_STATS *)psComponent)->weaponSubClass != WSC_BOMB
+	        && ((const WEAPON_STATS *)psComponent)->weaponSubClass != WSC_EMP))
 	{
 		ASSERT(ComponentIMD, "No ComponentIMD!");
 	}
@@ -128,9 +127,9 @@ UDWORD getComponentRadius(BASE_STATS *psComponent) // DISPLAY ONLY
 }
 
 
-UDWORD getResearchRadius(BASE_STATS *Stat) // DISPLAY ONLY
+UDWORD getResearchRadius(const BASE_STATS *Stat) // DISPLAY ONLY
 {
-	const iIMDShape *ResearchIMD = safeGetDisplayModelFromBase(((RESEARCH *)Stat)->pIMD);
+	const iIMDShape *ResearchIMD = safeGetDisplayModelFromBase(((const RESEARCH *)Stat)->pIMD);
 
 	if (ResearchIMD)
 	{
@@ -143,19 +142,19 @@ UDWORD getResearchRadius(BASE_STATS *Stat) // DISPLAY ONLY
 }
 
 
-UDWORD getStructureSizeMax(STRUCTURE *psStructure) // DISPLAY ONLY
+UDWORD getStructureSizeMax(const STRUCTURE *psStructure) // DISPLAY ONLY
 {
 	//radius based on base plate size
 	return MAX(psStructure->pStructureType->baseWidth, psStructure->pStructureType->baseBreadth);
 }
 
-UDWORD getStructureStatSizeMax(STRUCTURE_STATS *Stats) // DISPLAY ONLY
+UDWORD getStructureStatSizeMax(const STRUCTURE_STATS *Stats) // DISPLAY ONLY
 {
 	//radius based on base plate size
 	return MAX(Stats->baseWidth, Stats->baseBreadth);
 }
 
-UDWORD getStructureStatHeight(STRUCTURE_STATS *psStat) // DISPLAY ONLY
+UDWORD getStructureStatHeight(const STRUCTURE_STATS *psStat) // DISPLAY ONLY
 {
 	if (psStat->pIMD[0])
 	{
@@ -166,29 +165,29 @@ UDWORD getStructureStatHeight(STRUCTURE_STATS *psStat) // DISPLAY ONLY
 	return 0;
 }
 
-static void draw_player_3d_shape(uint32_t player_index, iIMDShape *shape, const glm::mat4 &modelMatrix)
+static void draw_player_3d_shape(uint32_t player_index, const iIMDShape *shape, const glm::mat4 &modelMatrix)
 {
-	for (iIMDShape *imd = getFactionDisplayIMD(getPlayerFaction(player_index), shape); imd != nullptr; imd = imd->next.get())
+	int team = getPlayerColour(player_index);
+	for (const iIMDShape *imd = getFactionDisplayIMD(getPlayerFaction(player_index), shape); imd != nullptr; imd = imd->next.get())
 	{
-		int team = getPlayerColour(player_index);
-		const PIELIGHT teamcolour = pal_GetTeamColour(team);
+		const PIELIGHT teamcolour = imd->getTeamColourForModel(team);
 		pie_Draw3DButton(imd, teamcolour, modelMatrix, glm::mat4(1.f));
 	}
 }
 
-void displayIMDButton(iIMDShape *IMDShape, const Vector3i *Rotation, const Vector3i *Position, int scale)
+void displayIMDButton(const iIMDShape *IMDShape, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	draw_player_3d_shape(selectedPlayer, IMDShape, setMatrix(Position, Rotation, scale));
 }
 
-static void sharedStructureButton(STRUCTURE_STATS *Stats, iIMDBaseShape *strBaseImd, const Vector3i *Rotation, const Vector3i *Position, int scale)
+static void sharedStructureButton(const STRUCTURE_STATS *Stats, const iIMDBaseShape *strBaseImd, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	if (!strBaseImd) { return; }
 
-	iIMDBaseShape *baseImd, *mountImd[MAX_WEAPONS], *weaponImd[MAX_WEAPONS];
+	const iIMDBaseShape *baseImd, *mountImd[MAX_WEAPONS], *weaponImd[MAX_WEAPONS];
 	Vector3i pos = *Position;
 
-	iIMDShape *strImd = strBaseImd->displayModel();
+	const iIMDShape *strImd = strBaseImd->displayModel();
 
 	if (!strImd)
 	{
@@ -282,12 +281,12 @@ static void sharedStructureButton(STRUCTURE_STATS *Stats, iIMDBaseShape *strBase
 	}
 }
 
-void displayStructureButton(STRUCTURE *psStructure, const Vector3i *rotation, const Vector3i *Position, int scale)
+void displayStructureButton(const STRUCTURE *psStructure, const Vector3i *rotation, const Vector3i *Position, int scale)
 {
 	sharedStructureButton(psStructure->pStructureType, psStructure->sDisplay.imd, rotation, Position, scale);
 }
 
-void displayStructureStatButton(STRUCTURE_STATS *Stats, const Vector3i *rotation, const Vector3i *Position, int scale)
+void displayStructureStatButton(const STRUCTURE_STATS *Stats, const Vector3i *rotation, const Vector3i *Position, int scale)
 {
 	ASSERT_OR_RETURN(, Stats != nullptr, "Stats is NULL");
 	if (Stats->pIMD.empty() || Stats->pIMD[0] == nullptr)
@@ -299,10 +298,10 @@ void displayStructureStatButton(STRUCTURE_STATS *Stats, const Vector3i *rotation
 
 // Render a component given a BASE_STATS structure.
 //
-void displayComponentButton(BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
+void displayComponentButton(const BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
-	iIMDShape *ComponentIMD = nullptr;
-	iIMDShape *MountIMD = nullptr;
+	const iIMDShape *ComponentIMD = nullptr;
+	const iIMDShape *MountIMD = nullptr;
 	int compID = StatIsComponent(Stat);
 
 	if (compID >= 0)
@@ -317,8 +316,8 @@ void displayComponentButton(BASE_STATS *Stat, const Vector3i *Rotation, const Ve
 
 	/* VTOL bombs are only stats allowed to have NULL ComponentIMD */
 	if (StatIsComponent(Stat) != COMP_WEAPON
-	    || (((WEAPON_STATS *)Stat)->weaponSubClass != WSC_BOMB
-	        && ((WEAPON_STATS *)Stat)->weaponSubClass != WSC_EMP))
+	    || (((const WEAPON_STATS *)Stat)->weaponSubClass != WSC_BOMB
+	        && ((const WEAPON_STATS *)Stat)->weaponSubClass != WSC_EMP))
 	{
 		ASSERT(ComponentIMD, "No ComponentIMD");
 	}
@@ -342,11 +341,11 @@ void displayComponentButton(BASE_STATS *Stat, const Vector3i *Rotation, const Ve
 
 // Render a research item given a BASE_STATS structure.
 //
-void displayResearchButton(BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
+void displayResearchButton(const BASE_STATS *Stat, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	ASSERT_OR_RETURN(, Stat != nullptr, "Stat is NULL");
-	iIMDBaseShape *ResearchIMD = ((RESEARCH *)Stat)->pIMD;
-	iIMDBaseShape *MountIMD = ((RESEARCH *)Stat)->pIMD2;
+	const iIMDBaseShape *ResearchIMD = ((const RESEARCH *)Stat)->pIMD;
+	const iIMDBaseShape *MountIMD = ((const RESEARCH *)Stat)->pIMD2;
 
 	ASSERT(ResearchIMD, "ResearchIMD is NULL");
 	if (ResearchIMD)
@@ -362,21 +361,21 @@ void displayResearchButton(BASE_STATS *Stat, const Vector3i *Rotation, const Vec
 }
 
 
-static inline iIMDBaseShape *getLeftPropulsionIMD(DROID *psDroid)
+static inline const iIMDBaseShape *getLeftPropulsionIMD(const DROID *psDroid)
 {
 	int bodyStat = psDroid->asBits[COMP_BODY];
 	int propStat = psDroid->asBits[COMP_PROPULSION];
 	return asBodyStats[bodyStat].ppIMDList[propStat * NUM_PROP_SIDES + LEFT_PROP];
 }
 
-static inline iIMDBaseShape *getRightPropulsionIMD(DROID *psDroid)
+static inline const iIMDBaseShape *getRightPropulsionIMD(const DROID *psDroid)
 {
 	int bodyStat = psDroid->asBits[COMP_BODY];
 	int propStat = psDroid->asBits[COMP_PROPULSION];
 	return asBodyStats[bodyStat].ppIMDList[propStat * NUM_PROP_SIDES + RIGHT_PROP];
 }
 
-void drawMuzzleFlash(WEAPON sWeap, iIMDShape *weaponImd, iIMDShape *flashImd, PIELIGHT buildingBrightness, int pieFlag, int iPieData, glm::mat4 modelMatrix, const glm::mat4 &viewMatrix, float heightAboveTerrain, UBYTE colour)
+void drawMuzzleFlash(WEAPON sWeap, const iIMDShape *weaponImd, const iIMDShape *flashImd, PIELIGHT buildingBrightness, int pieFlag, int iPieData, glm::mat4 modelMatrix, const glm::mat4 &viewMatrix, float heightAboveTerrain, UBYTE colour)
 {
 	if (!weaponImd || !flashImd || weaponImd->connectors.empty() || graphicsTime < sWeap.lastFired)
 	{
@@ -421,17 +420,25 @@ void drawMuzzleFlash(WEAPON sWeap, iIMDShape *weaponImd, iIMDShape *flashImd, PI
 /* Assumes matrix context is already set */
 // this is able to handle multiple weapon graphics now
 // removed mountRotation,they get such stuff from psObj directly now
-static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelMatrix2, const glm::mat4 &viewMatrix)
+static bool displayCompObj(const DROID *psDroid, bool bButton, const glm::mat4& modelMatrix2, const glm::mat4 &viewMatrix)
 {
 	iIMDBaseShape *psMoveAnim, *psStillAnim;
 	SDWORD				iConnector;
 	PROPULSION_STATS	*psPropStats;
 	SDWORD				pieFlag, iPieData;
+	SDWORD				shieldPieFlag = 0, iShieldPieData = 0;
 	PIELIGHT			brightness;
 	UDWORD				colour;
 	size_t	i = 0;
 	bool				didDrawSomething = false;
 
+	if (!bButton && psDroid->shieldPoints > 0 && droidGetMaxShieldPoints(psDroid) > 0)
+	{
+		float factor = static_cast<float>(psDroid->shieldPoints) / droidGetMaxShieldPoints(psDroid);
+		iShieldPieData = static_cast<SDWORD>(std::round(255.0f * factor));
+		shieldPieFlag = pie_FORCELIGHT | pie_TRANSLUCENT | pie_SHIELD;
+	}
+
 	glm::mat4 modifiedModelMatrix = modelMatrix2;
 
 	if (psDroid->timeLastHit - graphicsTime < ELEC_DAMAGE_DURATION && psDroid->lastHitWeapon == WSC_ELECTRONIC && !gamePaused())
@@ -444,7 +451,7 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 	}
 
 	/* get propulsion stats */
-	psPropStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION];
+	psPropStats = psDroid->getPropulsionStats();
 	ASSERT_OR_RETURN(didDrawSomething, psPropStats != nullptr, "invalid propulsion stats pointer");
 
 	//set pieflag for button object or ingame object
@@ -458,7 +465,7 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 		pieFlag = pie_SHADOW;
 		brightness = pal_SetBrightness(psDroid->illumination);
 		// NOTE: Beware of transporters that are offscreen, on a mission!  We should *not* be checking tiles at this point in time!
-		if (!isTransporter(psDroid) && !missionIsOffworld())
+		if (!psDroid->isTransporter() && !missionIsOffworld())
 		{
 			MAPTILE *psTile = worldTile(psDroid->pos.x, psDroid->pos.y);
 			if (psTile->jammerBits & alliancebits[psDroid->player])
@@ -485,13 +492,20 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 		modifiedModelMatrix *= glm::translate(glm::vec3(0.f, -world_coord(1) / 2.3f, 0.f));
 	}
 
-	iIMDBaseShape *psShapeProp = (leftFirst ? getLeftPropulsionIMD(psDroid) : getRightPropulsionIMD(psDroid));
+	const iIMDBaseShape *psShapeProp = (leftFirst ? getLeftPropulsionIMD(psDroid) : getRightPropulsionIMD(psDroid));
 	if (psShapeProp)
 	{
 		if (pie_Draw3DShape(psShapeProp->displayModel(), 0, colour, brightness, pieFlag, iPieData, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
 		{
 			didDrawSomething = true;
 		}
+		if (!bButton && psDroid->shieldPoints > 0)
+		{
+			if (pie_Draw3DShape(psShapeProp->displayModel(), 0, colour, brightness, shieldPieFlag, iShieldPieData, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
+			{
+				didDrawSomething = true;
+			}
+		}
 	}
 
 	/* set default components transparent */
@@ -507,11 +521,11 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 	}
 
 	/* Get the body graphic now*/
-	iIMDBaseShape *psBaseShapeBody = BODY_IMD(psDroid, psDroid->player);
-	iIMDShape *psShapeBody = (psBaseShapeBody) ? psBaseShapeBody->displayModel() : nullptr;
+	const iIMDBaseShape *psBaseShapeBody = BODY_IMD(psDroid, psDroid->player);
+	const iIMDShape *psShapeBody = (psBaseShapeBody) ? psBaseShapeBody->displayModel() : nullptr;
 	if (psShapeBody)
 	{
-		iIMDShape *strImd = psShapeBody;
+		const iIMDShape *strImd = psShapeBody;
 		if (psDroid->droidType == DROID_PERSON)
 		{
 			// previously, used to scale the model by 0.75 - no longer needed with the fixed models though...
@@ -526,16 +540,24 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 			{
 				didDrawSomething = true;
 			}
+			if (!bButton && psDroid->shieldPoints > 0)
+			{
+				if (drawShape(strImd, psDroid->timeAnimationStarted, colour, brightness, shieldPieFlag, iShieldPieData, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
+				{
+					didDrawSomething = true;
+				}
+			}
 			strImd = strImd->next.get();
 		}
 	}
 
 	/* Render animation effects based on movement or lack thereof, if any */
-	psMoveAnim = asBodyStats[psDroid->asBits[COMP_BODY]].ppMoveIMDList[psDroid->asBits[COMP_PROPULSION]];
-	psStillAnim = asBodyStats[psDroid->asBits[COMP_BODY]].ppStillIMDList[psDroid->asBits[COMP_PROPULSION]];
+	const auto* bodyStats = psDroid->getBodyStats();
+	psMoveAnim = bodyStats->ppMoveIMDList[psDroid->asBits[COMP_PROPULSION]];
+	psStillAnim = bodyStats->ppStillIMDList[psDroid->asBits[COMP_PROPULSION]];
 	if (!bButton && psMoveAnim && psDroid->sMove.Status != MOVEINACTIVE)
 	{
-		iIMDShape *displayModel = psMoveAnim->displayModel();
+		const iIMDShape *displayModel = psMoveAnim->displayModel();
 		if (pie_Draw3DShape(displayModel, getModularScaledGraphicsTime(displayModel->animInterval, displayModel->numFrames), colour, brightness, pie_ADDITIVE, 200, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
 		{
 			didDrawSomething = true;
@@ -543,7 +565,7 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 	}
 	else if (!bButton && psStillAnim) // standing still
 	{
-		iIMDShape *displayModel = psStillAnim->displayModel();
+		const iIMDShape *displayModel = psStillAnim->displayModel();
 		if (pie_Draw3DShape(displayModel, getModularScaledGraphicsTime(displayModel->animInterval, displayModel->numFrames), colour, brightness, 0, 0, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
 		{
 			didDrawSomething = true;
@@ -626,8 +648,8 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 					}
 
 					/* Get the mount graphic */
-					iIMDBaseShape *psBaseShape = WEAPON_MOUNT_IMD(psDroid, i);
-					iIMDShape *psShape = (psBaseShape) ? psBaseShape->displayModel() : nullptr;
+					const iIMDBaseShape *psBaseShape = WEAPON_MOUNT_IMD(psDroid, i);
+					const iIMDShape *psShape = (psBaseShape) ? psBaseShape->displayModel() : nullptr;
 
 					int recoilValue = getRecoil(psDroid->asWeaps[i]);
 					localModelMatrix *= glm::translate(glm::vec3(0.f, 0.f, recoilValue / 3.f));
@@ -639,6 +661,13 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 						{
 							didDrawSomething = true;
 						}
+						if (!bButton && psDroid->shieldPoints > 0)
+						{
+							if (pie_Draw3DShape(psShape, 0, colour, brightness, shieldPieFlag, iShieldPieData, localModelMatrix, viewMatrix, -localHeightAboveTerrain))
+							{
+								didDrawSomething = true;
+							}
+						}
 					}
 					localModelMatrix *= glm::translate(glm::vec3(0, 0, recoilValue));
 
@@ -672,8 +701,15 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 						{
 							didDrawSomething = true;
 						}
+						if (!bButton && psDroid->shieldPoints > 0)
+						{
+							if (pie_Draw3DShape(psShape, 0, colour, brightness, shieldPieFlag, iShieldPieData, localModelMatrix, viewMatrix, -localHeightAboveTerrain))
+							{
+								didDrawSomething = true;
+							}
+						}
 						auto flashBaseModel = MUZZLE_FLASH_PIE(psDroid, i);
-						iIMDShape *pMuzzleFlash = (flashBaseModel) ? flashBaseModel->displayModel() : nullptr;
+						const iIMDShape *pMuzzleFlash = (flashBaseModel) ? flashBaseModel->displayModel() : nullptr;
 						drawMuzzleFlash(psDroid->asWeaps[i], psShape, pMuzzleFlash, brightness, pieFlag, iPieData, localModelMatrix, viewMatrix, localHeightAboveTerrain);
 					}
 				}
@@ -720,8 +756,8 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 					break;
 				}
 
-				iIMDShape *psShape = (psBaseShape) ? psBaseShape->displayModel() : nullptr;
-				iIMDShape *psMountShape = (psBaseMountShape) ? psBaseMountShape->displayModel() : nullptr;
+				const iIMDShape *psShape = (psBaseShape) ? psBaseShape->displayModel() : nullptr;
+				const iIMDShape *psMountShape = (psBaseMountShape) ? psBaseMountShape->displayModel() : nullptr;
 
 				/*	Get the mounting graphic - we've already moved to the right position
 				Allegedly - all droids will have a mount graphic so this shouldn't
@@ -751,7 +787,7 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 				}
 
 				/* translate for construct mount point if cyborg */
-				if (cyborgDroid(psDroid) && psMountShape && !psMountShape->connectors.empty())
+				if (psDroid->isCyborg() && psMountShape && !psMountShape->connectors.empty())
 				{
 					localModelMatrix *= glm::translate(glm::vec3(psMountShape->connectors[0].xzy()));
 					localHeightAboveTerrain += psMountShape->connectors[0].z;
@@ -826,6 +862,13 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 		{
 			didDrawSomething = true;
 		}
+		if (!bButton && psDroid->shieldPoints > 0)
+		{
+			if (pie_Draw3DShape(psShapeProp->displayModel(), 0, colour, brightness, shieldPieFlag, iShieldPieData, modifiedModelMatrix, viewMatrix, -(psDroid->heightAboveMap)))
+			{
+				didDrawSomething = true;
+			}
+		}
 	}
 
 	return didDrawSomething;
@@ -834,7 +877,7 @@ static bool displayCompObj(DROID *psDroid, bool bButton, const glm::mat4& modelM
 
 // Render a composite droid given a DROID_TEMPLATE structure.
 //
-void displayComponentButtonTemplate(DROID_TEMPLATE *psTemplate, const Vector3i *Rotation, const Vector3i *Position, int scale)
+void displayComponentButtonTemplate(const DROID_TEMPLATE *psTemplate, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	const glm::mat4 matrix = setMatrix(Position, Rotation, scale);
 
@@ -855,7 +898,7 @@ void displayComponentButtonTemplate(DROID_TEMPLATE *psTemplate, const Vector3i *
 
 // Render a composite droid given a DROID structure.
 //
-void displayComponentButtonObject(DROID *psDroid, const Vector3i *Rotation, const Vector3i *Position, int scale)
+void displayComponentButtonObject(const DROID *psDroid, const Vector3i *Rotation, const Vector3i *Position, int scale)
 {
 	SDWORD		difference;
 
@@ -885,7 +928,7 @@ void displayComponentObject(DROID *psDroid, const glm::mat4 &viewMatrix, const g
 	position.z = -(st.pos.y);
 	position.y = st.pos.z;
 
-	if (isTransporter(psDroid))
+	if (psDroid->isTransporter())
 	{
 		position.y += bobTransporterHeight();
 	}
@@ -908,7 +951,7 @@ void displayComponentObject(DROID *psDroid, const glm::mat4 &viewMatrix, const g
 	}
 
 	// now check if the projected circle is within the screen boundaries
-	if(!clipDroidOnScreen(psDroid, perspectiveViewMatrix * modelMatrix))
+	if(!clipDroidOnScreen(psDroid, perspectiveViewMatrix * modelMatrix, (getIsCloseDistance()) ? 150 : 25))
 	{
 		return;
 	}
@@ -949,11 +992,11 @@ void displayComponentObject(DROID *psDroid, const glm::mat4 &viewMatrix, const g
 }
 
 
-void destroyFXDroid(DROID *psDroid, unsigned impactTime)
+void destroyFXDroid(DROID *psDroid, unsigned impactTime, Vector3f &velocity)
 {
 	for (int i = 0; i < 5; ++i)
 	{
-		iIMDBaseShape *psImd = nullptr;
+		const iIMDBaseShape *psImd = nullptr;
 
 		int maxHorizontalScatter = TILE_UNITS / 4;
 		int heightScatter = TILE_UNITS / 5;
@@ -1011,12 +1054,12 @@ void destroyFXDroid(DROID *psDroid, unsigned impactTime)
 		}
 		// Tell the effect system that it needs to use this player's color for the next effect
 		SetEffectForPlayer(psDroid->player);
-		addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_DR, true, psImd->displayModel(), getPlayerColour(psDroid->player), impactTime);
+		addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_DR, true, psImd->displayModel(), getPlayerColour(psDroid->player), impactTime, nullptr, &velocity);
 	}
 }
 
 
-void	compPersonToBits(DROID *psDroid)
+void	compPersonToBits(DROID *psDroid, Vector3f &velocity)
 {
 	Vector3i position;	//,rotation,velocity;
 	iIMDBaseShape	*headImd, *legsImd, *armImd, *bodyImd;
@@ -1028,7 +1071,7 @@ void	compPersonToBits(DROID *psDroid)
 		return;
 	}
 	/* get bits pointers according to whether baba or cyborg*/
-	if (cyborgDroid(psDroid))
+	if (psDroid->isCyborg())
 	{
 		// This is probably unused now, since there's a more appropriate effect for cyborgs.
 		headImd = getImdFromIndex(MI_CYBORG_HEAD);
@@ -1052,10 +1095,10 @@ void	compPersonToBits(DROID *psDroid)
 	/* Tell about player colour */
 	col = getPlayerColour(psDroid->player);
 
-	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, headImd->displayModel(), col, gameTime - deltaGameTime + 1);
-	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, legsImd->displayModel(), col, gameTime - deltaGameTime + 1);
-	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, armImd->displayModel(), col, gameTime - deltaGameTime + 1);
-	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, bodyImd->displayModel(), col, gameTime - deltaGameTime + 1);
+	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, headImd->displayModel(), col, gameTime - deltaGameTime + 1, nullptr, &velocity);
+	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, legsImd->displayModel(), col, gameTime - deltaGameTime + 1, nullptr, &velocity);
+	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, armImd->displayModel(), col, gameTime - deltaGameTime + 1, nullptr, &velocity);
+	addEffect(&position, EFFECT_GRAVITON, GRAVITON_TYPE_GIBLET, true, bodyImd->displayModel(), col, gameTime - deltaGameTime + 1, nullptr, &velocity);
 }
 
 
