@@ -27,6 +27,7 @@
 #include "lib/framework/input.h"
 #include "lib/framework/file.h"
 #include "lib/framework/physfs_ext.h"
+#include "lib/netplay/connection_provider_registry.h"
 #include "lib/netplay/netplay.h"
 #include "lib/sound/mixer.h"
 #include "lib/sound/sounddefs.h"
@@ -54,7 +55,6 @@
 #include "display.h"
 #include "keybind.h" // for MAP_ZOOM_RATE_STEP
 #include "loadsave.h" // for autosaveEnabled
-#include "clparse.h" // for autoratingUrl
 #include "terrain.h"
 #include "hci/groups.h"
 
@@ -68,7 +68,7 @@
 #define MASTERSERVERPORT	9990
 #define GAMESERVERPORT		2100
 #define BASECONFVERSION		1
-#define CURRCONFVERSION		2
+#define CURRCONFVERSION		3
 
 static const char *fileName = "config";
 
@@ -87,7 +87,6 @@ static inline std::string WZ_PHYSFS_getRealPath(const char *filename)
 
 static inline optional<uint64_t> WZ_PHYSFS_getFileSize(const char *filename)
 {
-#if defined(WZ_PHYSFS_2_1_OR_GREATER)
 	PHYSFS_Stat metaData = {};
 	if (PHYSFS_stat(filename, &metaData) == 0)
 	{
@@ -99,9 +98,6 @@ static inline optional<uint64_t> WZ_PHYSFS_getFileSize(const char *filename)
 		return nullopt;
 	}
 	return static_cast<uint64_t>(metaData.filesize);
-#else
-	return nullopt;
-#endif
 }
 
 // ////////////////////////////////////////////////////////////////////////////
@@ -310,6 +306,23 @@ bool loadConfig()
 		return iniSectionGetString(iniGeneral, key, defaultValue);
 	};
 
+	auto iniGetMouseKeyCode = [&iniGeneral](const std::string& key, optional<MOUSE_KEY_CODE> defaultValue) -> optional<MOUSE_KEY_CODE> {
+		auto intVal = iniSectionGetInteger(iniGeneral, key);
+		if (!intVal.has_value())
+		{
+			return defaultValue;
+		}
+		if (intVal.value() >= MOUSE_LMB && intVal.value() <= MOUSE_X2) // deliberately exclude mouse MOUSE_WUP + MOUSE_WDN
+		{
+			return static_cast<MOUSE_KEY_CODE>(intVal.value());
+		}
+		else
+		{
+			debug(LOG_WARNING, "Unsupported / invalid MOUSE_KEY_CODE value: %d", intVal.value());
+			return defaultValue;
+		}
+	};
+
 	auto iniGetPlayerLeaveMode = [&iniGeneral](const std::string& key, PLAYER_LEAVE_MODE defaultValue) -> optional<PLAYER_LEAVE_MODE> {
 		auto intVal = iniSectionGetInteger(iniGeneral, key);
 		if (!intVal.has_value())
@@ -329,6 +342,8 @@ bool loadConfig()
 
 	ActivityManager::instance().beginLoadingSettings();
 
+	auto configVersion = iniGetInteger("configVersion", BASECONFVERSION).value();
+
 	if (auto value = iniGetIntegerOpt("voicevol"))
 	{
 		sound_SetUIVolume(static_cast<float>(value.value()) / 100.0f);
@@ -393,7 +408,22 @@ bool loadConfig()
 	war_setSoundEnabled(iniGetBool("sound", true).value());
 	setInvertMouseStatus(iniGetBool("mouseflip", true).value());
 	setRightClickOrders(iniGetBool("RightClickOrders", false).value());
-	setMiddleClickRotate(iniGetBool("MiddleClickRotate", false).value());
+	setPanMouseKey(iniGetMouseKeyCode("mouseKeyPan", getPanMouseKey()));
+	if (!createdConfigFile && configVersion < 3)
+	{
+		// Upgrade old MiddleClickRotate value
+		auto oldMiddleClickRotateValue = iniGetBool("MiddleClickRotate", false).value();
+		if (!setRotateMouseKey((oldMiddleClickRotateValue) ? MOUSE_MMB : MOUSE_RMB) && !oldMiddleClickRotateValue)
+		{
+			// if unable to set (because of conflict, presumably with RightClickOrders), default to MMB
+			setRotateMouseKey(MOUSE_MMB);
+		}
+	}
+	else
+	{
+		setRotateMouseKey(iniGetMouseKeyCode("mouseKeyRotate", getRotateMouseKey()));
+	}
+	setEdgeScrollOutsideWindowBounds(iniGetBool("edgeScrollOutsideWindow", getEdgeScrollOutsideWindowBounds()).value());
 	if (auto value = iniGetIntegerOpt("cursorScale"))
 	{
 		war_setCursorScale(value.value());
@@ -410,8 +440,6 @@ bool loadConfig()
 	radarRotationArrow = iniGetBool("radarRotationArrow", true).value();
 	hostQuitConfirmation = iniGetBool("hostQuitConfirmation", true).value();
 	war_SetPauseOnFocusLoss(iniGetBool("PauseOnFocusLoss", false).value());
-	setAutoratingUrl(iniGetString("autoratingUrlV2", WZ_DEFAULT_PUBLIC_RATING_LOOKUP_SERVICE_URL).value());
-	setAutoratingEnable(iniGetBool("autorating", false).value());
 	NETsetMasterserverName(iniGetString("masterserver_name", "lobby.wz2100.net").value().c_str());
 	mpSetServerName(iniGetString("server_name", "").value());
 //	iV_font(ini.value("fontname", "DejaVu Sans").toString().toUtf8().constData(),
@@ -431,7 +459,7 @@ bool loadConfig()
 	war_setScanlineMode((SCANLINE_MODE)iniGetInteger("scanlines", SCANLINES_OFF).value());
 	seq_SetSubtitles(iniGetBool("subtitles", true).value());
 	setDifficultyLevel((DIFFICULTY_LEVEL)iniGetInteger("difficulty", DL_NORMAL).value());
-	if (!createdConfigFile && iniGetInteger("configVersion", BASECONFVERSION).value() < CURRCONFVERSION)
+	if (!createdConfigFile && configVersion < 2)
 	{
 		int level = (int)getDifficultyLevel() + 1;
 		if (level > static_cast<int>(AIDifficulty::INSANE))
@@ -487,7 +515,22 @@ bool loadConfig()
 			war_setWindowMode(static_cast<WINDOW_MODE>(fullscreenmode_int));
 		}
 	}
-	war_SetTrapCursor(iniGetBool("trapCursor", false).value());
+	if (!createdConfigFile && configVersion < 3)
+	{
+		// Upgrade old bool value to TrapCursorMode
+		// - map true -> Enabled and upgrade false -> Automatic
+		auto oldBoolValue = iniGetBool("trapCursor", false).value();
+		war_SetTrapCursor((oldBoolValue) ? TrapCursorMode::Enabled : TrapCursorMode::Automatic);
+	}
+	else
+	{
+		auto intTrapCursorValue = iniGetInteger("trapCursor", static_cast<int>(TrapCursorMode::Automatic)).value();
+		if (intTrapCursorValue < static_cast<int>(TrapCursorMode::Disabled) || intTrapCursorValue > static_cast<int>(TrapCursorMode::Automatic))
+		{
+			intTrapCursorValue = static_cast<int>(TrapCursorMode::Automatic);
+		}
+		war_SetTrapCursor(static_cast<TrapCursorMode>(intTrapCursorValue));
+	}
 	war_SetColouredCursor(iniGetBool("coloredCursor", true).value());
 	// this should be enabled on all systems by default
 	war_SetVsync(iniGetInteger("vsync", 1).value());
@@ -583,7 +626,9 @@ bool loadConfig()
 		pie_EnableFog(false);
 	}
 	war_setAutoLagKickSeconds(iniGetInteger("hostAutoLagKickSeconds", war_getAutoLagKickSeconds()).value());
+	war_setAutoLagKickAggressiveness(iniGetInteger("hostAutoLagKickAggressiveness", war_getAutoLagKickAggressiveness()).value());
 	war_setAutoDesyncKickSeconds(iniGetInteger("hostAutoDesyncKickSeconds", war_getAutoDesyncKickSeconds()).value());
+	war_setAutoNotReadyKickSeconds(iniGetInteger("hostAutoNotReadyKickSeconds", war_getAutoNotReadyKickSeconds()).value());
 	war_setDisableReplayRecording(iniGetBool("disableReplayRecord", war_getDisableReplayRecording()).value());
 	war_setMaxReplaysSaved(iniGetInteger("maxReplaysSaved", war_getMaxReplaysSaved()).value());
 	war_setOldLogsLimit(iniGetInteger("oldLogsLimit", war_getOldLogsLimit()).value());
@@ -622,6 +667,25 @@ bool loadConfig()
 		war_setPointLightPerPixelLighting(value.value_or(false));
 	}
 
+	std::string defAI = iniGetString("defaultSkirmishAI", DEFAULT_SKIRMISH_AI_SCRIPT_NAME).value();
+	setDefaultSkirmishAI(defAI);
+
+	war_setPlayAudioCue_GroupReporting(iniGetBool("audioCueGroupReporting", war_getPlayAudioCue_GroupReporting()).value());
+
+	auto hostConnProvider = war_getHostConnectionProvider();
+	if (iniGeneral.has("hostConnectionProvider"))
+	{
+		std::string netBackend = iniGetString("hostConnectionProvider", "tcp").value();
+		if (!net_backend_from_str(netBackend.c_str(), hostConnProvider))
+		{
+			hostConnProvider = ConnectionProviderType::TCP_DIRECT; // fall back to using TCP_DIRECT
+			const auto defConnProviderStr = to_string(hostConnProvider);
+			debug(LOG_WARNING, "Unsupported / invalid network backend: %s; defaulting to: %s",
+				netBackend.c_str(), defConnProviderStr.c_str());
+		}
+		war_setHostConnectionProvider(hostConnProvider);
+	}
+
 	ActivityManager::instance().endLoadingSettings();
 	return true;
 }
@@ -661,6 +725,14 @@ bool saveConfig()
 		}
 		iniGeneral.SetString(key, strVal);
 	};
+	auto iniSetMouseKeyOpt = [&iniGeneral](const std::string& key, optional<MOUSE_KEY_CODE> code) {
+		if (!code.has_value())
+		{
+			iniSectionSetInteger(iniGeneral, key, 0);
+			return;
+		}
+		iniSectionSetInteger(iniGeneral, key, static_cast<int>(code.value()));
+	};
 
 	// //////////////////////////
 	// voicevol, fxvol and cdvol
@@ -693,7 +765,9 @@ bool saveConfig()
 	iniSetInteger("nomousewarp", (int)getMouseWarp());		// mouse warp
 	iniSetInteger("coloredCursor", (int)war_GetColouredCursor());
 	iniSetInteger("RightClickOrders", (int)(getRightClickOrders()));
-	iniSetInteger("MiddleClickRotate", (int)(getMiddleClickRotate()));
+	iniSetMouseKeyOpt("mouseKeyPan", getPanMouseKey());
+	iniSetMouseKeyOpt("mouseKeyRotate", getRotateMouseKey());
+	iniSetInteger("edgeScrollOutsideWindow", (int)(getEdgeScrollOutsideWindowBounds()));
 	iniSetInteger("cursorScale", (int)war_getCursorScale());
 	iniSetInteger("textureCompression", (wz_texture_compression) ? 1 : 0);
 	iniSetInteger("showFPS", (int)showFPS);
@@ -705,7 +779,7 @@ bool saveConfig()
 	iniSetInteger("subtitles", (int)(seq_GetSubtitles()));		// subtitles
 	iniSetInteger("radarObjectMode", (int)bEnemyAllyRadarColor);   // enemy/allies radar view
 	iniSetInteger("radarTerrainMode", (int)radarDrawMode);
-	iniSetBool("trapCursor", war_GetTrapCursor());
+	iniSetInteger("trapCursor", (int)war_GetTrapCursor());
 	iniSetInteger("vsync", war_GetVsync());
 	iniSetInteger("displayScale", war_GetDisplayScale());
 	iniSetBool("autoAdjustDisplayScale", war_getAutoAdjustDisplayScale());
@@ -716,8 +790,6 @@ bool saveConfig()
 	iniSetBool("radarRotationArrow", radarRotationArrow);
 	iniSetBool("hostQuitConfirmation", hostQuitConfirmation);
 	iniSetBool("PauseOnFocusLoss", war_GetPauseOnFocusLoss());
-	iniSetString("autoratingUrlV2", getAutoratingUrl());
-	iniSetBool("autorating", getAutoratingEnable());
 	iniSetFromCString("masterserver_name", NETgetMasterserverName(), 255);
 	iniSetInteger("masterserver_port", (int)NETgetMasterserverPort());
 	iniSetString("server_name", mpGetServerName());
@@ -777,6 +849,7 @@ bool saveConfig()
 	iniSetBool("fog", pie_GetFogEnabled());
 	iniSetInteger("hostAutoLagKickSeconds", war_getAutoLagKickSeconds());
 	iniSetInteger("hostAutoDesyncKickSeconds", war_getAutoDesyncKickSeconds());
+	iniSetInteger("hostAutoNotReadyKickSeconds", war_getAutoNotReadyKickSeconds());
 	iniSetBool("disableReplayRecord", war_getDisableReplayRecording());
 	iniSetInteger("maxReplaysSaved", war_getMaxReplaysSaved());
 	iniSetInteger("oldLogsLimit", war_getOldLogsLimit());
@@ -787,6 +860,8 @@ bool saveConfig()
 	iniSetInteger("shadowFilterSize", (int)war_getShadowFilterSize());
 	iniSetInteger("shadowMapResolution", (int)war_getShadowMapResolution());
 	iniSetBool("pointLightsPerpixel", war_getPointLightPerPixelLighting());
+	iniSetString("defaultSkirmishAI", getDefaultSkirmishAI());
+	iniSetBool("audioCueGroupReporting", war_getPlayAudioCue_GroupReporting());
 	iniSetInteger("configVersion", CURRCONFVERSION);
 
 	// write out ini file changes
@@ -912,6 +987,7 @@ bool reloadMPConfig()
 	game.inactivityMinutes = war_getMPInactivityMinutes();
 	game.gameTimeLimitMinutes = war_getMPGameTimeLimitMinutes();
 	game.playerLeaveMode = war_getMPPlayerLeaveMode();
+	game.blindMode = BLIND_MODE::NONE;
 
 	// restore group menus enabled setting (as tutorial may override it)
 	setGroupButtonEnabled(war_getGroupsMenuEnabled());
