@@ -31,7 +31,6 @@
 #include "lib/framework/frameresource.h"
 #include "lib/framework/strres.h"
 #include "lib/framework/crc.h"
-#include "lib/gamelib/parser.h"
 #include "lib/ivis_opengl/bitimage.h"
 #include "lib/ivis_opengl/png_util.h"
 #include "lib/sound/audio.h"
@@ -64,10 +63,9 @@ uint32_t	DataHash[DATA_MAXDATA] = {0};
 *	This is almost the same routine that Pumpkin had, minus the ugly bug :)
 *	And minus the old algorithm and debugging trace, replaced with a simple CRC...
 */
-static UDWORD	hashBuffer(const uint8_t *pData, uint32_t size)
+static void hashBuffer(const uint8_t *pData, uint32_t size, uint32_t &crc)
 {
 	char nl = '\n';
-	uint32_t crc = 0;
 	uint32_t i, j;
 	uint32_t lines = 0;
 	uint32_t bytes = 0;
@@ -78,35 +76,26 @@ static UDWORD	hashBuffer(const uint8_t *pData, uint32_t size)
 
 		if (i != j)  // CRC non-empty lines only.
 		{
-			crc = crcSum(crc, pData + i, j - i);  // CRC the line.
-			crc = crcSum(crc, &nl, 1);            // CRC the line ending.
+			crc = wz::crc_update(crc, pData + i, j - i);  // CRC the line.
+			crc = wz::crc_update(crc, &nl, 1);            // CRC the line ending.
 
 			++lines;
 			bytes += j - i + 1;
 		}
 	}
 	debug(LOG_NET, "The size of the old buffer (%u bytes - %d stripped), New buffer size of %u bytes, %u non-empty lines.", size, size - bytes, bytes, lines);
-
-	return ~crc;
 }
 
 // create the hash for that data block.
 // Data should be converted to Network byte order
 void calcDataHash(const uint8_t *pBuffer, uint32_t size, uint32_t index)
 {
-	const uint32_t oldHash = DataHash[index];
-
 	if (!bMultiPlayer)
 	{
 		return;
 	}
 
-	DataHash[index] += hashBuffer(pBuffer, size);
-
-	if (!DataHash[index] && oldHash)
-	{
-		debug(LOG_NET, "The new hash is 0, the old hash was %u. We added the negated value!", oldHash);
-	}
+	hashBuffer(pBuffer, size, DataHash[index]);
 
 	debug(LOG_NET, "DataHash[%2u] = %08x", index, DataHash[index]);
 
@@ -124,7 +113,7 @@ void resetDataHash()
 	UDWORD i;
 	for (i = 0; i < DATA_MAXDATA; i++)
 	{
-		DataHash[i] = 0;
+		DataHash[i] = wz::crc_init();
 	}
 	debug(LOG_NET, "== Hash is reset ==");
 }
@@ -147,7 +136,7 @@ static bool bufferSBODYLoad(const char *fileName, void **ppData)
 	WzConfig ini(fileName, WzConfig::ReadOnlyAndRequired);
 	calcDataHash(ini, DATA_SBODY);
 
-	if (!loadBodyStats(ini) || !allocComponentList(COMP_BODY, numBodyStats))
+	if (!loadBodyStats(ini) || !allocComponentList(COMP_BODY, asBodyStats.size()))
 	{
 		return false;
 	}
@@ -170,7 +159,7 @@ static bool bufferSWEAPONLoad(const char *fileName, void **ppData)
 	calcDataHash(ini, DATA_SWEAPON);
 
 	if (!loadWeaponStats(ini)
-	    || !allocComponentList(COMP_WEAPON, numWeaponStats))
+	    || !allocComponentList(COMP_WEAPON, asWeaponStats.size()))
 	{
 		return false;
 	}
@@ -187,7 +176,7 @@ static bool bufferSCONSTRLoad(const char *fileName, void **ppData)
 	calcDataHash(ini, DATA_SCONSTR);
 
 	if (!loadConstructStats(ini)
-	    || !allocComponentList(COMP_CONSTRUCT, numConstructStats))
+	    || !allocComponentList(COMP_CONSTRUCT, asConstructStats.size()))
 	{
 		return false;
 	}
@@ -204,7 +193,7 @@ static bool bufferSECMLoad(const char *fileName, void **ppData)
 	calcDataHash(ini, DATA_SECM);
 
 	if (!loadECMStats(ini)
-	    || !allocComponentList(COMP_ECM, numECMStats))
+	    || !allocComponentList(COMP_ECM, asECMStats.size()))
 	{
 		return false;
 	}
@@ -220,7 +209,7 @@ static bool bufferSPROPLoad(const char *fileName, void **ppData)
 	WzConfig ini(fileName, WzConfig::ReadOnlyAndRequired);
 	calcDataHash(ini, DATA_SPROP);
 
-	if (!loadPropulsionStats(ini) || !allocComponentList(COMP_PROPULSION, numPropulsionStats))
+	if (!loadPropulsionStats(ini) || !allocComponentList(COMP_PROPULSION, asPropulsionStats.size()))
 	{
 		return false;
 	}
@@ -236,7 +225,7 @@ static bool bufferSSENSORLoad(const char *fileName, void **ppData)
 	calcDataHash(ini, DATA_SSENSOR);
 
 	if (!loadSensorStats(ini)
-	    || !allocComponentList(COMP_SENSOR, numSensorStats))
+	    || !allocComponentList(COMP_SENSOR, asSensorStats.size()))
 	{
 		return false;
 	}
@@ -252,7 +241,7 @@ static bool bufferSREPAIRLoad(const char *fileName, void **ppData)
 	WzConfig ini(fileName, WzConfig::ReadOnlyAndRequired);
 	calcDataHash(ini, DATA_SREPAIR);
 
-	if (!loadRepairStats(ini) || !allocComponentList(COMP_REPAIRUNIT, numRepairStats))
+	if (!loadRepairStats(ini) || !allocComponentList(COMP_REPAIRUNIT, asRepairStats.size()))
 	{
 		return false;
 	}
@@ -268,7 +257,7 @@ static bool bufferSBRAINLoad(const char *fileName, void **ppData)
 	WzConfig ini(fileName, WzConfig::ReadOnlyAndRequired);
 	calcDataHash(ini, DATA_SBRAIN);
 
-	if (!loadBrainStats(ini) || !allocComponentList(COMP_BRAIN, numBrainStats))
+	if (!loadBrainStats(ini) || !allocComponentList(COMP_BRAIN, asBrainStats.size()))
 	{
 		return false;
 	}
@@ -488,10 +477,36 @@ static bool dataResearchMsgLoad(const char *fileName, void **ppData)
 	return true;
 }
 
+static bool dataProximityMsgLoad(const char *fileName, void **ppData)
+{
+	WzString *ptr = loadProximityViewData(fileName);
+	if (!ptr)
+	{
+		return false;
+	}
+
+	// set the pointer so the release function gets called with it
+	*ppData = (void *)ptr;
+	return true;
+}
+
+static bool dataFlicMsgLoad(const char *fileName, void **ppData)
+{
+	WzString *ptr = loadFlicViewData(fileName);
+	if (!ptr)
+	{
+		return false;
+	}
+
+	// set the pointer so the release function gets called with it
+	*ppData = (void *)ptr;
+	return true;
+}
+
 // release the message viewdata
 static void dataSMSGRelease(void *pData)
 {
-	ASSERT(pData, "pData unexpectedly null");
+	ASSERT_OR_RETURN(, pData, "pData unexpectedly null");
 	WzString *pFilename = static_cast<WzString *>(pData);
 	viewDataShutDown(pFilename->toUtf8().c_str());
 	delete pFilename;
@@ -569,7 +584,7 @@ static void dataImageRelease(void *pData)
 /* Load an audio file */
 static bool dataAudioLoad(const char *fileName, void **ppData)
 {
-	if (audio_Disabled() == true)
+	if (audio_Disabled())
 	{
 		*ppData = nullptr;
 		// No error occurred (sound is just disabled), so we return true
@@ -585,28 +600,21 @@ static bool dataAudioLoad(const char *fileName, void **ppData)
 /* Load an audio file */
 static bool dataAudioCfgLoad(const char *fileName, void **ppData)
 {
-	bool success;
-	PHYSFS_file *fileHandle;
-
-	*ppData = nullptr;
-
+	std::string strName = fileName;
+	ASSERT_OR_RETURN(false, strName.find(".json") != std::string::npos, "Audio effect file must be JSON format!");
 	if (audio_Disabled())
 	{
 		return true;
 	}
-	debug(LOG_WZ, "Reading...[directory: %s] %s", WZ_PHYSFS_getRealDir_String(fileName).c_str(), fileName);
-	fileHandle = PHYSFS_openRead(fileName);
 
-	if (fileHandle == nullptr)
+	WzConfig ini(fileName, WzConfig::ReadOnlyAndRequired);
+	if (!loadAudioEffectFileData(ini))
 	{
 		return false;
 	}
 
-	success = ParseResourceFile(fileHandle);
-
-	PHYSFS_close(fileHandle);
-
-	return success;
+	*ppData = (void *)1;
+	return true;
 }
 
 /* Load a string resource file */
@@ -700,6 +708,8 @@ static const RES_TYPE_MIN_FILE FileResourceTypes[] =
 	{"JAVASCRIPT", jsLoad, nullptr},
 	{"SSTRUCT", bufferSSTRUCTLoad, dataSSTRUCTRelease},            //structure stats and associated files
 	{"RESCH", bufferRESCHLoad, dataRESCHRelease},                  //research stats files
+	{"PROX", dataProximityMsgLoad, dataSMSGRelease },
+	{"FLIC", dataFlicMsgLoad, dataSMSGRelease },
 };
 
 /* Pass all the data loading functions to the framework library */
