@@ -33,6 +33,7 @@ typedef std::function<void (const int)> playerCallbackFunc; // callback function
 #include "lib/ivis_opengl/pieclip.h"
 
 #include "message.h"
+#include "objmem.h"
 
 #include <nonstd/optional.hpp>
 using nonstd::optional;
@@ -202,7 +203,7 @@ enum  				  // Reticule button indecies.
 /* maximum array sizes */
 #define	MAXSTRUCTURES	65535	//bumped up from 80.  NOTE: was used for max # in build menus.
 #define	MAXRESEARCH		65535	//was 80 topic displayed   "           "
-#define	MAXFEATURES		80
+#define	MAXFEATURES		200		//was 80.
 #define	MAXCOMPONENT	200
 #define	MAXEXTRASYS		80
 
@@ -267,6 +268,10 @@ extern iIMDShape	*pNewDesignIMD;
 /* Initialise the in game interface */
 bool intInitialise();
 
+void intInformInterfaceObjectRemoved(const BASE_OBJECT *psObj);
+
+bool intAddRadarWidget();
+
 // Check of coordinate is in the build menu
 bool CoordInBuild(int x, int y);
 
@@ -294,12 +299,15 @@ bool intHelpOverlayIsUp();
 
 /* Add the reticule widgets to the widget screen */
 bool intAddReticule();
+bool intAddInGameOptionsButton();
 bool intShowGroupSelectionMenu();
 bool intAddPower();
 void intRemoveReticule();
+void intHideInGameOptionsButton();
 void setReticuleStats(int ButId, std::string tip = std::string(), std::string filename = std::string(), std::string filenameDown = std::string(), const playerCallbackFunc& callbackFunc = nullptr);
 void setReticulesEnabled(bool enabled);
 void setReticuleFlash(int ButId, bool flash);
+optional<std::string> getReticuleButtonDisplayFilename(int ButId);
 
 /* Set the map view point to the world coordinates x,y */
 void intSetMapPos(UDWORD x, UDWORD y);
@@ -345,7 +353,7 @@ void intRemoveStatsNoAnim();
 void intRemoveObjectNoAnim();
 
 /*sets which list of structures to use for the interface*/
-STRUCTURE *interfaceStructList();
+StructureList *interfaceStructList();
 
 //sets up the Transporter Screen as far as the interface is concerned
 void addTransporterInterface(DROID *psSelected, bool onMission);
@@ -377,7 +385,6 @@ void intRemoveProximityButton(PROXIMITY_DISPLAY *psProxDisp);
 /* Allows us to fool the widgets with a keypress */
 void	setKeyButtonMapping(UDWORD	val);
 
-STRUCTURE *intFindAStructure();
 DROID *intGotoNextDroidType(DROID *CurrDroid, DROID_TYPE droidType, bool AllowGroup);
 
 /// Returns the number of researches that selectedPlayer is not already researching, or 0 if there are no free laboratories.
