@@ -48,6 +48,7 @@
 #include "input/manager.h"
 #include "advvis.h"
 #include "atmos.h"
+#include "campaigninfo.h"
 #include "challenge.h"
 #include "cmddroid.h"
 #include "configuration.h"
@@ -58,6 +59,7 @@
 #include "display3d.h"
 #include "edit3d.h"
 #include "effects.h"
+#include "formation.h"
 #include "fpath.h"
 #include "frend.h"
 #include "frontend.h"
@@ -98,6 +100,12 @@
 #include "seqdisp.h"
 #include "version.h"
 #include "hci/teamstrategy.h"
+#include "screens/guidescreen.h"
+#include "titleui/widgets/gamebrowserform.h"
+#include "wzapi.h"
+
+#include "wzphysfszipioprovider.h"
+#include <wzmaplib/map_package.h>
 
 #include <algorithm>
 #include <unordered_map>
@@ -116,8 +124,7 @@ static std::vector<std::unique_ptr<wzSearchPath>> searchPathRegistry;
 static std::vector<std::string> searchPathMountErrors;
 
 static std::string inMemoryMapVirtualFilenameUID;
-static std::vector<uint8_t> inMemoryMapArchiveData;
-static size_t inMemoryMapArchiveMounted = 0;
+static std::shared_ptr<WzMapZipIO> inMemoryMapArchive;
 
 static optional<std::vector<TerrainShaderQuality>> cachedAvailableTerrainShaderQualityTextures;
 
@@ -137,16 +144,6 @@ typedef std::string MapName;
 typedef std::unordered_map<MapName, WZmapInfo> WZMapInfo_Map;
 WZMapInfo_Map WZ_Maps;
 
-enum MODS_PATHS: size_t
-{
-	MODS_MUSIC,
-	MODS_GLOBAL,
-	MODS_AUTOLOAD,
-	MODS_CAMPAIGN,
-	MODS_MULTIPLAY,
-	MODS_PATHS_MAX
-};
-
 static std::string getFullModPath(MODS_PATHS type)
 {
 	switch (type)
@@ -158,7 +155,8 @@ static std::string getFullModPath(MODS_PATHS type)
 		case MODS_AUTOLOAD:
 			return version_getVersionedModsFolderPath("autoload");
 		case MODS_CAMPAIGN:
-			return version_getVersionedModsFolderPath("campaign");
+			// With the new campaign mod packaging format, we no longer need a versioned path (as the mods contain version compatibility info)
+			return "mods/campaign";
 		case MODS_MULTIPLAY:
 			return version_getVersionedModsFolderPath("multiplay");
 		case MODS_PATHS_MAX:
@@ -177,7 +175,7 @@ static std::array<std::string, MODS_PATHS_MAX> buildFullModsPaths()
 	return result;
 }
 
-static const char* versionedModsPath(MODS_PATHS type)
+const char* versionedModsPath(MODS_PATHS type)
 {
 	static std::array<std::string, MODS_PATHS_MAX> cachedFullModsPaths = buildFullModsPaths();
 	return cachedFullModsPaths[type].c_str();
@@ -190,7 +188,6 @@ static const char* versionedModsPath(MODS_PATHS type)
 static bool InitialiseGlobals()
 {
 	frontendInitVars();	// Initialise frontend globals and statics.
-	statsInitVars();
 	structureInitVars();
 	if (!messageInitVars())
 	{
@@ -398,23 +395,20 @@ static void clearAllPhysFSSearchPaths()
 
 static void clearInMemoryMapFile(void *pData)
 {
-	ASSERT_OR_RETURN(, !inMemoryMapArchiveData.empty(), "Already freed??");
-	ASSERT_OR_RETURN(, inMemoryMapArchiveData.data() == pData, "Unexpected pointer received?");
+	ASSERT_OR_RETURN(, inMemoryMapArchive != nullptr, "Already freed??");
+	ASSERT_OR_RETURN(, inMemoryMapArchive.get() == pData, "Unexpected pointer received?");
 	if (!inMemoryMapVirtualFilenameUID.empty())
 	{
 		levRemoveDataSetByRealFileName(inMemoryMapVirtualFilenameUID.c_str(), nullptr);
 		WZ_Maps.erase(inMemoryMapVirtualFilenameUID);
 	}
 	inMemoryMapVirtualFilenameUID.clear();
-	inMemoryMapArchiveData.clear();
-	if (inMemoryMapArchiveMounted > 0)
-	{
-		--inMemoryMapArchiveMounted;
-	}
+	inMemoryMapArchive.reset();
 }
 
 static bool WZ_PHYSFS_MountSearchPathWrapper(const char *newDir, const char *mountPoint, int appendToPath)
 {
+	ASSERT_OR_RETURN(false, newDir != nullptr, "Null newDir");
 	if (PHYSFS_mount(newDir, mountPoint, appendToPath) != 0)
 	{
 		return true;
@@ -422,14 +416,12 @@ static bool WZ_PHYSFS_MountSearchPathWrapper(const char *newDir, const char *mou
 	else
 	{
 		// mount failed
-#if defined(WZ_PHYSFS_2_1_OR_GREATER)
 		auto errorCode = PHYSFS_getLastErrorCode();
 		if (errorCode != PHYSFS_ERR_NOT_FOUND)
 		{
 			const char* errorStr = PHYSFS_getErrorByCode(errorCode);
-			searchPathMountErrors.push_back(astringf("Failed to mount \"%s\" @ \"%s\": %s", newDir, mountPoint, (errorStr) ? errorStr : "<no details available?>"));
+			searchPathMountErrors.push_back(astringf("Failed to mount \"%s\" @ \"%s\": %s", newDir, (mountPoint) ? mountPoint : "/", (errorStr) ? errorStr : "<no details available?>"));
 		}
-#endif
 		return false;
 	}
 }
@@ -438,8 +430,6 @@ struct RebuildSearchPathCommand
 {
 	searchPathMode mode = mod_clean;
 	bool force = false;
-	optional<std::string> current_map;
-	optional<std::string> current_map_mount_point;
 	TerrainShaderQuality lastTerrainShaderQuality = TerrainShaderQuality::MEDIUM;
 };
 
@@ -447,20 +437,7 @@ static RebuildSearchPathCommand lastCommand;
 
 bool rebuildExistingSearchPathWithGraphicsOptionChange()
 {
-	char* current_map = nullptr;
-	char* current_map_mount_point = nullptr;
-	if (lastCommand.current_map.has_value())
-	{
-		current_map = strdup(lastCommand.current_map.value().c_str());
-	}
-	if (lastCommand.current_map_mount_point.has_value())
-	{
-		current_map_mount_point = strdup(lastCommand.current_map_mount_point.value().c_str());
-	}
-	bool result = rebuildSearchPath(lastCommand.mode, false /* do not force */, current_map, current_map_mount_point);
-	free(current_map);
-	free(current_map_mount_point);
-	return result;
+	return rebuildSearchPath(lastCommand.mode, false /* do not force */);
 }
 
 optional<std::string> getTerrainOverrideBaseSourcePath(TerrainShaderQuality quality)
@@ -582,14 +559,13 @@ std::vector<TerrainShaderQuality> getAvailableTerrainShaderQualityTextures()
  * Priority:
  * maps > mods > base > base.wz
  */
-bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map, const char* current_map_mount_point)
+bool rebuildSearchPath(searchPathMode mode, bool force)
 {
 	std::string tmpstr;
 	static searchPathMode current_mode = mod_clean;
 	auto currentTerrainShaderQuality = getTerrainShaderQuality();
 
 	if (mode != current_mode
-		|| (current_map != nullptr ? current_map : "") != lastCommand.current_map.value_or("")
 		|| force
 		|| lastCommand.lastTerrainShaderQuality != currentTerrainShaderQuality
 		|| (use_override_mods && override_mod_list != getModList()))
@@ -600,18 +576,8 @@ bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map,
 		}
 
 		current_mode = mode;
-		lastCommand.mode = mode; // store this separately, so it doesn't get overridden with mod_override below
+		lastCommand.mode = mode; // store this separately, so it doesn't get overridden by anything below
 		lastCommand.force = force;
-		lastCommand.current_map.reset();
-		if (current_map != nullptr)
-		{
-			lastCommand.current_map = current_map;
-		}
-		lastCommand.current_map_mount_point.reset();
-		if (current_map_mount_point != nullptr)
-		{
-			lastCommand.current_map_mount_point = current_map_mount_point;
-		}
 		lastCommand.lastTerrainShaderQuality = currentTerrainShaderQuality;
 
 		auto terrainQualityOverrideBasePath = getCurrentTerrainOverrideBaseSourcePath();
@@ -723,29 +689,6 @@ bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map,
 				tmpstr = curSearchPath->path + "sequences.wz";
 				WZ_PHYSFS_MountSearchPathWrapper(tmpstr.c_str(), NULL, PHYSFS_APPEND);
 			}
-			// Add the selected map first, for mapmod support
-			if (current_map != nullptr)
-			{
-				if (inMemoryMapVirtualFilenameUID.empty() || current_map != inMemoryMapVirtualFilenameUID)
-				{
-					// mount it as a normal physical map path
-					WzString realPathAndDir = WzString::fromUtf8(PHYSFS_getRealDir(current_map)) + current_map;
-					realPathAndDir.replace("/", PHYSFS_getDirSeparator()); // Windows fix
-					WZ_PHYSFS_MountSearchPathWrapper(realPathAndDir.toUtf8().c_str(), current_map_mount_point, PHYSFS_APPEND);
-				}
-				else if (!inMemoryMapArchiveData.empty())
-				{
-					// mount the in-memory map archive as a virtual file
-					if (PHYSFS_mountMemory_fixed(inMemoryMapArchiveData.data(), inMemoryMapArchiveData.size(), clearInMemoryMapFile, inMemoryMapVirtualFilenameUID.c_str(), current_map_mount_point, PHYSFS_APPEND) != 0)
-					{
-						inMemoryMapArchiveMounted++;
-					}
-				}
-				else
-				{
-					debug(LOG_ERROR, "Specified virtual map file, but no data?");
-				}
-			}
 			for (const auto& curSearchPath : searchPathRegistry)
 			{
 #ifdef DEBUG
@@ -812,8 +755,6 @@ bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map,
 			{
 				debug(LOG_POPUP, _("The required mod could not be loaded: %s\n\nWarzone will try to load the game without it."), override_mod_list.c_str());
 			}
-			clearOverrideMods();
-			current_mode = mod_override;
 		}
 
 		// User's home dir must be first so we always see what we write
@@ -839,11 +780,6 @@ bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map,
 
 		ActivityManager::instance().rebuiltSearchPath();
 	}
-	else if (use_override_mods)
-	{
-		// override mods are already the same as current mods, so no need to do anything
-		clearOverrideMods();
-	}
 	return true;
 }
 
@@ -876,85 +812,7 @@ static MapFileList listMapFiles()
 		return true; // continue
 	});
 
-	// save our current search path(s)
-	debug(LOG_WZ, "Map search paths:");
-	char **searchPath = PHYSFS_getSearchPath();
-	for (char **i = searchPath; *i != nullptr; i++)
-	{
-		debug(LOG_WZ, "    [%s]", *i);
-		oldSearchPath.push_back(*i);
-		WZ_PHYSFS_unmount(*i);
-	}
-	PHYSFS_freeList(searchPath);
-
-	for (const auto &realFileName : ret)
-	{
-		std::string realFilePathAndName = PHYSFS_getWriteDir() + realFileName.platformDependent;
-		if (PHYSFS_mount(realFilePathAndName.c_str(), NULL, PHYSFS_APPEND))
-		{
-			size_t numMapGamFiles = 0;
-			size_t numMapFolders = 0;
-			const std::string baseMapsPath("multiplay/maps/");
-			bool enumSuccess = WZ_PHYSFS_enumerateFiles("multiplay/maps", [&numMapGamFiles, &numMapFolders, &baseMapsPath, &realFilePathAndName](const char *file) -> bool {
-				if (!file)
-				{
-					return true; // continue
-				}
-				std::string isDir = baseMapsPath + file;
-				if (WZ_PHYSFS_isDirectory(isDir.c_str()))
-				{
-					if (baseMapsPath.size() == isDir.size())
-					{
-						// for some reason, file is an empty string - just skip
-						return true; // continue;
-					}
-					if (++numMapFolders > 1)
-					{
-						debug(LOG_ERROR, "Map packs are not supported! %s NOT added.", realFilePathAndName.c_str());
-						return false; // break;
-					}
-					return true; // continue;
-				}
-				std::string checkfile = file;
-				debug(LOG_WZ, "checking ... %s", file);
-				if (checkfile.substr(checkfile.find_last_of('.') + 1) == "gam")
-				{
-					if (++numMapGamFiles > 1)
-					{
-						debug(LOG_ERROR, "Map packs are not supported! %s NOT added.", realFilePathAndName.c_str());
-						return false; // break;
-					}
-				}
-				return true; // continue
-			});
-			if (!enumSuccess)
-			{
-				// Failed to enumerate contents - corrupt map archive (ignore it)
-				debug(LOG_ERROR, "Failed to enumerate - ignoring corrupt map file: %s", realFilePathAndName.c_str());
-				numMapGamFiles = std::numeric_limits<size_t>::max() - 1;
-				numMapFolders = std::numeric_limits<size_t>::max() - 1;
-			}
-			if (numMapGamFiles < 2 && numMapFolders < 2)
-			{
-				filtered.push_back(realFileName);
-			}
-			WZ_PHYSFS_unmount(realFilePathAndName.c_str());
-		}
-		else
-		{
-			debug(LOG_POPUP, "Could not mount %s, because: %s.\nPlease delete or move the file specified.", realFilePathAndName.c_str(), WZ_PHYSFS_getLastError());
-		}
-	}
-
-	// restore our search path(s) again
-	for (const auto &restorePaths : oldSearchPath)
-	{
-		PHYSFS_mount(restorePaths.c_str(), NULL, PHYSFS_APPEND);
-	}
-	debug(LOG_WZ, "Search paths restored");
-	printSearchPath();
-
-	return filtered;
+	return ret;
 }
 
 // Map processing
@@ -1001,166 +859,17 @@ bool CheckForRandom(char const *mapFile, char const *mapDataFile0)
 	return false;
 }
 
-// Mount the archive under the mountpoint, and enumerate the archive according to lookin
-static inline optional<WZmapInfo> CheckInMap(const char *archive, const std::string& mountpoint, const std::vector<std::string>& lookin_list)
+static inline WZmapInfo CheckInMap(WzMap::MapPackage& mapPackage)
 {
-	bool mapmod = false;
-	bool isRandom = false;
-
-	for (auto lookin_subdir : lookin_list)
-	{
-		std::string lookin = mountpoint;
-		if (!lookin_subdir.empty())
-		{
-			lookin.append("/");
-			lookin.append(lookin_subdir);
-		}
-		bool enumResult = WZ_PHYSFS_enumerateFiles(lookin.c_str(), [&](const char *file) -> bool {
-			std::string checkfile = file;
-			if (WZ_PHYSFS_isDirectory((lookin + "/" + checkfile).c_str()))
-			{
-				if (checkfile.compare("wrf") == 0 || checkfile.compare("stats") == 0 || checkfile.compare("components") == 0
-					|| checkfile.compare("effects") == 0 || checkfile.compare("messages") == 0
-					|| checkfile.compare("audio") == 0 || checkfile.compare("sequenceaudio") == 0 || checkfile.compare("misc") == 0
-					|| checkfile.compare("features") == 0 || checkfile.compare("script") == 0 || checkfile.compare("structs") == 0
-					|| checkfile.compare("tileset") == 0 || checkfile.compare("images") == 0 || checkfile.compare("texpages") == 0
-					|| checkfile.compare("skirmish") == 0 || checkfile.compare("shaders") == 0 || checkfile.compare("fonts") == 0
-					|| checkfile.compare("icons") == 0)
-				{
-					debug(LOG_WZ, "Detected: %s %s" , archive, checkfile.c_str());
-					mapmod = true;
-					return false; // break;
-				}
-			}
-			return true; // continue
-		});
-		if (!enumResult)
-		{
-			// failed to enumerate - just exit out
-			return nullopt;
-		}
-
-		std::string maps = lookin + "/multiplay/maps";
-		enumResult = WZ_PHYSFS_enumerateFiles(maps.c_str(), [&](const char *file) -> bool {
-			if (WZ_PHYSFS_isDirectory((maps + "/" + file).c_str()) && PHYSFS_exists((maps + "/" + file + "/game.js").c_str()))
-			{
-				isRandom = true;
-				return false; // break;
-			}
-			return true; // continue
-		});
-		if (!enumResult)
-		{
-			// failed to enumerate - just exit out
-			return nullopt;
-		}
-	}
-
+	bool mapmod = mapPackage.packageType() == WzMap::MapPackage::MapPackageType::Map_Mod;
+	bool isRandom = mapPackage.isScriptGeneratedMap();
 	return WZmapInfo(mapmod, isRandom);
 }
 
-static std::vector<std::string> map_lookin_list = { "", "multiplay" };
-
-// Process a map that has been mounted in the PhysFS virtual filesystem
-//
-// Verifies the index data, determines attributes, and adds to the level loading system so it can be loaded
-//
-// - archive: Directory or archive added to the path, in platform-dependent notation
-// - realFileName_platformIndependent:
-//     For actual map archives, this is the platform independent unique filename + parent path for the map
-//     For "virtual" map archives this is basically a lookup key that should be unique
-// - mountPoint: Location in the interpolated PhysFS tree that this archive was "mounted" (in platform-independent notation)
-bool processMap(const char* archive, const char* realFileName_platformIndependent, const std::string& mountPoint, bool rejectMapMods = false)
-{
-	auto WZmapInfoResult = CheckInMap(archive, mountPoint, map_lookin_list);
-	if (!WZmapInfoResult.has_value())
-	{
-		// failed to enumerate contents
-		return false;
-	}
-	if (rejectMapMods && WZmapInfoResult.value().isMapMod)
-	{
-		debug(LOG_WZ, "Rejecting map mod: %s", archive);
-		return false;
-	}
-
-	WzMapPhysFSIO mapIO;
-
-	bool containsMap = false;
-
-	// First pass: Look for new level.json (which are in multiplay/maps/<map name>)
-	std::string mapsDirPath = mapIO.pathJoin(mountPoint.c_str(), "multiplay/maps");
-	bool enumSuccess = WZ_PHYSFS_enumerateFolders(mapsDirPath, [&](const char *folder) -> bool {
-		if (!folder) { return true; }
-		if (*folder == '\0') { return true; }
-
-		std::string levelJSONPath = std::string("multiplay/maps/") + folder + "/level.json";
-		if (PHYSFS_exists(WzString::fromUtf8(mountPoint + "/" + levelJSONPath)) && loadLevFile_JSON(mountPoint, levelJSONPath, mod_multiplay, realFileName_platformIndependent))
-		{
-			containsMap = true;
-			return false; // stop enumerating
-		}
-		return true;
-	});
-	if (!enumSuccess)
-	{
-		// Failed to enumerate contents - corrupt map archive
-		return false;
-	}
-
-	// Or "flattened" self-contained maps (where level.json is in the root)
-	if (!containsMap)
-	{
-		if (PHYSFS_exists(WzString::fromUtf8(mountPoint + "/" + "level.json")) && loadLevFile_JSON(mountPoint, "level.json", mod_multiplay, realFileName_platformIndependent))
-		{
-			containsMap = true;
-		}
-	}
-
-	if (containsMap)
-	{
-		std::string MapName = realFileName_platformIndependent;
-		WZ_Maps.insert(WZMapInfo_Map::value_type(MapName, WZmapInfoResult.value()));
-		return true;
-	}
-
-	// Second pass: Look for older / classic maps (with *.addon.lev / *.xplayers.lev in the root)
-	enumSuccess = WZ_PHYSFS_enumerateFiles(mountPoint.c_str(), [&](const char *file) -> bool {
-		size_t len = strlen(file);
-		if ((len > 10 && !strcasecmp(file + (len - 10), ".addon.lev"))  // Do not add addon.lev again // <--- Err, what? The code has loaded .addon.lev for a while...
-			|| (len > 13 && !strcasecmp(file + (len - 13), ".xplayers.lev"))) // add support for X player maps using a new name to prevent conflicts.
-		{
-			std::string fullPath = mountPoint + "/" + file;
-			if (loadLevFile(mountPoint + "/" + file, mod_multiplay, true, realFileName_platformIndependent))
-			{
-				containsMap = true;
-				return false; // stop enumerating
-			}
-		}
-		return true; // continue
-	});
-
-	if (!enumSuccess)
-	{
-		// Failed to enumerate contents - corrupt map archive
-		return false;
-	}
-
-	if (containsMap)
-	{
-		std::string MapName = realFileName_platformIndependent;
-		WZ_Maps.insert(WZMapInfo_Map::value_type(MapName, WZmapInfoResult.value()));
-		return true;
-	}
-
-	return false;
-}
-
-#if defined(HAS_PHYSFS_IO_SUPPORT)
 bool setSpecialInMemoryMap(std::vector<uint8_t>&& mapArchiveData)
 {
 	ASSERT_OR_RETURN(false, !mapArchiveData.empty(), "Null map archive data passed?");
-	ASSERT_OR_RETURN(false, inMemoryMapArchiveMounted == 0, "In-memory map archive already mounted");
+	ASSERT_OR_RETURN(false, inMemoryMapArchive == nullptr, "In-memory map archive already set?");
 
 	// calculate a hash for the map data
 	Sha256 mapHash = sha256Sum(mapArchiveData.data(), mapArchiveData.size());
@@ -1168,62 +877,130 @@ bool setSpecialInMemoryMap(std::vector<uint8_t>&& mapArchiveData)
 	// generate a new unique filename UID for the in-memory map
 	inMemoryMapVirtualFilenameUID = "<in-memory>::mapArchive::" + mapHash.toString();
 
-	// store the map archive data
-	inMemoryMapArchiveData = std::move(mapArchiveData);
+	// open the in-memory zip archive
+	std::unique_ptr<std::vector<uint8_t>> pMapArchiveData = std::make_unique<std::vector<uint8_t>>(std::move(mapArchiveData));
+	inMemoryMapArchive = WzMapZipIO::openZipArchiveMemory(std::move(pMapArchiveData), false);
+	if (!inMemoryMapArchive)
+	{
+		debug(LOG_ERROR, "Failed to mount - corrupt / invalid map archive: %s", inMemoryMapVirtualFilenameUID.c_str());
+		inMemoryMapVirtualFilenameUID.clear();
+		return false;
+	}
 
-	// try to mount the in-memory archive
-	if (PHYSFS_mountMemory_fixed(inMemoryMapArchiveData.data(), inMemoryMapArchiveData.size(), nullptr, inMemoryMapVirtualFilenameUID.c_str(), "WZMap", PHYSFS_APPEND) == 0)
+	// open the map package
+	auto mapPackage = WzMap::MapPackage::loadPackage("", nullptr, inMemoryMapArchive);
+	if (!mapPackage)
 	{
-		// Failed to mount data - corrupt map archive
 		debug(LOG_ERROR, "Failed to mount - corrupt / invalid map file: %s", inMemoryMapVirtualFilenameUID.c_str());
 		inMemoryMapVirtualFilenameUID.clear();
-		inMemoryMapArchiveData.clear();
+		inMemoryMapArchive.reset();
 		return false;
 	}
 
 	// load it into the level-loading system
-	if (!processMap(inMemoryMapVirtualFilenameUID.c_str(), inMemoryMapVirtualFilenameUID.c_str(), "WZMap", true))
+	if (!levAddWzMap(mapPackage->levelDetails(), mod_multiplay, inMemoryMapVirtualFilenameUID.c_str()))
 	{
 		// Failed to enumerate contents - corrupt map archive
 		debug(LOG_ERROR, "Failed to enumerate - corrupt / invalid map file: %s", inMemoryMapVirtualFilenameUID.c_str());
 		inMemoryMapVirtualFilenameUID.clear();
-		inMemoryMapArchiveData.clear();
+		inMemoryMapArchive.reset();
 		return false;
 	}
 
-	if (WZ_PHYSFS_unmount(inMemoryMapVirtualFilenameUID.c_str()) == 0)
-	{
-		debug(LOG_ERROR, "Could not unmount %s, %s", inMemoryMapVirtualFilenameUID.c_str(), WZ_PHYSFS_getLastError());
-	}
+	auto WZmapInfoResult = CheckInMap(*mapPackage);
+	WZ_Maps.insert(WZMapInfo_Map::value_type(inMemoryMapVirtualFilenameUID, WZmapInfoResult));
 
 	// fix-up level hash
 	levSetFileHashByRealFileName(inMemoryMapVirtualFilenameUID.c_str(), mapHash);
 
 	return true;
 }
-#else
-bool setSpecialInMemoryMap(std::vector<uint8_t>&& mapArchiveData)
-{
-	// Sadly, the version of PhysFS used for compilation is too old
-	debug(LOG_INFO, "The version of PhysFS used for compilation is too old, and does not support PHYSFS_Io");
-	return false;
-}
-#endif
 
-bool buildMapList()
+std::shared_ptr<WzMapZipIO> getSpecialInMemoryMapArchive(const std::string& mapName)
+{
+	if (inMemoryMapVirtualFilenameUID.empty())
+	{
+		// no special in-memory map file loaded
+		return nullptr;
+	}
+	ASSERT_OR_RETURN(nullptr, inMemoryMapVirtualFilenameUID == mapName, "mapName (%s) does not match currently-loaded in-memory map: %s", mapName.c_str(), inMemoryMapVirtualFilenameUID.c_str());
+	return inMemoryMapArchive;
+}
+
+bool clearSpecialInMemoryMap()
+{
+	if (inMemoryMapVirtualFilenameUID.empty())
+	{
+		return false;
+	}
+
+	if (inMemoryMapArchive == nullptr)
+	{
+		return false;
+	}
+
+	debug(LOG_INFO, "Clearing in-memory map archive");
+	clearInMemoryMapFile(inMemoryMapArchive.get());
+	return true;
+}
+
+class WzMapLoadDebugLogger : public WzMap::LoggingProtocol
+{
+public:
+	virtual ~WzMapLoadDebugLogger() { }
+	virtual void printLog(WzMap::LoggingProtocol::LogLevel level, const char *function, int line, const char *str) override
+	{
+		code_part logPart = LOG_MAP;
+		switch (level)
+		{
+			case WzMap::LoggingProtocol::LogLevel::Info_Verbose:
+				logPart = LOG_NEVER;
+				break;
+			case WzMap::LoggingProtocol::LogLevel::Info:
+				logPart = LOG_NEVER;
+				break;
+			case WzMap::LoggingProtocol::LogLevel::Warning:
+				logPart = LOG_MAP;
+				break;
+			case WzMap::LoggingProtocol::LogLevel::Error:
+				logPart = (m_logErrors) ? LOG_ERROR : LOG_MAP;
+				break;
+		}
+		if (enabled_debug[logPart])
+		{
+			_debug(line, logPart, function, "%s", str);
+		}
+	}
+	void setLogErrors(bool enabled)
+	{
+		m_logErrors = enabled;
+	}
+private:
+	bool m_logErrors = false;
+};
+
+bool buildMapList(bool campaignOnly)
 {
 	if (!loadLevFile("gamedesc.lev", mod_campaign, false, nullptr))
 	{
 		return false;
 	}
-	loadLevFile("addon.lev", mod_multiplay, false, nullptr);
-	WZ_Maps.clear();
-	if (!inMemoryMapArchiveMounted && !inMemoryMapArchiveData.empty())
+	if (!campaignOnly)
 	{
-		debug(LOG_INFO, "Clearing in-memory map archive (since it isn't currently loaded)");
-		clearInMemoryMapFile(inMemoryMapArchiveData.data());
+		loadLevFile("addon.lev", mod_multiplay, false, nullptr);
+	}
+	WZ_Maps.clear();
+	if (inMemoryMapArchive != nullptr)
+	{
+		debug(LOG_INFO, "Clearing in-memory map archive");
+		clearInMemoryMapFile(inMemoryMapArchive.get());
+	}
+	if (campaignOnly)
+	{
+		return true;
 	}
 	MapFileList realFileNames = listMapFiles();
+	auto debugLoggerInstance = std::make_shared<WzMapLoadDebugLogger>();
 	for (auto &realFileName : realFileNames)
 	{
 		const char * pRealDirStr = PHYSFS_getRealDir(realFileName.platformIndependent.c_str());
@@ -1234,22 +1011,36 @@ bool buildMapList()
 		}
 		std::string realFilePathAndName = pRealDirStr + realFileName.platformDependent;
 
-		if (PHYSFS_mount(realFilePathAndName.c_str(), "WZMap", PHYSFS_APPEND) == 0)
+		auto zipReadSource = WzZipIOPHYSFSSourceReadProvider::make(realFileName.platformIndependent);
+		if (!zipReadSource)
 		{
-			debug(LOG_POPUP, "Could not mount %s, because: %s.\nPlease delete or move the file specified.", realFilePathAndName.c_str(), WZ_PHYSFS_getLastError());
-			continue; // skip
+			debug(LOG_ERROR, "Failed to open: %s", realFileName.platformIndependent.c_str());
+			continue;
 		}
 
-		if (!processMap(realFilePathAndName.c_str(), realFileName.platformIndependent.c_str(), "WZMap"))
+		debugLoggerInstance->setLogErrors(true);
+		auto mapZipIO = WzMapZipIO::openZipArchiveReadIOProvider(zipReadSource, debugLoggerInstance.get());
+		if (!mapZipIO)
 		{
-			// Failed to enumerate contents - corrupt map archive
-			debug(LOG_ERROR, "Failed to enumerate - corrupt / invalid map file: %s", realFilePathAndName.c_str());
+			debug(LOG_INFO, "Failed to open archive: %s.\nPlease delete or move the file specified.", realFilePathAndName.c_str());
+			continue;
+		}
+		debugLoggerInstance->setLogErrors(false);
+		auto mapPackage = WzMap::MapPackage::loadPackage("", debugLoggerInstance, mapZipIO);
+		if (!mapPackage)
+		{
+			debug(LOG_INFO, "Failed to load %s.\nPlease delete or move the file specified.", realFilePathAndName.c_str());
+			continue;
 		}
 
-		if (WZ_PHYSFS_unmount(realFilePathAndName.c_str()) == 0)
+		if (!levAddWzMap(mapPackage->levelDetails(), mod_multiplay, realFileName.platformIndependent.c_str()))
 		{
-			debug(LOG_ERROR, "Could not unmount %s, %s", realFilePathAndName.c_str(), WZ_PHYSFS_getLastError());
+			debug(LOG_ERROR, "Corrupt / invalid map file: %s", realFilePathAndName.c_str());
+			continue;
 		}
+
+		auto WZmapInfoResult = CheckInMap(*mapPackage);
+		WZ_Maps.insert(WZMapInfo_Map::value_type(realFileName.platformIndependent, WZmapInfoResult));
 	}
 
 	return true;
@@ -1305,6 +1096,12 @@ bool systemInitialise(unsigned int horizScalePercentage, unsigned int vertScaleP
 	pie_InitRadar();
 
 	readAIs();
+	// Reset config and order if default not found (user messed with config or maybe a mod AI is now missing).
+	const std::string configDefaultAI = defaultSkirmishAI;
+	if (configDefaultAI != getDefaultSkirmishAI())
+	{
+		readAIs();
+	}
 
 	initTerrainShaderType();
 
@@ -1322,6 +1119,8 @@ void systemShutdown()
 		closeLoadSaveOnShutdown(); // TODO: Ideally this would not be required here (refactor loadsave.cpp / frontend.cpp?)
 	}
 
+	NETclose();
+
 	seqReleaseAll();
 
 	pie_ShutdownRadar();
@@ -1349,7 +1148,7 @@ void systemShutdown()
 	debug(LOG_MAIN, "shutting down CD audio");
 	cdAudio_Close();
 
-	if (audio_Disabled() == false && !audio_Shutdown())
+	if (!audio_Disabled() && !audio_Shutdown())
 	{
 		debug(LOG_FATAL, "Unable to audio_Shutdown() cleanly!");
 		abort();
@@ -1366,6 +1165,8 @@ void systemShutdown()
 	pal_ShutDown();		// currently unused stub
 	frameShutDown();	// close screen / SDL / resources / cursors / trig
 	screenShutDown();
+	shutdownLobbyBrowserFetches();
+	netplayShutDown();	// MUST come after widgShutDown (as widget screens might have connections, etc)
 	gfx_api::context::get().shutdown();
 	cleanSearchPath();	// clean PHYSFS search paths
 	debug_exit();		// cleanup debug routines
@@ -1381,6 +1182,8 @@ void systemShutdown()
 
 bool frontendInitialise(const char *ResourceFile)
 {
+	frontendIsShuttingDown();
+
 	debug(LOG_WZ, "== Initializing frontend == : %s", ResourceFile);
 
 	if (!InitialiseGlobals())				// Initialise all globals and statics everywhere.
@@ -1476,9 +1279,11 @@ bool frontendShutdown()
 		closeLoadSaveOnShutdown(); // TODO: Ideally this would not be required here (refactor loadsave.cpp / frontend.cpp?)
 	}
 	interfaceShutDown();
+	shutdownGameGuide();
 
 	//do this before shutting down the iV library
 	resReleaseAllData();
+	FrontImages = nullptr;
 	frontendIsShuttingDown();
 
 	if (!objShutdown())
@@ -1551,6 +1356,11 @@ bool stageOneInitialise()
 		return false;
 	}
 
+	if (!formationInitialise())		// Initialise the formation system
+	{
+		return false;
+	}
+
 	// initialise the visibility stuff
 	if (!visInitialise())
 	{
@@ -1593,11 +1403,13 @@ bool stageOneShutDown()
 
 	pie_FreeShaders();
 
-	if (audio_Disabled() == false)
+	if (!audio_Disabled())
 	{
 		sound_CheckAllUnloaded();
 	}
 
+	shutdownGameGuide();
+
 	proj_Shutdown();
 
 	releaseMission();
@@ -1614,6 +1426,8 @@ bool stageOneShutDown()
 
 	grpShutDown();
 
+	formationShutDown();
+
 	ResearchRelease();
 
 	//free up the gateway stuff?
@@ -1632,8 +1446,28 @@ bool stageOneShutDown()
 	modelShutdown();
 	pie_TexShutDown();
 
+	bool needsLevReload = (hasOverrideMods() || hasCampaignMods()) && (game.type == LEVEL_TYPE::CAMPAIGN);
+	clearOverrideMods();
+	clearCampaignMods();
+	clearCamTweakOptions();
+
 	// Use mod_multiplay as the default (campaign might have set it to mod_singleplayer)
 	rebuildSearchPath(mod_multiplay, true);
+
+	if (needsLevReload)
+	{
+		// Clear & reload level list
+		levShutDown();
+		levInitialise();
+		pal_Init(); // Update palettes.
+		if (!buildMapList(false))
+		{
+			debug(LOG_FATAL, "Failed to rebuild map / level list?");
+		}
+	}
+
+	clearSpecialInMemoryMap();
+
 	pie_TexInit(); // restart it
 
 	initMiscVars();
@@ -1729,7 +1563,7 @@ bool stageTwoInitialise()
 		{
 			NETinitQueue(NETgameQueue(i));
 
-			if (!myResponsibility(i))
+			if (!myResponsibility(i) || !NetPlay.bComms)
 			{
 				NETsetNoSendOverNetwork(NETgameQueue(i));
 			}
@@ -1779,12 +1613,6 @@ bool stageTwoShutDown()
 		return false;
 	}
 
-
-	if (!ShutdownRadar())
-	{
-		return false;
-	}
-
 	interfaceShutDown();
 
 	cmdDroidShutDown();
@@ -1797,9 +1625,61 @@ bool stageTwoShutDown()
 		return false;
 	}
 
+	clearCampaignName();
+
 	return true;
 }
 
+static void displayLoadingErrors()
+{
+	std::vector<std::string> loadingErrors;
+
+	// warn the user if a mod might have corrupted models / files
+	auto modelLoadingErrorCount = getModelLoadingErrorCount();
+	if (modelLoadingErrorCount > 0)
+	{
+		debug(LOG_INFO, "Failure to load %zu game model(s) - please remove any outdated or broken mods", modelLoadingErrorCount);
+		loadingErrors.push_back(astringf(_("Failure to load %zu game model(s)"), modelLoadingErrorCount));
+	}
+	auto modelTextureLoadingFailures = getModelTextureLoadingFailuresCount();
+	if (modelTextureLoadingFailures > 0)
+	{
+		debug(LOG_INFO, "Failed to load textures for %zu models - please remove any outdated or broken mods", modelTextureLoadingFailures);
+		loadingErrors.push_back(astringf(_("Failure to load textures for %zu model(s)"), modelTextureLoadingFailures));
+	}
+	auto statModelLoadingFailures = getStatModelLoadingFailures();
+	if (statModelLoadingFailures > 0)
+	{
+		debug(LOG_INFO, "Failed to load model for %zu stats entries - please remove any outdated or broken mods", statModelLoadingFailures);
+		loadingErrors.push_back(astringf(_("Failure to load model for %zu stat(s)"), statModelLoadingFailures));
+	}
+
+	if (!loadingErrors.empty())
+	{
+		std::string modelErrorDescription = _("Warzone 2100 encountered error(s) loading game data:");
+		for (const auto& error : loadingErrors)
+		{
+			modelErrorDescription += "\n";
+			modelErrorDescription += "- ";
+			modelErrorDescription += error;
+		}
+		modelErrorDescription += "\n\n";
+		if (!getLoadedMods().empty())
+		{
+			modelErrorDescription += _("Please try removing any new mods - they may have issues or be incompatible with this version.");
+			modelErrorDescription += "\n\n";
+			modelErrorDescription += _("Loaded mod(s):");
+			modelErrorDescription += "\n";
+			modelErrorDescription += std::string("- ") + getModList();
+		}
+		else
+		{
+			modelErrorDescription += _("Base game files may be corrupt or outdated - please try reinstalling the game.");
+		}
+		wzDisplayDialog(Dialog_Error, _("Error Loading Game Data"), modelErrorDescription.c_str());
+	}
+}
+
 bool stageThreeInitialise()
 {
 	bool fromSave = (getSaveGameType() == GTYPE_SAVE_START || getSaveGameType() == GTYPE_SAVE_MIDMISSION);
@@ -1812,7 +1692,7 @@ bool stageThreeInitialise()
 	size_t modelTilesetIdx = static_cast<size_t>(currentMapTileset);
 	modelUpdateTilesetIdx(modelTilesetIdx);
 	enumerateLoadedModels([](const std::string &modelName, iIMDBaseShape &s){
-		for (iIMDShape *pDisplayShape = s.displayModel(); pDisplayShape != nullptr; pDisplayShape = pDisplayShape->next.get())
+		for (const iIMDShape *pDisplayShape = s.displayModel(); pDisplayShape != nullptr; pDisplayShape = pDisplayShape->next.get())
 		{
 			pDisplayShape->getTextures();
 		}
@@ -1835,6 +1715,7 @@ bool stageThreeInitialise()
 	effectResetUpdates();
 	initLighting(0, 0, mapWidth, mapHeight);
 	pie_InitLighting();
+	getCurrentLightmapData().reset(mapWidth, mapHeight);
 
 	if (fromSave)
 	{
@@ -1879,6 +1760,8 @@ bool stageThreeInitialise()
 		challengeActive = true;
 	}
 
+	// add radar to interface screen, and resize
+	intAddRadarWidget();
 	resizeRadar();
 
 	setAllPauseStates(false);
@@ -1887,16 +1770,21 @@ bool stageThreeInitialise()
 
 	if (getLevelLoadType() == GTYPE_SAVE_MIDMISSION || getLevelLoadType() == GTYPE_SAVE_START)
 	{
-		triggerEvent(TRIGGER_GAME_LOADED);
+		executeFnAndProcessScriptQueuedRemovals([]() { triggerEvent(TRIGGER_GAME_LOADED); });
 	}
 	else
 	{
+		initNoGoAreas();
+
 		const DebugInputManager& dbgInputManager = gInputManager.debugManager();
 		if (dbgInputManager.debugMappingsAllowed())
 		{
+			Cheated = true;
 			triggerEventCheatMode(true);
+			gInputManager.contexts().set(InputContext::DEBUG_MISC, InputContext::State::ACTIVE);
 		}
-		triggerEvent(TRIGGER_GAME_INIT);
+
+		executeFnAndProcessScriptQueuedRemovals([]() { triggerEvent(TRIGGER_GAME_INIT); });
 		playerBuiltHQ = structureExists(selectedPlayer, REF_HQ, true, false);
 	}
 
@@ -1906,15 +1794,22 @@ bool stageThreeInitialise()
 	// always start off with a refresh of the groups UI data
 	intGroupsChanged();
 
+	// warn the user if a mod might have corrupted models / files
+	displayLoadingErrors();
+
 	if (bMultiPlayer)
 	{
-		playerResponding();			// say howdy!
+		cameraToHome(selectedPlayer, false, fromSave);
+		playerResponding(); // say howdy!
 		if (NetPlay.bComms && !NETisReplay())
 		{
 			multiStartScreenInit();
 		}
 	}
 
+	// Call once again to update counts (if modified by earlier wzapi events)
+	countUpdate(false);
+
 	return true;
 }
 
@@ -1925,7 +1820,7 @@ bool stageThreeShutDown()
 {
 	debug(LOG_WZ, "== stageThreeShutDown ==");
 
-	setHostLaunch(HostLaunch::Normal);
+	resetHostLaunch();
 
 	removeSpotters();
 
@@ -1949,6 +1844,11 @@ bool stageThreeShutDown()
 	// make sure any button tips are gone.
 	widgReset();
 
+	if (!ShutdownRadar())
+	{
+		debug(LOG_FATAL, "Failed to shutdown radar");
+	}
+
 	audio_StopAll();
 
 	if (bMultiPlayer)
