@@ -25,6 +25,7 @@
  */
 #include "lib/framework/frame.h"
 #include "lib/framework/math_ext.h"
+#include "lib/framework/object_list_iteration.h"
 
 /* Includes direct access to render library */
 #include "lib/ivis_opengl/ivisdef.h"
@@ -81,7 +82,7 @@
 //the loop default value
 #define DEFAULT_LOOP		1
 
-static void StatGetResearchImage(BASE_STATS *psStat, AtlasImage *image, iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon);
+static void StatGetResearchImage(const BASE_STATS *psStat, AtlasImage *image, const iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon);
 
 
 static int FormOpenAudioID;	// ID of sfx to play when form opens.
@@ -451,7 +452,7 @@ void IntStatusButton::display(int xOffset, int yOffset)
 			case REF_RESEARCH:
 				if (structureIsResearchingPending(Structure))
 				{
-					iIMDShape *shape;
+					const iIMDShape *shape;
 					Stats = theStats;
 					if (!Stats)
 					{
@@ -599,7 +600,7 @@ void IntStatsButton::display(int xOffset, int yOffset)
 			}
 			else if (StatIsResearch(Stat))
 			{
-				iIMDShape *shape = nullptr;
+				const iIMDShape *shape = nullptr;
 				StatGetResearchImage(Stat, &image, &shape, &psResGraphic, true);
 				if (psResGraphic)
 				{
@@ -989,7 +990,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 	}
 
 	ImdType IMDType = imdObject.type;
-	void *Object = imdObject.ptr;
+	const void *Object = imdObject.ptr;
 	ASSERT_OR_RETURN(, Object != nullptr, "imdObject.ptr is null?");
 	if (IMDType == IMDTYPE_DROID || IMDType == IMDTYPE_DROIDTEMPLATE)
 	{
@@ -1029,11 +1030,11 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 
 		if (IMDType == IMDTYPE_DROID)
 		{
-			Radius = getComponentDroidRadius((DROID *)Object);
+			Radius = getComponentDroidRadius((const DROID *)Object);
 		}
 		else
 		{
-			Radius = getComponentDroidTemplateRadius((DROID_TEMPLATE *)Object);
+			Radius = getComponentDroidTemplateRadius((const DROID_TEMPLATE *)Object);
 		}
 
 		model.scale = DROID_BUT_SCALE;
@@ -1043,9 +1044,9 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 
 		if (IMDType == IMDTYPE_DROID)
 		{
-			if (isTransporter((DROID *)Object))
+			if (((const DROID*)Object)->isTransporter())
 			{
-				if (((DROID *)Object)->droidType == DROID_TRANSPORTER)
+				if (((const DROID *)Object)->droidType == DROID_TRANSPORTER)
 				{
 					model.scale = DROID_BUT_SCALE / 2;
 				}
@@ -1057,9 +1058,9 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else//(IMDType == IMDTYPE_DROIDTEMPLATE)
 		{
-			if (isTransporter((DROID_TEMPLATE *)Object))
+			if (isTransporter((const DROID_TEMPLATE *)Object))
 			{
-				if (((DROID_TEMPLATE *)Object)->droidType == DROID_TRANSPORTER)
+				if (((const DROID_TEMPLATE *)Object)->droidType == DROID_TRANSPORTER)
 				{
 					model.scale = DROID_BUT_SCALE / 2;
 				}
@@ -1073,11 +1074,11 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		//lefthand display droid buttons
 		if (IMDType == IMDTYPE_DROID)
 		{
-			displayComponentButtonObject((DROID *)Object, &model.rotation, &model.position, model.scale);
+			displayComponentButtonObject((const DROID *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else
 		{
-			displayComponentButtonTemplate((DROID_TEMPLATE *)Object, &model.rotation, &model.position, model.scale);
+			displayComponentButtonTemplate((const DROID_TEMPLATE *)Object, &model.rotation, &model.position, model.scale);
 		}
 	}
 	else
@@ -1118,10 +1119,10 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		// Decide which button grid size to use.
 		if (IMDType == IMDTYPE_COMPONENT)
 		{
-			Radius = getComponentRadius((BASE_STATS *)Object);
+			Radius = getComponentRadius((const BASE_STATS *)Object);
 			model.scale = rescaleButtonObject(Radius, COMP_BUT_SCALE, COMPONENT_RADIUS);
 			// NOTE: The Super transport is huge, and is considered a component type, so refit it to inside the button.
-			BASE_STATS *psStats = (BASE_STATS *)Object;
+			const BASE_STATS *psStats = (const BASE_STATS *)Object;
 			if (psStats->id.compare("SuperTransportBody") == 0)
 			{
 				model.scale = static_cast<int>(model.scale * .4f);
@@ -1133,7 +1134,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else if (IMDType == IMDTYPE_RESEARCH)
 		{
-			Radius = getResearchRadius((BASE_STATS *)Object);
+			Radius = getResearchRadius((const BASE_STATS *)Object);
 			if (Radius <= 100)
 			{
 				model.scale = rescaleButtonObject(Radius, COMP_BUT_SCALE, COMPONENT_RADIUS);
@@ -1153,7 +1154,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else if (IMDType == IMDTYPE_STRUCTURE)
 		{
-			basePlateSize = getStructureSizeMax((STRUCTURE *)Object);
+			basePlateSize = getStructureSizeMax((const STRUCTURE *)Object);
 			if (basePlateSize == 1)
 			{
 				model.scale = SMALL_STRUCT_SCALE;
@@ -1169,7 +1170,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else if (IMDType == IMDTYPE_STRUCTURESTAT)
 		{
-			basePlateSize = getStructureStatSizeMax((STRUCTURE_STATS *)Object);
+			basePlateSize = getStructureStatSizeMax((const STRUCTURE_STATS *)Object);
 			if (basePlateSize == 1)
 			{
 				model.scale = SMALL_STRUCT_SCALE;
@@ -1185,7 +1186,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else if (IMDType == IMDTYPE_FEATURE)
 		{
-			int imdRadius = ((iIMDShape *)Object)->radius;
+			int imdRadius = ((const iIMDShape *)Object)->sradius;
 
 			if (imdRadius <= 40)
 			{
@@ -1210,7 +1211,7 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		}
 		else
 		{
-			Radius = ((iIMDShape *)Object)->sradius;
+			Radius = ((const iIMDShape *)Object)->sradius;
 
 			if (Radius <= 128)
 			{
@@ -1236,27 +1237,27 @@ void IntFancyButton::displayIMD(AtlasImage image, ImdObject imdObject, int xOffs
 		/* all non droid buttons */
 		if (IMDType == IMDTYPE_COMPONENT)
 		{
-			displayComponentButton((BASE_STATS *)Object, &model.rotation, &model.position, model.scale);
+			displayComponentButton((const BASE_STATS *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else if (IMDType == IMDTYPE_RESEARCH)
 		{
-			displayResearchButton((BASE_STATS *)Object, &model.rotation, &model.position, model.scale);
+			displayResearchButton((const BASE_STATS *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else if (IMDType == IMDTYPE_STRUCTURE)
 		{
-			displayStructureButton((STRUCTURE *)Object, &model.rotation, &model.position, model.scale);
+			displayStructureButton((const STRUCTURE *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else if (IMDType == IMDTYPE_STRUCTURESTAT)
 		{
-			displayStructureStatButton((STRUCTURE_STATS *)Object, &model.rotation, &model.position, model.scale);
+			displayStructureStatButton((const STRUCTURE_STATS *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else if (IMDType == IMDTYPE_FEATURE)
 		{
-			displayIMDButton((iIMDShape *)Object, &model.rotation, &model.position, model.scale);
+			displayIMDButton((const iIMDShape *)Object, &model.rotation, &model.position, model.scale);
 		}
 		else
 		{
-			displayIMDButton((iIMDShape *)Object, &model.rotation, &model.position, model.scale);
+			displayIMDButton((const iIMDShape *)Object, &model.rotation, &model.position, model.scale);
 		}
 	}
 }
@@ -1372,14 +1373,13 @@ STRUCTURE *DroidGetBuildStructure(DROID *Droid)
 STRUCTURE *droidGetCommandFactory(DROID *psDroid)
 {
 	SDWORD		inc;
-	STRUCTURE	*psCurr;
 
 	for (inc = 0; inc < MAX_FACTORY; inc++)
 	{
 		if (psDroid->secondaryOrder & (1 << (inc + DSS_ASSPROD_SHIFT)))
 		{
 			// found an assigned factory - look for it in the lists
-			for (psCurr = apsStructLists[psDroid->player]; psCurr; psCurr = psCurr->psNext)
+			for (STRUCTURE* psCurr : apsStructLists[psDroid->player])
 			{
 				if ((psCurr->pStructureType->type == REF_FACTORY) &&
 				    (((FACTORY *)psCurr->pFunctionality)->
@@ -1392,7 +1392,7 @@ STRUCTURE *droidGetCommandFactory(DROID *psDroid)
 		if (psDroid->secondaryOrder & (1 << (inc + DSS_ASSPROD_CYBORG_SHIFT)))
 		{
 			// found an assigned factory - look for it in the lists
-			for (psCurr = apsStructLists[psDroid->player]; psCurr; psCurr = psCurr->psNext)
+			for (STRUCTURE* psCurr : apsStructLists[psDroid->player])
 			{
 				if ((psCurr->pStructureType->type == REF_CYBORG_FACTORY) &&
 				    (((FACTORY *)psCurr->pFunctionality)->
@@ -1405,7 +1405,7 @@ STRUCTURE *droidGetCommandFactory(DROID *psDroid)
 		if (psDroid->secondaryOrder & (1 << (inc + DSS_ASSPROD_VTOL_SHIFT)))
 		{
 			// found an assigned factory - look for it in the lists
-			for (psCurr = apsStructLists[psDroid->player]; psCurr; psCurr = psCurr->psNext)
+			for (STRUCTURE* psCurr : apsStructLists[psDroid->player])
 			{
 				if ((psCurr->pStructureType->type == REF_VTOL_FACTORY) &&
 				    (((FACTORY *)psCurr->pFunctionality)->
@@ -1542,7 +1542,7 @@ bool StatIsTemplate(BASE_STATS *Stat)
 	return Stat->hasType(STAT_TEMPLATE);
 }
 
-COMPONENT_TYPE StatIsComponent(BASE_STATS *Stat)
+COMPONENT_TYPE StatIsComponent(const BASE_STATS *Stat)
 {
 	switch (StatType(Stat->ref & STAT_MASK))
 	{
@@ -1558,7 +1558,7 @@ COMPONENT_TYPE StatIsComponent(BASE_STATS *Stat)
 	}
 }
 
-bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, iIMDShape **MountIMD) // DISPLAY ONLY
+bool StatGetComponentIMD(const BASE_STATS *Stat, SDWORD compID, const iIMDShape **CompIMD, const iIMDShape **MountIMD) // DISPLAY ONLY
 {
 	WEAPON_STATS		*psWStat;
 
@@ -1568,42 +1568,42 @@ bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, i
 	switch (compID)
 	{
 	case COMP_BODY:
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_BRAIN:
-		psWStat = ((BRAIN_STATS *)Stat)->psWeaponStat;
+		psWStat = ((const BRAIN_STATS *)Stat)->psWeaponStat;
 		*MountIMD = safeGetDisplayModelFromBase(psWStat->pMountGraphic);
 		*CompIMD = safeGetDisplayModelFromBase(psWStat->pIMD);
 		return true;
 
 	case COMP_WEAPON:
-		*MountIMD = safeGetDisplayModelFromBase(((WEAPON_STATS *)Stat)->pMountGraphic);
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*MountIMD = safeGetDisplayModelFromBase(((const WEAPON_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_SENSOR:
-		*MountIMD = safeGetDisplayModelFromBase(((SENSOR_STATS *)Stat)->pMountGraphic);
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*MountIMD = safeGetDisplayModelFromBase(((const SENSOR_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_ECM:
-		*MountIMD = safeGetDisplayModelFromBase(((ECM_STATS *)Stat)->pMountGraphic);
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*MountIMD = safeGetDisplayModelFromBase(((const ECM_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_CONSTRUCT:
-		*MountIMD = safeGetDisplayModelFromBase(((CONSTRUCT_STATS *)Stat)->pMountGraphic);
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*MountIMD = safeGetDisplayModelFromBase(((const CONSTRUCT_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_PROPULSION:
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_REPAIRUNIT:
-		*MountIMD = safeGetDisplayModelFromBase(((REPAIR_STATS *)Stat)->pMountGraphic);
-		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
+		*MountIMD = safeGetDisplayModelFromBase(((const REPAIR_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((const COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_NUMCOMPONENTS:
@@ -1614,28 +1614,28 @@ bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, i
 }
 
 
-bool StatIsResearch(BASE_STATS *Stat)
+bool StatIsResearch(const BASE_STATS *Stat)
 {
 	return Stat->hasType(STAT_RESEARCH);
 }
 
-static void StatGetResearchImage(BASE_STATS *psStat, AtlasImage *image, iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon) // DISPLAY ONLY
+static void StatGetResearchImage(const BASE_STATS *psStat, AtlasImage *image, const iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon) // DISPLAY ONLY
 {
-	if (drawTechIcon && ((RESEARCH *)psStat)->iconID != NO_RESEARCH_ICON)
+	if (drawTechIcon && ((const RESEARCH *)psStat)->iconID != NO_RESEARCH_ICON)
 	{
-		*image = AtlasImage(IntImages, ((RESEARCH *)psStat)->iconID);
+		*image = AtlasImage(IntImages, ((const RESEARCH *)psStat)->iconID);
 	}
 	//if the research has a Stat associated with it - use this as display in the button
-	if (((RESEARCH *)psStat)->psStat)
+	if (((const RESEARCH *)psStat)->psStat)
 	{
-		*ppGraphicData = ((RESEARCH *)psStat)->psStat;
+		*ppGraphicData = ((const RESEARCH *)psStat)->psStat;
 		//make sure the IMDShape is initialised
 		*Shape = nullptr;
 	}
 	else
 	{
 		//no stat so just just the IMD associated with the research
-		*Shape = safeGetDisplayModelFromBase(((RESEARCH *)psStat)->pIMD);
+		*Shape = safeGetDisplayModelFromBase(((const RESEARCH *)psStat)->pIMD);
 		//make sure the stat is initialised
 		*ppGraphicData = nullptr;
 	}
@@ -1692,10 +1692,8 @@ void IntTransportButton::display(int xOffset, int yOffset)
 /* Draws blips on radar to represent Proximity Display and damaged structures */
 void drawRadarBlips(int radarX, int radarY, float pixSizeH, float pixSizeV, const glm::mat4 &modelViewProjection)
 {
-	PROXIMITY_DISPLAY	*psProxDisp;
 	UWORD			imageID;
 	UDWORD			delay = 150;
-	UDWORD			i;
 	SDWORD width, height;
 	int		x = 0, y = 0;
 	static const uint16_t imagesEnemy[] = {IMAGE_RAD_ENMREAD, IMAGE_RAD_ENM1, IMAGE_RAD_ENM2, IMAGE_RAD_ENM3};
@@ -1708,118 +1706,92 @@ void drawRadarBlips(int radarX, int radarY, float pixSizeH, float pixSizeV, cons
 	width = scrollMaxX - scrollMinX;
 	height = scrollMaxY - scrollMinY;
 
-	// Check if it's time to remove beacons
-	bool removedAMessage = false;
-	for (i = 0; i < MAX_PLAYERS; i++)
-	{
-		/* Go through all the proximity Displays*/
-		for (psProxDisp = apsProxDisp[i]; psProxDisp != nullptr; psProxDisp = psProxDisp->psNext)
-		{
-			if (psProxDisp->psMessage->dataType == MSG_DATA_BEACON)
-			{
-				MESSAGE		*psCurrMsg = psProxDisp->psMessage;
-				VIEWDATA	*pViewData = psCurrMsg->pViewData;
-
-				ASSERT_OR_RETURN(, pViewData != nullptr, "Message without data!");
-
-				if (pViewData->type == VIEW_BEACON)
-				{
-					ASSERT_OR_RETURN(, pViewData->pData != nullptr, "Help message without data!");
-					if (pViewData->pData != nullptr && (((VIEW_PROXIMITY *)pViewData->pData)->timeAdded + 60000) <= gameTime)
-					{
-						debug(LOG_MSG, "blip timeout for %d, from %d", i, (((VIEW_PROXIMITY *)pViewData->pData)->sender));
-						removeMessage(psCurrMsg, i);	//remove beacon
-						removedAMessage = true;
-						break;	//there can only be 1 beacon per player
-					}
-				}
-			}
-		}
-	}
-	if (removedAMessage)
-	{
-		jsDebugMessageUpdate();
-	}
-
 	/* Go through all the proximity Displays */
-	for (psProxDisp = (selectedPlayer < MAX_PLAYERS) ? apsProxDisp[selectedPlayer] : nullptr; psProxDisp != nullptr; psProxDisp = psProxDisp->psNext)
+	if (selectedPlayer < MAX_PLAYERS)
 	{
-		unsigned        animationLength = ARRAY_SIZE(imagesEnemy) - 1;  // Same size as imagesResource and imagesArtifact.
-		const uint16_t *images;
-
-		if (psProxDisp->psMessage->player != selectedPlayer)
+		for (PROXIMITY_DISPLAY* psProxDisp : apsProxDisp[selectedPlayer])
 		{
-			continue;
-		}
+			unsigned        animationLength = ARRAY_SIZE(imagesEnemy) - 1;  // Same size as imagesResource and imagesArtifact.
+			const uint16_t* images;
 
-		if (psProxDisp->type == POS_PROXDATA)
-		{
-			PROX_TYPE proxType = ((VIEW_PROXIMITY *)psProxDisp->psMessage->pViewData->pData)->proxType;
-			images = imagesProxTypes[proxType];
-		}
-		else
-		{
-			const FEATURE *psFeature = castFeature(psProxDisp->psMessage->psObj);
-
-			ASSERT_OR_RETURN(, psFeature && psFeature->psStats, "Bad feature message");
-			if (psFeature && psFeature->psStats && psFeature->psStats->subType == FEAT_OIL_RESOURCE)
+			if (psProxDisp->psMessage->player != selectedPlayer)
 			{
-				images = imagesResource;
-				if (fireOnLocation(psFeature->pos.x, psFeature->pos.y))
+				continue;
+			}
+
+			if (psProxDisp->type == POS_PROXDATA)
+			{
+				if (psProxDisp->psMessage->pViewData == nullptr)
 				{
-					images = imagesBurningResource;
-					animationLength = ARRAY_SIZE(imagesBurningResource) - 1;  // Longer animation for burning oil wells.
+					continue; // if no data - ignore message
 				}
+				PROX_TYPE proxType = ((VIEW_PROXIMITY*)psProxDisp->psMessage->pViewData->pData)->proxType;
+				images = imagesProxTypes[proxType];
 			}
 			else
 			{
-				images = imagesArtifact;
-			}
-		}
+				const FEATURE* psFeature = castFeature(psProxDisp->psMessage->psObj);
 
-		// Draw the 'blips' on the radar - use same timings as radar blips if the message is read - don't animate
-		if (psProxDisp->psMessage->read)
-		{
-			imageID = images[0];
-		}
-		else
-		{
-			// Draw animated
-			if (realTime - psProxDisp->timeLastDrawn > delay)
+				ASSERT_OR_RETURN(, psFeature && psFeature->psStats, "Bad feature message");
+				if (psFeature && psFeature->psStats && psFeature->psStats->subType == FEAT_OIL_RESOURCE)
+				{
+					images = imagesResource;
+					if (fireOnLocation(psFeature->pos.x, psFeature->pos.y))
+					{
+						images = imagesBurningResource;
+						animationLength = ARRAY_SIZE(imagesBurningResource) - 1;  // Longer animation for burning oil wells.
+					}
+				}
+				else
+				{
+					images = imagesArtifact;
+				}
+			}
+
+			// Draw the 'blips' on the radar - use same timings as radar blips if the message is read - don't animate
+			if (psProxDisp->psMessage->read)
 			{
-				++psProxDisp->strobe;
-				psProxDisp->timeLastDrawn = realTime;
+				imageID = images[0];
+			}
+			else
+			{
+				// Draw animated
+				if (realTime - psProxDisp->timeLastDrawn > delay)
+				{
+					++psProxDisp->strobe;
+					psProxDisp->timeLastDrawn = realTime;
+				}
+				psProxDisp->strobe %= animationLength;
+				imageID = images[1 + psProxDisp->strobe];
 			}
-			psProxDisp->strobe %= animationLength;
-			imageID = images[1 + psProxDisp->strobe];
-		}
 
-		if (psProxDisp->type == POS_PROXDATA)
-		{
-			const VIEW_PROXIMITY *psViewProx = (VIEW_PROXIMITY *)psProxDisp->psMessage->pViewData->pData;
+			if (psProxDisp->type == POS_PROXDATA)
+			{
+				const VIEW_PROXIMITY* psViewProx = (VIEW_PROXIMITY*)psProxDisp->psMessage->pViewData->pData;
 
-			x = static_cast<int>((psViewProx->x / TILE_UNITS - scrollMinX) * pixSizeH);
-			y = static_cast<int>((psViewProx->y / TILE_UNITS - scrollMinY) * pixSizeV);
-		}
-		else if (psProxDisp->type == POS_PROXOBJ)
-		{
-			x = static_cast<int>((psProxDisp->psMessage->psObj->pos.x / TILE_UNITS - scrollMinX) * pixSizeH);
-			y = static_cast<int>((psProxDisp->psMessage->psObj->pos.y / TILE_UNITS - scrollMinY) * pixSizeV);
-		}
-		else
-		{
-			ASSERT(false, "Bad message type");
-			continue;
-		}
+				x = static_cast<int>((psViewProx->x / TILE_UNITS - scrollMinX) * pixSizeH);
+				y = static_cast<int>((psViewProx->y / TILE_UNITS - scrollMinY) * pixSizeV);
+			}
+			else if (psProxDisp->type == POS_PROXOBJ)
+			{
+				x = static_cast<int>((psProxDisp->psMessage->psObj->pos.x / TILE_UNITS - scrollMinX) * pixSizeH);
+				y = static_cast<int>((psProxDisp->psMessage->psObj->pos.y / TILE_UNITS - scrollMinY) * pixSizeV);
+			}
+			else
+			{
+				ASSERT(false, "Bad message type");
+				continue;
+			}
 
-		// NOTE:  On certain missions (limbo & expand), there is still valid data that is stored outside the
-		// normal radar/mini-map view.  We must now calculate the radar/mini-map's bounding box, and clip
-		// everything outside the box.
-		if ((x + radarX) < width * pixSizeV / 2 && (x + radarX) > -width * pixSizeV / 2
-		    && (y + radarY) < height * pixSizeH / 2 && (y + radarY) > -height * pixSizeH / 2)
-		{
-			// Draw the 'blip'
-			iV_DrawImage(IntImages, imageID, x + radarX, y + radarY, modelViewProjection);
+			// NOTE:  On certain missions (limbo & expand), there is still valid data that is stored outside the
+			// normal radar/mini-map view.  We must now calculate the radar/mini-map's bounding box, and clip
+			// everything outside the box.
+			if ((x + radarX) < width * pixSizeV / 2 && (x + radarX) > -width * pixSizeV / 2
+				&& (y + radarY) < height * pixSizeH / 2 && (y + radarY) > -height * pixSizeH / 2)
+			{
+				// Draw the 'blip'
+				iV_DrawImage(IntImages, imageID, x + radarX, y + radarY, modelViewProjection);
+			}
 		}
 	}
 	if (audio_GetPreviousQueueTrackRadarBlipPos(&x, &y))
