@@ -22,7 +22,7 @@
 #include "lib/framework/crc.h"
 #include "lib/gamelib/gtime.h"
 #include "lib/netplay/netplay.h"
-#include "lib/netplay/netsocket.h"
+#include "lib/netplay/tcp/netsocket.h"
 #include "../activity.h"
 #include "../frontend.h"
 #include "../multiint.h"
@@ -96,7 +96,7 @@ static void asyncGetDiscordDefaultUserAvatar(const std::string& discord_user_dis
 		std::vector<unsigned char> memoryBuffer((unsigned char *)data->memory, ((unsigned char*)data->memory) + data->size);
 		callback(memoryBuffer);
 	};
-	urlRequest.onFailure = [callback](const std::string& url, URLRequestFailureType type, optional<HTTPResponseDetails> transferDetails) {
+	urlRequest.onFailure = [callback](const std::string& url, URLRequestFailureType type, std::shared_ptr<HTTPResponseDetails> transferDetails) {
 		callback(nullopt);
 	};
 	urlRequest.maxDownloadSizeLimit = 4 * 1024 * 1024; // response should never be > 4 MB
@@ -149,7 +149,7 @@ static void asyncGetDiscordUserAvatar(const DiscordUser* request, const std::fun
 		std::vector<unsigned char> memoryBuffer((unsigned char *)data->memory, ((unsigned char*)data->memory) + data->size);
 		callback(memoryBuffer);
 	};
-	urlRequest.onFailure = [callback, discord_user_discriminator](const std::string& url, URLRequestFailureType type, optional<HTTPResponseDetails> transferDetails) {
+	urlRequest.onFailure = [callback, discord_user_discriminator](const std::string& url, URLRequestFailureType type, std::shared_ptr<HTTPResponseDetails> transferDetails) {
 		// fallback
 		asyncGetDiscordDefaultUserAvatar(discord_user_discriminator, callback);
 	};
@@ -206,14 +206,10 @@ static void joinGameImpl(const std::vector<JoinConnectionDescription>& joinConne
 	}
 	// join the game!
 	NetPlay.bComms = true; // use network = true
-	NETinit(true);
 	// Ensure the joinGame has a place to return to
 	changeTitleMode(TITLE);
-	JoinGameResult result = joinGame(joinConnectionDetails);
-	if (result != JoinGameResult::JOINED)
-	{
-		cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
-	}
+	joinGame(joinConnectionDetails);
+	cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
 }
 
 static void findAndJoinLobbyGameImpl(const std::string& lobbyAddress, unsigned int lobbyPort, uint32_t lobbyGameId)
@@ -225,45 +221,46 @@ static void findAndJoinLobbyGameImpl(const std::string& lobbyAddress, unsigned i
 	contentStr += "\n\n";
 	contentStr += _("This may take a moment...");
 	notification.contentText = contentStr;
-	notification.onDisplay = [lobbyAddress, lobbyPort, lobbyGameId](const WZ_Notification&) {
+	std::string lobbyAddressCopy = lobbyAddress;
+	notification.onDisplay = [lobbyAddressCopy, lobbyPort, lobbyGameId](const WZ_Notification&) {
 		// once the notification is completely displayed, trigger the lookup & join
+		wzAsyncExecOnMainThread([lobbyAddressCopy, lobbyPort, lobbyGameId]() {
+			const auto currentGameMode = ActivityManager::instance().getCurrentGameMode();
+			if (currentGameMode != ActivitySink::GameMode::MENUS)
+			{
+				// Can't join a game while in a game - abort
+				cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
+				displayCantJoinWhileInGameNotification();
+				return;
+			}
 
-		const auto currentGameMode = ActivityManager::instance().getCurrentGameMode();
-		if (currentGameMode != ActivitySink::GameMode::MENUS)
-		{
-			// Can't join a game while in a game - abort
-			cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
-			displayCantJoinWhileInGameNotification();
-			return;
-		}
+			// `findLobbyGame()` will automatically call `NETinit()`, if needed.
+			// Obviously this wouldn't be a good idea to do *during* a game, hence the check above to
+			// ensure we're still in the menus...
+			ingame.side = InGameSide::MULTIPLAYER_CLIENT;
+			auto joinConnectionDetails = findLobbyGame(lobbyAddressCopy, lobbyPort, lobbyGameId);
 
-		// For now, it is necessary to call `NETinit(true)` before calling findLobbyGame
-		// Obviously this wouldn't be a good idea to do *during* a game, hence the check above to
-		// ensure we're still in the menus...
-		NETinit(true);
-		ingame.side = InGameSide::MULTIPLAYER_CLIENT;
-		auto joinConnectionDetails = findLobbyGame(lobbyAddress, lobbyPort, lobbyGameId);
+			if (joinConnectionDetails.empty())
+			{
+				cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
+				debug(LOG_ERROR, "Join code: Failed to find game in the lobby server: %s:%u", lobbyAddressCopy.c_str(), lobbyPort);
+				std::string contentText = _("Failed to find game in the lobby server: ");
+				contentText += lobbyAddressCopy + ":" + std::to_string(lobbyPort) + "\n\n";
+				contentText += _("The game may have already started, or the host may have disbanded the game lobby.");
+				WZ_Notification notification;
+				notification.duration = 0;
+				notification.contentTitle = _("Failed to Find Game");
+				notification.contentText = contentText;
+				notification.largeIcon = WZ_Notification_Image("images/notifications/exclamation_triangle.png");
+				notification.tag = std::string(JOIN_NOTIFICATION_TAG_PREFIX "failedtoconnect");
+				addNotification(notification, WZ_Notification_Trigger::Immediate());
+				return;
+			}
 
-		if (joinConnectionDetails.empty())
-		{
-			cancelOrDismissNotificationsWithTag(JOIN_FIND_AND_CONNECT_TAG);
-			debug(LOG_ERROR, "Join code: Failed to find game in the lobby server: %s:%u", lobbyAddress.c_str(), lobbyPort);
-			std::string contentText = _("Failed to find game in the lobby server: ");
-			contentText += lobbyAddress + ":" + std::to_string(lobbyPort) + "\n\n";
-			contentText += _("The game may have already started, or the host may have disbanded the game lobby.");
-			WZ_Notification notification;
-			notification.duration = 0;
-			notification.contentTitle = _("Failed to Find Game");
-			notification.contentText = contentText;
-			notification.largeIcon = WZ_Notification_Image("images/notifications/exclamation_triangle.png");
-			notification.tag = std::string(JOIN_NOTIFICATION_TAG_PREFIX "failedtoconnect");
-			addNotification(notification, WZ_Notification_Trigger::Immediate());
-			return;
-		}
+			ActivityManager::instance().willAttemptToJoinLobbyGame(lobbyAddressCopy, lobbyPort, lobbyGameId, joinConnectionDetails);
 
-		ActivityManager::instance().willAttemptToJoinLobbyGame(lobbyAddress, lobbyPort, lobbyGameId, joinConnectionDetails);
-
-		joinGameImpl(joinConnectionDetails);
+			joinGameImpl(joinConnectionDetails);
+		});
 	};
 	notification.largeIcon = WZ_Notification_Image("images/notifications/connect_wait.png");
 	notification.tag = JOIN_FIND_AND_CONNECT_TAG;
@@ -396,10 +393,10 @@ static void joinGameFromSecret_v1(const std::string joinSecretStr)
 				return;
 			}
 			std::vector<unsigned char> ipNetworkBinaryFormat = EmbeddedJSONSignature::b64Decode(b64UrlSafeTob64(connectionDetails[0].toUtf8()));
-			std::string ipAddressStr = ipv6_NetBinary_To_AddressString(ipNetworkBinaryFormat);
+			std::string ipAddressStr = tcp::ipv6_NetBinary_To_AddressString(ipNetworkBinaryFormat);
 			if (ipAddressStr.empty())
 			{
-				ipAddressStr = ipv4_NetBinary_To_AddressString(ipNetworkBinaryFormat);
+				ipAddressStr = tcp::ipv4_NetBinary_To_AddressString(ipNetworkBinaryFormat);
 			}
 			if (ipAddressStr.empty())
 			{
@@ -742,21 +739,41 @@ void DiscordRPCActivitySink::setJoinInformation(const ActivitySink::MultiplayerG
 				{
 					return;
 				}
+
+				const std::string* pExternalIPv4Address = nullptr;
+				unsigned int externalIPv4Port = 0;
+
+				// Prefer external IP information acquired from port-mapping, if available
+				for (const auto& extAddress : listeningInterfaces.knownExternalAddresses)
+				{
+					if (extAddress.type == ListeningInterfaces::IPType::IPv4 && !pExternalIPv4Address)
+					{
+						pExternalIPv4Address = &extAddress.ipAddress;
+						externalIPv4Port = extAddress.port;
+					}
+				}
+
+				if (pExternalIPv4Address == nullptr)
+				{
+					pExternalIPv4Address = &ipv4Address;
+					externalIPv4Port = listeningInterfaces.ipv4_port;
+				}
+
 				// Convert ip address strings to binary, in network-byte-order format, then base64-encode
 				// Append the port as a separate string component
 				std::string joinSecretDetails;
-				if (!ipv4Address.empty())
+				if (!pExternalIPv4Address->empty())
 				{
-					auto ipv4AddressBinaryForm = ipv4_AddressString_To_NetBinary(ipv4Address);
+					auto ipv4AddressBinaryForm = tcp::ipv4_AddressString_To_NetBinary(*pExternalIPv4Address);
 					if (!ipv4AddressBinaryForm.empty())
 					{
 						joinSecretDetails += b64Tob64UrlSafe(EmbeddedJSONSignature::b64Encode(ipv4AddressBinaryForm));
-						joinSecretDetails += std::string(":") + std::to_string(listeningInterfaces.ipv4_port);
+						joinSecretDetails += std::string(":") + std::to_string(externalIPv4Port);
 					}
 				}
 				if (!ipv6Address.empty())
 				{
-					auto ipv6AddressBinaryForm = ipv6_AddressString_To_NetBinary(ipv6Address);
+					auto ipv6AddressBinaryForm = tcp::ipv6_AddressString_To_NetBinary(ipv6Address);
 					if (!ipv6AddressBinaryForm.empty())
 					{
 						if (!joinSecretDetails.empty())
@@ -783,7 +800,7 @@ void DiscordRPCActivitySink::setJoinInformation(const ActivitySink::MultiplayerG
 				std::string joinSecretStr = std::string("v1/i/") + uniqueGameStr + "/" + joinSecretDetails;
 
 				// construct a unique party id from the uniqueGameStr + ip addresses, hashed
-				std::string rawPartyId = std::string("direct_connection:") + uniqueGameStr + ":" + ipv4Address + ":" + std::to_string(listeningInterfaces.ipv4_port) + ":" + ipv6Address + ":" + std::to_string(listeningInterfaces.ipv6_port);
+				std::string rawPartyId = std::string("direct_connection:") + uniqueGameStr + ":" + *pExternalIPv4Address + ":" + std::to_string(externalIPv4Port) + ":" + ipv6Address + ":" + std::to_string(listeningInterfaces.ipv6_port);
 				std::string partyIdStr = hashAndB64EncodePartyId(rawPartyId);
 
 				wzAsyncExecOnMainThread([pSink, joinSecretStr, partyIdStr](){
