@@ -27,6 +27,7 @@
 
 #include "lib/framework/frame.h"
 #include "lib/framework/strres.h"
+#include "lib/framework/object_list_iteration.h"
 #include "lib/widget/widget.h"
 #include "lib/widget/gridlayout.h"
 #include "lib/widget/button.h"
@@ -136,7 +137,7 @@ static bool				playCurrent;
 
 /* functions declarations ****************/
 static const char* getMessageTitle(const MESSAGE& message);
-static void StartMessageSequences(MESSAGE *psMessage, bool Start);
+static bool StartMessageSequences(MESSAGE *psMessage, bool Start);
 /*Displays the buttons used on the intelligence map */
 class IntMessageButton : public IntFancyButton
 {
@@ -227,6 +228,14 @@ void WzMessageView::geometryChanged()
 
 bool WzMessageView::initialize(MESSAGE *psMessage)
 {
+	auto title = std::make_shared<W_LABEL>();
+	title->setGeometry(0, 0, INTMAP_RESEARCHWIDTH, INTMAP_TITLEHEIGHT);
+	title->setFontColour(WZCOL_YELLOW);
+	title->setTextAlignment(WLAB_ALIGNCENTRE);
+	title->setFont(font_regular, WZCOL_YELLOW);
+	title->setString(getMessageTitle(*psMessage));
+	attach(title);
+
 	/* Add the close box */
 	W_BUTINIT sButInit;
 	sButInit.id = IDINTMAP_CLOSE;
@@ -258,14 +267,6 @@ bool WzMessageView::initialize(MESSAGE *psMessage)
 		});
 	});
 
-	auto title = std::make_shared<W_LABEL>();
-	title->setGeometry(0, 0, INTMAP_RESEARCHWIDTH, INTMAP_TITLEHEIGHT);
-	title->setFontColour(WZCOL_YELLOW);
-	title->setTextAlignment(WLAB_ALIGNCENTRE);
-	title->setFont(font_regular, WZCOL_YELLOW);
-	title->setString(getMessageTitle(*psMessage));
-	attach(title);
-
 	auto grid = std::make_shared<GridLayout>();
 	attach(grid);
 	grid->setGeometry(1, INTMAP_TITLEHEIGHT, INTMAP_RESEARCHWIDTH - 2, INTMAP_RESEARCHHEIGHT - 1 - INTMAP_TITLEHEIGHT);
@@ -274,6 +275,7 @@ bool WzMessageView::initialize(MESSAGE *psMessage)
 	messages->setItemSpacing(3);
 	messages->setBackgroundColor(WZCOL_TRANSPARENT_BOX);
 	messages->setPadding({10, 10, 10, 10});
+	messages->setGeometry(0, 0, grid->width(), 0); // set initial width so that grid calculations can properly calculate required height
 
 	if (psMessage->type != MSG_RESEARCH && psMessage->pViewData->type == VIEW_RPL)
 	{
@@ -284,7 +286,7 @@ bool WzMessageView::initialize(MESSAGE *psMessage)
 			{
 				auto message = std::make_shared<Paragraph>();
 				message->setFontColour(WZCOL_TEXT_BRIGHT);
-				message->addText(msg.toUtf8());
+				message->addText(msg);
 				messages->addItem(message);
 			}
 		}
@@ -331,7 +333,7 @@ bool WzMessageView::initialize(MESSAGE *psMessage)
 	{
 		auto message = std::make_shared<Paragraph>();
 		message->setFontColour(WZCOL_TEXT_BRIGHT);
-		message->addText(msg.toUtf8());
+		message->addText(msg);
 		messages->addItem(message);
 	}
 
@@ -539,7 +541,7 @@ void W_INTELLIGENCEOVERLAY_FORM::initialize(bool _playCurrent, bool animate)
 		/* Add the message buttons */
 
 		//add each button
-		for (MESSAGE *psMessage = apsMessages[selectedPlayer]; psMessage != nullptr; psMessage = psMessage->psNext)
+		for (MESSAGE *psMessage : apsMessages[selectedPlayer])
 		{
 			/*if (psMessage->type == MSG_TUTORIAL)
 			{
@@ -688,11 +690,8 @@ void W_INTELLIGENCEOVERLAY_FORM::intIntelButtonPressed(const std::shared_ptr<Int
 				{
 					intShowMessageView(psMessage);
 				}
-				// only attempt to show videos if they are installed
-				if (seq_hasVideos())
-				{
-					StartMessageSequences(psMessage, true);
-				}
+				// attempt to show video(s)
+				StartMessageSequences(psMessage, true);
 			}
 			else if (psMessage->pViewData->type == VIEW_RES)
 			{
@@ -834,7 +833,7 @@ bool intAddIntelMap()
 }
 
 // Add all the Video Sequences for a message
-static void StartMessageSequences(MESSAGE *psMessage, bool Start)
+static bool StartMessageSequences(MESSAGE *psMessage, bool Start)
 {
 	bool bLoop = false;
 
@@ -843,10 +842,10 @@ static void StartMessageSequences(MESSAGE *psMessage, bool Start)
 	//should never have a proximity message here
 	if (psMessage->type == MSG_PROXIMITY)
 	{
-		return;
+		return false;
 	}
 
-	ASSERT_OR_RETURN(, psMessage->pViewData != nullptr, "Invalid ViewData pointer");
+	ASSERT_OR_RETURN(false, psMessage->pViewData != nullptr, "Invalid ViewData pointer");
 
 	if (psMessage->pViewData->type == VIEW_RPL)
 	{
@@ -877,10 +876,12 @@ static void StartMessageSequences(MESSAGE *psMessage, bool Start)
 		//play first full screen video
 		if (Start == true)
 		{
-			seq_StartNextFullScreenVideo();
+			if (!seq_StartNextFullScreenVideo())
+			{
+				return false;
+			}
 		}
 	}
-
 	else if (psMessage->pViewData->type == VIEW_RES)
 	{
 		VIEW_RESEARCH		*psViewReplay;
@@ -893,30 +894,36 @@ static void StartMessageSequences(MESSAGE *psMessage, bool Start)
 		//play first full screen video
 		if (Start == true)
 		{
-			seq_StartNextFullScreenVideo();
+			if (!seq_StartNextFullScreenVideo())
+			{
+				return false;
+			}
 		}
 	}
+	else
+	{
+		return false;
+	}
 
+	return true;
 }
 
 static void intCleanUpIntelMap()
 {
-	MESSAGE		*psMessage, *psNext;
 	bool removedAMessage = false;
 
 	if (selectedPlayer < MAX_PLAYERS)
 	{
 		//remove any research messages that have been read
-		for (psMessage = apsMessages[selectedPlayer]; psMessage != nullptr; psMessage =
-				 psNext)
+		mutating_list_iterate(apsMessages[selectedPlayer], [&removedAMessage](MESSAGE* psMessage)
 		{
-			psNext = psMessage->psNext;
 			if (psMessage->type == MSG_RESEARCH && psMessage->read)
 			{
 				removeMessage(psMessage, selectedPlayer);
 				removedAMessage = true;
 			}
-		}
+			return IterationResult::CONTINUE_ITERATION;
+		});
 	}
 	if (removedAMessage)
 	{
@@ -1205,20 +1212,16 @@ void addVideoText(SEQ_DISPLAY *psSeqDisplay, UDWORD sequence)
 /*sets psCurrentMsg for the Intelligence screen*/
 void setCurrentMsg()
 {
-	MESSAGE *psMsg, *psLastMsg;
-
 	ASSERT_OR_RETURN(, selectedPlayer < MAX_PLAYERS, "Unsupported selectedPlayer: %" PRIu32 "", selectedPlayer);
 
-	psLastMsg = nullptr;
-	for (psMsg = apsMessages[selectedPlayer]; psMsg != nullptr; psMsg =
-	         psMsg->psNext)
+	for (auto it = apsMessages[selectedPlayer].rbegin(), end = apsMessages[selectedPlayer].rend(); it != end; ++it)
 	{
-		if (psMsg->type != MSG_PROXIMITY)
+		if ((*it)->type != MSG_PROXIMITY)
 		{
-			psLastMsg = psMsg;
+			psCurrentMsg = *it;
+			return;
 		}
 	}
-	psCurrentMsg = psLastMsg;
 }
 
 /*sets which states need to be paused when the intelligence screen is up*/
@@ -1269,11 +1272,11 @@ void displayImmediateMessage(MESSAGE *psMessage)
 		This has to be changed to support a script calling a message in the intelligence screen
 	*/
 
-	// only attempt to show videos if they are installed
-	if (seq_hasVideos())
+	// attempt to show video(s)
+	psCurrentMsg = psMessage;
+	if (!StartMessageSequences(psMessage, true))
 	{
-	    psCurrentMsg = psMessage;
-	    StartMessageSequences(psMessage, true);
+		psCurrentMsg = nullptr;
 	}
 	// remind the player that the message can be seen again from
 	// the intelligence screen
