@@ -482,7 +482,10 @@ enum INTFAC_TYPE
 	IMAGE_BUT_EMPTY_DOWN,
 	IMAGE_BUT_INNER_GLOW,
 	IMAGE_INDICATOR_DOT,
-	IMAGE_INDICATOR_DOT_EXPAND
+	IMAGE_INDICATOR_DOT_EXPAND,
+	IMAGE_INTFAC_SIDEBAR_LIST,
+	IMAGE_INGAMEOPTIONS_UP,
+	IMAGE_INGAMEOPTIONS_DOWN,
 };
 
 #endif //__INCLUDED_SRC_INTFAC_H__
