@@ -18,10 +18,12 @@
 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 #include <cstring>
+#include <nonstd/optional.hpp>
 
 #include "lib/framework/frame.h"
 #include "lib/framework/wzapp.h"
 #include "lib/framework/rational.h"
+#include "lib/framework/object_list_iteration.h"
 #include "lib/framework/physfs_ext.h"
 #include "objects.h"
 #include "levels.h"
@@ -74,10 +76,15 @@
 #include "loadsave.h"
 #include "game.h"
 #include "droid.h"
+#include "move.h"
 #include "spectatorwidgets.h"
+#include "campaigninfo.h"
 
 #include "activity.h"
 
+#include "screens/ingameopscreen.h"
+#include "titleui/options/optionsforms.h"
+
 /*
 	KeyBind.c
 	Holds all the functions that can be mapped to a key.
@@ -91,7 +98,8 @@ bool	bMovePause = false;
 bool		bAllowOtherKeyPresses = true;
 char	beaconMsg[MAX_PLAYERS][MAX_CONSOLE_STRING_LENGTH];		//beacon msg for each player
 
-static STRUCTURE	*psOldRE = nullptr;
+using ExtractorListIter = optional<ExtractorList::const_iterator>;
+static ExtractorListIter psOldRE = {};
 static char	sCurrentConsoleText[MAX_CONSOLE_STRING_LENGTH];			//remember what user types in console for beacon msg
 
 #define QUICKSAVE_CAM_FOLDER "savegames/campaign/QuickSave"
@@ -121,6 +129,22 @@ static void noMPCheatMsg()
 	addConsoleMessage(_("Sorry, that cheat is disabled in multiplayer games."), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
 }
 
+bool shouldTrapCursor()
+{
+	auto result = war_GetTrapCursor();
+	switch (result)
+	{
+		case TrapCursorMode::Disabled:
+			return false;
+		case TrapCursorMode::Enabled:
+			return true;
+		case TrapCursorMode::Automatic:
+			// automatic mode - if in fullscreen mode (including desktop full), or maximized, trap the cursor
+			return (wzIsFullscreen() || wzIsMaximized());
+	}
+	return false; // silence compiler warning
+}
+
 // --------------------------------------------------------------------------
 void kf_AutoGame()
 {
@@ -209,7 +233,7 @@ void kf_DamageMe()
 	{
 		return; // no-op
 	}
-	for (DROID *psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID *psDroid : apsDroidLists[selectedPlayer])
 	{
 		if (psDroid->selected)
 		{
@@ -225,11 +249,11 @@ void kf_DamageMe()
 			}
 		}
 	}
-	for (STRUCTURE *psStruct = apsStructLists[selectedPlayer]; psStruct; psStruct = psStruct->psNext)
+	for (STRUCTURE *psStruct : apsStructLists[selectedPlayer])
 	{
 		if (psStruct->selected)
 		{
-			int val = psStruct->body - ((structureBody(psStruct) / 100) * 20);
+			int val = psStruct->body - ((psStruct->structureBody() / 100) * 20);
 			if (val > 0)
 			{
 				psStruct->body = val;
@@ -245,17 +269,13 @@ void kf_DamageMe()
 
 void	kf_TraceObject()
 {
-	DROID		*psCDroid, *psNDroid;
-	STRUCTURE	*psCStruct, *psNStruct;
-
 	if (selectedPlayer >= MAX_PLAYERS)
 	{
 		return; // no-op
 	}
 
-	for (psCDroid = apsDroidLists[selectedPlayer]; psCDroid; psCDroid = psNDroid)
+	for (const DROID* psCDroid : apsDroidLists[selectedPlayer])
 	{
-		psNDroid = psCDroid->psNext;
 		if (psCDroid->selected)
 		{
 			objTraceEnable(psCDroid->id);
@@ -263,9 +283,8 @@ void	kf_TraceObject()
 			return;
 		}
 	}
-	for (psCStruct = apsStructLists[selectedPlayer]; psCStruct; psCStruct = psNStruct)
+	for (const STRUCTURE* psCStruct : apsStructLists[selectedPlayer])
 	{
-		psNStruct = psCStruct->psNext;
 		if (psCStruct->selected)
 		{
 			objTraceEnable(psCStruct->id);
@@ -307,9 +326,9 @@ void	kf_HalveHeights()
 {
 	MAPTILE	*psTile;
 
-	for (int i = 0; i < mapWidth; ++i)
+	for (int j = 0; j < mapHeight; ++j)
 	{
-		for (int j = 0; j < mapHeight; ++j)
+		for (int i = 0; i < mapWidth; ++i)
 		{
 			psTile = mapTile(i, j);
 			psTile->height /= 2;
@@ -357,14 +376,12 @@ void	kf_FaceWest()
 /* Writes out debug info about all the selected droids */
 void	kf_DebugDroidInfo()
 {
-	DROID	*psDroid;
-
 	if (selectedPlayer >= MAX_PLAYERS)
 	{
 		return; // no-op
 	}
 
-	for (psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (const DROID* psDroid : apsDroidLists[selectedPlayer])
 	{
 		if (psDroid->selected)
 		{
@@ -390,48 +407,61 @@ void kf_CloneSelected(int limit)
 		return; // no-op
 	}
 
-	for (DROID *psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	bool selectedAnything = false;
+	const DROID* droidToClone = nullptr;
+	for (const DROID* d : apsDroidLists[selectedPlayer])
 	{
-		if (psDroid->selected)
+		if (!d->selected)
 		{
-			enumerateTemplates(selectedPlayer, [psDroid, &sTemplate](DROID_TEMPLATE * psTempl) {
-				if (psTempl->name.compare(psDroid->aName) == 0)
-				{
-					sTemplate = psTempl;
-					return false; // stop enumerating
-				}
-				return true;
-			});
-
-			if (!sTemplate)
-			{
-				debug(LOG_ERROR, "Cloning vat has been destroyed. We can't find the template for this droid: %s, id:%u, type:%d!", psDroid->aName, psDroid->id, psDroid->droidType);
-				return;
-			}
-
-			// create a new droid army
-			for (int i = 0; i < limit; i++)
-			{
-				Vector2i pos = psDroid->pos.xy() + iSinCosR(40503 * i, iSqrt(50 * 50 * (i + 1)));  // 40503 = 65536/φ (A bit more than a right angle)
-				DROID *psNewDroid = buildDroid(sTemplate, pos.x, pos.y, psDroid->player, false, nullptr);
-				if (psNewDroid)
-				{
-					addDroid(psNewDroid, apsDroidLists);
-					triggerEventDroidBuilt(psNewDroid, nullptr);
-				}
-				else if (!bMultiMessages)
-				{
-					debug(LOG_ERROR, "Cloning has failed for template:%s id:%d", getID(sTemplate), sTemplate->multiPlayerID);
-				}
-			}
-			std::string msg = astringf(_("Player %u is cheating a new droid army of: %d × %s."), selectedPlayer, limit, psDroid->aName);
-			sendInGameSystemMessage(msg.c_str());
-			Cheated = true;
-			audio_PlayTrack(ID_SOUND_NEXUS_LAUGH1);
-			return;
+			continue;
 		}
-		debug(LOG_INFO, "Nothing was selected?");
+		selectedAnything = true;
+		enumerateTemplates(selectedPlayer, [psDroid = d, &sTemplate](DROID_TEMPLATE* psTempl) {
+			if (psTempl->name.compare(psDroid->aName) == 0)
+			{
+				sTemplate = psTempl;
+				return false; // stop enumerating
+			}
+			return true;
+		});
+		if (!sTemplate)
+		{
+			debug(LOG_ERROR, "Cloning vat has been destroyed. We can't find the template for this droid: %s, id:%u, type:%d!", d->aName, d->id, d->droidType);
+			break;
+		}
+		droidToClone = d;
+		// Break out of the loop if we've successfully found the associated droid template.
+		break;
 	}
+	if (!selectedAnything)
+	{
+		debug(LOG_INFO, "Nothing was selected?");
+		return;
+	}
+	if (!sTemplate)
+	{
+		return;
+	}
+	ASSERT(droidToClone != nullptr, "No droid to clone found");
+	// create a new droid army
+	for (int i = 0; i < limit; i++)
+	{
+		Vector2i pos = droidToClone->pos.xy() + iSinCosR(40503 * i, iSqrt(50 * 50 * (i + 1)));  // 40503 = 65536/φ (A bit more than a right angle)
+		DROID* psNewDroid = buildDroid(sTemplate, pos.x, pos.y, droidToClone->player, false, nullptr);
+		if (psNewDroid)
+		{
+			addDroid(psNewDroid, apsDroidLists);
+			triggerEventDroidBuilt(psNewDroid, nullptr);
+		}
+		else if (!bMultiMessages)
+		{
+			debug(LOG_ERROR, "Cloning has failed for template:%s id:%d", getID(sTemplate), sTemplate->multiPlayerID);
+		}
+	}
+	std::string msg = astringf(_("Player %u is cheating a new droid army of: %d × %s."), selectedPlayer, limit, droidToClone->aName);
+	sendInGameSystemMessage(msg.c_str());
+	Cheated = true;
+	audio_PlayTrack(ID_SOUND_NEXUS_LAUGH1);
 }
 
 void kf_MakeMeHero()
@@ -449,12 +479,12 @@ void kf_MakeMeHero()
 		return; // no-op
 	}
 
-	for (DROID *psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID *psDroid : apsDroidLists[selectedPlayer])
 	{
-		if (psDroid->selected && psDroid->droidType == DROID_COMMAND)
+		if (psDroid->selected && (psDroid->droidType == DROID_COMMAND || psDroid->droidType == DROID_SENSOR))
 		{
-			psDroid->experience = 8 * 65536 * 128;
-		} 
+			psDroid->experience = 16 * 65536 * 128;
+		}
 		else if (psDroid->selected)
 		{
 			psDroid->experience = 4 * 65536 * 128;
@@ -478,7 +508,7 @@ void kf_TeachSelected()
 		return; // no-op
 	}
 
-	for (DROID *psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID *psDroid : apsDroidLists[selectedPlayer])
 	{
 		if (psDroid->selected)
 		{
@@ -503,7 +533,7 @@ void kf_Unselectable()
 		return; // no-op
 	}
 
-	for (DROID *psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID *psDroid : apsDroidLists[selectedPlayer])
 	{
 		if (psDroid->selected)
 		{
@@ -511,7 +541,7 @@ void kf_Unselectable()
 			psDroid->selected = false;
 		}
 	}
-	for (STRUCTURE *psStruct = apsStructLists[selectedPlayer]; psStruct; psStruct = psStruct->psNext)
+	for (STRUCTURE *psStruct : apsStructLists[selectedPlayer])
 	{
 		if (psStruct->selected)
 		{
@@ -563,19 +593,6 @@ void	kf_BifferBaker()
 	          selectedPlayer, _("Hard as nails!!!"));
 	sendInGameSystemMessage(cmsg.c_str());
 }
-// --------------------------------------------------------------------------
-void	kf_SetEasyLevel()
-{
-	// Bail out if we're running a _true_ multiplayer game (to prevent MP cheating)
-	if (runningMultiplayer())
-	{
-		noMPCheatMsg();
-		return;
-	}
-
-	setDifficultyLevel(DL_EASY);
-	addConsoleMessage(_("Takings thing easy!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
-}
 
 // --------------------------------------------------------------------------
 void	kf_UpThePower()
@@ -608,7 +625,7 @@ void	kf_MaxPower()
 }
 
 // --------------------------------------------------------------------------
-void	kf_SetNormalLevel()
+void kf_SetDifficultyLevel(const DIFFICULTY_LEVEL level)
 {
 	// Bail out if we're running a _true_ multiplayer game (to prevent MP cheating)
 	if (runningMultiplayer())
@@ -617,21 +634,16 @@ void	kf_SetNormalLevel()
 		return;
 	}
 
-	setDifficultyLevel(DL_NORMAL);
-	addConsoleMessage(_("Back to normality!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
-}
-// --------------------------------------------------------------------------
-void	kf_SetHardLevel()
-{
-	// Bail out if we're running a _true_ multiplayer game (to prevent MP cheating)
-	if (runningMultiplayer())
+	setDifficultyLevel(level);
+	switch (level)
 	{
-		noMPCheatMsg();
-		return;
+		case DL_SUPER_EASY: addConsoleMessage(_("A power fantasy!"), LEFT_JUSTIFY, SYSTEM_MESSAGE); break;
+		case DL_EASY: addConsoleMessage(_("Taking things easy!"), LEFT_JUSTIFY, SYSTEM_MESSAGE); break;
+		case DL_NORMAL: addConsoleMessage(_("Back to normality!"), LEFT_JUSTIFY, SYSTEM_MESSAGE); break;
+		case DL_HARD: addConsoleMessage(_("Getting tricky!"), LEFT_JUSTIFY, SYSTEM_MESSAGE); break;
+		case DL_INSANE: addConsoleMessage(_("In a nightmare!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);break;
+		default: break;
 	}
-
-	setDifficultyLevel(DL_HARD);
-	addConsoleMessage(_("Getting tricky!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
 }
 // --------------------------------------------------------------------------
 void	kf_DoubleUp()
@@ -742,14 +754,13 @@ void kf_ListDroids()
 	}
 	for (int i = 0; i < MAX_PLAYERS; i++)
 	{
-		for (DROID *psDroid = apsDroidLists[i]; psDroid; psDroid = psDroid->psNext)
+		for (const DROID *psDroid : apsDroidLists[i])
 		{
 			const auto x = map_coord(psDroid->pos.x);
 			const auto y = map_coord(psDroid->pos.y);
 			debug(LOG_INFO, "droid %i;%s;%i;%i;%i", i, psDroid->aName, psDroid->droidType, x, y);
 		}
 	}
-	
 }
 
 
@@ -905,25 +916,21 @@ void kf_MapCheck()
 	}
 #endif
 
-	DROID		*psDroid;
-	STRUCTURE	*psStruct;
-	FLAG_POSITION	*psCurrFlag;
-
 	if (selectedPlayer >= MAX_PLAYERS) { return; }
 
-	for (psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID* psDroid : apsDroidLists[selectedPlayer])
 	{
 		psDroid->pos.z = map_Height(psDroid->pos.x, psDroid->pos.y);
 	}
 
-	for (psStruct = apsStructLists[selectedPlayer]; psStruct; psStruct = psStruct->psNext)
+	for (STRUCTURE* psStruct : apsStructLists[selectedPlayer])
 	{
 		alignStructure(psStruct);
 	}
 
-	for (psCurrFlag = apsFlagPosLists[selectedPlayer]; psCurrFlag; psCurrFlag = psCurrFlag->psNext)
+	for (auto& psFlag : apsFlagPosLists[selectedPlayer])
 	{
-		psCurrFlag->coords.z = map_Height(psCurrFlag->coords.x, psCurrFlag->coords.y) + ASSEMBLY_POINT_Z_PADDING;
+		psFlag->coords.z = map_Height(psFlag->coords.x, psFlag->coords.y) + ASSEMBLY_POINT_Z_PADDING;
 	}
 }
 
@@ -1054,7 +1061,7 @@ void	kf_PitchForward()
 /* Resets pitch to default */
 void	kf_ResetPitch()
 {
-	playerPos.r.x = DEG(360 - 20);
+	playerPos.r.x = DEG(360 + INITIAL_STARTING_PITCH);
 	setViewDistance(STARTDISTANCE);
 }
 
@@ -1064,9 +1071,17 @@ void kf_ShowMappings()
 {
 	if (!InGameOpUp && !isInGamePopupUp)
 	{
-		kf_addInGameOptions();
-		intProcessInGameOptions(INTINGAMEOP_OPTIONS);
-		intProcessInGameOptions(INTINGAMEOP_KEYMAP);
+		// Open new Options screen and jump to Controls section
+		auto optionsBrowser = createOptionsBrowser(true);
+		if (!optionsBrowser->switchToOptionsForm(OptionsBrowserForm::Modes::Controls))
+		{
+			debug(LOG_INFO, "Failed to open Controls options form?");
+			return;
+		}
+		showInGameOptionsScreen(psWScreen, optionsBrowser, []() {
+			// the setting for group menu display may have been modified
+			intShowGroupSelectionMenu();
+		});
 	}
 }
 
@@ -1079,11 +1094,10 @@ void kf_SelectGrouping(UDWORD groupNumber)
 	SPECTATOR_NO_OP();
 
 	bool	bAlreadySelected;
-	DROID	*psDroid;
 	bool	Selected;
 
 	bAlreadySelected = false;
-	for (psDroid = apsDroidLists[selectedPlayer]; psDroid != nullptr; psDroid = psDroid->psNext)
+	for (DROID* psDroid : apsDroidLists[selectedPlayer])
 	{
 		/* Wipe out the ones in the wrong group */
 		if (psDroid->selected && psDroid->group != groupNumber)
@@ -1110,7 +1124,7 @@ void kf_SelectGrouping(UDWORD groupNumber)
 	intGroupsChanged(groupNumber);
 
 	/* play group audio but only if they weren't already selected - AM */
-	if (Selected && !bAlreadySelected)
+	if (Selected && !bAlreadySelected && war_getPlayAudioCue_GroupReporting())
 	{
 		audio_QueueTrack(ID_SOUND_GROUP_0 + groupNumber);
 		audio_QueueTrack(ID_SOUND_REPORTING);
@@ -1137,7 +1151,7 @@ MappableFunction kf_AssignGrouping_N(const unsigned int n)
 		/* not supported if a spectator */
 		SPECTATOR_NO_OP();
 
-		assignDroidsToGroup(selectedPlayer, n, true);
+		assignObjectToGroup(selectedPlayer, n, true);
 	};
 }
 
@@ -1147,7 +1161,7 @@ MappableFunction kf_AddGrouping_N(const unsigned int n)
 		/* not supported if a spectator */
 		SPECTATOR_NO_OP();
 
-		assignDroidsToGroup(selectedPlayer, n, false);
+		assignObjectToGroup(selectedPlayer, n, false);
 	};
 }
 
@@ -1157,7 +1171,7 @@ MappableFunction kf_RemoveFromGrouping()
 		/* not supported if a spectator */
 		SPECTATOR_NO_OP();
 
-		removeDroidsFromGroup(selectedPlayer);
+		removeObjectFromGroup(selectedPlayer);
 	};
 }
 
@@ -1301,15 +1315,12 @@ void	kf_ToggleGodMode()
 	{
 		ASSERT_OR_RETURN(, selectedPlayer < MAX_PLAYERS, "Cannot disable godMode for spectator-only slots");
 
-		FEATURE	*psFeat = apsFeatureLists[0];
-
 		godMode = false;
 		setRevealStatus(pastReveal);
 		// now hide the features
-		while (psFeat)
+		for (BASE_OBJECT* psFeat : apsFeatureLists[0])
 		{
 			psFeat->visible[selectedPlayer] = 0;
-			psFeat = psFeat->psNext;
 		}
 
 		// and the structures
@@ -1317,12 +1328,9 @@ void	kf_ToggleGodMode()
 		{
 			if (playerId != selectedPlayer)
 			{
-				STRUCTURE *psStruct = apsStructLists[playerId];
-
-				while (psStruct)
+				for (STRUCTURE* psStruct : apsStructLists[playerId])
 				{
 					psStruct->visible[selectedPlayer] = 0;
-					psStruct = psStruct->psNext;
 				}
 			}
 		}
@@ -1360,12 +1368,34 @@ MappableFunction kf_ScrollCamera(const int horizontal, const int vertical)
 	};
 }
 
+static TrapCursorMode oldTrapCursorEnabledValue = TrapCursorMode::Enabled;
 void kf_toggleTrapCursor()
 {
-	bool trap = !war_GetTrapCursor();
-	war_SetTrapCursor(trap);
-	(trap ? wzGrabMouse : wzReleaseMouse)();
-	std::string msg = astringf(_("Trap cursor %s"), trap ? "ON" : "OFF");
+	auto value = war_GetTrapCursor();
+	if (value == TrapCursorMode::Disabled)
+	{
+		value = oldTrapCursorEnabledValue;
+	}
+	else if (value == TrapCursorMode::Automatic && !shouldTrapCursor())
+	{
+		// set to auto, but will not automatically trap the cursor - toggle to enabled
+		value = TrapCursorMode::Enabled;
+	}
+	else
+	{
+		oldTrapCursorEnabledValue = value;
+		value = TrapCursorMode::Disabled;
+	}
+	war_SetTrapCursor(value);
+	(shouldTrapCursor() ? wzGrabMouse : wzReleaseMouse)();
+	const char* pValueStr = "";
+	switch (value)
+	{
+		case TrapCursorMode::Disabled: pValueStr = "OFF"; break;
+		case TrapCursorMode::Enabled: pValueStr = "ON"; break;
+		case TrapCursorMode::Automatic: pValueStr = "AUTO"; break;
+	}
+	std::string msg = astringf(_("Trap cursor %s"), pValueStr);
 	addConsoleMessage(msg.c_str(), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
 }
 
@@ -1389,11 +1419,8 @@ void	kf_TogglePauseMode()
 		setAudioPause(true);
 		setScrollPause(true);
 
-		// If cursor trapping is enabled allow the cursor to leave the window
-		if (war_GetTrapCursor())
-		{
-			wzReleaseMouse();
-		}
+		// If cursor trapping is enabled, allow the cursor to leave the window when paused
+		wzReleaseMouse();
 
 		/* And stop the clock */
 		gameTimeStop();
@@ -1471,7 +1498,7 @@ void	kf_TogglePauseMode()
 		setScrollPause(false);
 
 		// Re-enable cursor trapping if it is enabled
-		if (war_GetTrapCursor())
+		if (shouldTrapCursor())
 		{
 			wzGrabMouse();
 		}
@@ -1505,6 +1532,10 @@ void	kf_FinishAllResearch()
 	{
 		if (!IsResearchCompleted(&asPlayerResList[selectedPlayer][j]))
 		{
+			if (asResearch[j].excludeFromCheats)
+			{
+				continue;
+			}
 			if (bMultiMessages)
 			{
 				SendResearch(selectedPlayer, j, false);
@@ -1512,7 +1543,6 @@ void	kf_FinishAllResearch()
 			}
 			else
 			{
-				MakeResearchCompleted(&asPlayerResList[selectedPlayer][j]);
 				researchResult(j, selectedPlayer, false, nullptr, false);
 			}
 		}
@@ -1524,8 +1554,6 @@ void	kf_FinishAllResearch()
 
 void kf_Reload()
 {
-	STRUCTURE	*psCurr;
-
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
@@ -1537,15 +1565,18 @@ void kf_Reload()
 		return;
 	}
 #endif
-
-	for (psCurr = interfaceStructList(); psCurr; psCurr = psCurr->psNext)
+	auto* intStrList = interfaceStructList();
+	if (intStrList)
 	{
-		if (isLasSat(psCurr->pStructureType) && psCurr->selected)
+		for (STRUCTURE* psCurr : *intStrList)
 		{
-			unsigned int firePause = weaponFirePause(&asWeaponStats[psCurr->asWeaps[0].nStat], psCurr->player);
+			if (isLasSat(psCurr->pStructureType) && psCurr->selected)
+			{
+				unsigned int firePause = weaponFirePause(*psCurr->getWeaponStats(0), psCurr->player);
 
-			psCurr->asWeaps[0].lastFired -= firePause;
-			CONPRINTF("%s", _("Selected buildings instantly recharged!"));
+				psCurr->asWeaps[0].lastFired -= firePause;
+				CONPRINTF("%s", _("Selected buildings instantly recharged!"));
+			}
 		}
 	}
 }
@@ -1554,8 +1585,6 @@ void kf_Reload()
 // finish all the research for the selected player
 void	kf_FinishResearch()
 {
-	STRUCTURE	*psCurr;
-
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
@@ -1568,29 +1597,42 @@ void	kf_FinishResearch()
 	}
 #endif
 
-	for (psCurr = interfaceStructList(); psCurr; psCurr = psCurr->psNext)
+	auto* intStrList = interfaceStructList();
+	if (intStrList)
 	{
-		if (psCurr->pStructureType->type == REF_RESEARCH)
+		for (STRUCTURE* psCurr : *intStrList)
 		{
-			BASE_STATS	*pSubject;
-
-			// find out what we are researching here
-			pSubject = ((RESEARCH_FACILITY *)psCurr->pFunctionality)->psSubject;
-			if (pSubject)
+			if (psCurr->pStructureType->type == REF_RESEARCH)
 			{
-				int rindex = ((RESEARCH *)pSubject)->index;
-				if (bMultiMessages)
+				BASE_STATS* pSubject;
+
+				// find out what we are researching here
+				pSubject = ((RESEARCH_FACILITY*)psCurr->pFunctionality)->psSubject;
+				if (pSubject)
 				{
-					SendResearch(selectedPlayer, rindex, true);
-					// Wait for our message before doing anything.
+					int rindex = ((RESEARCH*)pSubject)->index;
+					PLAYER_RESEARCH *plrRes = &asPlayerResList[selectedPlayer][rindex];
+					if (!IsResearchCompleted(plrRes))
+					{
+						if (bMultiMessages)
+						{
+							SendResearch(selectedPlayer, rindex, true);
+							// Wait for our message before doing anything.
+						}
+						else
+						{
+							researchResult(rindex, selectedPlayer, true, psCurr, true);
+						}
+						std::string cmsg = astringf(_("(Player %u) is using cheat :%s %s"), selectedPlayer, _("Researched"), getLocalizedStatsName(pSubject));
+						sendInGameSystemMessage(cmsg.c_str());
+						intResearchFinished(psCurr);
+					}
+					else
+					{
+						debug(LOG_ERROR, "Research already completed for player %" PRIu32 "?: %s", (int)selectedPlayer, getStatsName(pSubject));
+						continue;
+					}
 				}
-				else
-				{
-					researchResult(rindex, selectedPlayer, true, psCurr, true);
-				}
-				std::string cmsg = astringf(_("(Player %u) is using cheat :%s %s"), selectedPlayer, _("Researched"), getLocalizedStatsName(pSubject));
-				sendInGameSystemMessage(cmsg.c_str());
-				intResearchFinished(psCurr);
 			}
 		}
 	}
@@ -1635,32 +1677,38 @@ void	kf_JumpToResourceExtractor()
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
-	if (psOldRE && (STRUCTURE *)psOldRE->psNextFunc)
+	if (apsExtractorLists[selectedPlayer].empty())
 	{
-		psOldRE = psOldRE->psNextFunc;
+		addConsoleMessage(_("Unable to locate any oil derricks!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
+		return;
+	}
+
+	if (!psOldRE.has_value() || std::next(*psOldRE) == apsExtractorLists[selectedPlayer].end())
+	{
+		// Reset the pointer if `psOldRE` is either not initialized yet or points to the last element.
+		psOldRE.emplace(apsExtractorLists[selectedPlayer].begin());
 	}
 	else
 	{
-		psOldRE = apsExtractorLists[selectedPlayer];
+		++(*psOldRE);
 	}
 
-	if (psOldRE)
+	if (**psOldRE != nullptr)
 	{
 		playerPos.r.y = 0; // face north
-		setViewPos(map_coord(psOldRE->pos.x), map_coord(psOldRE->pos.y), true);
+		setViewPos(map_coord((**psOldRE)->pos.x), map_coord((**psOldRE)->pos.y), true);
 	}
 	else
 	{
 		addConsoleMessage(_("Unable to locate any oil derricks!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
 	}
-
 }
 
 void keybindInformResourceExtractorRemoved(const STRUCTURE* psResourceExtractor)
 {
-	if (psOldRE == psResourceExtractor)
+	if (psOldRE.has_value() && *psOldRE != apsExtractorLists[selectedPlayer].end() && **psOldRE == psResourceExtractor)
 	{
-		psOldRE = nullptr;
+		psOldRE.reset();
 	}
 }
 
@@ -1856,10 +1904,8 @@ MappableFunction kf_SelectNextFactory(const STRUCTURE_TYPE factoryType, const bo
 
 		selNextSpecifiedBuilding(factoryType, bJumpToSelected);
 
-		STRUCTURE* psCurrent;
-
 		//deselect factories of other types
-		for (psCurrent = apsStructLists[selectedPlayer]; psCurrent; psCurrent = psCurrent->psNext)
+		for (STRUCTURE* psCurrent : apsStructLists[selectedPlayer])
 		{
 			const STRUCTURE_TYPE currentType = psCurrent->pStructureType->type;
 			const bool bIsAnotherTypeOfFactory = currentType != factoryType && std::any_of(FACTORY_TYPES.begin(), FACTORY_TYPES.end(), [currentType](const STRUCTURE_TYPE type) {
@@ -1910,9 +1956,6 @@ MappableFunction kf_SelectNextPowerStation(const bool bJumpToSelected)
 // --------------------------------------------------------------------------
 void	kf_KillEnemy()
 {
-	DROID		*psCDroid, *psNDroid;
-	STRUCTURE	*psCStruct, *psNStruct;
-
 #ifndef DEBUG
 	// Bail out if we're running a _true_ multiplayer game (to prevent MP cheating)
 	if (runningMultiplayer())
@@ -1936,15 +1979,13 @@ void	kf_KillEnemy()
 		if (playerId != selectedPlayer && !aiCheckAlliances(selectedPlayer, playerId))
 		{
 			// wipe out all the droids
-			for (psCDroid = apsDroidLists[playerId]; psCDroid; psCDroid = psNDroid)
+			for (const DROID* psCDroid : apsDroidLists[playerId])
 			{
-				psNDroid = psCDroid->psNext;
 				SendDestroyDroid(psCDroid);
 			}
 			// wipe out all their structures
-			for (psCStruct = apsStructLists[playerId]; psCStruct; psCStruct = psNStruct)
+			for (STRUCTURE* psCStruct : apsStructLists[playerId])
 			{
-				psNStruct = psCStruct->psNext;
 				SendDestroyStructure(psCStruct);
 			}
 		}
@@ -1954,9 +1995,6 @@ void	kf_KillEnemy()
 // kill all the selected objects
 void kf_KillSelected()
 {
-	DROID		*psCDroid, *psNDroid;
-	STRUCTURE	*psCStruct, *psNStruct;
-
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
@@ -1977,17 +2015,15 @@ void kf_KillSelected()
 	audio_PlayTrack(ID_SOUND_COLL_DIE);
 	Cheated = true;
 
-	for (psCDroid = apsDroidLists[selectedPlayer]; psCDroid; psCDroid = psNDroid)
+	for (const DROID* psCDroid : apsDroidLists[selectedPlayer])
 	{
-		psNDroid = psCDroid->psNext;
 		if (psCDroid->selected)
 		{
 			SendDestroyDroid(psCDroid);
 		}
 	}
-	for (psCStruct = apsStructLists[selectedPlayer]; psCStruct; psCStruct = psNStruct)
+	for (STRUCTURE* psCStruct : apsStructLists[selectedPlayer])
 	{
-		psNStruct = psCStruct->psNext;
 		if (psCStruct->selected)
 		{
 			SendDestroyStructure(psCStruct);
@@ -2089,7 +2125,7 @@ MappableFunction kf_OrderDroid(const DroidOrderType order)
 		/* not supported if a spectator */
 		SPECTATOR_NO_OP();
 
-		for (DROID* psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+		for (DROID* psDroid : apsDroidLists[selectedPlayer])
 		{
 			if (psDroid->selected)
 			{
@@ -2118,8 +2154,6 @@ void	kf_ToggleVisibility()
 // --------------------------------------------------------------------------
 static void kfsf_SetSelectedDroidsState(SECONDARY_ORDER sec, SECONDARY_STATE state)
 {
-	DROID	*psDroid;
-
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
@@ -2128,10 +2162,10 @@ static void kfsf_SetSelectedDroidsState(SECONDARY_ORDER sec, SECONDARY_STATE sta
 	// _not_ be disallowed in multiplayer games.
 
 	// This code is similar to SetSecondaryState() in intorder.cpp. Unfortunately, it seems hard to un-duplicate the code.
-	for (psDroid = apsDroidLists[selectedPlayer]; psDroid; psDroid = psDroid->psNext)
+	for (DROID* psDroid : apsDroidLists[selectedPlayer])
 	{
 		// Only set the state if it's not a transporter.
-		if (psDroid->selected && !isTransporter(psDroid))
+		if (psDroid->selected && !psDroid->isTransporter())
 		{
 			secondarySetState(psDroid, sec, state);
 		}
@@ -2142,24 +2176,15 @@ static void kfsf_SetSelectedDroidsState(SECONDARY_ORDER sec, SECONDARY_STATE sta
 // --------------------------------------------------------------------------
 void	kf_TriggerRayCast()
 {
-	DROID	*psDroid;
-	bool	found;
-
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
-	found = false;
-	for (psDroid = apsDroidLists[selectedPlayer]; psDroid && !found;
-	     psDroid = psDroid->psNext)
+	auto selectedDroidIt = std::find_if(apsDroidLists[selectedPlayer].begin(), apsDroidLists[selectedPlayer].end(), [](DROID* d)
 	{
-		if (psDroid->selected)
-		{
-			found = true;
-		}
-		/* NOP */
-	}
+		return d->selected;
+	});
 
-	if (found)
+	if (selectedDroidIt != apsDroidLists[selectedPlayer].end())
 	{
 //		getBlockHeightDirToEdgeOfGrid(UDWORD x, UDWORD y, UBYTE direction, UDWORD *height, UDWORD *dist)
 //		getBlockHeightDirToEdgeOfGrid(psOther->pos.x,psOther->pos.y,psOther->direction,&height,&dist);
@@ -2170,24 +2195,22 @@ void	kf_TriggerRayCast()
 // --------------------------------------------------------------------------
 void	kf_CentreOnBase()
 {
-	STRUCTURE	*psStruct;
-	bool		bGotHQ;
+	bool		bGotHQ = false;
 	UDWORD		xJump = 0, yJump = 0;
 
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
 	/* Got through our buildings */
-	for (psStruct = apsStructLists[selectedPlayer], bGotHQ = false;	// start
-	     psStruct && !bGotHQ;											// terminate
-	     psStruct = psStruct->psNext)									// iteration
+	for (const STRUCTURE* pStruct : apsStructLists[selectedPlayer])
 	{
 		/* Have we got a HQ? */
-		if (psStruct->pStructureType->type == REF_HQ)
+		if (pStruct->pStructureType->type == REF_HQ)
 		{
 			bGotHQ = true;
-			xJump = psStruct->pos.x;
-			yJump = psStruct->pos.y;
+			xJump = pStruct->pos.x;
+			yJump = pStruct->pos.y;
+			break;
 		}
 	}
 
@@ -2213,14 +2236,25 @@ void	kf_CentreOnBase()
 // --------------------------------------------------------------------------
 void kf_ToggleFormationSpeedLimiting()
 {
-	addConsoleMessage(_("Formation speed limiting has been removed from the game due to bugs."), LEFT_JUSTIFY, SYSTEM_MESSAGE);
+	bool resultingValue = false;
+	if (moveToggleFormationSpeedLimiting(selectedPlayer, &resultingValue))
+	{
+		if (resultingValue)
+		{
+			addConsoleMessage(_("Formation speed limiting ON"),LEFT_JUSTIFY, SYSTEM_MESSAGE);
+		}
+		else
+		{
+			addConsoleMessage(_("Formation speed limiting OFF"),LEFT_JUSTIFY, SYSTEM_MESSAGE);
+		}
+	}
 }
 
 // --------------------------------------------------------------------------
 void	kf_RightOrderMenu()
 {
-	DROID	*psDroid, *psGotOne = nullptr;
-	bool	bFound;
+	DROID	*psGotOne = nullptr;
+	bool	bFound = false;
 
 	// if menu open, then close it!
 	if (widgGetFromID(psWScreen, IDORDER_FORM) != nullptr)
@@ -2232,13 +2266,13 @@ void	kf_RightOrderMenu()
 	/* not supported if a spectator */
 	SPECTATOR_NO_OP();
 
-	for (psDroid = apsDroidLists[selectedPlayer], bFound = false;
-	     psDroid && !bFound; psDroid = psDroid->psNext)
+	for (DROID* psDroid : apsDroidLists[selectedPlayer])
 	{
 		if (psDroid->selected) // && droidOnScreen(psDroid,0))
 		{
 			bFound = true;
 			psGotOne = psDroid;
+			break;
 		}
 	}
 	if (bFound)
@@ -2573,6 +2607,15 @@ void kf_QuickSave()
 	{
 		return;
 	}
+	if (getCamTweakOption_AutosavesOnly())
+	{
+		console(_("QuickSave not allowed in Autosaves-Only mode"));
+		return;
+	}
+	if (NETisReplay())
+	{
+		return; // Bail out if we're running a replay
+	}
 
 	const char *filename = bMultiPlayer ? QUICKSAVE_SKI_FILENAME : QUICKSAVE_CAM_FILENAME;
 	const char *quickSaveFolder = bMultiPlayer ? QUICKSAVE_SKI_FOLDER : QUICKSAVE_CAM_FOLDER;
@@ -2602,6 +2645,11 @@ void kf_QuickLoad()
 	{
 		return;
 	}
+	if (getCamTweakOption_AutosavesOnly())
+	{
+		console(_("QuickLoad not allowed in Autosaves-Only mode"));
+		return;
+	}
 
 	const char *filename = bMultiPlayer ? QUICKSAVE_SKI_FILENAME : QUICKSAVE_CAM_FILENAME;
 	// check for .json version, because that's what going to be loaded anyway
@@ -2614,8 +2662,6 @@ void kf_QuickLoad()
 		setWidgetsStatus(true);
 		intResetScreen(false);
 		wzSetCursor(CURSOR_DEFAULT);
-		int campaign = getCampaign(filename);
-		setCampaignNumber(campaign);
 
 		loopMissionState = LMS_LOADGAME;
 		sstrcpy(saveGameName, filename);
@@ -2642,5 +2688,5 @@ void kf_ToggleSpecOverlays()
 
 void keybindShutdown()
 {
-	psOldRE = nullptr;
+	psOldRE.reset();
 }
