@@ -59,7 +59,7 @@ struct MAPTILE
 	uint8_t         tileInfoBits;
 	PlayerMask      tileExploredBits;
 	PlayerMask      sensorBits;             ///< bit per player, who can see tile with sensor
-	uint8_t         watchers[MAX_PLAYERS];  // player sees through fog of war here with this many objects
+	uint16_t        watchers[MAX_PLAYERS];  // player sees through fog of war here with this many objects
 	uint16_t        texture;                // Which graphics texture is on this tile
 	int32_t         height;                 ///< The height at the top left of the tile
 	BASE_OBJECT *   psObject;               // Any object sitting on the location (e.g. building)
@@ -68,20 +68,21 @@ struct MAPTILE
 	uint16_t        fireEndTime;            ///< The (uint16_t)(gameTime / GAME_TICKS_PER_UPDATE) that BITS_ON_FIRE should be cleared.
 	int32_t         waterLevel;             ///< At what height is the water for this tile
 	PlayerMask      jammerBits;             ///< bit per player, who is jamming tile
-	uint8_t         sensors[MAX_PLAYERS];   ///< player sees this tile with this many radar sensors
-	uint8_t         jammers[MAX_PLAYERS];   ///< player jams the tile with this many objects
+	uint16_t        sensors[MAX_PLAYERS];   ///< player sees this tile with this many radar sensors
+	uint16_t        jammers[MAX_PLAYERS];   ///< player jams the tile with this many objects
 
 	// DISPLAY ONLY (NOT for use in game calculations)
 	uint8_t         ground;                 ///< The ground type used for the terrain renderer
 	uint8_t         illumination;           // How bright is this tile? = diffuseSunLight * ambientOcclusion
 	uint8_t			ambientOcclusion;		// ambient occlusion. from 1 (max occlusion) to 254 (no occlusion), similar to illumination.
 	float           level;                  ///< The visibility level of the top left of the tile, for this client. for terrain lightmap
-	PIELIGHT        colour;					// color in terrain lightmap, based on tile.level and near light sources
 };
 
 /* The size and contents of the map */
 extern SDWORD	mapWidth, mapHeight;
 
+
+
 extern std::unique_ptr<MAPTILE[]> psMapTiles;
 extern float waterLevel;
 extern char *tilesetDir;
@@ -390,6 +391,7 @@ bool mapShutdown();
 /* Load the map data */
 bool mapLoad(char const *filename);
 struct ScriptMapData;
+bool loadTerrainTypeMap(const std::shared_ptr<WzMap::TerrainTypeData>& ttypeData);
 bool mapLoadFromWzMapData(std::shared_ptr<WzMap::MapData> mapData);
 
 // used to reload decal + ground types types when switching terrain overrides
@@ -404,10 +406,13 @@ public:
 	{ }
 public:
 	virtual std::unique_ptr<WzMap::BinaryIOStream> openBinaryStream(const std::string& filename, WzMap::BinaryIOStream::OpenMode mode) override;
-	virtual bool loadFullFile(const std::string& filename, std::vector<char>& fileData) override;
+	virtual WzMap::IOProvider::LoadFullFileResult loadFullFile(const std::string& filename, std::vector<char>& fileData, uint32_t maxFileSize = 0, bool appendNullCharacter = false) override;
 	virtual bool writeFullFile(const std::string& filename, const char *ppFileData, uint32_t fileSize) override;
 	virtual bool makeDirectory(const std::string& directoryPath) override;
 	virtual const char* pathSeparator() const override;
+	virtual bool fileExists(const std::string& filename) override;
+
+	bool folderExists(const std::string& dirPath);
 
 	virtual bool enumerateFiles(const std::string& basePath, const std::function<bool (const char* file)>& enumFunc) override;
 	virtual bool enumerateFolders(const std::string& basePath, const std::function<bool (const char* file)>& enumFunc) override;
@@ -523,6 +528,13 @@ WZ_DECL_ALWAYS_INLINE static inline bool worldOnMap(Vector2i pos)
 	return worldOnMap(pos.x, pos.y);
 }
 
+static inline void makeTileRubbleTexture(MAPTILE *psTile, const unsigned int x, const unsigned int y, const unsigned int newTexture)
+{
+	psTile->texture = TileNumber_texture(psTile->texture) | newTexture;
+	SET_TILE_DECAL(psTile);
+	markTileDirty(x, y);
+}
+
 
 /* Intersect a line with the map and report tile intersection points */
 bool map_Intersect(int *Cx, int *Cy, int *Vx, int *Vy, int *Sx, int *Sy);
@@ -563,7 +575,7 @@ bool fireOnLocation(unsigned int x, unsigned int y);
 
 /**
  * Transitive sensor check for tile. Has to be here rather than
- * visibility.h due to header include order issues.
+ * visibility.h due to header include order issues. -- (For DISPLAY-ONLY purposes - *NOT* game-state calculations!)
  */
 WZ_DECL_ALWAYS_INLINE static inline bool hasSensorOnTile(MAPTILE *psTile, unsigned player)
 {
