@@ -38,12 +38,13 @@ bool runMultiRequester(UDWORD id, UDWORD *mode, WzString *chosen, LEVEL_DATASET
 void displayRequestOption(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset);
 
 // multimenu
-void intProcessMultiMenu(UDWORD id);
-bool intRunMultiMenu();
 bool intCloseMultiMenu();
 void intCloseMultiMenuNoAnim();
 bool intAddMultiMenu();
 
+class IntFormAnimated;
+std::shared_ptr<IntFormAnimated> intCreateMultiMenuForm(std::function<void ()> closeButtonHandler = nullptr);
+
 void multiMenuScreenSizeDidChange(unsigned int oldWidth, unsigned int oldHeight, unsigned int newWidth, unsigned int newHeight);
 
 extern bool		MultiMenuUp;
