@@ -38,6 +38,7 @@
 #include "multistat.h"
 #include "multirecv.h"
 #include "stdinreader.h"
+#include "multilobbycommands.h"
 
 #include <array>
 
@@ -66,6 +67,11 @@ static std::array<optional<PingChallengeBytes>, MAX_CONNECTED_PLAYERS> pingChall
 // We use setMultiStats() to broadcast the score when needed.
 bool sendScoreCheck()
 {
+	if (game.blindMode != BLIND_MODE::NONE)
+	{
+		// no-op in blind mode
+		return false;
+	}
 	// Broadcast any changes in other players, but not in FRONTEND!!!
 	// Detection for this no longer uses title mode, but instead game mode, because that makes more sense
 	if (GetGameMode() == GS_NORMAL)
@@ -215,11 +221,11 @@ bool sendPing()
 			{
 				pingChallenges[i] = generatePingChallenge(i);
 
-				NETbeginEncode(NETnetQueue(i), NET_PING);
-				NETuint8_t(&player);
-				NETbool(&isNew);
-				NETbin(pingChallenges[i].value().data(), PING_CHALLENGE_BYTES);
-				NETend();
+				auto w = NETbeginEncode(NETnetQueue(i), NET_PING);
+				NETuint8_t(w, player);
+				NETbool(w, isNew);
+				NETbin(w, pingChallenges[i].value().data(), PING_CHALLENGE_BYTES);
+				NETend(w);
 
 				// Note when we sent the ping
 				PingSend[i] = realTime;
@@ -231,11 +237,11 @@ bool sendPing()
 		// Just generate and broadcast the same ping challenge to all other players
 		pingChallenges[0] = generatePingChallenge(0);
 
-		NETbeginEncode(NETbroadcastQueue(), NET_PING);
-		NETuint8_t(&player);
-		NETbool(&isNew);
-		NETbin(pingChallenges[0].value().data(), PING_CHALLENGE_BYTES);
-		NETend();
+		auto w = NETbeginEncode(NETbroadcastQueue(), NET_PING);
+		NETuint8_t(w, player);
+		NETbool(w, isNew);
+		NETbin(w, pingChallenges[0].value().data(), PING_CHALLENGE_BYTES);
+		NETend(w);
 
 		// Note when we sent the ping
 		for (int i = 0; i < MAX_CONNECTED_PLAYERS; ++i)
@@ -260,18 +266,18 @@ bool recvPing(NETQUEUE queue)
 	uint8_t challenge[PING_CHALLENGE_BYTES];
 	EcKey::Sig challengeResponse;
 
-	NETbeginDecode(queue, NET_PING);
-	NETuint8_t(&sender);
-	NETbool(&isNew);
+	auto r = NETbeginDecode(queue, NET_PING);
+	NETuint8_t(r, sender);
+	NETbool(r, isNew);
 	if (isNew)
 	{
-		NETbin(challenge, PING_CHALLENGE_BYTES);
+		NETbin(r, challenge, PING_CHALLENGE_BYTES);
 	}
 	else
 	{
-		NETbytes(&challengeResponse);
+		NETbytes(r, challengeResponse);
 	}
-	NETend();
+	NETend(r);
 
 	if (sender >= MAX_CONNECTED_PLAYERS)
 	{
@@ -288,16 +294,16 @@ bool recvPing(NETQUEUE queue)
 	// If this is a new ping, respond to it
 	if (isNew)
 	{
-		challengeResponse = getMultiStats(us).identity.sign(&challenge, PING_CHALLENGE_BYTES);
+		challengeResponse = getLocalSharedIdentity().sign(&challenge, PING_CHALLENGE_BYTES);
 
-		NETbeginEncode(NETnetQueue(sender), NET_PING);
+		auto w = NETbeginEncode(NETnetQueue(sender), NET_PING);
 		// We are responding to a new ping
 		isNew = false;
 
-		NETuint8_t(&us);
-		NETbool(&isNew);
-		NETbytes(&challengeResponse);
-		NETend();
+		NETuint8_t(w, us);
+		NETbool(w, isNew);
+		NETbytes(w, challengeResponse);
+		NETend(w);
 	}
 	// They are responding to one of our pings
 	else
@@ -330,6 +336,18 @@ bool recvPing(NETQUEUE queue)
 			// Record that they've verified the identity
 			ingame.VerifiedIdentity[sender] = true;
 
+			if (NetPlay.isHost && game.blindMode != BLIND_MODE::NONE)
+			{
+				// check if verified identity is an admin, and handle changes to admin status
+				bool oldIsAdminStatus = NetPlay.players[sender].isAdmin;
+				NetPlay.players[sender].isAdmin = identityMatchesAdmin(senderIdentity);
+				if (oldIsAdminStatus != NetPlay.players[sender].isAdmin)
+				{
+					// then send info about admin status changes to all players
+					NETBroadcastPlayerInfo(sender);
+				}
+			}
+
 			// Output to stdinterface, if enabled
 			std::string senderPublicKeyB64 = base64Encode(senderIdentity.toBytes(EcKey::Public));
 			std::string senderIdentityHash = senderIdentity.publicHashString();
