@@ -26,6 +26,15 @@
 
 #include "../multiplay.h"	// for JoinConnectionDescription
 
-bool startJoiningAttempt(char* playerName, std::vector<JoinConnectionDescription> connection_list, bool asSpectator = false);
+struct ExpectedHostProperties
+{
+	optional<EcKey::Key> hostPublicKey;
+	optional<std::string> gamePassword;
+};
+
+bool startJoiningAttempt(char* playerName, std::vector<JoinConnectionDescription> connection_list, bool asSpectator = false, const ExpectedHostProperties& expectedHostProps = ExpectedHostProperties());
+bool startJoinRedirectAttempt(char* playerName, std::vector<JoinConnectionDescription> connection_list, bool asSpectator = false, const ExpectedHostProperties& expectedHostProps = ExpectedHostProperties());
+void resetJoinRedirectTracking();
 void shutdownJoiningAttempt();
 
+std::shared_ptr<WIDGET> createJoiningIndeterminateProgressWidget(iV_fonts fontID);
