@@ -47,14 +47,14 @@
 #define DEFAULT_DROID_RESISTANCE	150
 
 /* The stores for the different stats */
-BODY_STATS		*asBodyStats;
-BRAIN_STATS		*asBrainStats;
-PROPULSION_STATS	*asPropulsionStats;
-SENSOR_STATS		*asSensorStats;
-ECM_STATS		*asECMStats;
-REPAIR_STATS		*asRepairStats;
-WEAPON_STATS		*asWeaponStats;
-CONSTRUCT_STATS		*asConstructStats;
+std::vector<BODY_STATS> asBodyStats;
+std::vector<BRAIN_STATS> asBrainStats;
+std::vector<PROPULSION_STATS> asPropulsionStats;
+std::vector<SENSOR_STATS> asSensorStats;
+std::vector<ECM_STATS> asECMStats;
+std::vector<REPAIR_STATS> asRepairStats;
+std::vector<WEAPON_STATS> asWeaponStats;
+std::vector<CONSTRUCT_STATS> asConstructStats;
 std::vector<PROPULSION_TYPES> asPropulsionTypes;
 static int		*asTerrainTable;
 
@@ -62,16 +62,6 @@ static int		*asTerrainTable;
 WEAPON_MODIFIER		asWeaponModifier[WE_NUMEFFECTS][PROPULSION_TYPE_NUM];
 WEAPON_MODIFIER		asWeaponModifierBody[WE_NUMEFFECTS][SIZE_NUM];
 
-/* The number of different stats stored */
-UDWORD		numBodyStats;
-UDWORD		numBrainStats;
-UDWORD		numPropulsionStats;
-UDWORD		numSensorStats;
-UDWORD		numECMStats;
-UDWORD		numRepairStats;
-UDWORD		numWeaponStats;
-UDWORD		numConstructStats;
-
 //stores for each players component states - can be either UNAVAILABLE, REDUNDANT, FOUND or AVAILABLE
 UBYTE		*apCompLists[MAX_PLAYERS][COMP_NUMCOMPONENTS];
 
@@ -80,6 +70,7 @@ UBYTE		*apStructTypeLists[MAX_PLAYERS];
 
 static std::unordered_map<WzString, BASE_STATS *> lookupStatPtr;
 static std::unordered_map<WzString, COMPONENT_STATS *> lookupCompStatPtr;
+static size_t statModelLoadingFailures = 0;
 
 static bool getMovementModel(const WzString &movementModel, MOVEMENT_MODEL *model);
 static bool statsGetAudioIDFromString(const WzString &szStatName, const WzString &szWavName, int *piWavID);
@@ -105,52 +96,38 @@ static void deallocTerrainTable()
 *******************************************************************************/
 
 /* Macro to allocate memory for a set of stats */
-#define ALLOC_STATS(numEntries, list, listSize, type) \
+#define ALLOC_STATS(numEntries, list, type) \
 	ASSERT((numEntries) < ~STAT_MASK + 1, "Number of stats entries too large for " #type);\
-	if ((list))	delete [] (list);	\
-	(list) = new type[numEntries]; \
-	(listSize) = (numEntries); \
+	if (!list.empty()) list.clear(); \
+	list.resize(numEntries); \
 	return true
 
-/*Macro to Deallocate stats*/
-#define STATS_DEALLOC(list, listSize) \
-	delete [] (list); \
-	listSize = 0; \
-	(list) = NULL
-
-void statsInitVars()
-{
-	/* The number of different stats stored */
-	numBodyStats = 0;
-	numBrainStats = 0;
-	numPropulsionStats = 0;
-	numSensorStats = 0;
-	numECMStats = 0;
-	numRepairStats = 0;
-	numWeaponStats = 0;
-	numConstructStats = 0;
-}
-
 /*Deallocate all the stats assigned from input data*/
 bool statsShutDown()
 {
 	lookupStatPtr.clear();
 	lookupCompStatPtr.clear();
 
-	STATS_DEALLOC(asWeaponStats, numWeaponStats);
-	STATS_DEALLOC(asBrainStats, numBrainStats);
-	STATS_DEALLOC(asPropulsionStats, numPropulsionStats);
-	STATS_DEALLOC(asRepairStats, numRepairStats);
-	STATS_DEALLOC(asConstructStats, numConstructStats);
-	STATS_DEALLOC(asECMStats, numECMStats);
-	STATS_DEALLOC(asSensorStats, numSensorStats);
-	STATS_DEALLOC(asBodyStats, numBodyStats);
+	asWeaponStats.clear();
+	asBrainStats.clear();
+	asPropulsionStats.clear();
+	asRepairStats.clear();
+	asConstructStats.clear();
+	asECMStats.clear();
+	asSensorStats.clear();
+	asBodyStats.clear();
 	deallocPropulsionTypes();
 	deallocTerrainTable();
 
+	statModelLoadingFailures = 0;
+
 	return true;
 }
 
+size_t getStatModelLoadingFailures()
+{
+	return statModelLoadingFailures;
+}
 
 /*******************************************************************************
 *		Allocate stats functions
@@ -158,44 +135,44 @@ bool statsShutDown()
 /* Allocate Weapon stats */
 bool statsAllocWeapons(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asWeaponStats, numWeaponStats, WEAPON_STATS);
+	ALLOC_STATS(numStats, asWeaponStats, WEAPON_STATS);
 }
 /* Allocate Body Stats */
 bool statsAllocBody(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asBodyStats, numBodyStats, BODY_STATS);
+	ALLOC_STATS(numStats, asBodyStats, BODY_STATS);
 }
 /* Allocate Brain Stats */
 bool statsAllocBrain(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asBrainStats, numBrainStats, BRAIN_STATS);
+	ALLOC_STATS(numStats, asBrainStats, BRAIN_STATS);
 }
 /* Allocate Propulsion Stats */
 bool statsAllocPropulsion(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asPropulsionStats, numPropulsionStats, PROPULSION_STATS);
+	ALLOC_STATS(numStats, asPropulsionStats, PROPULSION_STATS);
 }
 /* Allocate Sensor Stats */
 bool statsAllocSensor(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asSensorStats, numSensorStats, SENSOR_STATS);
+	ALLOC_STATS(numStats, asSensorStats, SENSOR_STATS);
 }
 /* Allocate Ecm Stats */
 bool statsAllocECM(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asECMStats, numECMStats, ECM_STATS);
+	ALLOC_STATS(numStats, asECMStats, ECM_STATS);
 }
 
 /* Allocate Repair Stats */
 bool statsAllocRepair(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asRepairStats, numRepairStats, REPAIR_STATS);
+	ALLOC_STATS(numStats, asRepairStats, REPAIR_STATS);
 }
 
 /* Allocate Construct Stats */
 bool statsAllocConstruct(UDWORD	numStats)
 {
-	ALLOC_STATS(numStats, asConstructStats, numConstructStats, CONSTRUCT_STATS);
+	ALLOC_STATS(numStats, asConstructStats, CONSTRUCT_STATS);
 }
 
 /*******************************************************************************
@@ -284,6 +261,10 @@ static iIMDBaseShape *statsGetIMD(WzConfig &json, BASE_STATS *psStats, const WzS
 		WzString filename = json_variant(value).toWzString();
 		deprecatedModelUpgrade(filename); // FUTURE TODO: Use output of this to auto-upgrade old model references? (Check for empty string)
 		retval = modelGet(filename);
+		if (retval == nullptr)
+		{
+			++statModelLoadingFailures;
+		}
 		ASSERT(retval != nullptr, "Cannot find the PIE model %s for stat %s in %s",
 		       filename.toUtf8().c_str(), getStatsName(psStats), json.fileName().toUtf8().c_str());
 	}
@@ -652,12 +633,12 @@ bool loadBodyStats(WzConfig &ini)
 	// separate function
 
 	// allocate space
-	for (int numStats = 0; numStats < numBodyStats; ++numStats)
+	for (int numStats = 0; numStats < asBodyStats.size(); ++numStats)
 	{
 		BODY_STATS *psBodyStat = &asBodyStats[numStats];
-		psBodyStat->ppIMDList.resize(numPropulsionStats * NUM_PROP_SIDES, nullptr);
-		psBodyStat->ppMoveIMDList.resize(numPropulsionStats * NUM_PROP_SIDES, nullptr);
-		psBodyStat->ppStillIMDList.resize(numPropulsionStats * NUM_PROP_SIDES, nullptr);
+		psBodyStat->ppIMDList.resize(asPropulsionStats.size() * NUM_PROP_SIDES, nullptr);
+		psBodyStat->ppMoveIMDList.resize(asPropulsionStats.size() * NUM_PROP_SIDES, nullptr);
+		psBodyStat->ppStillIMDList.resize(asPropulsionStats.size() * NUM_PROP_SIDES, nullptr);
 	}
 	for (size_t i = 0; i < list.size(); ++i)
 	{
@@ -673,7 +654,7 @@ bool loadBodyStats(WzConfig &ini)
 		}
 		ini.beginGroup("propulsionExtraModels");
 		//get the body stats
-		for (numStats = 0; numStats < numBodyStats; ++numStats)
+		for (numStats = 0; numStats < asBodyStats.size(); ++numStats)
 		{
 			psBodyStat = &asBodyStats[numStats];
 			if (list[i].compare(psBodyStat->id) == 0)
@@ -681,7 +662,7 @@ bool loadBodyStats(WzConfig &ini)
 				break;
 			}
 		}
-		if (numStats == numBodyStats) // not found
+		if (numStats == asBodyStats.size()) // not found
 		{
 			debug(LOG_FATAL, "Invalid body name %s", list[i].toUtf8().c_str());
 			return false;
@@ -689,7 +670,7 @@ bool loadBodyStats(WzConfig &ini)
 		std::vector<WzString> keys = ini.childKeys();
 		for (size_t j = 0; j < keys.size(); j++)
 		{
-			for (numStats = 0; numStats < numPropulsionStats; numStats++)
+			for (numStats = 0; numStats < asPropulsionStats.size(); numStats++)
 			{
 				PROPULSION_STATS *psPropulsionStat = &asPropulsionStats[numStats];
 				if (keys[j].compare(psPropulsionStat->id) == 0)
@@ -697,7 +678,7 @@ bool loadBodyStats(WzConfig &ini)
 					break;
 				}
 			}
-			if (numStats == numPropulsionStats)
+			if (numStats == asPropulsionStats.size())
 			{
 				debug(LOG_FATAL, "Invalid propulsion name %s", keys[j].toUtf8().c_str());
 				return false;
@@ -739,6 +720,17 @@ bool loadBrainStats(WzConfig &ini)
 		psStats->weight = ini.value("weight", 0).toInt();
 		psStats->base.maxDroids = ini.value("maxDroids").toInt();
 		psStats->base.maxDroidsMult = ini.value("maxDroidsMult").toInt();
+
+		auto cmdExpRange = ini.json("cmdExpRange");
+		if (!cmdExpRange.is_null())
+		{
+			ASSERT(cmdExpRange.is_array(), "cmdExpRange is not an array");
+			for (const auto& v : cmdExpRange)
+			{
+				psStats->cmdExpRange.push_back(v.get<int>());
+			}
+		}
+
 		auto rankNames = ini.json("ranks");
 		ASSERT(rankNames.is_array(), "ranks is not an array");
 		for (const auto& v : rankNames)
@@ -764,7 +756,7 @@ bool loadBrainStats(WzConfig &ini)
 			int weapon = getCompFromName(COMP_WEAPON, ini.value("turret").toWzString());
 			if (weapon >= 0)
 			{
-				psStats->psWeaponStat = asWeaponStats + weapon;
+				psStats->psWeaponStat = &asWeaponStats[weapon];
 			}
 			else
 			{
@@ -1688,88 +1680,88 @@ bool getWeaponClass(const WzString& weaponClassStr, WEAPON_CLASS *weaponClass)
 	ASSERT_OR_RETURN(retVal, player >= 0 && player < MAX_PLAYERS, "Invalid player: %" PRIu32 "", player);
 
 /*Access functions for the upgradeable stats of a weapon*/
-int weaponFirePause(const WEAPON_STATS *psStats, int player)
+int weaponFirePause(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].firePause;
+	return psStats.upgrade[player].firePause;
 }
 
 /* Reload time is reduced for weapons with salvo fire */
-int weaponReloadTime(const WEAPON_STATS *psStats, int player)
+int weaponReloadTime(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].reloadTime;
+	return psStats.upgrade[player].reloadTime;
 }
 
-int weaponLongHit(const WEAPON_STATS *psStats, int player)
+int weaponLongHit(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].hitChance;
+	return psStats.upgrade[player].hitChance;
 }
 
-int weaponShortHit(const WEAPON_STATS *psStats, int player)
+int weaponShortHit(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].shortHitChance;
+	return psStats.upgrade[player].shortHitChance;
 }
 
-int weaponDamage(const WEAPON_STATS *psStats, int player)
+int weaponDamage(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].damage;
+	return psStats.upgrade[player].damage;
 }
 
-int weaponRadDamage(const WEAPON_STATS *psStats, int player)
+int weaponRadDamage(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].radiusDamage;
+	return psStats.upgrade[player].radiusDamage;
 }
 
-int weaponPeriodicalDamage(const WEAPON_STATS *psStats, int player)
+int weaponPeriodicalDamage(const WEAPON_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].periodicalDamage;
+	return psStats.upgrade[player].periodicalDamage;
 }
 
-int sensorRange(const SENSOR_STATS *psStats, int player)
+int sensorRange(const SENSOR_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].range;
+	return psStats.upgrade[player].range;
 }
 
-int ecmRange(const ECM_STATS *psStats, int player)
+int ecmRange(const ECM_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].range;
+	return psStats.upgrade[player].range;
 }
 
-int repairPoints(const REPAIR_STATS *psStats, int player)
+int repairPoints(const REPAIR_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].repairPoints;
+	return psStats.upgrade[player].repairPoints;
 }
 
-int constructorPoints(const CONSTRUCT_STATS *psStats, int player)
+int constructorPoints(const CONSTRUCT_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].constructPoints;
+	return psStats.upgrade[player].constructPoints;
 }
 
-int bodyPower(const BODY_STATS *psStats, int player)
+int bodyPower(const BODY_STATS& psStats, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
-	return psStats->upgrade[player].power;
+	return psStats.upgrade[player].power;
 }
 
-int bodyArmour(const BODY_STATS *psStats, int player, WEAPON_CLASS weaponClass)
+int bodyArmour(const BODY_STATS& psStats, int player, WEAPON_CLASS weaponClass)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
 	switch (weaponClass)
 	{
 	case WC_KINETIC:
-		return psStats->upgrade[player].armour;
+		return psStats.upgrade[player].armour;
 	case WC_HEAT:
-		return psStats->upgrade[player].thermal;
+		return psStats.upgrade[player].thermal;
 	case WC_NUM_WEAPON_CLASSES:
 		break;
 	}
@@ -1778,17 +1770,17 @@ int bodyArmour(const BODY_STATS *psStats, int player, WEAPON_CLASS weaponClass)
 }
 
 //calculates the weapons ROF based on the fire pause and the salvos
-int weaponROF(const WEAPON_STATS *psStat, int player)
+int weaponROF(const WEAPON_STATS& psStat, int player)
 {
 	ASSERT_PLAYER_OR_RETURN(0, player);
 	int rof = 0;
 	// if there are salvos
 	if (player >= 0
-	    && psStat->upgrade[player].numRounds
-	    && psStat->upgrade[player].reloadTime != 0)
+	    && psStat.upgrade[player].numRounds
+	    && psStat.upgrade[player].reloadTime != 0)
 	{
 		// Rounds per salvo multiplied with the number of salvos per minute
-		rof = psStat->upgrade[player].numRounds * 60 * GAME_TICKS_PER_SEC / weaponReloadTime(psStat, player);
+		rof = psStat.upgrade[player].numRounds * 60 * GAME_TICKS_PER_SEC / weaponReloadTime(psStat, player);
 	}
 
 	if (rof == 0)
@@ -1838,8 +1830,8 @@ SENSOR_STATS *objActiveRadar(const BASE_OBJECT *psObj)
 			return nullptr;
 		}
 		compIndex = ((const DROID *)psObj)->asBits[COMP_SENSOR];
-		ASSERT_OR_RETURN(nullptr, compIndex < numSensorStats, "Invalid range referenced for numSensorStats, %d > %d", compIndex, numSensorStats);
-		psStats = asSensorStats + compIndex;
+		ASSERT_OR_RETURN(nullptr, compIndex < asSensorStats.size(), "Invalid range referenced for numSensorStats, %d > %zu", compIndex, asSensorStats.size());
+		psStats = &asSensorStats[compIndex];
 		break;
 	case OBJ_STRUCTURE:
 		psStats = ((const STRUCTURE *)psObj)->pStructureType->pSensor;
@@ -1865,7 +1857,7 @@ bool objRadarDetector(const BASE_OBJECT *psObj)
 	else if (psObj->type == OBJ_DROID)
 	{
 		const DROID *psDroid = (const DROID *)psObj;
-		SENSOR_STATS *psSensor = getSensorStats(psDroid);
+		SENSOR_STATS *psSensor = psDroid->getSensorStats();
 
 		return (psSensor && psSensor->type == RADAR_DETECTOR_SENSOR);
 	}
