@@ -21,12 +21,16 @@
 
 #include "lib/framework/wzglobal.h" // required for config.h
 #include "lib/framework/wzapp.h"
+#include "lib/framework/wzpaths.h"
 #include "lib/netplay/netpermissions.h"
 #include "multiint.h"
 #include "multistat.h"
+#include "multiplay.h"
 #include "multilobbycommands.h"
 #include "clparse.h"
 #include "main.h"
+#include "multivote.h"
+#include "hci/teamstrategy.h"
 
 #include <string>
 #include <atomic>
@@ -89,8 +93,9 @@ wzAsyncExecOnMainThread([]{ \
 
 static WZ_Command_Interface wz_cmd_interface = WZ_Command_Interface::None;
 static std::string wz_cmd_interface_param;
+static bool hasQueuedRoomStatusJSONOutput = false;
 
-WZ_Command_Interface wz_command_interface()
+inline WZ_Command_Interface wz_command_interface()
 {
 	return wz_cmd_interface;
 }
@@ -293,7 +298,9 @@ bool getInputLine(int fd, bool isSocketFd, optional<std::string> &nextLine)
 	}
 	if (bytesRead == 0)
 	{
-		return true;
+		// read() returned 0, or EOF - pipe closed?
+		debug(LOG_INFO, "read returned 0 / EOF - pipe closed?");
+		return false;
 	}
 	actualAvailableBytes += static_cast<size_t>(bytesRead);
 	nextLine = getNextLineFromBuffer();
@@ -467,6 +474,24 @@ int cmdOutputThreadFunc(void *)
 	return 0;
 }
 
+static bool checkPlayerIdentityMatchesString(const EcKey& identity, const std::string& playerIdentityStrCopy)
+{
+	if (identity.empty())
+	{
+		return (playerIdentityStrCopy == "0"); // special case for empty identity, in case that happens...
+	}
+
+	// Check playerIdentityStrCopy versus both the (b64) public key and the public hash
+	std::string checkIdentityHash = identity.publicHashString();
+	std::string checkPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
+	if (playerIdentityStrCopy == checkPublicKeyB64 || playerIdentityStrCopy == checkIdentityHash)
+	{
+		return true;
+	}
+
+	return false;
+}
+
 static bool applyToActivePlayerWithIdentity(const std::string& playerIdentityStrCopy, const std::function<void (uint32_t playerIdx)>& func)
 {
 	bool foundActivePlayer = false;
@@ -477,29 +502,10 @@ static bool applyToActivePlayerWithIdentity(const std::string& playerIdentityStr
 			continue;
 		}
 
-		bool matchingPlayer = false;
-		auto& identity = getMultiStats(i).identity;
-		if (identity.empty())
+		bool matchingPlayer = checkPlayerIdentityMatchesString(getMultiStats(i).identity, playerIdentityStrCopy);
+		if (!matchingPlayer && game.blindMode != BLIND_MODE::NONE)
 		{
-			if (playerIdentityStrCopy == "0") // special case for empty identity, in case that happens...
-			{
-				matchingPlayer = true;
-			}
-			else
-			{
-				continue;
-			}
-		}
-
-		if (!matchingPlayer)
-		{
-			// Check playerIdentityStrCopy versus both the (b64) public key and the public hash
-			std::string checkIdentityHash = identity.publicHashString();
-			std::string checkPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
-			if (playerIdentityStrCopy == checkPublicKeyB64 || playerIdentityStrCopy == checkIdentityHash)
-			{
-				matchingPlayer = true;
-			}
+			matchingPlayer = checkPlayerIdentityMatchesString(getVerifiedJoinIdentity(i), playerIdentityStrCopy);
 		}
 
 		if (matchingPlayer)
@@ -574,10 +580,11 @@ static bool changeHostChatPermissionsForActivePlayerWithIdentity(const std::stri
 		}
 		displayRoomSystemMessage(msg.c_str());
 
-		std::string playerPublicKeyB64 = base64Encode(getMultiStats(i).identity.toBytes(EcKey::Public));
-		std::string playerIdentityHash = getMultiStats(i).identity.publicHashString();
+		const auto& identity = getOutputPlayerIdentity(i);
+		std::string playerPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
+		std::string playerIdentityHash = identity.publicHashString();
 		std::string playerVerifiedStatus = (ingame.VerifiedIdentity[i]) ? "V" : "?";
-		std::string playerName = NetPlay.players[i].name;
+		std::string playerName = getPlayerName(i);
 		std::string playerNameB64 = base64Encode(std::vector<unsigned char>(playerName.begin(), playerName.end()));
 		wz_command_interface_output("WZEVENT: hostChatPermissions=%s: %" PRIu32 " %" PRIu32 " %s %s %s %s %s\n", (freeChatEnabled) ? "Y" : "N", i, gameTime, playerPublicKeyB64.c_str(), playerIdentityHash.c_str(), playerVerifiedStatus.c_str(), playerNameB64.c_str(), NetPlay.players[i].IPtextAddress);
 	});
@@ -597,7 +604,7 @@ static bool kickActivePlayerWithIdentity(const std::string& playerIdentityStrCop
 			wz_command_interface_output("WZCMD error: Can't kick host!\n");
 			return;
 		}
-		const char *pPlayerName = getPlayerName(i);
+		const char *pPlayerName = getPlayerName(i, true);
 		std::string playerNameStr = (pPlayerName) ? pPlayerName : (std::string("[p") + std::to_string(i) + "]");
 		kickPlayer(i, kickReasonStrCopy.c_str(), ERROR_KICKED, banPlayer);
 		auto KickMessage = astringf("Player %s was kicked by the administrator.", playerNameStr.c_str());
@@ -605,6 +612,108 @@ static bool kickActivePlayerWithIdentity(const std::string& playerIdentityStrCop
 	});
 }
 
+static optional<KickRedirectInfo> parseCmdInterfaceRedirectStringToRedirectInfo(const std::string& redirectStrCopy)
+{
+	// Parse the redirect string:
+	// <tcp/gns>:<new_port>:<0/1=spectator>:<optional:gamepassword>
+
+	JoinConnectionDescription::JoinConnectionType connectionType;
+	uint16_t newPort = 0;
+	bool asSpectator = false;
+
+	auto redirectComponents = splitAtAnyDelimiter(redirectStrCopy, ":");
+	ASSERT_OR_RETURN(nullopt, redirectComponents.size() >= 3, "Invalid redirect string");
+
+	auto optConnectionType = JoinConnectionDescription::connectiontype_from_string(redirectComponents[0]);
+	ASSERT_OR_RETURN(nullopt, optConnectionType.has_value(), "Unrecognized / unsupported connection type: \"%s\"", redirectComponents[0].c_str());
+	connectionType = optConnectionType.value();
+
+	try {
+		auto portNumber = std::stoul(redirectComponents[1], nullptr, 10);
+		ASSERT_OR_RETURN(nullopt, newPort < std::numeric_limits<uint16_t>::max(), "Invalid port: %ul", newPort);
+		newPort = static_cast<uint16_t>(portNumber);
+	}
+	catch (const std::exception& e) {
+		ASSERT_OR_RETURN(nullopt, false, "Invalid port specified: \"%s\"", redirectComponents[1].c_str());
+	}
+
+	ASSERT_OR_RETURN(nullopt, newPort > 1024, "Invalid port (%ul) - cannot redirect to privileged port <= 1024", static_cast<unsigned int>(newPort));
+
+	try {
+		auto specValue = std::stoul(redirectComponents[2], nullptr, 10);
+		asSpectator = specValue != 0;
+	}
+	catch (const std::exception& e) {
+		ASSERT_OR_RETURN(nullopt, false, "Invalid spec value specified: \"%s\"", redirectComponents[2].c_str());
+	}
+
+	std::string gamePassword;
+	for (size_t i = 3; i < redirectComponents.size(); ++i)
+	{
+		if (!gamePassword.empty())
+		{
+			gamePassword.push_back(':');
+		}
+		gamePassword += redirectComponents[i];
+	}
+
+	KickRedirectInfo redirectInfo;
+	redirectInfo.connList.push_back(JoinConnectionDescription(connectionType, "=", newPort)); // special "=" value is treated as "same address" on the client side
+	redirectInfo.gamePassword = gamePassword;
+	redirectInfo.asSpectator = asSpectator;
+	return redirectInfo;
+}
+
+static bool redirectActivePlayerWithIdentity(const std::string& playerIdentityStrCopy, const std::string& redirectStrCopy)
+{
+	ASSERT_OR_RETURN(false, ingame.localJoiningInProgress && !ingame.TimeEveryoneIsInGame.has_value(), "Game must not have started yet!");
+
+	// Parse the redirect string:
+	auto redirectInfoOpt = parseCmdInterfaceRedirectStringToRedirectInfo(redirectStrCopy);
+	if (!redirectInfoOpt.has_value())
+	{
+		return false;
+	}
+
+	return applyToActivePlayerWithIdentity(playerIdentityStrCopy, [&](uint32_t i) {
+		if (i == NetPlay.hostPlayer)
+		{
+			wz_command_interface_output("WZCMD error: Can't redirect host!\n");
+			return;
+		}
+		kickRedirectPlayer(i, redirectInfoOpt.value());
+	});
+}
+
+static bool chatActivePlayerWithIdentity(const std::string& playerIdentityStrCopy, const std::string& chatmsgstr)
+{
+	if (!NetPlay.isHostAlive)
+	{
+		// can't send this message when the host isn't alive
+		wz_command_interface_output("WZCMD error: Failed to send chat direct message because host isn't yet hosting!\n");
+	}
+
+	return applyToActivePlayerWithIdentity(playerIdentityStrCopy, [&](uint32_t i) {
+		sendRoomSystemMessageToSingleReceiver(chatmsgstr.c_str(), i);
+	});
+}
+
+static void handleCmdInterfaceConnectionClosed()
+{
+	if (!ingame.localJoiningInProgress || ingame.TimeEveryoneIsInGame.has_value())
+	{
+		// do nothing once game has fired up
+		return;
+	}
+
+	// if in lobby...
+	if (NETgetAsyncJoinApprovalRequired())
+	{
+		debug(LOG_INFO, "Shutting down lobby due to closed cmdinterface connection + async join approval required");
+		wzQuit(1);
+	}
+}
+
 int cmdInputThreadFunc(void *)
 {
 	fseek(stdin, 0, SEEK_END);
@@ -640,6 +749,9 @@ int cmdInputThreadFunc(void *)
 				if (!getInputLine(readFd, readFdIsSocket, nextLine))
 				{
 					errlog("WZCMD FAILURE: get input line failed! (did peer close the connection?)\n");
+					wzAsyncExecOnMainThread([]() {
+						handleCmdInterfaceConnectionClosed();
+					});
 					return 1;
 				}
 				break;
@@ -761,6 +873,35 @@ int cmdInputThreadFunc(void *)
 				});
 			}
 		}
+		else if(!strncmpl(line, "redirect identity "))
+		{
+			// redirect identity <identity> <tcp/gns>:<new_port>:<0/1=spectator>:<optional:gamepassword>
+			char playeridentitystring[1024] = {0};
+			char redirectstr[1024] = {0};
+			int r = sscanf(line, "redirect identity %1023s %1023[^\n]s", playeridentitystring, redirectstr);
+			if (r != 2)
+			{
+				wz_command_interface_output_onmainthread("WZCMD error: Failed to get player public key or hash, and/or redirect str!\n");
+			}
+			else
+			{
+				std::string playerIdentityStrCopy(playeridentitystring);
+				std::string redirectStrCopy = redirectstr;
+				wzAsyncExecOnMainThread([playerIdentityStrCopy, redirectStrCopy] {
+					if (!ingame.localJoiningInProgress || ingame.TimeEveryoneIsInGame.has_value())
+					{
+						// can't redirect once game has fired up - only in lobby
+						wz_command_interface_output("WZCMD error: Failed to execute redirect command - must be in lobby\n");
+						return;
+					}
+					bool foundActivePlayer = redirectActivePlayerWithIdentity(playerIdentityStrCopy, redirectStrCopy);
+					if (!foundActivePlayer)
+					{
+						wz_command_interface_output("WZCMD info: redirect identity %s: failed to find currently-connected player with matching public key or hash\n", playerIdentityStrCopy.c_str());
+					}
+				});
+			}
+		}
 		else if(!strncmpl(line, "permissions set connect:allow "))
 		{
 			char playeridentitystring[1024] = {0};
@@ -927,6 +1068,7 @@ int cmdInputThreadFunc(void *)
 			else
 			{
 				std::string chatmsgstr(chatmsg);
+				convertEscapedNewlines(chatmsgstr);
 				wzAsyncExecOnMainThread([chatmsgstr] {
 					if (!NetPlay.isHostAlive)
 					{
@@ -950,53 +1092,9 @@ int cmdInputThreadFunc(void *)
 			{
 				std::string playerIdentityStrCopy(playeridentitystring);
 				std::string chatmsgstr(chatmsg);
+				convertEscapedNewlines(chatmsgstr);
 				wzAsyncExecOnMainThread([playerIdentityStrCopy, chatmsgstr] {
-					if (!NetPlay.isHostAlive)
-					{
-						// can't send this message when the host isn't alive
-						wz_command_interface_output("WZCMD error: Failed to send chat direct message because host isn't yet hosting!\n");
-					}
-
-					bool foundActivePlayer = false;
-					for (uint32_t i = 0; i < MAX_CONNECTED_PLAYERS; i++)
-					{
-						auto player = NetPlay.players[i];
-						if (!isHumanPlayer(i))
-						{
-							continue;
-						}
-
-						bool msgThisPlayer = false;
-						auto& identity = getMultiStats(i).identity;
-						if (identity.empty())
-						{
-							if (playerIdentityStrCopy == "0") // special case for empty identity, in case that happens...
-							{
-								msgThisPlayer = true;
-							}
-							else
-							{
-								continue;
-							}
-						}
-
-						if (!msgThisPlayer)
-						{
-							// Check playerIdentityStrCopy versus both the (b64) public key and the public hash
-							std::string checkIdentityHash = identity.publicHashString();
-							std::string checkPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
-							if (playerIdentityStrCopy == checkPublicKeyB64 || playerIdentityStrCopy == checkIdentityHash)
-							{
-								msgThisPlayer = true;
-							}
-						}
-
-						if (msgThisPlayer)
-						{
-							sendRoomSystemMessageToSingleReceiver(chatmsgstr.c_str(), i);
-							foundActivePlayer = true;
-						}
-					}
+					bool foundActivePlayer = chatActivePlayerWithIdentity(playerIdentityStrCopy, chatmsgstr);
 					if (!foundActivePlayer)
 					{
 						wz_command_interface_output("WZCMD info: chat direct %s: failed to find currently-connected player with matching public key or hash\n", playerIdentityStrCopy.c_str());
@@ -1009,8 +1107,8 @@ int cmdInputThreadFunc(void *)
 			char action[1024] = {0};
 			char uniqueJoinID[1024] = {0};
 			char rejectionMessage[MAX_JOIN_REJECT_REASON] = {0};
-			unsigned int rejectionReason = static_cast<unsigned int>(ERROR_NOERROR);
-			int r = sscanf(line, "join %1023s %1023s %u %2047[^\n]s", action, uniqueJoinID, &rejectionReason, rejectionMessage);
+			unsigned int uintVal = static_cast<unsigned int>(ERROR_NOERROR);
+			int r = sscanf(line, "join %1023s %1023s %u %2047[^\n]s", action, uniqueJoinID, &uintVal, rejectionMessage);
 			if (r < 2 || r > 4)
 			{
 				wz_command_interface_output_onmainthread("WZCMD error: Failed to parse join command!\n");
@@ -1018,26 +1116,69 @@ int cmdInputThreadFunc(void *)
 			else
 			{
 				optional<AsyncJoinApprovalAction> approve = nullopt;
+				optional<uint8_t> explicitPlayerIdx = nullopt;
+				LOBBY_ERROR_TYPES rejectedReason = ERROR_NOERROR;
 				if (strcmp(action, "approve") == 0)
 				{
 					approve = AsyncJoinApprovalAction::Approve;
+					if (r >= 3)
+					{
+						explicitPlayerIdx = uintVal;
+					}
 				}
 				else if (strcmp(action, "reject") == 0)
 				{
 					approve = AsyncJoinApprovalAction::Reject;
+					if (uintVal < static_cast<unsigned int>(std::numeric_limits<uint8_t>::max()))
+					{
+						rejectedReason = static_cast<LOBBY_ERROR_TYPES>(uintVal);
+					}
 				}
 				else if (strcmp(action, "approvespec") == 0)
 				{
 					approve = AsyncJoinApprovalAction::ApproveSpectators;
 				}
-				if (approve.has_value() && rejectionReason < static_cast<unsigned int>(std::numeric_limits<uint8_t>::max()))
+				if (approve.has_value())
 				{
 					auto approveValue = approve.value();
 					std::string uniqueJoinIDCopy(uniqueJoinID);
 					std::string rejectionMessageCopy(rejectionMessage);
 					convertEscapedNewlines(rejectionMessageCopy);
-					wzAsyncExecOnMainThread([uniqueJoinIDCopy, approveValue, rejectionReason, rejectionMessageCopy] {
-						if (!NETsetAsyncJoinApprovalResult(uniqueJoinIDCopy, approveValue, static_cast<LOBBY_ERROR_TYPES>(rejectionReason), rejectionMessageCopy))
+					wzAsyncExecOnMainThread([uniqueJoinIDCopy, approveValue, explicitPlayerIdx, rejectedReason, rejectionMessageCopy]() mutable {
+
+						if (rejectedReason == ERROR_REDIRECT)
+						{
+							// Parse the rejection message as a cmdinterface redirect string
+							auto redirectInfoOpt = parseCmdInterfaceRedirectStringToRedirectInfo(rejectionMessageCopy);
+							if (redirectInfoOpt.has_value())
+							{
+								// Convert it to json redirect string
+								std::string redirectStr;
+								try {
+									nlohmann::json obj = redirectInfoOpt.value();
+									redirectStr = obj.dump(-1, ' ', false, nlohmann::json::error_handler_t::replace);
+								}
+								catch (const std::exception& e) {
+									debug(LOG_ERROR, "Failed to encode redirect string with error: %s", e.what());
+									redirectStr.clear();
+								}
+
+								if (!redirectStr.empty() && redirectStr.size() <= MAX_KICK_REASON)
+								{
+									rejectionMessageCopy = redirectStr;
+								}
+								else
+								{
+									wz_command_interface_output("WZCMD error: Encoded redirect string invalid length - redirect failed\n");
+								}
+							}
+							else
+							{
+								wz_command_interface_output("WZCMD error: Failed to parse redirect string\n");
+							}
+						}
+
+						if (!NETsetAsyncJoinApprovalResult(uniqueJoinIDCopy, approveValue, explicitPlayerIdx, rejectedReason, rejectionMessageCopy))
 						{
 							wz_command_interface_output("WZCMD info: Could not find currently-waiting join with specified uniqueJoinID\n");
 						}
@@ -1049,25 +1190,49 @@ int cmdInputThreadFunc(void *)
 				}
 			}
 		}
-		else if(!strncmpl(line, "autobalance"))
-		{
-			wzAsyncExecOnMainThread([] {
-				if (autoBalancePlayersCmd())
-				{
-					wz_command_interface_output("WZCMD info: autobalanced players\n");
-				}
-				else
-				{
-					wz_command_interface_output("WZCMD error: autobalance failed\n");
-				}
-			});
-		}
 		else if(!strncmpl(line, "status"))
 		{
 			wzAsyncExecOnMainThread([] {
 				wz_command_interface_output_room_status_json();
 			});
 		}
+		else if(!strncmpl(line, "set host ready "))
+		{
+			unsigned hostReadyVal = 0;
+			int r = sscanf(line, "set host ready %u", &hostReadyVal);
+			if (r != 1)
+			{
+				wz_command_interface_output_onmainthread("WZCMD error: Failed to get host ready value!\n");
+			}
+			else
+			{
+				bool hostReady = false;
+				if (hostReadyVal == 1 || hostReadyVal == 0)
+				{
+					hostReady = static_cast<bool>(hostReadyVal);
+				}
+				else
+				{
+					wz_command_interface_output_onmainthread("WZCMD error: Unsupported set host ready value!\n");
+					continue;
+				}
+
+				wzAsyncExecOnMainThread([hostReady] {
+					if (!NetPlay.isHostAlive)
+					{
+						wz_command_interface_output("WZCMD error: Unable to change host ready status because host isn't yet hosting!\n");
+						return;
+					}
+					if (!NetPlay.isHost)
+					{
+						wz_command_interface_output("WZCMD error: Unable to change host ready status when not the host!\n");
+						return;
+					}
+
+					sendReadyRequest(selectedPlayer, hostReady);
+				});
+			}
+		}
 		else if(!strncmpl(line, "shutdown now"))
 		{
 			inexit = true;
@@ -1469,8 +1634,29 @@ static void WzCmdInterfaceDumpHumanPlayerVarsImpl(uint32_t player, bool gameHasF
 
 	if (!gameHasFiredUp)
 	{
-		// in lobby, output "ready" status
+		// in lobby:
+		auto currentTime = std::chrono::steady_clock::now();
+
+		// output "ready" status
 		j["ready"] = static_cast<int>(p.ready);
+
+		if (NetPlay.isHost)
+		{
+			// output seconds since join
+			if (ingame.joinTimes[player].has_value())
+			{
+				j["joinedfor"] = std::chrono::duration_cast<std::chrono::seconds>(currentTime - ingame.joinTimes[player].value()).count();
+			}
+
+			// output seconds since _last_ ready (i.e. if currently ready, how long we've been ready this time)
+			if (p.ready && ingame.lastReadyTimes[player].has_value())
+			{
+				j["readyfor"] = std::chrono::duration_cast<std::chrono::seconds>(currentTime - ingame.lastReadyTimes[player].value()).count();
+			}
+
+			// output _total_ seconds spent not ready (at this snapshot)
+			j["notreadyfor"] = calculateSecondsNotReadyForPlayer(player, currentTime);
+		}
 	}
 	else
 	{
@@ -1493,10 +1679,15 @@ static void WzCmdInterfaceDumpHumanPlayerVarsImpl(uint32_t player, bool gameHasF
 		else
 		{
 			j["status"] = "left";
+
+			if (ingame.playerLeftGameTime[player].has_value())
+			{
+				j["playerLeftGameTime"] = ingame.playerLeftGameTime[player].value();
+			}
 		}
 	}
 
-	const auto& identity = getMultiStats(player).identity;
+	const auto& identity = (game.blindMode != BLIND_MODE::NONE) ? getVerifiedJoinIdentity(player) : getMultiStats(player).identity;
 	if (!identity.empty())
 	{
 		j["pk"] = base64Encode(identity.toBytes(EcKey::Public));
@@ -1522,15 +1713,27 @@ static void WzCmdInterfaceDumpHumanPlayerVarsImpl(uint32_t player, bool gameHasF
 	{
 		j["host"] = 1;
 	}
+
+	if (!gameHasFiredUp)
+	{
+		// in lobby, output player multiopt prefs
+		j["prefs"] = getMultiOptionPrefValuesJSON(player);
+	}
 }
 
-void wz_command_interface_output_room_status_json()
+void wz_command_interface_output_room_status_json(bool queued)
 {
 	if (!wz_command_interface_enabled())
 	{
 		return;
 	}
 
+	if (queued)
+	{
+		hasQueuedRoomStatusJSONOutput = true;
+		return;
+	}
+
 	bool gameHasFiredUp = (GetGameMode() == GS_NORMAL);
 
 	auto root = nlohmann::ordered_json::object();
@@ -1539,7 +1742,11 @@ void wz_command_interface_output_room_status_json()
 	auto data = nlohmann::ordered_json::object();
 	if (gameHasFiredUp)
 	{
-		if (ingame.TimeEveryoneIsInGame.has_value())
+		if (ingame.endTime.has_value())
+		{
+			data["state"] = "ended";
+		}
+		else if (ingame.TimeEveryoneIsInGame.has_value())
 		{
 			data["state"] = "started";
 		}
@@ -1557,10 +1764,12 @@ void wz_command_interface_output_room_status_json()
 		auto lobbyGameId = NET_getCurrentHostedLobbyGameId();
 		if (lobbyGameId != 0)
 		{
-			data["lobbyid"] = lobbyGameId;
+			data["lobbyid"] = std::to_string(lobbyGameId);
 		}
 	}
 	data["map"] = game.map;
+	data["blind"] = static_cast<uint8_t>(game.blindMode);
+	data["unixtime"] = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now().time_since_epoch()).count();
 
 	root["data"] = std::move(data);
 
@@ -1573,7 +1782,7 @@ void wz_command_interface_output_room_status_json()
 			auto j = nlohmann::ordered_json::object();
 
 			j["pos"] = p.position;
-			j["team"] = p.team;
+			j["team"] = checkedGetPlayerTeam(player);
 			j["col"] = p.colour;
 			j["fact"] = static_cast<int32_t>(p.faction);
 
@@ -1658,4 +1867,20 @@ void wz_command_interface_output_room_status_json()
 	std::string statusJSONStr = std::string("__WZROOMSTATUS__") + root.dump(-1, ' ', false, nlohmann::ordered_json::error_handler_t::replace) + "__ENDWZROOMSTATUS__";
 	statusJSONStr.append("\n");
 	wz_command_interface_output_str(statusJSONStr.c_str());
+
+	hasQueuedRoomStatusJSONOutput = false;
+}
+
+void wz_command_interface_process_queued_status_output()
+{
+	if (!wz_command_interface_enabled())
+	{
+		return;
+	}
+
+	if (hasQueuedRoomStatusJSONOutput)
+	{
+		wz_command_interface_output_room_status_json(false);
+		hasQueuedRoomStatusJSONOutput = false;
+	}
 }
